indexing

	description: "Pairs of objects."
	names: "pair", "tuple"
	representation: "fixed"
	access: "fixed"
	size: "fixed"
	contents: "generic"
	date: "$Date: 2006-06-23 10:34:45 -0400 (Fri, 23 Jun 2006) $"
	revision: "$Revision: 226 $"
	author: "Todd Plessel, US EPA/LM, plessel.todd@epa.gov"
	status: "reviewed", "tested"

class

	PAIR [G, H]

inherit

	ANY
		redefine
			is_equal
		end
	
creation
	make, put, replace
	
feature -- Initialization

	make, put, replace (the_first: G; the_second: H) is
			-- Make pair with the given components.
		do
			first := the_first
			second := the_second
		ensure
			first_set: first = the_first
			second_set: second = the_second
		end

feature -- Access

	first: G
			-- First item in pair.

	second: H
			-- Second item in pair.

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Do `Current' and other have equal-valued items?
		do
			Result := first.is_equal (other.first) and then
			          second.is_equal (other.second)
		end

feature -- Element change

	put_first, replace_first (new_first: G) is
			-- Replace first.
		do
			first := new_first
		ensure
			first_set: first = new_first
		end

	put_second, replace_second (new_second: H) is
			-- Replace second.
		do
			second := new_second
		ensure
			second_set: second = new_second
		end

end -- class PAIR
