indexing

	description: "Arrays that can be down-sized (i.e., whose count can be %
		% decreased though their area size remains at the 'high-water mark' %
		% until de/re-allocation via `discard_items' or `make'.)."
	names: "trim", "array", "resize", "shrink"
	representation: "array"
	access: "index"
	size: "fixed"
	contents: "generic"
	date: "$Date: 2006-06-23 10:34:45 -0400 (Fri, 23 Jun 2006) $"
	revision: "$Revision: 226 $"
	author: "Todd Plessel, US EPA/LM, plessel.todd@epa.gov"
	status: "tested, reviewed"

class

	TRIMMABLE_ARRAY [G]

inherit

	ARRAY [G]
		redefine
			resize
		end

creation

	make

feature -- Resizing

	resize (new_lower, new_upper: INTEGER) is
			-- Reset index range to [`new_lower', `new_upper'],
			-- effectively "removing" items outside the new range.
		require else
			new_upper_large_enough: new_upper >= new_lower - 1
		do
			if new_upper < new_lower then
				make (new_lower, new_upper)
			else
				Precursor (new_lower, new_upper)
				trim (new_upper)
			end
		ensure then
			lower_set: lower = new_lower
			upper_set: upper = new_upper
			count_set: count = 1 + new_upper - new_lower
			-- `item' (i) = old (`item' (i)) for i in [`new_lower', `new_upper']
		end

	trim (new_upper: INTEGER) is
			-- Reduce upper to `new_upper',
			-- effectively "removing" items outside of [`lower', `new_upper'].
		require
			new_upper_small_enough: new_upper <= upper
			new_upper_large_enough: new_upper >= lower - 1
		do
			if new_upper < lower then
				make (lower, new_upper)
			end

			upper := new_upper
		ensure
			upper_set: upper = new_upper
			-- `item' (i) = old (`item' (i)) for i in [`lower', `new_upper']
		end

end -- class TRIMMABLE_ARRAY
