indexing

	description: "Triples of objects."
	names: "triple", "tuple"
	representation: "fixed"
	access: "fixed"
	size: "fixed"
	contents: "generic"
	date: "$Date: 2006-07-27 17:30:52 -0400 (Thu, 27 Jul 2006) $"
	revision: "$Revision: 238 $"
	author: "Todd Plessel, US EPA/LM, plessel.todd@epa.gov"
	status: "reviewed", "tested"

class

	TRIPLE [T1, T2, T3]

inherit

	PAIR [T1, T2]
		rename
			make as pair_make,
			put as pair_put,
			replace as pair_replace
		export
			{NONE}
				pair_make, pair_put, pair_replace
		redefine
			is_equal
		end
creation
	default_create, make, put, replace


feature -- Initialization

	make, put, replace (the_first: T1; the_second: T2; the_third: T3) is
			-- Make triple with the given components.
		do
			pair_make (the_first, the_second)
			third := the_third
		ensure then
			first_set: first = the_first
			second_set: second = the_second
			third_set: third = the_third
		end

feature -- Access

	third: T3
			-- Third item in triple.

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Do `Current' and other have equal-valued items?
		do
			Result := first.is_equal (other.first) and then
						second.is_equal (other.second) and then
						third.is_equal (other.third)
		end

feature -- Element change

	put_third, replace_third (new_third: T3) is
			-- Replace third.
		do
			third := new_third
		ensure
			third_set: third = new_third
		end

end -- class TRIPLE
