

indexing
	description: "Provides facilities for computing memory usage"
	patterns: "none"
	date: "$Date: 2006-12-08 13:32:17 -0500 (Fri, 08 Dec 2006) $"
	revision: "$Revision: 273 $"
	status: "unreviewed"




class 

	MEMORY_REPORTER

inherit

	ANY
	

	INTERNAL
		rename
			class_name as internal_class_name
		export
			{NONE} all
		end


	MEMORY
		export
			{NONE} all
		end
	
feature -- Access
	
	class_name: STRING is
			-- Returns the name of the class
		do
			Result := generator
		ensure then
			Result_set: Result /= Void
			Result_has_non_zero_length: Result.count > 1
		end 

feature -- Measurement
	
	total_memory_usage: INTEGER is
			-- Compute the total memory for the system
		do

			Result := memory_statistics (Total_memory).type



		ensure
			result_greater_than_zero: Result >= 0
		end
	
	memory_usage: INTEGER is
			-- Compute the memory usage for `Current' using fields
		do
			Result := object_memory_usage (Current)
			Result := Result + detailed_memory_usage
			
			debug ("memory")
				io.put_string ("Size using fields is: ")
				io.put_integer (Result)
				io.put_string ("%N%N")
			end
		ensure
			result_greater_than_zero: Result >= 0
		end
	
feature {NONE} -- Implementation
	
	detailed_memory_usage: INTEGER is
			-- Compute additional memory usage as required
		do
			Result := 0
		ensure 
			Result_positive: Result >= 0
		end 










	object_memory_usage (object: ANY): INTEGER is
			-- Compute the memory usage for `object' using fields
		local
			i: INTEGER
			the_size, field_size: INTEGER
			a: TO_SPECIAL [ANY]
		do
			from
				i := 1
				the_size := 4
				debug ("memory") 
					io.put_string ("%N")
					io.put_string (internal_class_name (object))
					io.put_string ("%N")
				end
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + field_count (object)
			variant
				i_increasing: 1 + field_count (object) - i
			until
				i > field_count (object)
			loop
				
				debug ("memory")
					io.put_string ("%T")
					io.put_string (field_name (i, object))
					io.put_string (": ")
				end
				
				field_size := 0
				if field (i, object) /= Void then
					field_size := physical_size (field (i, object))
					if field_type (i, object) = Reference_type then
						a ?= field (i, object)
						if a /= Void then
							field_size := field_size + physical_size (a.area)
						end
					end
				else
					-- This assumes that the size of the type tag is 4
					field_size := field_size + 4
				end
				
				debug ("memory")
					io.put_integer (field_size)
					io.put_string ("%N")
				end
				
				the_size := the_size + field_size
				
				i := i + 1
			end
			
			Result := the_size
		ensure
			result_greater_than_zero: Result > 0
		end

	
end -- MEMORY_REPORTER

