
indexing
	description: "Class for gathering timing statistics"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	TIMER

inherit
	MEMORY
		redefine
			dispose, default_create
		end

creation
	default_create

feature -- Initialization

	default_create is
			-- Initialize the timer structure
		do
			Precursor
			
			start_time := 0
			stop_time := 0
			timer_stopped := true

			if tms_pointer /= default_pointer then
				dispose
			end
			
			tms_pointer := c_malloc (tms_structure_size)
		ensure then
			tms_pointer_set: tms_pointer /= default_pointer
		end

feature -- Access

	timer_stopped: BOOLEAN
			-- Has the timer been stopped
	
	elapsed_time: DOUBLE is
			-- The elapsed time between calls to `start_timer' and `stop_timer'
		require 
			timer_stopped_has_been_stopped: timer_stopped
		do
			Result := (stop_time - start_time) / c_clock_ticks
		ensure 
			Result_positive: Result >= 0
		end 
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal

	dispose is
			-- Remove external memory
		do
			c_free (tms_pointer)
			tms_pointer := default_pointer
		end 

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	start_timer is
			-- Start the timer
		do
			timer_stopped := false
			start_time := c_times (tms_pointer)
		ensure 
			start_time_not_zero: start_time > 0
			timer_is_running:  not timer_stopped		
		end 

	stop_timer is
			-- Start the timer
		do
			stop_time := c_times (tms_pointer)
			timer_stopped := true
		ensure 
			stop_time_not_zero: stop_time > 0
			timer_stopped: timer_stopped		
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	start_time: INTEGER_64
	stop_time: INTEGER_64

	tms_pointer: POINTER
	
	c_times (p: POINTER): INTEGER_64 is
			-- Set current date and time in `p', pointer to a `struct tms' area.
		external
			"C macro use <sys/times.h>"
		alias
			"times"
		end

	c_clock_ticks: INTEGER_64 is
			-- Set current date and time in `p', pointer to a `struct timeb' area.
		external
			"C macro use  <unistd.h>"
		alias
			"sysconf(_SC_CLK_TCK)"
		end

feature {NONE} -- `struct tms' encapsulation
		
	tms_structure_size: INTEGER is
			-- Size of `struct timeb'.
		external
			"C macro use <sys/times.h>"
		alias
			"sizeof(struct tms)"
		end
		
		c_calloc (nmemb, size: INTEGER): POINTER is
						-- C calloc
				external
						"C macro use <stdlib.h>"
				alias
						"calloc"
				end

		c_malloc (size: INTEGER): POINTER is
						-- C malloc
				external
						"C macro use  <stdlib.h>"
				alias
						"malloc"
				end

		c_free (p: POINTER) is
						-- C free
				external
						"C macro use  <stdlib.h>"
				alias
						"free"
				end

invariant
	tms_pointer_not_default: tms_pointer /= default_pointer

end -- TIMER

