

indexing
	description: "Colors represented as rgb triples"
	names: "RGB", "HSV"
	representation: "fixed"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"
	patterns: "none"


class 

	RGB_COLOR

inherit

	ANY
		undefine
			default_create, copy, out, is_equal
		end
	
	LINEAR_TUPLE_3D
		rename
			x as red, y as green, z as blue, 
			make as linear_tuple_make, fill as linear_tuple_fill,
			zero as black, unit as white, is_zero as is_black,
			put as linear_tuple_put, scale as linear_tuple_scale,
			magnitude as luminance
			
		export
			{RGB_COLOR}
			infix "#", prefix "-", prefix "+", linear_tuple_make,
			linear_tuple_fill, two_norm, magnitude_squared,
			one_norm, max_norm, put_x, put_y, put_z, normalize,
			all_finite, linear_tuple_scale
		redefine
			default_create
		end
	
creation
	default_create, make
	
feature -- Initialization
	
	default_create is
			-- Create color as white 
		do
			make (1, 1, 1)
		end 

	make (initial_red, initial_green, initial_blue: DOUBLE) is
			-- Initial cartesian component coordinates.
		require
			first_argument_in_range: initial_red >= 0.0
			second_argument_in_range: initial_green >= 0.0
			third_argument_in_range: initial_blue >= 0.0
		do
			linear_tuple_make (initial_red, initial_green, initial_blue)
		end

	fill (initial_value: DOUBLE) is
			-- Set each `item' to `initial_value'.
		require
			initial_value_in_range: initial_value >= 0.0
		do
			linear_tuple_fill (initial_value)
		end

feature -- Access
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	put_red (new_red: DOUBLE) is
			-- Set the red component.
		require
			new_red_in_range: new_red >= 0.0
		do
			put_x (new_red)
		end

	put_green (new_green: DOUBLE) is
			-- Set the green component.
		require
			new_green_in_range: new_green >= 0.0
		do
			put_y (new_green)
		end

	put_blue (new_blue: DOUBLE) is
			-- Set the blue component.
		require
			new_blue_in_range: new_blue >= 0.0
		do
			put_z (new_blue)
		end

	put (value: DOUBLE; index: INTEGER) is
			-- Assign `value' to component at `index'.
		require
			value_in_range: value >= 0.0
		do
			linear_tuple_put (value, index)
		end
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
	
	to_rgba: RGBA_COLOR is
			-- Convert `Current' to an RGBA with an alpha of 1
		do
			Result.make (red, green, blue, 1.0)
		end
	
feature -- Duplication
feature -- Basic operations
	
	scale, infix "*" (scalar: DOUBLE): RGB_COLOR is
			-- Scale `Current' by `scalar'
		require
			scalar_is_positive: scalar >= 0.0
		do
--			Result.make (clamp_to_range (red * scalar, 0.0, 1.0),
--						 clamp_to_range (green * scalar, 0.0, 1.0),
--						 clamp_to_range (blue * scalar, 0.0, 1.0))
			create Result.make (red * scalar, green * scalar, blue * scalar)
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	Opening_out_delimiter: CHARACTER is '('
			-- First character of result of `out'.

	Closing_out_delimiter: CHARACTER is ')'
			-- Second character of result of `out'.

invariant
	
	red_component_in_range: red >= 0.0
	green_component_in_range: green >= 0.0
	blue_component_in_range: blue >= 0.0
	
end -- RGB_COLOR

