

indexing
	description: "OpenGL based SceneGraph viewer"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"


class

	SCENE_VIEWER

inherit

	GTKGL_WIDGET
		rename
			make as widget_make
		end

	GTK_SHARED_MAIN
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

	GTKGL_SHARED_INIT
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

	EWG_IMPORTED_EXTERNAL_ROUTINES
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

	KL_SHARED_EXCEPTIONS
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

	GDK_EVENT_MASK_ENUM_EXTERNAL
		export
			{NONE} all
		undefine
			default_create, is_equal, copy, is_valid_enum
		end

	GDK_MODIFIER_TYPE_ENUM_EXTERNAL
		rename
			is_valid_enum as is_valid_event_mask_enum
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

	KEY_CODES
		export
			{NONE} all
		undefine
			is_equal, copy
		end

	NUMERIC_CONVERSION
		export
			{NONE} all
		undefine
			default_create, is_equal, copy, out
		end

creation
	make, make_with_camera

feature -- Initialization

	make is
			-- Create a viewer.
		local
			c: SCENE_PERSPECTIVE_CAMERA
		do
			create c
			c.set_position_xyz (0, 0, 8)
			c.set_clipping (1.0, 100.0)

			make_with_camera (c)
		end

	make_with_camera (c: SCENE_CAMERA) is
			-- Create a viewer.
		require
			camera_not_void: c /= Void
			toolkit_initialized: gtk_main.is_toolkit_initialized
		local
			g: SCENE_GROUP
			color: RGBA_COLOR
			color_element: SCENE_COLOR_ELEMENT
		do
			default_create

			first_pass := true

			if not gtkgl_init.is_toolkit_initialized then
				gtkgl_init.initialize_toolkit
			end

			if not gtkgl_init.is_toolkit_initialized then
				print ("Unable to init GTKGL%N")
				Exceptions.die (1)
			end

			widget_make
			request_size (c.x_resolution, c.y_resolution)

			-- quaternions
			create view_quat.make (1, 4)
			create view_quat_diff.make (1, 4)

			in_motion := false

			create root_node.make

			-- Setup the arcball
			create arcball.make (c.x_resolution, c.y_resolution)
			create tmp_pt
			create last_pt

			-- Add the event handlers
			create expose_event_signal_receiver.make (Current)
			connect_expose_event_signal_receiver (expose_event_signal_receiver)

			--create configure_event_signal_receiver.make (Current, arcball)
			--connect_configure_event_signal_receiver (configure_event_signal_receiver)

			create button_press_event_signal_receiver.make (Current)
			connect_button_press_event_signal_receiver (button_press_event_signal_receiver)

			create motion_notify_event_signal_receiver.make (Current)
			connect_motion_notify_event_signal_receiver (motion_notify_event_signal_receiver)

			set_events (event_flags)

			--create scale_xform
			-- Swap to right-handed coordinates
			--scale_xform.scale (1, 1, -1)
			--c.set_world_to_camera (scale_xform)

			set_camera (c)
			create g.make
			root_node.add_child (view_axis)
			root_node.add_child (g)

			create gl_drawer
			create culler
			create scene_ambient_color.make (0.2, 0.2, 0.2,1)
----			create scene_ambient_color.make (0.2,0.2,0.2,1)

			create color.make (1, 1, 0.5, 1)
			create color_element.make (color)

			create information_string
			information_string.replace_element (color_element)
			information_string.set_screenspace_camera (camera)
			information_string.set_screenspace_drawing (true)

			create tmp_string.make (10)

			create timer

			create formatter.make (6, 2)

			show_as_frames_per_sec := true
			show_information := false

			max_time_samples := 10
			time_sample_index := 1
			time_sample_count := 0
			create time_samples.make (1, max_time_samples)

			timer.start_timer
		end

feature -- Access

	scene_graph: SCENE_GROUP is
			-- The root of the scene_graph
		do
			Result ?= root_node.item (2)
			check
				item_2_is_a_group: Result /= Void
			end
		end

	camera: SCENE_CAMERA
			-- The camera for viewing

	viewing_axis_visible: BOOLEAN is
			-- Are the viewing axes visible?
		do
			Result := view_axis.traversable
		end

	scene_ambient_color: RGBA_COLOR
			-- The color of the global scene ambience

	culler: CULL_ACTION
			-- The culling traverser

feature -- Measurement
feature -- Comparison
feature -- Status report

	max_time_samples: INTEGER
			-- The number of time samples to use in the window

feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_scene_ambient_color (new_color: RGBA_COLOR) is
			-- Set `scene_ambient_color' to `new_color'
		do
			scene_ambient_color := new_color
		ensure
			scene_ambient_color_set: scene_ambient_color = new_color
		end

	set_scene_graph (root: SCENE_GROUP) is
			-- Set the child of `root_node' to `root'
		require
			root_not_void: root /= Void
		local
			g: SCENE_GROUP
		do
			g ?= root_node.item (2)
			check
				item_2_is_a_group: g /= Void
			end

			if g.count = 0 then
				g.add_child (root)
			else
				g.replace_child (root, 1)
			end
		end

	set_camera (new_camera: SCENE_CAMERA) is
			-- Set `camera' to `new_camera'
		require
			new_camera_not_void: new_camera /= Void
		do
			camera := new_camera
--			container_widget.set_minimum_size (camera.x_resolution,
--										   camera.y_resolution)
			arcball.set_width (camera.x_resolution)
			arcball.set_height (camera.y_resolution)
			request_size (camera.x_resolution, camera.y_resolution)
		ensure
			camera_set: camera = new_camera
		end

	viewing_axis_visibility (on: BOOLEAN) is
			-- Set the viewing axis visibility
		do
			view_axis.set_traversable (on)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	on_key_press_event (a_widget: GTK_WIDGET; a_event: GDK_KEY_EVENT): BOOLEAN is
			-- Handle key press events
		do
			if a_event.keyval = key_lower_v.code then
				view_all
				a_widget.queue_draw
			elseif a_event.keyval = key_upper_f.code then
				show_information := not show_information
				a_widget.queue_draw
			elseif a_event.keyval = key_upper_p.code then
				show_as_frames_per_sec := not show_as_frames_per_sec
				a_widget.queue_draw
			elseif a_event.keyval = key_upper_d.code then
				gl_drawer.set_display_list_usage (not gl_drawer.use_display_lists)
				a_widget.queue_draw
			elseif a_event.keyval = key_upper_b.code then
				gl_drawer.set_show_extents (not gl_drawer.show_extents)
				viewing_axis_visibility (not gl_drawer.show_extents)
				a_widget.queue_draw
			end

			Result := true
		end

	view_all is
			-- Center the scene graph in the viewer
		do
			camera.view_all (scene_graph)
		end


feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SCENE_VIEWER_MOTION_NOTIFY_EVENT_SIGNAL_RECEIVER} -- Implementation

	on_motion_notify_event (a_widget: GTK_WIDGET; a_motion_event: GDK_MOTION_EVENT): BOOLEAN is
			-- Called when the mouse button is moved
		local
			x, y: INTEGER
			the_state: INTEGER
			xform: TRANSFORM
		do
			if a_motion_event.is_hint then
				a_motion_event.window.determine_pointer
				x := a_motion_event.window.last_pointer_x
				y := a_motion_event.window.last_pointer_y
				the_state := a_motion_event.window.last_pointer_state
			else
				x := real_64_truncated_to_integer (a_motion_event.x.rounded)
				y := real_64_truncated_to_integer (a_motion_event.y.rounded)
				the_state := a_motion_event.state
			end

			if external_memory.bitwise_integer_and_external (the_state, gdk_button1_mask) /= 0 then
				tmp_pt.make (x, y)
				arcball.rotate_event (tmp_pt)


				xform := arcball.trackball_matrix

				debug ("gui")
					io.put_string ("%NMotion Transform%N")
					io.put_string (xform.out)
					io.put_string ("%N")
				end

				rotate_camera (xform)

				camera.update_view_volume
				a_widget.queue_draw

			elseif external_memory.bitwise_integer_and_external (the_state, gdk_button2_mask) /= 0 then
				tmp_pt.make (x, y)
				arcball.pan_event (camera, tmp_pt)

				a_widget.queue_draw
			elseif external_memory.bitwise_integer_and_external (the_state, gdk_button3_mask) /= 0 then
				tmp_pt.make (x, y)
				arcball.dolly_event (camera, tmp_pt)

				a_widget.queue_draw
			end

			Result := true
		end

feature {SCENE_VIEWER_BUTTON_PRESS_EVENT_SIGNAL_RECEIVER} -- Implementation

	on_button_press_event (a_widget: GTK_WIDGET; a_event: GDK_BUTTON_EVENT): BOOLEAN is
			-- Called when a mouse button is pressed
		do
			last_pt.make (a_event.x, a_event.y)
			arcball.start_motion (last_pt)

			Result := true
		end

feature {SCENE_VIEWER_EXPOSE_EVENT_SIGNAL_RECEIVER} -- Implementation

	on_expose_event (a_widget: GTK_WIDGET; a_event: GDK_EXPOSE_EVENT): BOOLEAN is
			-- Called when we need to redraw the GTKGL_widget
		local
			the_time: DOUBLE
			i: INTEGER
			draw_pool: SCENE_DRAWABLE_POOL
		do
			if gl_drawer.context_enabled then
				if first_pass then
					initialize_shaders
					first_pass := false
				end

				culler.default_create

				root_node.traverse (culler)

				gl_drawer.begin_statistics

				if gl_begin then
					gl_drawer.enable_two_sided_lighting
					gl_drawer.set_scene_ambient_color (scene_ambient_color)
					gl_drawer.enable (gl_drawer.rd_normalize)
					gl_drawer.disable (gl_drawer.rd_cull_face)
					gl_drawer.enable (gl_drawer.rd_depth_test)
					gl_drawer.enable (gl_drawer.rd_lighting)
					gl_drawer.clear (gl_drawer.integer_or (gl_drawer.rd_color_buffer_bit,
														   gl_drawer.rd_depth_buffer_bit))

					gl_drawer.set_global_lights (culler.global_lights)
					gl_drawer.global_lights.reset (gl_drawer)
					draw_pool ?= culler.pool

						check
							pool_not_void: draw_pool /= Void
						end

					camera.draw (draw_pool, gl_drawer)

					timer.stop_timer

					the_time := 1.0 / timer.elapsed_time
					time_samples.put (the_time, time_sample_index)
					time_sample_count := max_time_samples.min (time_sample_count + 1)
					time_sample_index := time_sample_index + 1
					if time_sample_index = max_time_samples + 1 then
						time_sample_index := 1
					end

					if show_information then
						from
							i := 1
							the_time := 0.0
						invariant
							i_at_least_one: i >= 1
							i_small_enough: i <= 1 + time_sample_count
						variant
							i_increasing: 1 + time_sample_count - i
						until
							i >= time_sample_count
						loop
							the_time := the_time + time_samples.item (i)
							i := i + 1
						end

						the_time := the_time / time_sample_count

						if show_as_frames_per_sec then
							output_information_string ("Frames per sec", the_time)
						else
							the_time := the_time * gl_drawer.polygon_count
							output_information_string ("Polygons per sec", the_time)
						end
					end

					timer.start_timer

					swap_buffers

					debug ("gui")
						io.put_string ("Polygon count = ")
						io.put_integer (gl_drawer.polygon_count)
						io.put_string ("%N")
					end

					gl_end

					Result := true
				end
			else
				Result := gl_begin
				gl_end
				-- Force a redraw now that the context is initialized
				a_widget.queue_draw
			end
		end

feature {NONE} -- Implementation

	time_samples: ARRAY [DOUBLE]
	time_sample_index: INTEGER
	time_sample_count: INTEGER

	root_node: SCENE_TRANSFORM
			-- The root of the scene_graph

	arcball: TRACKBALL
			-- Arcball to map mouse clicks to scene interaction

	expose_event_signal_receiver: SCENE_VIEWER_EXPOSE_EVENT_SIGNAL_RECEIVER
	button_press_event_signal_receiver: SCENE_VIEWER_BUTTON_PRESS_EVENT_SIGNAL_RECEIVER
	motion_notify_event_signal_receiver: SCENE_VIEWER_MOTION_NOTIFY_EVENT_SIGNAL_RECEIVER
	configure_event_signal_receiver: GL_WIDGET_CONFIGURE_EVENT_SIGNAL_RECEIVER
			-- Receivers for event signals

	timer: TIMER

	gl_drawer: GL

	in_motion: BOOLEAN

	first_pass: BOOLEAN

	formatter: DOUBLE_FORMATTER

	tmp_pt, last_pt: POINT2
	view_quat_diff, view_quat: ARRAY [REAL]
	begin_x, begin_y: INTEGER
			-- Local variables for arcball

	tmp_string: STRING

	show_information: BOOLEAN

	show_as_frames_per_sec: BOOLEAN

	information_string: SCENE_BILLBOARD_TEXT

	event_flags: INTEGER is
		do
			Result := external_memory.bitwise_integer_or_external (gdk_exposure_mask, gdk_leave_notify_mask)
			Result := external_memory.bitwise_integer_or_external (Result, gdk_button_press_mask)
			Result := external_memory.bitwise_integer_or_external (Result, gdk_pointer_motion_mask)
			Result := external_memory.bitwise_integer_or_external (Result, gdk_pointer_motion_hint_mask)
			Result := external_memory.bitwise_integer_or_external (Result, gdk_key_press_mask)
		end

	initialize_shaders is
			-- Force the initialization of any shaders in the scene
			-- graph
		local
			shader_action: GLSL_SHADER_ACTION
			shader_list: DS_ARRAYED_LIST [SCENE_GEOMETRY]
			geometry: SCENE_GEOMETRY
			shader: SCENE_SHADER_ELEMENT
			glsl_shader: SCENE_GLSL_SHADER_ELEMENT
		do
			gl_drawer.initialize_extensions
			if gl_drawer.extensions_initialized then
				io.put_string ("Extensions successfully initialized!")
				io.put_string ("%N")
				create shader_action
				root_node.traverse (shader_action)
				if shader_action.count > 0 then
					shader_list := shader_action.shader_list
					from
						shader_list.start
					until
						shader_list.after
					loop
						geometry := shader_list.item_for_iteration
						if geometry.valid_key ("SCENE_GLSL_SHADER_ELEMENT") then
							glsl_shader ?= geometry.element_item ("SCENE_GLSL_SHADER_ELEMENT")
							glsl_shader.initialize_shaders (gl_drawer)
							set_parameters
							glsl_shader.enable_shader (true)
						elseif geometry.valid_key ("SCENE_SHADER_ELEMENT") then
							-- Create a GLSL Shader from the
							-- RenderMan version
							shader ?= geometry.element_item ("SCENE_SHADER_ELEMENT")
							create glsl_shader.make_from_shader (shader)
							glsl_shader.initialize_shaders (gl_drawer)
							glsl_shader.create_parameters (gl_drawer)
							glsl_shader.enable_shader (true)
							geometry.replace_element (glsl_shader)
						end
						shader_list.forth
					end
				end
			end
		end

	set_parameters is
			-- Override in sub-classes to set GLSL uniform parameters
		do

		end

	output_information_string (tag: STRING; value: DOUBLE) is
			-- Output an annotation string to the display
		do
			tmp_string.wipe_out
			tmp_string.append (tag)
			tmp_string.append (" ")
			tmp_string.append_string (formatter.formatted (value))
			information_string.set_text_string (tmp_string)
			information_string.draw (gl_drawer)
		end

	view_axis: SCENE_TRANSFORM is
			-- A set of X, Y, Z axis at the view center
		local
			pt1, pt2: POINT3
			sl: SCENE_LINES
			pl: POLYLINE
			pl_array: DS_ARRAYED_LIST [POLYLINE]
			shp: SCENE_SHAPE
			c: RGBA_COLOR
			appearance: SCENE_COLOR_ELEMENT
		once
			create Result.make
			create pl_array.make (3)

			-- X axis
			create pt1.make (0, 0, 0)
			create pt2.make (1, 0, 0)
			create pl.make_simple (pt1, pt2)
			pl_array.force_last (pl)
			create c.make (1, 0, 0, 1)
			create appearance.make (c)

			-- Y axis
			create pt1.make (0, 0, 0)
			create pt2.make (0, 1, 0)
			create pl.make_simple (pt1, pt2)
			pl_array.force_last (pl)
			create c.make (0, 1, 0, 1)
			appearance.add_item (c)

			-- Z axis
			create pt1.make (0, 0, 0)
			create pt2.make (0, 0, 1)
			create pl.make_simple (pt1, pt2)
			pl_array.force_last (pl)
			create c.make (0, 0, 1, 1)
			appearance.add_item (c)

			create sl.make (pl_array)

			sl.set_line_width (2.0)
			sl.set_color_mapping (sl.per_object)
			sl.replace_element (appearance)

			-- Mark the axes as annotations so they're not
			-- rendered in all views
			sl.mark_as_annotation (true)

			create shp.make (sl)

			Result.add_child (shp)

		ensure
			Result_not_void: Result /= Void
			Result_has_one_child: Result.count = 1
		end

	rotate_camera (xform: TRANSFORM) is
			-- rotates the camera around point of interest
		local
			radius: DOUBLE
			center: POINT3
			forward: VECTOR3
			camera_rotation: TRANSFORM
		do
			camera_rotation := camera.orientation.rotation_matrix
			debug ("gui")
				io.put_string ("Camera Rotation%N")
				io.put_string (camera_rotation.out)
				io.put_string ("%N")
			end
			radius := (camera.position - camera.focal_point).magnitude
			create forward.make (-camera_rotation.a13,
						   -camera_rotation.a23,
							-camera_rotation.a33)
			forward := forward.scale (radius)
			center := camera.position + forward

			camera_rotation := xform
			camera.orientation.make_from_transformation (camera_rotation)

			forward.make (camera_rotation.a13,
						  camera_rotation.a23,
						  camera_rotation.a33)
			forward := forward.scale (radius)
			center := center + forward
			camera.position.make (center.x, center.y, center.z)

			debug ("gui")
				io.put_string ("Camera Rotation%N")
				io.put_string (camera_rotation.out)
				io.put_string ("%N")
			end
		end

invariant
	root_node_not_void: root_node /= Void
	root_node_has_two_children: root_node.count = 2
	view_axis_is_first_child: view_axis = root_node.item (1)
	ambient_color_not_void: scene_ambient_color /= Void
	culler_not_void: culler /= Void
	gl_drawer_not_void: gl_drawer /= Void

end -- SCENE_VIEWER

