

indexing
	description: "Abstract class for reading and writing images"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2006-09-25 16:21:23 -0400 (Mon, 25 Sep 2006) $"
	revision: "$Revision: 268 $"



deferred class 

	IMAGE_IO

inherit

	ANY
		redefine
			default_create
		end
	
feature -- Initialization

	default_create is
			-- Initialize the variables
		do
			one := 255
			minimum := 0
			maximum := 255
			dither_value := 0.5
		end

	make (fn: STRING) is
			-- Create file object with `fn' as file name.
		require
			string_exists: fn /= Void
			string_not_empty: not fn.is_empty
		do
			default_create
			
			name := fn
			mode := closed_file
		ensure
			file_named: name = fn
			file_closed: is_closed
		end

	make_open_read (fn: STRING) is
			-- Create file object with `fn' as file name
			-- and open file in read mode.
		require
			string_exists: fn /= Void
			string_not_empty: not fn.is_empty
		do
			default_create
			
			make (fn)
			open_read
		ensure
			file_named: name = fn
			open_read: is_open_read
		end

	make_open_write (fn: STRING) is
			-- Create file object with `fn' as file name
			-- and open file for writing;
			-- create it if it does not exist.
		require
			string_exists: fn /= Void
			string_not_empty: not fn.is_empty
		do
			default_create
			
			make (fn)
			open_write
		ensure
			file_named: name = fn
			open_write: is_open_write
		end

feature -- Access

	name: STRING
			-- File name

	file_pointer: POINTER
			-- File pointer as required in C

	image_data: IMAGE
			-- The image data in the file

	mode: INTEGER
			-- The file mode

	read_file, write_file, closed_file: INTEGER is unique
			-- The possible `mode's

	one, minimum, maximum: INTEGER
			-- Quantization parameters

	dither_value: DOUBLE
			-- Amount of dithering
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	print_header is
			-- Print the header to the screen
		require 
			opened_for_reading: is_open_read
		deferred
		end 
	
	width: INTEGER is
			-- The width of the image
		deferred
		ensure
			Result_positive: Result >= 0
		end
	
	height: INTEGER is
			-- The height of the image
		deferred
		ensure
			Result_positive: Result >= 0
		end
	
	samples_per_pixel: INTEGER is
			-- The number of samples per pixel
		deferred
		ensure
			Result_positive: Result >= 0
		end
	
	bits_per_sample: INTEGER is
			-- The number of bits per sample
		deferred
		ensure
			Result_positive: Result >= 0
		end
	
	is_closed: BOOLEAN is
			-- Is file closed?
		do
			Result := mode = closed_file
		end

	is_open_read: BOOLEAN is
			-- Is file open for reading?
		do
			Result := mode = read_file
		end

	is_open_write: BOOLEAN is
			-- Is file open for writing?
		do
			Result := mode = write_file
		end

feature -- Status setting

	open_read is
			-- Open file in read-only mode.
		require
			is_closed: is_closed
		deferred
		ensure
			open_read: is_open_read
		end

	open_write is
			-- Open file in write-only mode;
			-- create it if it does not exist.
		deferred
		ensure
			open_write: is_open_write
		end

	fd_open_read (fd: INTEGER) is
			-- Open file of descriptor `fd' in read-only mode.
		deferred
		ensure
			open_read: is_open_read
		end

	fd_open_write (fd: INTEGER) is
			-- Open file of descriptor `fd' in write mode.
		deferred
		ensure
			open_write: is_open_write
		end

	close is
			-- Close file.
		do
			file_close (file_pointer)
			mode := closed_file
		ensure then
			is_closed: is_closed
		end

feature -- Cursor movement
feature -- Element change
	
	set_image (new_image: IMAGE) is
			-- Set `image_data' to `new_image'
		require 
			not_opened_for_reading: not is_open_read
			new_image_not_void: new_image /= Void
		do
			image_data := new_image
		ensure 
			image_data_set: image_data = new_image
		end 

	set_quantization (new_one, new_min, new_max: INTEGER; new_dither_value: DOUBLE) is
			-- Set the quantization parameters
		do
			one := new_one
			minimum := new_min
			maximum := new_max
			dither_value := new_dither_value
		end
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	read_data is
			-- Reads the image data from the file
		require 
			opened_for_reading: is_open_read
		deferred
		ensure
			image_data_not_void: image_data /= Void
		end
	
	write_data is
			-- Writes the image data to the file
		require 
			opened_for_writing: is_open_write
		deferred
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Inapplicable
	
	file_close (file: POINTER) is
			-- Close `file'.
		external
			"C use <stdio.h>"
		alias
			"fclose"
		end

	file_dopen (fd: INTEGER; how: POINTER): POINTER is
			-- File pointer for file of descriptor `fd' in mode `how'
			-- (which must fit the way `fd' was obtained).
		external
			"C use <stdio.h>"
		alias
			"fdopen"
		end

end -- IMAGE_IO

