

indexing
	description: "Class for reading and writing TIFF images"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2006-11-21 10:58:10 -0500 (Tue, 21 Nov 2006) $"
	revision: "$Revision: 269 $"



class 
	
	TIFF_IMAGE
	
inherit
	
	IMAGE_IO
		redefine
			close
		end
	
	TIFF_CONSTANTS
		undefine
			default_create
		end
	
	NUMERIC_CONVERSION
		undefine
			default_create
		end
	
	BASIC_NUMERIC
		undefine
			default_create
		end
			
creation
	make_open_read, make_open_write

feature -- Initialization

feature -- Access
	
	compression: INTEGER is 
			-- The type of compression
		do
			Result := compression_none
		end
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	get_integer_field (tag: INTEGER): INTEGER is
			-- Get a field from the TIFF file
		require 
			opened_for_reading: is_open_read
		do
			Result := TIFFGetIntegerField (tiff_pointer, tag)
		end 
	
	print_header is
			-- Print the header to the screen
		do
			TIFFPrintDirectory (tiff_pointer, io.output.file_pointer, 0)
		end 
	
	is_tiled: BOOLEAN is
			-- Is this TIFF file tiled?
		require 
			opened_for_reading: is_open_read
		do
			if TIFFIsTiled (tiff_pointer) = 0 then
				Result := false
			else
				Result := true
			end
		end 
	
	width: INTEGER is
			-- The width of the image
		do
			if is_open_read then
				Result := get_integer_field (tifftag_imagewidth)
			elseif image_data /= Void then
				Result := image_data.width
			else
				Result := the_width
			end
		end
	
	height: INTEGER is
			-- The height of the image
		do
			if is_open_read then
				Result := get_integer_field (tifftag_imagelength)
			elseif image_data /= Void then
				Result := image_data.height
			else
				Result := the_height
			end
		end
	
	samples_per_pixel: INTEGER is
			-- The number of samples per pixel
		do
			if is_open_read then
				Result := TIFFSamplesPerPixel (tiff_pointer)
				--Result := get_integer_field (tifftag_samplesperpixel)
			elseif image_data /= Void then
				Result := image_data.pixel_depth
			else
				Result := the_pixel_depth
			end
		end
	
	bits_per_sample: INTEGER is
			-- The number of bits per sample
		do
			if is_open_read then
				Result := TIFFBitsPerSample (tiff_pointer)
				--Result := get_integer_field (tifftag_bitspersample)
			else
				Result := 8
			end
		end
		
feature -- Status setting

	set_integer_field (tag, value: INTEGER) is
			-- Set a field in the TIFF file
		require 
			opened_for_writing: is_open_write
		local
			error_flag: INTEGER
		do
			error_flag := TIFFSetIntegerField (tiff_pointer, tag, value)
		end 
	
feature -- Cursor movement
feature -- Element change
	
	open_read is
			-- Open the TIFF file, `name', for `mode'
		local
			file_mode: STRING
			n, m: ANY
		do
			create file_mode.make_from_string ("r")
			
			n := name.to_c
			m := file_mode.to_c
			tiff_pointer := TIFFOpen ($n, $m)
			fd_open_read (TIFFFileno (tiff_pointer))
			mode := read_file
		end 

	open_write is
			-- Open file in write-only mode;
			-- create it if it does not exist.
		local
			file_mode: STRING
			n, m: ANY
		do
			create file_mode.make_from_string ("w")
			
			n := name.to_c
			m := file_mode.to_c
			tiff_pointer := TIFFOpen ($n, $m)
			fd_open_write (TIFFFileno (tiff_pointer))
			mode := write_file
		end
		
	fd_open_read (fd: INTEGER) is
			-- Open file of descriptor `fd' in read-only mode.
		local
			file_mode: STRING
			m: ANY
		do
			create file_mode.make_from_string ("r")
			m := file_mode.to_c
			file_pointer := file_dopen (fd, $m)
			mode := read_file
		end

	fd_open_write (fd: INTEGER) is
			-- Open file of descriptor `fd' in write mode.
		local
			file_mode: STRING
			m: ANY
		do
			create file_mode.make_from_string ("w")
			m := file_mode.to_c
			file_pointer := file_dopen (fd, $m)
			mode := write_file
		end

	close is
			-- Close file.
		do
			TIFFClose (tiff_pointer)
			
			mode := closed_file
		end

	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	read_data is
			-- Reads the image data from the TIFF file
		local
			config: INTEGER
		do
			config := TIFFPlanarConfig( tiff_pointer )
			--config := get_integer_field (tifftag_planarconfig)
			
			if is_tiled then
			else
				if config = planarconfig_contig then
					read_contiguous_strip_data
				else
				end
			end
		end 
	
	write_data is
			-- Write the image data to a TIFF file
		do
			-- Add options for other modes
			write_contiguous_strip_data
		end 

feature -- Miscellaneous
feature -- Obsolete
feature {NONE} -- Inapplicable
feature {NONE} -- Implementation
	
	the_width: INTEGER
	the_height: INTEGER
	the_pixel_depth: INTEGER
			-- Values set for writing
	
	tiff_pointer: POINTER
			-- Pointer to TIFF object
	
	read_contiguous_strip_data is
			-- Read data from the TIFF file as a single, contiguous, 
			-- stripped image
		require 
			opened_for_reading: is_open_read
			not_tiled: not is_tiled
			is_contiguous: TIFFPlanarConfig( tiff_pointer ) = 
						   planarconfig_contig
		local
			buffer: ARRAY [CHARACTER]
			file_mode: STRING
			buffer_pointer: ANY
			scanline_size, row, nrow, rows_per_strip: INTEGER
			i, k, strip_index, strip_size: INTEGER
		do
			scanline_size := TIFFScanlineSize (tiff_pointer)
			strip_size := TIFFStripSize (tiff_pointer) + 2
			create buffer.make (1, strip_size)
			
			buffer_pointer := buffer.to_c
			
			rows_per_strip := get_integer_field (tifftag_rowsperstrip)
			
			create image_data.make (height, width, samples_per_pixel)
			
			from
				row := 0
				k := 1
			invariant
				row_at_least_one: row >= 0
				row_small_enough: row <= height + rows_per_strip
			variant
				row_increasing: height - row + rows_per_strip
			until
				row >= height
			loop
				if row + rows_per_strip > height then
					nrow := height - row
				else
					nrow := rows_per_strip
				end
				
				strip_index := TIFFComputeStrip (tiff_pointer, row, 0)
				
				if TIFFReadEncodedStrip (tiff_pointer, strip_index , 
										 $buffer_pointer, 
										 nrow * scanline_size) < 0 then
					io.put_string ("Error reading from TIFF file!%N")
					row := height
				end
				
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + nrow * scanline_size
				variant
					i_increasing: 1 + nrow * scanline_size - i
				until
					i > nrow * scanline_size
				loop
					image_data.image_data.put (real_64_truncated_to_real_32 ((buffer @ i).code / 255),
												   image_data.swap_index (k))
 
					i := i + 1
					k := k + 1
				end
				row := row + rows_per_strip
			end
		ensure 
			default: true
		end 

	write_contiguous_strip_data is
			-- Write data to the TIFF file as a single, contiguous, 
			-- stripped image
		require 
			opened_for_writing: is_open_write
		local
			vv: ARRAY [INTEGER]
			v: ARRAY [INTEGER_8]
			vr: ARRAY [REAL]
			val: REAL
			index, i, j, flag, bits: INTEGER
			a: ANY
			write_as_floating_point: BOOLEAN
		do
			set_integer_field (tifftag_imagewidth, image_data.width)
			set_integer_field (tifftag_imagelength, image_data.height)
			if minimum = 0 and maximum = 0 and one = 0 then
				bits := 32
				write_as_floating_point := true
			else
				bits := real_64_rounded_to_integer (log10 (maximum) / log10 (2.0))
				write_as_floating_point := false
			end
			set_integer_field (tifftag_bitspersample, bits)
			set_integer_field (tifftag_compression, compression)
			
			if image_data.pixel_depth = 1 then
				set_integer_field (tifftag_photometric, photometric_minisblack)
			else
				set_integer_field (tifftag_photometric, photometric_rgb)
			end
			
			set_integer_field (tifftag_orientation, orientation_topleft)
			set_integer_field (tifftag_samplesperpixel, image_data.pixel_depth)
			
			if image_data.pixel_depth > 3 then
				create vv.make (1, 1)
				vv.put (extrasample_unassalpha, 1)
				a := vv.to_c
				flag := TIFFSetIntegerArrayField(tiff_pointer, tifftag_extrasamples, 1, $a)
			end
			
			set_integer_field (tifftag_planarconfig, planarconfig_contig)
			set_integer_field (tifftag_rowsperstrip, 
							   TIFFDefaultStripSize(tiff_pointer, 0))
			
			from
				i := 1
				create v.make (1, image_data.width * image_data.pixel_depth)
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + image_data.height
			variant
				i_increasing: 1 + image_data.height - i
			until
				i > image_data.height
			loop
				index := image_data.swap_index ((i-1) * image_data.width * image_data.pixel_depth + 1)
				if write_as_floating_point then
					from
						j := 1
					invariant
						j_at_least_one: j >= 1
						j_small_enough: j <= 1 + image_data.width * image_data.pixel_depth
					variant
						j_increasing: 1 + image_data.width * image_data.pixel_depth - j
					until
						j > image_data.width * image_data.pixel_depth
					loop
						val := image_data.image_data.item (index + j - 1) 
						vr.put (val, j)
						j := j + 1
					end
					
					a := vr.to_c
				else
					from
						j := 1
					invariant
						j_at_least_one: j >= 1
						j_small_enough: j <= 1 + image_data.width * image_data.pixel_depth
					variant
						j_increasing: 1 + image_data.width * image_data.pixel_depth - j
					until
						j > image_data.width * image_data.pixel_depth
					loop
						val := image_data.image_data.item (index + j - 1) * one + dither_value
						flag := real_64_truncated_to_integer (clamp_to_range (val, minimum, maximum))
						v.put (flag.to_integer_8, j)
						j := j + 1
					end
					
					a := v.to_c
				end
				
				flag := TIFFWriteScanline(tiff_pointer, $a, i-1, 0)
				
				i := i + 1
			end
		end
	
feature {NONE} -- Implementation - GetField, SetField typed wrappers
	
	TIFFSetIntegerField (tif: POINTER; tag, value: INTEGER): INTEGER is
		external
			"C use <TIFFWrapper.h>"
		alias
			"TIFFSetIntegerField"
		end
	
	TIFFSetIntegerArrayField (tif: POINTER; tag, value: INTEGER; buf: POINTER): INTEGER is
		external
			"C use <TIFFWrapper.h>"
		alias
			"TIFFSetIntegerArrayField"
		end
	
	TIFFGetIntegerField (tif: POINTER; tag: INTEGER): INTEGER is
		external
			"C use <TIFFWrapper.h>"
		alias
			"TIFFGetIntegerField"
		end

	TIFFSamplesPerPixel (tif: POINTER): INTEGER is
		external
			"C use <TIFFWrapper.h>"
		alias
			"TIFFSamplesPerPixel"
		end

	TIFFBitsPerSample (tif: POINTER): INTEGER is
		external
			"C use <TIFFWrapper.h>"
		alias
			"TIFFBitsPerSample"
		end

	TIFFPlanarConfig (tif: POINTER): INTEGER is
		external
			"C use <TIFFWrapper.h>"
		alias
			"TIFFPlanarConfig"
		end

feature {NONE} -- Implementation

	TIFFGetVersion: POINTER is
		external
			"C: const char* | <tiffio.h>"
		alias
			"TIFFGetVersion"
		end 

	TIFFFindCODEC (scheme: INTEGER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFFindCODEC"
		end
	
	TIFFRegisterCODEC (scheme: INTEGER; tiff_name, init: POINTER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRegisterCODEC"
		end
	
	TIFFUnRegisterCODEC (c: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFUnRegisterCODEC"
		end
	

	TIFFmalloc (s: INTEGER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"_TIFFmalloc"
		end
	
	TIFFrealloc (p:POINTER; s: INTEGER): POINTER is
		external
			"C (tdata_t, tsize_t): tdata_t	| <tiffio.h>"
		alias
			"_TIFFrealloc"
		end
	
	TIFFmemset (p:POINTER; v, s: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"_TIFFmemset"
		end
	
	TIFFmemcpy (d, s, c: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"_TIFFmemcpy"
		end
	
	TIFFmemcmp (p1, p2, c: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"_TIFFmemcmp"
		end
	
	TIFFfree (p:POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"_TIFFfree"
		end
	

	TIFFClose (tif: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFClose"
		end
	
	TIFFFlush (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFFlush"
		end
	
	TIFFFlushData (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFFlushData"
		end
	
	TIFFReadDirectory (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadDirectory"
		end
	
	TIFFScanlineSize (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFScanlineSize"
		end
	
	TIFFRasterScanlineSize (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRasterScanlineSize"
		end
	
	TIFFStripSize (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFStripSize"
		end
	
	TIFFVStripSize (tif: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFVStripSize"
		end
	
	TIFFTileRowSize (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFTileRowSize"
		end
	
	TIFFTileSize (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFTileSize"
		end
	
	TIFFVTileSize (tif: POINTER; nrows: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFVTileSize"
		end
	
	TIFFDefaultStripSize (tif: POINTER; request: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFDefaultStripSize"
		end
	
	TIFFDefaultTileSize (tif, tw, th: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFDefaultTileSize"
		end
	
	TIFFFileno (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFFileno"
		end
	
	TIFFGetMode (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFGetMode"
		end
	
	TIFFIsTiled (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFIsTiled"
		end
	
	TIFFIsByteSwapped (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFIsByteSwapped"
		end
	
	TIFFIsUpSampled (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFIsUpSampled"
		end
	
	TIFFIsMSB2LSB (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFIsMSB2LSB"
		end
	
	TIFFCurrentRow (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCurrentRow"
		end
	
	TIFFCurrentDirectory (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCurrentDirectory"
		end
	
	TIFFNumberOfDirectories (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFNumberOfDirectories"
		end
	
	TIFFCurrentDirOffset (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCurrentDirOffset"
		end
	
	TIFFCurrentStrip (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCurrentStrip"
		end
	
	TIFFCurrentTile (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCurrentTile"
		end
	
	TIFFReadBufferSetup (tif, bp: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadBufferSetup"
		end
	
	TIFFWriteBufferSetup (tif, bp: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteBufferSetup"
		end
	
	TIFFWriteCheck (tif: POINTER; tiles: INTEGER; module: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteCheck"
		end
	
	TIFFCreateDirectory (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCreateDirectory"
		end
	
	TIFFLastDirectory (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFLastDirectory"
		end
	
	TIFFSetDirectory (tif: POINTER; dirn: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSetDirectory"
		end
	
	TIFFSetSubDirectory (tif: POINTER; diroff: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSetSubDirectory"
		end
	
	TIFFUnlinkDirectory (tif: POINTER; dirn: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFUnlinkDirectory"
		end
	
	TIFFWriteDirectory (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteDirectory"
		end
	
	TIFFRewriteDirectory (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRewriteDirectory"
		end
	
	TIFFReassignTagToIgnore (task, TIFFtagID: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReassignTagToIgnore"
		end
	

	TIFFPrintDirectory (tif, fd: POINTER; flags: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFPrintDirectory"
		end
	
	TIFFReadScanline (tif, buf: POINTER; row, sample: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadScanline"
		end
	
	TIFFWriteScanline (tif, buf: POINTER; row, sample: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteScanline"
		end
	
	TIFFReadRGBAImage (tif: POINTER; rwidth, rheight: INTEGER; 
					   raster: POINTER; stop: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadRGBAImage"
		end
	

	TIFFReadRGBAStrip (tif: POINTER; row: INTEGER; raster: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadRGBAStrip"
		end
	
	TIFFReadRGBATile (tif: POINTER; col, row: INTEGER; 
					   raster: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadRGBATile"
		end
	
	TIFFRGBAImageOK (tif, emsg: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRGBAImageOK"
		end
	
	TIFFRGBAImageBegin (img, tif: POINTER; stop: INTEGER; emsg: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRGBAImageBegin"
		end
	
	TIFFRGBAImageGet (img, raster: POINTER; w, h: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRGBAImageGet"
		end
	
	TIFFRGBAImageEnd (img: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFRGBAImageEnd"
		end
	
	TIFFOpen (tiff_name, tiff_mode: POINTER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFOpen"
		end
	
	TIFFFdOpen (fd: INTEGER; tiff_name, tiff_mode: POINTER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFFdOpen"
		end
	
	TIFFFileName (tif: POINTER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFFileName"
		end
	
	TIFFComputeTile (tif: POINTER; x, y ,z, s: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFComputeTile"
		end
	
	TIFFCheckTile (tif: POINTER; x, y ,z, s: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFCheckTile"
		end
	
	TIFFNumberOfTiles (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFNumberOfTiles"
		end
	
	TIFFReadTile (tif, buf: POINTER; x, y ,z, s: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadTile"
		end
	
	TIFFWriteTile (tif, buf: POINTER; x, y ,z, s: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteTile"
		end
	
	TIFFComputeStrip (tif: POINTER; row, sample: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFComputeStrip"
		end
	
	TIFFNumberOfStrips (tif: POINTER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFNumberOfStrips"
		end
	
	TIFFReadEncodedStrip (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadEncodedStrip"
		end
	
	TIFFReadRawStrip (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadRawStrip"
		end
	
	TIFFReadEncodedTile (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadEncodedTile"
		end
	
	TIFFReadRawTile (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReadRawTile"
		end
	
	TIFFWriteEncodedStrip (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteEncodedStrip"
		end
	
	TIFFWriteRawStrip (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteRawStrip"
		end
	
	TIFFWriteEncodedTile (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteEncodedTile"
		end
	
	TIFFWriteRawTile (tif: POINTER; the_strip: INTEGER; buf: POINTER; size: INTEGER): INTEGER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFWriteRawTile"
		end
	
	TIFFSetWriteOffset (tif: POINTER; toff: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSetWriteOffset"
		end
	
	TIFFSwabShort (wp: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSwabShort"
		end
	
	TIFFSwabLong (lp: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSwabLong"
		end
	
	TIFFSwabDouble (dp: POINTER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSwabDouble"
		end
	
	TIFFSwabArrayOfShort (wp: POINTER; n: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSwabArrayOfShort"
		end
	
	TIFFSwabArrayOfLong (lp: POINTER; n: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSwabArrayOfLong"
		end
	
	TIFFSwabArrayOfDouble (dp: POINTER; n: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFSwabArrayOfDouble"
		end
	
	TIFFReverseBits (cp: POINTER; n: INTEGER) is
		external
			"C use <tiffio.h>"
		alias
			"TIFFReverseBits"
		end
	
	TIFFGetBitRevTable (reversed: INTEGER): POINTER is
		external
			"C use <tiffio.h>"
		alias
			"TIFFGetBitRevTable"
		end
	

end -- TIFF_IMAGE

