indexing

	description:
		"Files viewed as persistent sequences of ASCII characters"

	status: "See notice at end of class"
	date: "$Date: 2006-06-23 10:34:45 -0400 (Fri, 23 Jun 2006) $"
	revision: "$Revision: 226 $"

class IO_INDENTED_MEDIUM

inherit
	
	KL_STDOUT_FILE
		rename
			put_double as old_put_double,
			put_real as old_put_real
		redefine
			make, old_put_new_line, put_new_line, old_put_string
		end
	
creation

	make

feature -- Initialization

	make is
			-- Open the medium as standard out
		do
			Precursor
			create indent_string.make (1)
			create temp_string.make (1)
		end
	
feature -- Access
	
	indentation_prefix: STRING is "    "
			-- The character used for indenting
	
	depth: INTEGER
			-- The depth of the indentation
	
feature -- Element Change
	
	set_indentation (new_depth: INTEGER) is
			-- Set the indentation to depth * 
			-- indentation_prefix.count characters
		require 
			new_depth_is_positive: new_depth >= 0
		local
			i: INTEGER
		do
			depth := new_depth
			
			indent_string.wipe_out
			
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + depth
			variant
				i_increasing: 0 + depth - i
			until
				i >= depth
			loop
				indent_string.append (indentation_prefix)
				
				i := i + 1
			end			
		ensure 
			depth_is_set: depth = new_depth
		end 

	
	push is
			-- Increase depth in tree
		do
			set_indentation (depth + 1)
		end 

	pop is
			-- Increase depth in tree
		require 
			depth_not_zero: depth /= 0
		do
			set_indentation (depth - 1)
		end 

feature -- Output
	
	indent is
			-- Indent the position
		do
			put_string (indent_string)
		end

	put_double (d: DOUBLE) is
			-- Write `val' at the current position
		do
			put_string (d.out)
		end 

	put_real (r: REAL) is
			-- Write `val' at the current position
		do
			put_string (r.out)
		end 

	put_new_line is
			-- Write an indented new_line
		do
			Precursor
			put_string (indent_string)
		end

feature {NONE} -- Implementation
	
	indent_string: STRING
			-- The character string to be used for indenting
	
	temp_string: STRING
			-- Used in the put_string routine for temporary 
			-- processing. Declared as a class feature to avoid 
			-- memory allocation	

	old_put_new_line is
			-- Write an indented new_line
		do
			Precursor
			put_string (indent_string)
		end

	old_put_string (s: STRING) is
			-- Write `s' at current position. Indenting at new_lines (%N)
		local
			start_index, end_index : INTEGER
		do
			if s.has ('%N') and s.count > 1 then
				from
					start_index := 1
					end_index := s.index_of ('%N', start_index)
				invariant
					start_index_at_least_one: start_index >= 1
					start_index_small_enough: start_index <= 1 + s.count
				variant
					start_index_increasing: 1 + s.count - start_index
				until
					start_index >= s.count or end_index = 0
				loop
					temp_string := s.substring (start_index, end_index - 1)
					Precursor (temp_string)
					Precursor ("%N")
					
					start_index := end_index + 1
					if start_index < s.count then
						end_index := s.index_of ('%N', start_index)
					end
				end
				
				if start_index < s.count then
					temp_string := s.substring (start_index, s.count)
					Precursor (temp_string)
				end
				
			else
				Precursor (s)
			end
		end
	
invariant
	
	indent_string_not_void: indent_string /= Void
	indent_string_count_is_positive: indent_string.count >= 0
	indentation_prefix_not_void: indentation_prefix /= Void
	indentation_prefix_count_is_positive: indentation_prefix.count >= 1
	depth_is_positive: depth >= 0
	
end -- class IO_INDENTED_MEDIUM

