
indexing

	description: "Lists implemented with arrays using large memory"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"

deferred class

	LARGE_ARRAYED_LIST [G]

inherit

	ANY
		redefine
			copy, is_equal, default_create, out
		end

	KL_IMPORTED_ANY_ROUTINES
		export
			{NONE} all
		redefine
			copy, is_equal, default_create, out
		end

	DEBUG_OUTPUT
		undefine
			is_equal, out, copy, default_create
		end

	BIT_SIZES
		undefine
			is_equal, out, copy, default_create
		end
	
feature {NONE} -- Initialization

	default_create is
			-- Initialize with the default number of elements
		do
			make_default
		end

	make (n: INTEGER_64) is
			-- Create an empty list and allocate
			-- memory space for at least `n' items.
			-- Use `=' as comparison criterion.
		require
			positive_n: n >= 0
		do
			create storage
			storage.allocate ((n + 1) * size_of_object)
			capacity := n + 1
			internal_cursor := new_cursor

			is_shared := false
			create_default_item
			create_local_storage
		ensure
			-- No longer true for arrayed_lists of expandeds
			--empty: is_empty
			capacity_set: capacity = n + 1
			before: before
		end

	make_equal (n: INTEGER_64) is
			-- Create an empty list and allocate
			-- memory space for at least `n' items.
			-- Use `equal' as comparison criterion.
		require
			positive_n: n >= 0
		do
			make (n)
			create equality_tester
		ensure
			empty: is_empty
			capacity_set: capacity = n + 1
			before: before
		end

	make_from_linear (other: DS_LINEAR [G]) is
			-- Create a new list and fill it with items of `other'.
			-- Use `=' as comparison criterion.
		require
			other_not_void: other /= Void
		local
			other_cursor: DS_LINEAR_CURSOR [G]
			i, nb: INTEGER_64
		do
			nb := other.count
			make (nb)
			count := nb
			from
				i := 1
				other_cursor := other.new_cursor
				other_cursor.start
			until
				i > nb
			loop
				put (other_cursor.item, i)
				other_cursor.forth
				i := i + 1
			end
		ensure
			count_set: count = other.count
			capacity_set: capacity = count
			before: before
		end

	make_from_array (other: ARRAY [G]) is
			-- Create a new list and fill it with items of `other'.
			-- Use `=' as comparison criterion.
		require
			other_not_void: other /= Void
		local
			i: INTEGER
			j, nb: INTEGER_64
		do
			nb := other.count
			make (nb)
			count := nb
			from
				j := 1
				i := other.lower
			until
				j > nb
			loop
				replace (other.item (i), j)
				j := j + 1
				i := i + 1
			end
		ensure
			count_set: count = other.count
			capacity_set: capacity = count + 1
			before: before
		end

	make_shared_from_list (other: LARGE_ARRAYED_LIST [G]) is
			-- Create a new list sharing the data with `other'
		require
			other_is_valid: other /= Void
		do
			if storage /= Void then
				storage.dispose
			end
			
			create storage.share_from_pointer (other.storage.item, other.storage.count)
			capacity := other.capacity
			count := other.count
			internal_cursor := new_cursor

			is_shared := false
			create_default_item
		ensure
			empty: not is_empty
			capacity_set: capacity = other.capacity
			count_set: count = other.count
			before: before
		end

	make_default is
			-- Create an empty list and allocate memory
			-- space for at least `default_capacity' items.
			-- Use `=' as comparison criterion.
		do
			make (default_capacity)
		ensure then
			before: before
		end

feature -- Initialization

	fill is
			-- Initialize the array
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i >= capacity
			loop
				force (default_item, i)
				i := i + 1
			end
		end

feature -- Access

	storage: LARGE_MEMORY_POINTER
			-- Storage for items of the list

	infix "@", item (i: INTEGER_64): G is
			-- Item at index `i'
			-- (Performance: O(1).)
		require
			valid_index: 1 <= i and i <= count
		do
			Result := internal_item (i)
		end

	item_for_iteration: G is
			-- Item at internal cursor position
		require
			not_off: not off
		do
			Result := cursor_item (internal_cursor)
		end

	first: G is
			-- First item in list
			-- (Performance: O(1).)
		require
			not_empty: not is_empty
		do
			Result := internal_item (1)
		ensure
			definition: Result = item (1)
			has_first: has (Result)
		end

	last: G is
			-- Last item in list
			-- (Performance: O(1).)
		require
			not_empty: not is_empty
		do
			Result := internal_item (count)
		ensure
			definition: Result = item (count)
			has_last: has (Result)
		end

	index: INTEGER_64 is
			-- Index of current internal cursor position
		do
			Result := cursor_index (internal_cursor)
		ensure
			valid_index: valid_index (Result)
		end

	new_cursor: LARGE_ARRAYED_LIST_CURSOR [G] is
			-- New external cursor for traversal
		do
			create Result.make (Current)
		ensure
			cursor_not_void: Result /= Void
			valid_cursor: valid_cursor (Result)
		end

	equality_tester: KL_EQUALITY_TESTER [G]
			-- Equality tester;
			-- A void equality tester means that `='
			-- will be used as comparison criterion.

feature -- Cursor movement

	go_i_th (i: INTEGER_64) is
			-- Move internal cursor to `i'-th position.
		require
			valid_index: valid_index (i)
		do
			cursor_go_i_th (internal_cursor, i)
		ensure
			moved: index = i
		end

	start is
			-- Move internal cursor to first position.
		do
			cursor_start (internal_cursor)
		ensure
			empty_behavior: is_empty implies after
			not_empty_behavior: not is_empty implies is_first
		end

	forth is
			-- Move internal cursor to next position.
		require
			not_after: not after
		do
			cursor_forth (internal_cursor)
		end

	search_forth (v: G) is
			-- Move internal cursor to first position at or after current
			-- position where `item_for_iteration' and `v' are equal.
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
			-- Move `after' if not found.
		require
			not_off: not off or after
		do
			cursor_search_forth (internal_cursor, v)
		end

	go_after is
			-- Move internal cursor to `after' position.
		do
			cursor_go_after (internal_cursor)
		ensure
			after: after
		end

	finish is
			-- Move internal cursor to last position.
		do
			cursor_finish (internal_cursor)
		ensure
			empty_behavior: is_empty implies before
			not_empty_behavior: not is_empty implies is_last
		end

	back is
			-- Move internal cursor to previous position.
		require
			not_before: not before
		do
			cursor_back (internal_cursor)
		end

	search_back (v: G) is
			-- Move internal cursor to first position at or before current
			-- position where `item_for_iteration' and `v' are equal.
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
			-- Move `before' if not found.
		require
			not_off: not off or before
		do
			cursor_search_back (internal_cursor, v)
		end

	go_before is
			-- Move internal cursor to `before' position.
		do
			cursor_go_before (internal_cursor)
		ensure
			before: before
		end

feature -- Measurement

	count: INTEGER_64
			-- Number of items in list
			-- (Performance: O(1).)

	capacity: INTEGER_64
			-- Maximum number of items in list

	default_capacity: INTEGER is
			-- Initial capacity in `make_default'
			-- (Default value: 10)
		once
			Result := 10
		ensure
			default_capacity_positive: Result >= 0
		end

	occurrences (v: G): INTEGER_64 is
			-- Number of times `v' appears in list
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
		local
			i: INTEGER_64
			a_tester: like equality_tester
		do
			i := count
			a_tester := equality_tester
			if a_tester /= Void then
				from  until i < 1 loop
					if a_tester.test (internal_item (i), v) then
						Result := Result + 1
					end
					i := i - 1
				end
			else
					-- Use `=' as comparison criterion.
				from  until i < 1 loop
					if internal_item (i) = v then
						Result := Result + 1
					end
					i := i - 1
				end
			end
		ensure
			positive: Result >= 0
			has: has (v) implies Result >= 1
		end

feature -- Status report

	is_shared: BOOLEAN
			-- Is this objects data area shared with another
			-- instance? If yes, then the size of the object may not
			-- be changed. There is no way to "unshare" an object.

	valid_index (i: INTEGER_64): BOOLEAN is
			-- Is `i' a valid index value?
		do
			Result := 0 <= i and i <= (count + 1)
		ensure
			definition: Result = (0 <= i and i <= (count + 1))
		end

	off: BOOLEAN is
			-- Is there no item at internal cursor position?
		do
			Result := cursor_off (internal_cursor)
		end

	is_empty: BOOLEAN is
			-- Is container empty?
		do
			Result := count = 0
		end

	is_first: BOOLEAN is
			-- Is internal cursor on first item?
		do
			Result := cursor_is_first (internal_cursor)
		ensure
			not_empty: Result implies not is_empty
			not_off: Result implies not off
			definition: Result implies (item_for_iteration = first)
		end

	after: BOOLEAN is
			-- Is there no valid position to right of internal cursor?
		do
			Result := cursor_after (internal_cursor)
		end

	has (v: G): BOOLEAN is
			-- Does list include `v'?
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
		local
			i: INTEGER_64
			a_tester: like equality_tester
		do
			i := count
			a_tester := equality_tester
			if a_tester /= Void then
				from until i < 1 loop
					if a_tester.test (internal_item (i), v) then
						Result := True
						i := 0 -- Jump out of the loop.
					else
						i := i - 1
					end
				end
			else
					-- Use `=' as comparison criterion.
				from until i < 1 loop
					if internal_item (i) = v then
						Result := True
						i := 0 -- Jump out of the loop.
					else
						i := i - 1
					end
				end
			end
		ensure
			not_empty: Result implies not is_empty
		end

	extendible (n: INTEGER_64): BOOLEAN is
			-- May list be extended with `n' items?
		require
			positive_n: n >= 0
		do
			Result := capacity >= count + n
		ensure then
			enough_space: Result implies (capacity >= count + n)
		end

	is_last: BOOLEAN is
			-- Is internal cursor on last item?
		do
			Result := cursor_is_last (internal_cursor)
		ensure
			not_empty: Result implies not is_empty
			not_off: Result implies not off
			definition: Result implies (item_for_iteration = last)
		end

	before: BOOLEAN is
			-- Is there no valid position to left of internal cursor?
		do
			Result := cursor_before (internal_cursor)
		end

	valid_cursor (a_cursor: LARGE_ARRAYED_LIST_CURSOR [G]): BOOLEAN is
			-- Is `a_cursor' a valid cursor?
		require
			a_cursor_not_void: a_cursor /= Void
		do
			Result := a_cursor.container = Current
		end

feature -- Duplication

	copy (other: like Current) is
			-- Copy `other' to current list.
			-- Move all cursors `off' (unless `other = Current').
			-- (Performance: O(other.count).)
		local
			old_cursor: like new_cursor
		do
			capacity := other.capacity
			count := other.count
			--is_shared := true
			if other /= Current then
				old_cursor := internal_cursor
				move_all_cursors_after
				create storage
				storage.copy (other.storage)
				if old_cursor /= Void then
					internal_cursor := old_cursor
				else
						-- Set `internal_cursor' to Void before calling
						-- `new_cursor' to avoid an invariant violation.
					internal_cursor := Void
					internal_cursor := new_cursor
				end
			end
		end

	base_address: POINTER is
			-- The address of the first element
		do
			Result := storage.item
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is list equal to `other'?
			-- Do not take cursor positions, capacity
			-- nor `equality_tester' into account.
			-- (Performance: O(count).)
		local
			i, nb: INTEGER_64
		do
			if Current = other then
				Result := True
			elseif ANY_.same_types (Current, other) and other.count = count then
				from
					i := 1
					nb := count
					Result := True
				until
					not Result or i > nb
				loop
					Result := (internal_item (i).is_equal (other.internal_item (i)))
					i := i + 1
				end
			end
		end

feature -- Element change

	replace (v: G; i: INTEGER_64) is
			-- Replace item at `i'-th position by `v'.
			-- Do not move cursors.
			-- (Performance: O(1).)
		do
			internal_put (v, i)
		end

	put_first (v: G) is
			-- Add `v' to beginning of list.
			-- Do not move cursors.
			-- (Performance: O(count).)
		require
			not_shared: not is_shared
			extendible: extendible (1)
		do
			put (v, 1)
		ensure
			one_more: count = old count + 1
			inserted: first.is_equal (v)
		end

	put_last (v: G) is
			-- Add `v' to end of list.
			-- Do not move cursors.
			-- (Performance: O(1).)
		require
			not_shared: not is_shared
		do
			count := count + 1
			internal_put (v, count)
		ensure
			one_more: count = old count + 1
			inserted: last.is_equal (v)
		end

	put (v: G; i: INTEGER_64) is
			-- Add `v' at `i'-th position.
			-- Do not move cursors.
			-- (Performance: O(count-i).)
		require
			not_shared: not is_shared
			extendible: extendible (1)
			valid_index: 1 <= i and i <= (count + 1)
		do
			if i = (count + 1) then
				put_last (v)
			else
				move_right (i, 1)
				move_cursors_right (i, 1)
				internal_put (v, i)
			end
		ensure
			one_more: count = old count + 1
			inserted: item (i).is_equal (v)
		end

	put_left (v: G) is
			-- Add `v' to left of `index' position.
			-- Do not move cursors.
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			put_left_cursor (v, internal_cursor)
		end

	put_left_cursor (v: G; a_cursor: like new_cursor) is
			-- Add `v' to left of `a_cursor' position.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.put_left (v)'.)
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			put (v, a_cursor.index)
		end

	put_right (v: G) is
			-- Add `v' to right of `index' position.
			-- Do not move cursors.
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			put_right_cursor (v, internal_cursor)
		end

	put_right_cursor (v: G; a_cursor: like new_cursor) is
			-- Add `v' to right of `a_cursor' position.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.put_right (v)'.)
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			put (v, a_cursor.index + 1)
		end

	force_first (v: G) is
			-- Add `v' to beginning of list.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(count)[+resizing].)
		require
			not_shared: not is_shared
		do
			if not extendible (1) then
				resize (new_capacity (count + 1))
			end
			put (v, 1)
		ensure
			one_more: count = old count + 1
			inserted: last = v
		end

	force_last (v: G) is
			-- Add `v' to end of list.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(1)[+resizing].)
		require
			not_shared: not is_shared
		do
			if not extendible (1) then
				resize (new_capacity (count + 1))
			end
			count := count + 1
			internal_put (v, count)
		ensure
			one_more: count = old count + 1
			inserted: last.is_equal (v)
		end

	force (v: G; i: INTEGER_64) is
			-- Add `v' at `i'-th position.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(count-i)[+resizing].)
		require
			valid_index: 1 <= i and i <= (count + 1)
			not_shared: not is_shared
		do
			if not extendible (1) then
				resize (new_capacity (count + 1))
			end
			put (v, i)
		ensure
			one_more: count = old count + 1
			inserted: item (i) .is_equal (v)
		end

	force_left (v: G) is
			-- Add `v' to left of the current position.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(count-a_cursor.index)[+resizing].)
		require
			not_shared: not is_shared
			not_before: not before
		do
			force_left_cursor (v, internal_cursor)
		ensure
			one_more: count = old count + 1
		end

	force_left_cursor (v: G; a_cursor: like new_cursor) is

			-- Add `v' to left of `a_cursor' position.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.force_left (v)'.)
			-- (Performance: O(count-a_cursor.index)[+resizing].)
		require
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_before: not a_cursor.before
			not_shared: not is_shared
		do
			if not extendible (1) then
				resize (new_capacity (count + 1))
			end
			put (v, a_cursor.index)
		ensure
			one_more: count = old count + 1
		end

	force_right (v: G) is
			-- Add `v' to right of the current position.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(count-a_cursor.index)[+resizing].)
		require
			not_shared: not is_shared
			not_after: not after
		do
			force_right_cursor (v, internal_cursor)
		ensure
			one_more: count = old count + 1
		end

	force_right_cursor (v: G; a_cursor: like new_cursor) is
			-- Add `v' to right of `a_cursor' position.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.force_right (v)'.)
			-- (Performance: O(count-a_cursor.index)[+resizing].)
		require
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_after: not a_cursor.after
			not_shared: not is_shared
		do
			if not extendible (1) then
				resize (new_capacity (count + 1))
			end
			put (v, a_cursor.index + 1)
		ensure
			one_more: count = old count + 1
		end

	replace_at (v: G) is
			-- Replace item at internal cursor position by `v'.
			-- Do not move cursors.
		require
			not_off: not off
		do
			internal_cursor.replace (v)
		ensure
			same_count: count = old count
			replaced: item_for_iteration.is_equal (v)
		end

	replace_at_cursor (v: G; a_cursor: like new_cursor) is
			-- Replace item at `a_cursor' position by `v'.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.replace (v)'.)
		require
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_off: not a_cursor.off
		do
			a_cursor.replace (v)
		ensure
			same_count: count = old count
			replaced: a_cursor.item.is_equal (v)
		end

	extend_first (other: DS_LINEAR [G]) is
			-- Add items of `other' to beginning of list.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
			-- (Performance: O(count+other.count).)
		require
			not_shared: not is_shared
		do
			extend (other, 1)
		end

	extend_last (other: DS_LINEAR [G]) is
			-- Add items of `other' to end of list.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
			-- (Performance: O(other.count).)
		require
			not_shared: not is_shared
		local
			i: INTEGER_64
			other_cursor: DS_LINEAR_CURSOR [G]
		do
			i := count + 1
			other_cursor := other.new_cursor
			from other_cursor.start until other_cursor.after loop
				internal_put (other_cursor.item, i)
				i := i + 1
				other_cursor.forth
			end
				-- Set `count' only at this stage in order to cope
				-- properly with the case where `other = Current'.
			count := count + other.count
		end

	extend (other: DS_LINEAR [G]; i: INTEGER_64) is
			-- Add items of `other' at `i'-th position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
			-- (Performance: O(count-i+other.count).)
		require
			other_not_void: other /= Void
			extendible: extendible (other.count)
			valid_index: 1 <= i and i <= (count + 1)
			not_shared: not is_shared
		local
			k, n: INTEGER_64
			other_cursor: DS_LINEAR_CURSOR [G]
		do
			if i = (count + 1) then
				extend_last (other)
			else
				n := other.count
				move_right (i, n)
				move_cursors_right (i, n)
				k := i
				other_cursor := other.new_cursor
				from other_cursor.start until other_cursor.after loop
					internal_put (other_cursor.item, k)
					k := k + 1
					other_cursor.forth
				end
			end
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (i) = other.first)
		end

	extend_left (other: DS_LINEAR [G]) is
			-- Add items of `other' to left of internal cursor position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
		require
			other_not_void: other /= Void
			extendible: extendible (other.count)
			not_before: not before
			not_shared: not is_shared
		do
			extend_left_cursor (other, internal_cursor)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (old index) = other.first)
		end

	extend_left_cursor (other: DS_LINEAR [G]; a_cursor: like new_cursor) is
			-- Add item of `other' to left of `a_cursor' position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
			-- (Synonym of a_cursor.extend_left (other).)
			-- (Performance: O(count-a_cursor.index+other.count).)
		require
			other_not_void: other /= Void
			extendible: extendible (other.count)
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_before: not a_cursor.before
			not_shared: not is_shared
		do
			extend (other, a_cursor.index)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (old (a_cursor.index)) = other.first)
		end

	extend_right (other: DS_LINEAR [G]) is
			-- Add items of `other' to right of internal cursor position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
		require
			other_not_void: other /= Void
			extendible: extendible (other.count)
			not_after: not after
			not_shared: not is_shared
		do
			extend_right_cursor (other, internal_cursor)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (index + 1) = other.first)
		end

	extend_right_cursor (other: DS_LINEAR [G]; a_cursor: like new_cursor) is
			-- Add item of `other' to right of `a_cursor' position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
			-- (Synonym of a_cursor.extend_right (other).)
			-- (Performance: O(count-a_cursor.index+other.count).)
		require
			other_not_void: other /= Void
			extendible: extendible (other.count)
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_after: not a_cursor.after
			not_shared: not is_shared
		do
			extend (other, a_cursor.index + 1)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (a_cursor.index + 1) = other.first)
		end

	append_first (other: DS_LINEAR [G]) is
			-- Add items of `other' to beginning of list.
			-- Keep items of `other' in the same order.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(count+other.count)[+resizing].)
		require
			not_shared: not is_shared
		do
			append (other, 1)
		end

	append_last (other: DS_LINEAR [G]) is
			-- Add items of `other' to end of list.
			-- Keep items of `other' in the same order.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(other.count)[+resizing].)
		require
			not_shared: not is_shared
		local
			nb: INTEGER_64
		do
			nb := other.count
			if not extendible (nb) then
				resize (new_capacity (count + nb))
			end
			extend_last (other)
		end

	append (other: DS_LINEAR [G]; i: INTEGER_64) is
			-- Add items of `other' at `i'-th position.
			-- Keep items of `other' in the same order.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Performance: O(count-i+other.count)[+resizing].)
		require
			other_not_void: other /= Void
			valid_index: 1 <= i and i <= (count + 1)
			not_shared: not is_shared
		local
			nb: INTEGER_64
		do
			nb := other.count
			if not extendible (nb) then
				resize (new_capacity (count + nb))
			end
			extend (other, i)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (i) = other.first)
		end

	append_left (other: DS_LINEAR [G]) is
			-- Add items of `other' to left of internal cursor position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
		require
			other_not_void: other /= Void
			not_before: not before
			not_shared: not is_shared
		do
			append_left_cursor (other, internal_cursor)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (old index) = other.first)
		end

	append_left_cursor (other: DS_LINEAR [G]; a_cursor: like new_cursor) is
			-- Add item of `other' to left of `a_cursor' position.
			-- Keep items of `other' in the same order.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.append_left (other)'.)
			-- (Performance: O(count-a_cursor.index+other.count)[+resizing].)
		require
			other_not_void: other /= Void
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_before: not a_cursor.before
			not_shared: not is_shared
		do
			append (other, a_cursor.index)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (old (a_cursor.index)) = other.first)
		end

	append_right (other: DS_LINEAR [G]) is
			-- Add items of `other' to right of internal cursor position.
			-- Keep items of `other' in the same order.
			-- Do not move cursors.
		require
			other_not_void: other /= Void
			not_after: not after
			not_shared: not is_shared
		do
			append_right_cursor (other, internal_cursor)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (index + 1) = other.first)
		end

	append_right_cursor (other: DS_LINEAR [G]; a_cursor: like new_cursor) is
			-- Add item of `other' to right of `a_cursor' position.
			-- Keep items of `other' in the same order.
			-- Resize container if needed.
			-- Do not move cursors.
			-- (Synonym of `a_cursor.append_right (other)'.)
			-- (Performance: O(count-a_cursor.index+other.count)[+resizing].)
		require
			other_not_void: other /= Void
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			not_after: not a_cursor.after
			not_shared: not is_shared
		do
			append (other, a_cursor.index + 1)
		ensure
			new_count: count = old count + other.count
			same_order: (not other.is_empty) implies (item (a_cursor.index + 1) = other.first)
		end

feature -- removal

	remove_first is
			-- Remove item at beginning of list.
			-- Move any cursors at this position `forth'.
			-- (Performance: O(count).)
		require
			not_shared: not is_shared
		do
			remove (1)
		end

	remove_last is
			-- Remove item at end of list.
			-- Move any cursors at this position `forth'.
			-- (Performance: O(1).)
		require
			not_shared: not is_shared
		do
			move_last_cursors_after
			internal_put (default_item, count)
			count := count - 1
		end

	remove (i: INTEGER_64) is
			-- Remove item at `i'-th position.
			-- Move any cursors at this position `forth'.
			-- (Performance: O(count-i).)
		require
			not_shared: not is_shared
		do
			if i = count then
				remove_last
			else
				move_cursors_left (i + 1)
				move_left (i + 1, 1)
				internal_put (default_item, count + 1)
			end
		end

	remove_at is
			-- Remove item at the current position.
			-- Move any cursors at this position `forth'.
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			remove_at_cursor (internal_cursor)
		end

	remove_at_cursor (a_cursor: like new_cursor) is
			-- Remove item at `a_cursor' position.
			-- Move any cursors at this position `forth'.
			-- (Synonym of `a_cursor.remove'.)
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			remove (a_cursor.index)
		end

	remove_left_cursor (a_cursor: like new_cursor) is
			-- Remove item to left of `a_cursor' position.
			-- Move any cursors at this position `forth'.
			-- (Synonym of `a_cursor.remove_left'.)
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			remove (a_cursor.index - 1)
		end

	remove_right_cursor (a_cursor: like new_cursor) is
			-- Remove item to right of `a_cursor' position.
			-- Move any cursors at this position `forth'.
			-- (Synonym of `a_cursor.remove_right'.)
			-- (Performance: O(count-a_cursor.index).)
		require
			not_shared: not is_shared
		do
			remove (a_cursor.index + 1)
		end

	prune_first (n: INTEGER_64) is
			-- Remove `n' first items from list.
			-- Move all cursors `off'.
			-- (Performance: O(count-n).)
		require
			not_shared: not is_shared
		do
			prune (n, 1)
		end

	prune_last (n: INTEGER_64) is
			-- Remove `n' last items from list.
			-- Move all cursors `off'.
			-- (Performance: O(1).)
		require
			not_shared: not is_shared
		do
			move_all_cursors_after
			clear_items (count - n + 1, count)
			count := count - n
		end

	prune (n: INTEGER_64; i: INTEGER_64) is
			-- Remove `n' items at and after `i'-th position.
			-- Move all cursors `off'.
			-- (Performance: O(count-i-n).)
		require
			not_shared: not is_shared
		do
			if (i + n) = (count + 1) then
				prune_last (n)
			else
				move_all_cursors_after
				move_left (i + n, n)
				clear_items (count + 1, count + n)
			end
		end

	prune_left (n: INTEGER_64) is
			-- Remove `n' items to left of internal cursor position.
			-- Move all cursors `off'.
		require
			not_shared: not is_shared
			valid_n: 0 <= n and n < index
		do
			prune_left_cursor (n, internal_cursor)
		ensure
			new_count: count = old count - n
		end

	prune_left_cursor (n: INTEGER_64; a_cursor: like new_cursor) is
			-- Remove `n' items to left of `a_cursor' position.
			-- Move all cursors `off'.
			-- (Synonym of `a_cursor.prune_left (n)'.)
			-- (Performance: O(count-a_cursor.index).)
		require
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			valid_n: 0 <= n and n < a_cursor.index
			not_shared: not is_shared
		do
			prune (n, a_cursor.index - n)
		ensure
			new_count: count = old count - n
		end

	prune_right (n: INTEGER_64) is
			-- Remove `n' items to right of internal cursor position.
			-- Move all cursors `off'.
		require
			valid_n: 0 <= n and n <= (count - index)
			not_shared: not is_shared
		do
			prune_right_cursor (n, internal_cursor)
		ensure
			new_count: count = old count - n
		end

	prune_right_cursor (n: INTEGER_64; a_cursor: like new_cursor) is
			-- Remove `n' items to right of `a_cursor' position.
			-- Move all cursors `off'.
			-- (Synonym of `a_cursor.prune_right (n)'.)
			-- (Performance: O(count-a_cursor.index-n).)
		require
			cursor_not_void: a_cursor /= Void
			valid_cursor: valid_cursor (a_cursor)
			valid_n: 0 <= n and n <= (count - a_cursor.index)
			not_shared: not is_shared
		do
			prune (n, a_cursor.index + 1)
		ensure
			new_count: count = old count - n
		end

	keep_first (n: INTEGER_64) is
			-- Keep `n' first items in list.
			-- Move all cursors `off'.
			-- (Performance: O(1).)
		do
			move_all_cursors_after
			clear_items (n + 1, count)
			count := n
		end

	keep_last (n: INTEGER_64) is
			-- Keep `n' last items in list.
			-- Move all cursors `off'.
			-- (Performance: O(n).)
		do
			prune_first (count - n)
		end

	delete (v: G) is
			-- Remove all occurrences of `v'.
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
			-- Move all cursors `off'.
			-- (Performance: O(count).)
		require
			not_shared: not is_shared
		local
			i, j, nb: INTEGER_64
			old_count: INTEGER_64
			a_tester: like equality_tester
		do
			move_all_cursors_after
			if not is_empty then
				nb := count
				a_tester := equality_tester
				if a_tester /= Void then
					from i := 1 until i > nb loop
						from
						until
							i > nb or else
							not a_tester.test (internal_item (i), v)
						loop
							i := i + 1
						end
						from
						until
							i > nb or else
							a_tester.test (internal_item (i), v)
						loop
							j := j + 1
							internal_put (internal_item (i), j)
							i := i + 1
						end
					end
				else
						-- Use `=' as comparison criterion.
					from i := 1 until i > nb loop
						from
						until
							i > nb or else
							internal_item (i) /= v
						loop
							i := i + 1
						end
						from
						until
							i > nb or else
							internal_item (i) = v
						loop
							j := j + 1
							internal_put (internal_item (i), j)
							i := i + 1
						end
					end
				end
				old_count := count
				count := j
				clear_items (count + 1, old_count)
			end
		end

	wipe_out is
			-- Remove all items from list.
			-- Move all cursors `off'.
			-- (Performance: O(1).)
		require
			not_shared: not is_shared
		do
			move_all_cursors_after
			clear_items (1, count)
			count := 0
		end

feature -- Resizing

	resize (n: INTEGER_64) is
			-- Resize list so that it can contain
			-- at least `n' items. Do not lose any item.
		require
			not_shared: not is_shared
		do
			storage.resize ((n + 1) *  size_of_object)
			capacity := n + 1
		end

feature -- Conversion

	out: STRING is
			-- Printable representation
		local
			i: INTEGER_64
		do
			from
				i := 1
				create Result.make (40)
			until
				i > count or i > 10
			loop
				Result.append (item (i).out)
				if i < count then
					Result.append (", ")
				end

				i := i + 1
			end
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := out
		end

feature {NONE} -- Implementation

	move_right (i, offset: INTEGER_64) is
			-- Move items at and after `i' position
			-- by `offset' positions to the right.
		require
			valid_index: 1 <= i and i <= count + 1
			positive_offset: offset >= 0
			extendible: extendible (offset)
		local
			j: INTEGER_64
		do
			from
				j := count
				count := count + offset
			until
				j < i
			loop
				internal_put (internal_item (j), j + offset)
				j := j - 1
			end
		ensure
			count_set: count = old count + offset
		end

	move_left (i, offset: INTEGER_64) is
			-- Move items at and after `i' position
			-- by `offset' positions to the left.
		require
			valid_index: 1 <= i and i <= count + 1
			positive_offset: offset >= 0
			constraint: offset < i
		local
			j, nb: INTEGER_64
		do
			from
				j := i
				nb := count
			until
				j > nb
			loop
				internal_put (internal_item (j), j - offset)
				j := j + 1
			end
			count := count - offset
		ensure
			count_set: count = old count - offset
		end

	clear_items (s, e: INTEGER_64) is
			-- Clear items in `storage' within bounds `s'..`e'.
		require
			s_large_enough: s >= 1
			e_small_enough: e <= capacity
			valid_bound: s <= e + 1
		local
			i: INTEGER_64
		do
			from i := s until i > e loop
				internal_put (default_item, i)
				i := i + 1
			end
		end

feature {LARGE_ARRAYED_LIST} -- Implementation

	internal_cursor: like new_cursor
			-- Internal cursor

	internal_item (i: INTEGER_64): G is
			-- Retrive the item from storage, translating to the
			-- appropriate byte position
		require
			i_positive: i > 0
			i_less_than_count: i <= count
		deferred
		end

	internal_put (val: G; i: INTEGER_64) is
			-- Put the item into storage, translating to the
			-- appropriate byte position
		require
			i_positive: i > 0
			i_less_than_count: i <= capacity
		deferred
		end

	item_size: INTEGER is
			-- What is the size of an individual item being stored?
			-- This is meaningless for compund objects.
		deferred
		end

	size_of_object: INTEGER is
			-- What is the size (in bytes) of the object being stored?
		deferred
		end

	create_default_item is
			-- Create a default version of the generic parameter
		deferred
		ensure
			default_item_not_void: default_item /= Void
		end

	create_local_storage is
			-- Create local storage for the clients
		do
		end

feature {NONE} -- Configuration

	default_item: G
			-- A default object

	new_capacity (n: INTEGER_64): INTEGER_64 is
			-- New capacity which could accommodate at least
			-- `n' items (Used as argument of `resize'.)
		require
			n_large_enough: n >= capacity
		do
			Result := (n + 1) * 3 // 2
		ensure
			definition: Result >= n
		end

feature {NONE} -- Cursor movement

	move_all_cursors_after is
			-- Move `after' all cursors.
		local
			a_cursor, next_cursor: like new_cursor
		do
			from a_cursor := internal_cursor until (a_cursor = Void) loop
				a_cursor.set_position (after_position)
				next_cursor := a_cursor.next_cursor
				a_cursor.set_next_cursor (Void)
				a_cursor := next_cursor
			end
		end

	move_last_cursors_after is
			-- Move `after' all cursors at last position.
		local
			i: INTEGER_64
			a_cursor, previous_cursor, next_cursor: like new_cursor
		do
			i := count
			a_cursor := internal_cursor
			if a_cursor.position = i then
				a_cursor.set_position (after_position)
			end
			previous_cursor := a_cursor
			a_cursor := a_cursor.next_cursor
			from until (a_cursor = Void) loop
				if a_cursor.position = i then
					a_cursor.set_position (after_position)
					next_cursor := a_cursor.next_cursor
					previous_cursor.set_next_cursor (next_cursor)
					a_cursor.set_next_cursor (Void)
					a_cursor := next_cursor
				else
					previous_cursor := a_cursor
					a_cursor := a_cursor.next_cursor
				end
			end
		end

	move_cursors_left (i: INTEGER_64) is
			-- Move left by one position all cursors
			-- at or after index `i'.
		require
			valid_i: 2 <= i and i <= count
		local
			j: INTEGER_64
			a_cursor: like new_cursor
		do
			from a_cursor := internal_cursor until (a_cursor = Void) loop
				j := a_cursor.position
				if j >= i then
					a_cursor.set_position (j - 1)
				end
				a_cursor := a_cursor.next_cursor
			end
		end

	move_cursors_right (i, offset: INTEGER_64) is
			-- Move right by `offset' positions all cursors
			-- at or after index `i'.
		require
			valid_index: 1 <= i and i <= (count - offset + 1)
			positive_offset: offset >= 0
		local
			a_cursor: like new_cursor
			j: INTEGER_64
		do
			from a_cursor := internal_cursor until (a_cursor = Void) loop
				j := a_cursor.position
				if j >= i then
					a_cursor.set_position (j + offset)
				end
				a_cursor := a_cursor.next_cursor
			end
		end

feature {LARGE_ARRAYED_LIST_CURSOR} -- Cursor implementation

	add_traversing_cursor (a_cursor: like new_cursor) is
			-- Add `a_cursor' to the list of traversing cursors
			-- (i.e. cursors associated with current container
			-- and which are not currently `off').
		require
			a_cursor_not_void: a_cursor /= Void
		do
			if a_cursor /= internal_cursor then
				a_cursor.set_next_cursor (internal_cursor.next_cursor)
				internal_cursor.set_next_cursor (a_cursor)
			end
		end

	remove_traversing_cursor (a_cursor: like new_cursor) is
			-- Remove `a_cursor' from the list of traversing cursors
			-- (i.e. cursors associated with current container
			-- and which are not currently `off').
		require
			a_cursor_not_void: a_cursor /= Void
		local
			current_cursor, previous_cursor: like new_cursor
		do
			if a_cursor /= internal_cursor then
				from
					previous_cursor := internal_cursor
					current_cursor := previous_cursor.next_cursor
				until
					current_cursor = a_cursor or
						current_cursor = Void
				loop
					previous_cursor := current_cursor
					current_cursor := current_cursor.next_cursor
				end
				if current_cursor = a_cursor then
					previous_cursor.set_next_cursor (a_cursor.next_cursor)
					a_cursor.set_next_cursor (Void)
				end
			end
		end

	cursor_off (a_cursor: like new_cursor): BOOLEAN is
			-- Is there no item at `a_cursor' position?
		do
			Result := cursor_after (a_cursor) or cursor_before (a_cursor)
		end

	cursor_item (a_cursor: like new_cursor): G is
			-- Item at `a_cursor' position
			-- (Performance: O(1).)
		do
			Result := item (a_cursor.position)
		end

	cursor_index (a_cursor: like new_cursor): INTEGER_64 is
			-- Index of `a_cursor''s current position
			-- (Performance: O(1).)
		do
			Result := a_cursor.position
			if Result = after_position then
				Result := count + 1
			end
		end

	cursor_after (a_cursor: like new_cursor): BOOLEAN is
			-- Is there no valid position to right of `a_cursor'?
		do
			Result := (a_cursor.position = after_position)
		end

	cursor_before (a_cursor: like new_cursor): BOOLEAN is
			-- Is there no valid position to left of `a_cursor'?
		do
			Result := a_cursor.position = 0
		end

	cursor_is_first (a_cursor: like new_cursor): BOOLEAN is
			-- Is `a_cursor' on first item?
		do
			Result := not is_empty and a_cursor.position = 1
		end

	cursor_is_last (a_cursor: like new_cursor): BOOLEAN is
			-- Is `a_cursor' on last item?
		do
			Result := not is_empty and a_cursor.position = count
		end

	cursor_same_position (a_cursor, other: like new_cursor): BOOLEAN is
			-- Is `a_cursor' at same position as `other'?
		do
			Result := (a_cursor.position = other.position)
		end

	cursor_start (a_cursor: like new_cursor) is
			-- Move `a_cursor' to first position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			if is_empty then
				a_cursor.set_position (after_position)
			else
				a_cursor.set_position (1)
				if was_off then
					add_traversing_cursor (a_cursor)
				end
			end
		end

	cursor_finish (a_cursor: like new_cursor) is
			-- Move `a_cursor' to last position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			a_cursor.set_position (count)
			if count /= 0 and was_off then
				add_traversing_cursor (a_cursor)
			end
		end

	cursor_forth (a_cursor: like new_cursor) is
			-- Move `a_cursor' to next position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
			p: INTEGER_64
		do
			p := a_cursor.position
			was_off := (p = 0)
			p := p + 1
			if p > count then
				p := after_position
				if not was_off then
					remove_traversing_cursor (a_cursor)
				end
			elseif was_off then
				add_traversing_cursor (a_cursor)
			end
			a_cursor.set_position (p)
		end

	cursor_back (a_cursor: like new_cursor) is
			-- Move `a_cursor' to previous position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
			p: INTEGER_64
		do
			p := a_cursor.position
			if p = after_position then
				was_off := True
				p := count
			else
				p := p - 1
			end
			a_cursor.set_position (p)
			if p = 0 then
				if not was_off then
					remove_traversing_cursor (a_cursor)
				end
			elseif was_off then
				add_traversing_cursor (a_cursor)
			end
		end

	cursor_search_forth (a_cursor: like new_cursor; v: G) is
			-- Move `a_cursor' to first position at or after its current
			-- position where `cursor_item (a_cursor)' and `v' are equal.
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
			-- Move `after' if not found.
		local
			i, nb: INTEGER_64
			a_tester: like equality_tester
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			i := a_cursor.index
			nb := count
			a_tester := equality_tester
			if a_tester /= Void then
				from until
					i > nb or else a_tester.test (item (i), v)
				loop
					i := i + 1
				end
			else
					-- Use `=' as comparison criterion.
				from until
					i > nb or else item (i) = v
				loop
					i := i + 1
				end
			end
			if i > nb then
				a_cursor.set_position (after_position)
				if not was_off then
					remove_traversing_cursor (a_cursor)
				end
			else
				a_cursor.set_position (i)
				if was_off then
					add_traversing_cursor (a_cursor)
				end
			end
		end

	cursor_search_back (a_cursor: like new_cursor; v: G) is
			-- Move `a_cursor' to first position at or before its current
			-- position where `cursor_item (a_cursor)' and `v' are equal.
			-- (Use `equality_tester''s comparison criterion
			-- if not void, use `=' criterion otherwise.)
			-- Move `before' if not found.
			-- (Performance: O(a_cursor.index).)
		local
			i: INTEGER_64
			a_tester: like equality_tester
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			i := a_cursor.position
			a_tester := equality_tester
			if a_tester /= Void then
				from until
					i < 1 or else a_tester.test (item (i), v)
				loop
					i := i - 1
				end
			else
					-- Use `=' as comparison criterion.
				from until
					i < 1 or else item (i) = v
				loop
					i := i - 1
				end
			end
			a_cursor.set_position (i)
			if i = 0 then
				if not was_off then
					remove_traversing_cursor (a_cursor)
				end
			elseif was_off then
				add_traversing_cursor (a_cursor)
			end
		end

	cursor_go_after (a_cursor: like new_cursor) is
			-- Move `a_cursor' to `after' position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			a_cursor.set_position (after_position)
			if not was_off then
				remove_traversing_cursor (a_cursor)
			end
		end

	cursor_go_before (a_cursor: like new_cursor) is
			-- Move `a_cursor' to `before' position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			a_cursor.set_position (0)
			if not was_off then
				remove_traversing_cursor (a_cursor)
			end
		end

	cursor_go_to (a_cursor, other: like new_cursor) is
			-- Move `a_cursor' to `other''s position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			a_cursor.set_position (other.position)
			if not a_cursor.off then
				if was_off then
					add_traversing_cursor (a_cursor)
				end
			elseif not was_off then
				remove_traversing_cursor (a_cursor)
			end
		end

	cursor_go_i_th (a_cursor: like new_cursor; i: INTEGER_64) is
			-- Move `a_cursor' to `i'-th position.
			-- (Performance: O(1).)
		local
			was_off: BOOLEAN
		do
			was_off := a_cursor.off
			if i > count then
				a_cursor.set_position (after_position)
				if not was_off then
					remove_traversing_cursor (a_cursor)
				end
			else
				a_cursor.set_position (i)
				if i = 0 then
					if not was_off then
						remove_traversing_cursor (a_cursor)
					end
				elseif was_off then
					add_traversing_cursor (a_cursor)
				end
			end
		end

	after_position: INTEGER_64 is -1
			-- Special value for after cursor position

invariant

	capacity_definition: capacity >= storage.count // size_of_object

end
