

indexing

	description:

		"Cursors for arrayed list traversals"

	library: "Gobo Eiffel Structure Library"
	copyright: "Copyright (c) 1999-2001, Eric Bezault and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2006-06-23 10:34:45 -0400 (Fri, 23 Jun 2006) $"
	revision: "$Revision: 226 $"

class

	LARGE_ARRAYED_LIST_CURSOR [G]

creation

	make

feature {NONE} -- Initialization

	make (a_list: like container) is
			-- Create a new cursor for `a_list'.
		require
			a_list_not_void: a_list /= Void
		do
			container := a_list
			-- position := 0
		ensure
			container_set: container = a_list
			before: before
		end

feature -- Access

	item: G is
			-- Item at cursor position
		require
			not_off: not off
		do
			Result := container.cursor_item (Current)
		end

	container: LARGE_ARRAYED_LIST [G]
			-- List traversed

	index: INTEGER_64 is
			-- Index of current position
		do
			Result := container.cursor_index (Current)
		end

feature -- Element change

	replace (v: G) is
			-- Replace item at cursor position by `v'.
			-- (Performance: O(1).)
		do
			container.replace (v, position)
		end

feature -- Status report

	off: BOOLEAN is
			-- Is there no item at cursor position?
		do
			Result := container.cursor_off (Current)
		end

	is_first: BOOLEAN is
			-- Is cursor on first item?
		do
			Result := container.cursor_is_first (Current)
		ensure
			not_empty: Result implies not container.is_empty
			not_off: Result implies not off
		end

	after: BOOLEAN is
			-- Is there no valid position to right of cursor?
		do
			Result := container.cursor_after (Current)
		end

	is_last: BOOLEAN is
			-- Is cursor on last item?
		do
			Result := container.cursor_is_last (Current)
		ensure
			not_empty: Result implies not container.is_empty
			not_off: Result implies not off
		end

	before: BOOLEAN is
			-- Is there no valid position to left of cursor?
		do
			Result := container.cursor_before (Current)
		end

feature -- Cursor movement

	start is
			-- Move cursor to first position.
		do
			container.cursor_start (Current)
		ensure
			empty_behavior: container.is_empty implies after
			first_or_after: is_first xor after
		end

	forth is
			-- Move cursor to next position.
		require
			not_after: not after
		do
			container.cursor_forth (Current)
		end

	search_forth (v: G) is
			-- Move to first position at or after current
			-- position where `item' and `v' are equal.
			-- (Use `equality_tester''s criterion from `container'
			-- if not void, use `=' criterion otherwise.)
			-- Move `after' if not found.
		require
			not_off: not off or after
		do
			container.cursor_search_forth (Current, v)
		end

	go_after is
			-- Move cursor to `after' position.
		do
			container.cursor_go_after (Current)
		ensure
			after: after
		end

	finish is
			-- Move cursor to last position.
		do
			container.cursor_finish (Current)
		ensure
			empty_behavior: container.is_empty implies before
			last_or_before: is_last xor before
		end

	back is
			-- Move cursor to previous position.
		require
			not_before: not before
		do
			container.cursor_back (Current)
		end

	search_back (v: G) is
			-- Move to first position at or before current
			-- position where `item' and `v' are equal.
			-- (Use `equality_tester''s criterion from `container'
			-- if not void, use `=' criterion otherwise.)
			-- Move `before' if not found.
		require
			not_off: not off or before
		do
			container.cursor_search_back (Current, v)
		end

	go_before is
			-- Move cursor to `before' position.
		do
			container.cursor_go_before (Current)
		ensure
			before: before
		end

feature {LARGE_ARRAYED_LIST, LARGE_ARRAYED_LIST_CURSOR} -- Implementation

	position: INTEGER_64
			-- Internal position in list

feature {LARGE_ARRAYED_LIST} -- Implementation

	set_next_cursor (a_cursor: like next_cursor) is
			-- Set `next_cursor' to `a_cursor'.
		do
			next_cursor := a_cursor
		ensure
			next_cursor_set: next_cursor = a_cursor
		end

	set_position (i: INTEGER_64) is
			-- Set `position' to `i'.
		require
			valid_i: valid_position (i)
		do
			position := i
		ensure
			position_set: position = i
		end

	set_after is
			-- Set `position' to after position.
		do
			position := container.after_position
		ensure
			after: after
		end

	set_before is
			-- Set `position' to before position.
		do
			position := 0
		ensure
			before: before
		end

	valid_position (p: INTEGER_64): BOOLEAN is
			-- Is `p' a valid value for `position'?
		do
			Result := p = container.after_position or
				(0 <= p and p <= container.count)
		ensure
			not_off: (1 <= p and p <= container.count) implies Result
			before: (p = 0) implies Result
			after: (p = container.after_position) implies Result
		end

feature {LARGE_ARRAYED_LIST} -- Implementation

	next_cursor: LARGE_ARRAYED_LIST_CURSOR [G]
			-- Next cursor
			-- (Used by `container' to keep track of traversing
			-- cursors (i.e. cursors associated with `container'
			-- and which are not currently `off').)

invariant

-- The following assertion are commented out because
-- some Eiffel compilers check invariants even when the
-- execution of the creation procedure is not completed.
-- (In this case, this is `container' which is not fully
-- created yet, breaking its invariant.)

--	valid_position: valid_position (position)

end
