

indexing
	description: "Arrayed list of points"
	names: "list", "point"
	representation: "array", "list"
	access: "sequential", "indexed"
	size: "resizable"
	contents: "POINT2"
	patterns: "none"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	author: "Mark A. Bolstad"
	status: "unreviewed"


class 

	POINT2_ARRAYED_LIST

inherit

	LARGE_ARRAYED_LIST [POINT2]

	PLATFORM
		undefine
			default_create, is_equal, copy, out
		end
	
creation
	default_create, make, make_equal, make_default,
	make_from_linear, make_from_array, make_shared

feature {POINT2_ARRAYED_LIST} -- Implementation

	make_shared (other: DOUBLE_ARRAYED_LIST) is
			-- Create a list that shares items from `other'
			-- Use `=' as comparison criterion.
		require
			other_is_valid: other /= Void
		do
			create storage.share_from_pointer (other.storage.item, other.storage.count)
			capacity := other.storage.count // size_of_object
			count := other.count // item_stride
			internal_cursor := new_cursor
		ensure
			empty: not is_empty
			capacity_set: capacity = other.storage.count // size_of_object
			before: before
		end

	internal_item (i: INTEGER_64): POINT2 is
			-- Retrive the item from storage, translating to the 
			-- appropriate byte position
		do
			create Result
			Result.put_x (storage.read_double ((item_stride * (i - 1) )* item_size))
			Result.put_y (storage.read_double ((item_stride * (i - 1) + 1)* item_size))
		end
	
	internal_put (val: POINT2; i: INTEGER_64) is
			-- Put the item into storage, translating to the 
			-- appropriate byte position
		do
			storage.put_double (val.x, (item_stride * (i - 1) )* item_size)
			storage.put_double (val.y, (item_stride * (i - 1) + 1)* item_size)
		end

	item_size: INTEGER is
			-- Put the item into storage, translating to the 
			-- appropriate byte position
		do
			Result := real64_bits // 8
		end
	
	size_of_object: INTEGER is
			-- What is the size (in bytes) of the object being stored?
		do
			Result := item_size * item_stride
		end	
	
	create_default_item is
			-- Create a default version of the generic parameter
		do
			create default_item
		end
	
feature {NONE} -- Implementation

	item_stride: INTEGER is 2
			-- How many `item_size' elements between objects.
	
end -- POINT2_ARRAYED_LIST

