

indexing
	description: "Arrayed list of points"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"


class 

	POINT3_ARRAYED_LIST

inherit

	LARGE_ARRAYED_LIST [POINT3]
		redefine
			create_local_storage
		end

	PLATFORM
		undefine
			default_create, is_equal, copy, out
		end
	
creation
	default_create, make, make_equal, make_default,
	make_from_linear, make_from_array, make_shared

feature {POINT3_ARRAYED_LIST} -- Implementation

	make_shared (other: DOUBLE_ARRAYED_LIST) is
			-- Create a list that shares items from `other'
			-- Use `=' as comparison criterion.
		require
			other_is_valid: other /= Void
		do
			create storage.share_from_pointer (other.storage.item, other.storage.count)
			capacity := other.storage.count // size_of_object
			count := other.count // item_stride
			internal_cursor := new_cursor
		ensure
			empty: not is_empty
			capacity_set: capacity = other.storage.count // size_of_object
			before: before
		end

	internal_item (i: INTEGER_64): POINT3 is
			-- Retrive the item from storage, translating to the 
			-- appropriate byte position
		do
			create Result
			storage.read_doubles ((item_stride * (i - 1) )* item_size, 3, tmp_array)
			Result.put_x (tmp_array.item (1))
			Result.put_y (tmp_array.item (2))
			Result.put_z (tmp_array.item (3))
		end
	
	internal_put (val: POINT3; i: INTEGER_64) is
			-- Put the item into storage, translating to the 
			-- appropriate byte position
		do
			storage.put_double (val.x, (item_stride * (i - 1) )* item_size)
			storage.put_double (val.y, (item_stride * (i - 1) + 1)* item_size)
			storage.put_double (val.z, (item_stride * (i - 1) + 2)* item_size)
		end

	item_size: INTEGER is
			-- Put the item into storage, translating to the 
			-- appropriate byte position
		do
			Result := real64_bits // 8
		end
	
	size_of_object: INTEGER is
			-- What is the size (in bytes) of the object being stored?
		do
			Result := item_size * item_stride
		end	
	
	create_default_item is
			-- Create a default version of the generic parameter
		do
			create default_item
		end
	
feature {NONE} -- Implementation

	create_local_storage is
			-- Create local storage for the clients
		do
			if tmp_array = Void then
				create tmp_array.make (1, 3)
			end
		end

	item_stride: INTEGER is 3
			-- How many `item_size' elements between objects.

	tmp_array: ARRAY [DOUBLE]
			-- Temporary array for passing data to the large memory 
			-- structure

invariant
	tmp_array_not_void: tmp_array /= Void
	tmp_array_has_three_elements: tmp_array.count = 3
	
end -- POINT3_ARRAYED_LIST

