

indexing
	description: "Arrayed list of RGBA colors"
	names: "list", "color"
	representation: "array", "list"
	access: "sequential", "indexed"
	size: "resizable"
	contents: "RGBA_COLOR"
	patterns: "none"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	author: "Mark A. Bolstad"
	status: "unreviewed"


class

	RGBA_COLOR_ARRAYED_LIST

inherit

	LARGE_ARRAYED_LIST [RGBA_COLOR]
		redefine
			make
		end

	PLATFORM
		undefine
			default_create, is_equal, copy, out
		end

creation
	default_create, make, make_equal, make_default,
	make_from_linear, make_from_array, make_shared

feature {NONE} -- Initialization

	make (n: INTEGER_64) is
			-- Create an empty list and allocate
			-- memory space for at least `n' items.
			-- Use `=' as comparison criterion.
		do
			Precursor {LARGE_ARRAYED_LIST} (n)

			create local_storage.make (1, 4)
		end

feature {RGBA_COLOR_ARRAYED_LIST} -- Implementation

	make_shared (other: DOUBLE_ARRAYED_LIST) is
			-- Create a list that shares items from `other'
			-- Use `=' as comparison criterion.
		require
			other_is_valid: other /= Void
		do
			create storage.share_from_pointer (other.storage.item, other.storage.count)
			capacity := other.storage.count // size_of_object
			count := other.count // item_stride
			internal_cursor := new_cursor
		ensure
			empty: not is_empty
			capacity_set: capacity = other.storage.count // size_of_object
			before: before
		end

	internal_item (i: INTEGER_64): RGBA_COLOR is
			-- Retrive the item from storage, translating to the
			-- appropriate byte position
		local
			loc: INTEGER_64
		do
			create Result
			loc := (item_stride * (i - 1)) * item_size
			storage.read_doubles (loc, item_stride, local_storage)
			Result.put_red (local_storage.item (1))
			Result.put_green (local_storage.item (2))
			Result.put_blue (local_storage.item (3))
			Result.put_alpha (local_storage.item (4))
		end

	internal_put (val: RGBA_COLOR; i: INTEGER_64) is
			-- Put the item into storage, translating to the
			-- appropriate byte position
		do
			storage.put_double (val.red, (item_stride * (i - 1) )* item_size)
			storage.put_double (val.green, (item_stride * (i - 1) + 1)* item_size)
			storage.put_double (val.blue, (item_stride * (i - 1) + 2)* item_size)
			storage.put_double (val.alpha, (item_stride * (i - 1) + 3)* item_size)
		end

	item_size: INTEGER is
			-- Put the item into storage, translating to the
			-- appropriate byte position
		do
			Result := real64_bits // 8
		end

	size_of_object: INTEGER is
			-- What is the size (in bytes) of the object being stored?
		do
			Result := item_size * item_stride
		end

	create_default_item is
			-- Create a default version of the generic parameter
		do
			create default_item
		end

feature {NONE} -- Implementation

	item_stride: INTEGER is 4
			-- How many `item_size' elements between objects.

	local_storage: ARRAY [DOUBLE]
			-- local storage for reading

invariant

	local_storage_not_void: local_storage /= Void
	local_storage_count_is_four: local_storage.count = 4

end -- RGBA_COLOR_ARRAYED_LIST

