indexing

	description: "Arbitrary-length real-valued vectors."
	names: "vector"
	representation: "ARRAY", "DOUBLE"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"

class

	GENERAL_VECTOR

inherit

	VECTOR

	LINEAR_TUPLE_ND

	LINEAR_TUPLE_ND_IMP
	
creation

	make, make_from_array, make_from_c

feature -- Basic operations

	infix "+" (other: like Current): like Current is
			-- Sum with `other'.
		local
			index: INTEGER
			left, right: DOUBLE
		do
			Result := Current.deep_twin

			from
				index := 1
			invariant
				index_at_least_one: index >= 1
				index_small_enough: index <= 1 + count
			variant
				index_increasing: 1 + count - index
			until
				index > count
			loop
				left := item (index)
				right := other.item (index)

				if left = -right then
						-- Avoid nan (e.g., -inf + inf).
					Result.put (0.0, index)
				else
					Result.put (left + right, index)
				end

				index := index + 1
			end
		end

	infix "-" (other: like Current): like Current is
			-- Difference with `other'.
		local
			index: INTEGER
			left, right: DOUBLE
		do
			Result := Current.deep_twin

			from
				index := 1
			invariant
				index_at_least_one: index >= 1
				index_small_enough: index <= 1 + count
			variant
				index_increasing: 1 + count - index
			until
				index > count
			loop
				left := item (index)
				right := other.item (index)

				if left = right then
						-- Avoid nan (e.g., inf - inf).
					Result.put (0.0, index)
				else
					Result.put (left - right, index)
				end

				index := index + 1
			end
		end

end -- class GENERAL_VECTOR
