
indexing
	description: "N element linear algebra objects."
	names: "linear tuple"
	representation: "array"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"
	patterns: "none"

deferred class

	LINEAR_TUPLE_ND_IMP

inherit

	LINEAR_TUPLE

feature -- Initialization

	make (the_count: INTEGER) is
			-- Make a zero vector with given number of components.
		do
			create elements.make (1, the_count)
		end

	make_from_array (values: ARRAY [DOUBLE]) is
			-- Make a vector from an array of component values.
		do
			make (values.count)
			elements := values.twin
		ensure then
			duplicate:elements.is_equal (values)
		end

feature -- Access

	item (index: INTEGER): DOUBLE is
			-- Coordinate/component at `index'.
		do
			Result := elements.item (index)
		end

feature -- Measurement

	count: INTEGER is
			-- The number of elements.
		do
			Result := elements.count
		end

feature -- Element change

	put (value: DOUBLE; index: INTEGER) is
			-- Assign `value' to component/coordinate at `index'.
		do
			elements.put (value, index)
		end

feature -- Conversion

	to_c: POINTER is
			-- Base of values for passing to C routines.
--		local
--			p: EXTERNAL_POINTER [DOUBLE]
		do
--			Result := p.get_pointer (elements)
		end

feature {NONE} -- Implementation

	elements: ARRAY [DOUBLE]
			-- The vector components.

invariant

	elements_not_void: elements /= Void
	count_correct: elements.count = count

end -- LINEAR_TUPLE_ND_IMP

