indexing

	description: "4 x 4 real matrices."
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "reviewed", "untested", "partial", "unimplemented: ^"

class

	MATRIX4

inherit
	
	SQUARE_MATRIX
		redefine
			is_equal, default_create, to_array_argument
		end
	
creation
	default_create, make_with_elements, make_from_c, make_from_array, make_identity
	
feature -- Initialization
	
	default_create is
			-- Create a matrix of zeros
		do
			update_required := true
		end
	
	make_with_elements (x1, y1, z1, w1,
						x2, y2, z2, w2,
						x3, y3, z3, w3,
						x4, y4, z4, w4: DOUBLE) is
			-- Create with the given elements.
		do
			update_required := true
			
			a11 := x1
			a12 := y1
			a13 := z1
			a14 := w1
			a21 := x2
			a22 := y2
			a23 := z2
			a24 := w2
			a31 := x3
			a32 := y3
			a33 := z3
			a34 := w3
			a41 := x4
			a42 := y4
			a43 := z4
			a44 := w4
		ensure
			a11_is_x1: about_equal (a11, x1)
			a12_is_y1: about_equal (a12, y1)
			a13_is_z1: about_equal (a13, z1)
			a14_is_w1: about_equal (a14, w1)
			a21_is_x2: about_equal (a21, x2)
			a22_is_y2: about_equal (a22, y2)
			a23_is_z2: about_equal (a23, z2)
			a24_is_w2: about_equal (a24, w2)
			a31_is_x3: about_equal (a31, x3)
			a32_is_y3: about_equal (a32, y3)
			a33_is_z3: about_equal (a33, z3)
			a34_is_w3: about_equal (a34, w3)
			a41_is_x1: about_equal (a41, x4)
			a42_is_y1: about_equal (a42, y4)
			a43_is_z1: about_equal (a43, z4)
			a44_is_w1: about_equal (a44, w4)
		end

	make_from_c (address: POINTER; the_count: INTEGER) is
			-- Make from pointer to row-major array of 16 C doubles.
		do
			update_required := true
			fill (0.0)
			copy_doubles ($a11, address, the_count.min (16))
		end

	make_from_array (values: ARRAY [DOUBLE]) is
			-- Make with these component values.
			-- Default implementation uses `to_c' and so must be
			-- redefined by expanded heirs.
		do
			update_required := true
			a11 := values.item (1)
			a12 := values.item (2)
			a13 := values.item (3)
			a14 := values.item (4)
			a21 := values.item (5)
			a22 := values.item (6)
			a23 := values.item (7)
			a24 := values.item (8)
			a31 := values.item (9)
			a32 := values.item (10)
			a33 := values.item (11)
			a34 := values.item (12)
			a41 := values.item (13)
			a42 := values.item (14)
			a43 := values.item (15)
			a44 := values.item (16)
		end
	
	make_from_list (values: DOUBLE_ARRAYED_LIST) is
			-- Make with these component values.
		do
			update_required := true
			a11 := values.item (1)
			a12 := values.item (2)
			a13 := values.item (3)
			a14 := values.item (4)
			a21 := values.item (5)
			a22 := values.item (6)
			a23 := values.item (7)
			a24 := values.item (8)
			a31 := values.item (9)
			a32 := values.item (10)
			a33 := values.item (11)
			a34 := values.item (12)
			a41 := values.item (13)
			a42 := values.item (14)
			a43 := values.item (15)
			a44 := values.item (16)
		end
	
	make_identity is
			-- Make `Current' the neutral element for "*".
		do
			fill (0.0)
			a11 := 1.0
			a22 := 1.0
			a33 := 1.0
			a44 := 1.0
		ensure then
			diagonals_are_one:
				a11 = 1.0 and a22 = 1.0 and a33 = 1.0 and a44 = 1.0
			rest_are_zero:
				a12 = 0.0 and a13 = 0.0 and a14 = 0.0 and
				a21 = 0.0 and a23 = 0.0 and a24 = 0.0 and
				a31 = 0.0 and a32 = 0.0 and a34 = 0.0 and
				a41 = 0.0 and a42 = 0.0 and a43 = 0.0
		end

	fill (initial_value: DOUBLE) is
			-- Set each element equal to `value'.
		do
			update_required := true
			
			a11 := initial_value
			a12 := initial_value
			a13 := initial_value
			a14 := initial_value
			a21 := initial_value
			a22 := initial_value
			a23 := initial_value
			a24 := initial_value
			a31 := initial_value
			a32 := initial_value
			a33 := initial_value
			a34 := initial_value
			a41 := initial_value
			a42 := initial_value
			a43 := initial_value
			a44 := initial_value
		ensure then
			a11_is_value: about_equal (a11, initial_value)
			a12_is_value: about_equal (a12, initial_value)
			a13_is_value: about_equal (a13, initial_value)
			a14_is_value: about_equal (a14, initial_value)
			a21_is_value: about_equal (a21, initial_value)
			a22_is_value: about_equal (a22, initial_value)
			a23_is_value: about_equal (a23, initial_value)
			a24_is_value: about_equal (a24, initial_value)
			a31_is_value: about_equal (a31, initial_value)
			a32_is_value: about_equal (a32, initial_value)
			a33_is_value: about_equal (a33, initial_value)
			a34_is_value: about_equal (a34, initial_value)
			a41_is_value: about_equal (a41, initial_value)
			a42_is_value: about_equal (a42, initial_value)
			a43_is_value: about_equal (a43, initial_value)
			a44_is_value: about_equal (a44, initial_value)
		end

feature -- Access

	a11, a12, a13, a14,
	a21, a22, a23, a24,
	a31, a32, a33, a34,
	a41, a42, a43, a44: DOUBLE
			-- The matrix elements subscripted by (row, column).

	item (row, column: INTEGER): DOUBLE is
			-- Element at coordinates (`row', `column').
		do
			inspect
				row
			when 1 then
				inspect
					column
				when 1 then
					Result := a11
				when 2 then
					Result := a12
				when 3 then
					Result := a13
				when 4 then
					Result := a14
				end
			when 2 then
				inspect
					column
				when 1 then
					Result := a21
				when 2 then
					Result := a22
				when 3 then
					Result := a23
				when 4 then
					Result := a24
				end
			when 3 then
				inspect
					column
				when 1 then
					Result := a31
				when 2 then
					Result := a32
				when 3 then
					Result := a33
				when 4 then
					Result := a34
				end
			when 4 then
				inspect
					column
				when 1 then
					Result := a41
				when 2 then
					Result := a42
				when 3 then
					Result := a43
				when 4 then
					Result := a44
				end
			end
		end

	zero: MATRIX4 is
			-- Neutral element for "+" and "-".
		once
			create Result
		end

	one: MATRIX4 is
			-- Neutral element for "*".
		once
			create Result.make_identity
		end

feature -- Measurement

	column_count: INTEGER is 4
			-- Number of columns.

	row_count: INTEGER is 4
			-- Number of rows.
	
	count: INTEGER is 16
			-- Total number of elements
	
	frobenius_norm, two_norm: DOUBLE is
			-- Square-root of the sum of the squares of the elements.
		do
			Result := square_root (
						a11 * a11 + a12 * a12 + a13 * a13 + a14 * a14 +
						a21 * a21 + a22 * a22 + a23 * a23 + a24 * a24 +
						a31 * a31 + a32 * a32 + a33 * a33 + a34 * a34 +
						a41 * a41 + a42 * a42 + a43 * a43 + a44 * a44)
		end

	one_norm: DOUBLE is
			-- Maximum absolute column sum of the elements.
		local
			column_1_sum, column_2_sum, column_3_sum, column_4_sum: DOUBLE
		do
			column_1_sum := absolute_value (a11) + absolute_value (a21) +
							absolute_value (a31) + absolute_value (a41)
			column_2_sum := absolute_value (a12) + absolute_value (a22) +
							absolute_value (a32) + absolute_value (a42)
			column_3_sum := absolute_value (a13) + absolute_value (a23) +
							absolute_value (a33) + absolute_value (a43)
			column_4_sum := absolute_value (a14) + absolute_value (a24) +
							absolute_value (a34) + absolute_value (a44)
			Result := column_1_sum.max (column_2_sum).max
						(column_3_sum).max (column_4_sum)
		end

	max_norm: DOUBLE is
			-- Maximum absolute row sum of the elements.
		local
			row_1_sum, row_2_sum, row_3_sum, row_4_sum: DOUBLE
		do
			row_1_sum := absolute_value (a11) + absolute_value (a21) +
							absolute_value (a31) + absolute_value (a41)
			row_2_sum := absolute_value (a12) + absolute_value (a22) +
							absolute_value (a32) + absolute_value (a42)
			row_3_sum := absolute_value (a13) + absolute_value (a23) +
							absolute_value (a33) + absolute_value (a43)
			row_4_sum := absolute_value (a14) + absolute_value (a24) +
							absolute_value (a34) + absolute_value (a44)
			Result := row_1_sum.max (row_2_sum).max (row_3_sum).max (row_4_sum)
		end

feature -- Comparison

	is_equal (other: MATRIX4): BOOLEAN is
			-- Is each corresponding `item' within `Tolerance'?
		do
			Result := (about_equal (a11, other.a11) and then
						about_equal (a12, other.a12) and then
						about_equal (a13, other.a13) and then
						about_equal (a14, other.a14) and then

						about_equal (a21, other.a21) and then
						about_equal (a22, other.a22) and then
						about_equal (a23, other.a23) and then
						about_equal (a24, other.a24) and then

						about_equal (a31, other.a31) and then
						about_equal (a32, other.a32) and then
						about_equal (a33, other.a33) and then
						about_equal (a34, other.a34) and then

						about_equal (a41, other.a41) and then
						about_equal (a42, other.a42) and then
						about_equal (a43, other.a43) and then
						about_equal (a44, other.a44))
		end

feature -- Status report

	is_singular: BOOLEAN is
			-- Is the matrix non-invertable?
		local
			determinant_magnitude: DOUBLE
		do
			determinant_magnitude := absolute_value (determinant)
			Result := is_nan (determinant_magnitude) or else
						determinant_magnitude < Tolerance * Tolerance
		end

feature -- Element change

	put (value: like item; row, column: INTEGER) is
			-- Make `item' (`row', `column') = `value'.
		do
			update_required := true
			
			inspect
				row
			when 1 then
				inspect
					column
				when 1 then
					a11 := value
				when 2 then
					a12 := value
				when 3 then
					a13 := value
				when 4 then
					a14 := value
				end
			when 2 then
				inspect
					column
				when 1 then
					a21 := value
				when 2 then
					a22 := value
				when 3 then
					a23 := value
				when 4 then
					a24 := value
				end
			when 3 then
				inspect
					column
				when 1 then
					a31 := value
				when 2 then
					a32 := value
				when 3 then
					a33 := value
				when 4 then
					a34 := value
				end
			when 4 then
				inspect
					column
				when 1 then
					a41 := value
				when 2 then
					a42 := value
				when 3 then
					a43 := value
				when 4 then
					a44 := value
				end
			end
		end	

feature -- Transformation

	transpose is
			-- Transpose the current matrix.
		local
			tmp: DOUBLE
		do
			update_required := true
			
			tmp := a12
			a12 := a21
			a21 := tmp

			tmp := a13
			a13 := a31
			a31 := tmp

			tmp := a14
			a14 := a41
			a41 := tmp

			tmp := a23
			a23 := a32
			a32 := tmp

			tmp := a24
			a24 := a42
			a42 := tmp

			tmp := a34
			a34 := a43
			a43 := tmp
		ensure then
			a11_is_correct: about_equal (a11, old a11)
			a12_is_correct: about_equal (a12, old a21)
			a13_is_correct: about_equal (a13, old a31)
			a14_is_correct: about_equal (a14, old a41)
			a21_is_correct: about_equal (a21, old a12)
			a22_is_correct: about_equal (a22, old a22)
			a23_is_correct: about_equal (a23, old a32)
			a24_is_correct: about_equal (a24, old a42)
			a31_is_correct: about_equal (a31, old a13)
			a32_is_correct: about_equal (a32, old a23)
			a33_is_correct: about_equal (a33, old a33)
			a34_is_correct: about_equal (a34, old a43)
			a41_is_correct: about_equal (a41, old a14)
			a42_is_correct: about_equal (a42, old a24)
			a43_is_correct: about_equal (a43, old a34)
			a44_is_correct: about_equal (a44, old a44)
		end

	inverse: MATRIX4 is
			-- Attempt to invert `Current'.
			-- If successful (matrix was non-singular), `inverted' is True,
			-- otherwise `inverted' is False and the matrix is unchanged.
		do
			if update_required then
				cached_inverse := Current.twin
				cached_inverse.invert
				update_required := false
			end
			
			Result := cached_inverse
		end
	
	invert is
			-- Attempt to invert `Current'.
			-- If successful (matrix was non-singular), `inverted' is True,
			-- otherwise `inverted' is False and the matrix is unchanged.
		local
			the_determinant: DOUBLE
			tmp_matrix: like Current
		do
			inverted := False
			
			the_determinant := determinant
			
			if the_determinant /= 0.0 then

				the_determinant := 1.0 / the_determinant

				if the_determinant /= 0.0 then
						-- Determinant wasn't infinite.
					tmp_matrix := Current.twin
					
					scaled_adjugate (the_determinant)
					inverted := all_finite and then not is_singular
					
					if not inverted then
						Current.copy (tmp_matrix)
					else
						update_required := true
					end
				end
			end

		end
	
feature -- Conversion

	to_array_argument (a: DOUBLE_ARRAYED_LIST) is
			-- Replace the items of `a' with the row-major elements of `Current'.
		local
			index: INTEGER_64
		do
			index := 1
			a.replace (a11, index)
			a.replace (a12, index + 1)
			a.replace (a13, index + 2)
			a.replace (a14, index + 3)

			a.replace (a21, index + 4)
			a.replace (a22, index + 5)
			a.replace (a23, index + 6)
			a.replace (a24, index + 7)

			a.replace (a31, index + 8)
			a.replace (a32, index + 9)
			a.replace (a33, index + 10)
			a.replace (a34, index + 11)

			a.replace (a41, index + 12)
			a.replace (a42, index + 13)
			a.replace (a43, index + 14)
			a.replace (a44, index + 15)
		end

feature -- Basic operations

	to_real, determinant: DOUBLE is
			-- Compute the determinant.
		do
			Result := a11 * determinant3 (a22, a23, a24,
											a32, a33, a34,
											a42, a43, a44)
					- a21 * determinant3 (a12, a13, a14,
											a32, a33, a34,
											a42, a43, a44)
					+ a31 * determinant3 (a12, a13, a14,
											a22, a23, a24,
											a42, a43, a44)
					- a41 * determinant3 (a12, a13, a14,
											a22, a23, a24,
											a32, a33, a34)
		end

	infix "*" (o: MATRIX4): MATRIX4 is
			-- Product by `o'.
		do
			create Result.make_with_elements (
						a11 * o.a11 + a12 * o.a21 + a13 * o.a31 + a14 * o.a41,
						a11 * o.a12 + a12 * o.a22 + a13 * o.a32 + a14 * o.a42,
						a11 * o.a13 + a12 * o.a23 + a13 * o.a33 + a14 * o.a43,
						a11 * o.a14 + a12 * o.a24 + a13 * o.a34 + a14 * o.a44,
						a21 * o.a11 + a22 * o.a21 + a23 * o.a31 + a24 * o.a41,
						a21 * o.a12 + a22 * o.a22 + a23 * o.a32 + a24 * o.a42,
						a21 * o.a13 + a22 * o.a23 + a23 * o.a33 + a24 * o.a43,
						a21 * o.a14 + a22 * o.a24 + a23 * o.a34 + a24 * o.a44,
						a31 * o.a11 + a32 * o.a21 + a33 * o.a31 + a34 * o.a41,
						a31 * o.a12 + a32 * o.a22 + a33 * o.a32 + a34 * o.a42,
						a31 * o.a13 + a32 * o.a23 + a33 * o.a33 + a34 * o.a43,
						a31 * o.a14 + a32 * o.a24 + a33 * o.a34 + a34 * o.a44,
						a41 * o.a11 + a42 * o.a21 + a43 * o.a31 + a44 * o.a41,
						a41 * o.a12 + a42 * o.a22 + a43 * o.a32 + a44 * o.a42,
						a41 * o.a13 + a42 * o.a23 + a43 * o.a33 + a44 * o.a43,
						a41 * o.a14 + a42 * o.a24 + a43 * o.a34 + a44 * o.a44)
		end

	infix "+" (other: MATRIX4): MATRIX4 is
			-- Sum with `other' (commutative).
		do
			create Result.make_with_elements (
						filter_nan (a11 + other.a11),
						filter_nan (a12 + other.a12),
						filter_nan (a13 + other.a13),
						filter_nan (a14 + other.a14),
						filter_nan (a21 + other.a21),
						filter_nan (a22 + other.a22),
						filter_nan (a23 + other.a23),
						filter_nan (a24 + other.a24),
						filter_nan (a31 + other.a31),
						filter_nan (a32 + other.a32),
						filter_nan (a33 + other.a33),
						filter_nan (a34 + other.a34),
						filter_nan (a41 + other.a41),
						filter_nan (a42 + other.a42),
						filter_nan (a43 + other.a43),
						filter_nan (a44 + other.a44))
		ensure then
			result_a11_is_sum: about_equal (Result.a11,
											filter_nan (a11 + other.a11))
			result_a12_is_sum: about_equal (Result.a12,
											filter_nan (a12 + other.a12))
			result_a13_is_sum: about_equal (Result.a13,
											filter_nan (a13 + other.a13))
			result_a14_is_sum: about_equal (Result.a14,
											filter_nan (a14 + other.a14))
			result_a21_is_sum: about_equal (Result.a21,
											filter_nan (a21 + other.a21))
			result_a22_is_sum: about_equal (Result.a22,
											filter_nan (a22 + other.a22))
			result_a23_is_sum: about_equal (Result.a23,
											filter_nan (a23 + other.a23))
			result_a24_is_sum: about_equal (Result.a24,
											filter_nan (a24 + other.a24))
			result_a31_is_sum: about_equal (Result.a31,
											filter_nan (a31 + other.a31))
			result_a32_is_sum: about_equal (Result.a32,
											filter_nan (a32 + other.a32))
			result_a33_is_sum: about_equal (Result.a33,
											filter_nan (a33 + other.a33))
			result_a34_is_sum: about_equal (Result.a34,
											filter_nan (a34 + other.a34))
			result_a41_is_sum: about_equal (Result.a41,
											filter_nan (a41 + other.a41))
			result_a42_is_sum: about_equal (Result.a42,
											filter_nan (a42 + other.a42))
			result_a43_is_sum: about_equal (Result.a43,
											filter_nan (a43 + other.a43))
			result_a44_is_sum: about_equal (Result.a44,
											filter_nan (a44 + other.a44))
		end

	infix "-" (other: MATRIX4): MATRIX4 is
			-- Result of subtracting `other'.
		do
			create Result.make_with_elements (
						filter_nan (a11 - other.a11),
						filter_nan (a12 - other.a12),
						filter_nan (a13 - other.a13),
						filter_nan (a14 - other.a14),
						filter_nan (a21 - other.a21),
						filter_nan (a22 - other.a22),
						filter_nan (a23 - other.a23),
						filter_nan (a24 - other.a24),
						filter_nan (a31 - other.a31),
						filter_nan (a32 - other.a32),
						filter_nan (a33 - other.a33),
						filter_nan (a34 - other.a34),
						filter_nan (a41 - other.a41),
						filter_nan (a42 - other.a42),
						filter_nan (a43 - other.a43),
						filter_nan (a44 - other.a44))
		ensure then
			result_a11_is_difference: about_equal (Result.a11,
												   filter_nan (a11 - other.a11))
			result_a12_is_difference: about_equal (Result.a12,
												   filter_nan (a12 - other.a12))
			result_a13_is_difference: about_equal (Result.a13,
												   filter_nan (a13 - other.a13))
			result_a14_is_difference: about_equal (Result.a14,
												   filter_nan (a14 - other.a14))
			result_a21_is_difference: about_equal (Result.a21,
												   filter_nan (a21 - other.a21))
			result_a22_is_difference: about_equal (Result.a22,
												   filter_nan (a22 - other.a22))
			result_a23_is_difference: about_equal (Result.a23,
												   filter_nan (a23 - other.a23))
			result_a24_is_difference: about_equal (Result.a24,
												   filter_nan (a24 - other.a24))
			result_a31_is_difference: about_equal (Result.a31,
												   filter_nan (a31 - other.a31))
			result_a32_is_difference: about_equal (Result.a32,
												   filter_nan (a32 - other.a32))
			result_a33_is_difference: about_equal (Result.a33,
												   filter_nan (a33 - other.a33))
			result_a34_is_difference: about_equal (Result.a34,
												   filter_nan (a34 - other.a34))
			result_a41_is_difference: about_equal (Result.a41,
												   filter_nan (a41 - other.a41))
			result_a42_is_difference: about_equal (Result.a42,
												   filter_nan (a42 - other.a42))
			result_a43_is_difference: about_equal (Result.a43,
												   filter_nan (a43 - other.a43))
			result_a44_is_difference: about_equal (Result.a44,
												   filter_nan (a44 - other.a44))
		end

	infix "^" (exponent: NUMERIC): like Current is
			-- Current raised to the power `exponent'.
		do
				check
					implemented: False
				end
		end

	prefix "+": MATRIX4 is
			-- Unary plus.
		do
---			Result := Current
			create Result.make_with_elements (a11, a12, a13, a14,
										a21, a22, a23, a24,
										a31, a32, a33, a34,
										a41, a42, a43, a44)
		end

	prefix "-": MATRIX4 is
			-- Unary minus
		do
			create Result.make_with_elements (-a11, -a12, -a13, -a14,
										-a21, -a22, -a23, -a24,
										-a31, -a32, -a33, -a34,
										-a41, -a42, -a43, -a44)
		ensure then
			result_a11_is_negated: about_equal (result.a11, -a11)
			result_a12_is_negated: about_equal (result.a12, -a12)
			result_a13_is_negated: about_equal (result.a13, -a13)
			result_a14_is_negated: about_equal (result.a14, -a14)
			result_a21_is_negated: about_equal (result.a21, -a21)
			result_a22_is_negated: about_equal (result.a22, -a22)
			result_a23_is_negated: about_equal (result.a23, -a23)
			result_a24_is_negated: about_equal (result.a24, -a24)
			result_a31_is_negated: about_equal (result.a31, -a31)
			result_a32_is_negated: about_equal (result.a32, -a32)
			result_a33_is_negated: about_equal (result.a33, -a33)
			result_a34_is_negated: about_equal (result.a34, -a34)
			result_a41_is_negated: about_equal (result.a41, -a41)
			result_a42_is_negated: about_equal (result.a42, -a42)
			result_a43_is_negated: about_equal (result.a43, -a43)
			result_a44_is_negated: about_equal (result.a44, -a44)
		end

	scale (scalar: DOUBLE): MATRIX4 is
			-- Result of multiplying each element by `scalar'.
		do
			create Result.make_with_elements (
						a11 * scalar, a12 * scalar, a13 * scalar, a14 * scalar,
						a21 * scalar, a22 * scalar, a23 * scalar, a24 * scalar,
						a31 * scalar, a32 * scalar, a33 * scalar, a34 * scalar,
						a41 * scalar, a42 * scalar, a43 * scalar, a44 * scalar)
		ensure then
			result_a11_is_scaled: about_equal (result.a11, a11 * scalar)
			result_a12_is_scaled: about_equal (result.a12, a12 * scalar)
			result_a13_is_scaled: about_equal (result.a13, a13 * scalar)
			result_a14_is_scaled: about_equal (result.a14, a14 * scalar)
			result_a21_is_scaled: about_equal (result.a21, a21 * scalar)
			result_a22_is_scaled: about_equal (result.a22, a22 * scalar)
			result_a23_is_scaled: about_equal (result.a23, a23 * scalar)
			result_a24_is_scaled: about_equal (result.a24, a24 * scalar)
			result_a31_is_scaled: about_equal (result.a31, a31 * scalar)
			result_a32_is_scaled: about_equal (result.a32, a32 * scalar)
			result_a33_is_scaled: about_equal (result.a33, a33 * scalar)
			result_a34_is_scaled: about_equal (result.a34, a34 * scalar)
			result_a41_is_scaled: about_equal (result.a41, a41 * scalar)
			result_a42_is_scaled: about_equal (result.a42, a42 * scalar)
			result_a43_is_scaled: about_equal (result.a43, a43 * scalar)
			result_a44_is_scaled: about_equal (result.a44, a44 * scalar)
		end

feature {NONE} -- Implementation
	
	update_required: BOOLEAN
			-- Has the matrix been updated?
	
	cached_inverse: like Current
			-- Cached inverse computation
	
	scaled_adjugate (scalar: DOUBLE) is
			-- Form the scaled transpose of the cofactor matrix.
		local
			ta11, ta12, ta13, ta14,
			ta21, ta22, ta23, ta24,
			ta31, ta32, ta33, ta34,
			ta41, ta42, ta43, ta44: DOUBLE
		do
			ta11 := a11
			ta12 := a12
			ta13 := a13
			ta14 := a14

			ta21 := a21
			ta22 := a22
			ta23 := a23
			ta24 := a24

			ta31 := a31
			ta32 := a32
			ta33 := a33
			ta34 := a34

			ta41 := a41
			ta42 := a42
			ta43 := a43
			ta44 := a44

				-- Compute the transpose of the cofactors:

			a11 := scalar * determinant3(ta22, ta23, ta24,
										ta32, ta33, ta34,
										ta42, ta43, ta44)
			a12 := scalar * -determinant3(ta12, ta13, ta14,
										ta32, ta33, ta34,
										ta42, ta43, ta44)
			a13 := scalar * determinant3(ta12, ta13, ta14,
										ta22, ta23, ta24,
										ta42, ta43, ta44)
			a14 := scalar * -determinant3(ta12, ta13, ta14,
										ta22, ta23, ta24,
										ta32, ta33, ta34)

			a21 := scalar * -determinant3(ta21, ta23, ta24,
										ta31, ta33, ta34,
										ta41, ta43, ta44)
			a22 := scalar * determinant3(ta11, ta13, ta14,
										ta31, ta33, ta34,
										ta41, ta43, ta44)
			a23 := scalar * -determinant3(ta11, ta13, ta14,
										ta21, ta23, ta24,
										ta41, ta43, ta44)
			a24 := scalar * determinant3(ta11, ta13, ta14,
										ta21, ta23, ta24,
										ta31, ta33, ta34)

			a31 := scalar * determinant3(ta21, ta22, ta24,
										ta31, ta32, ta34,
										ta41, ta42, ta44)
			a32 := scalar * -determinant3(ta11, ta12, ta14,
										ta31, ta32, ta34,
										ta41, ta42, ta44)
			a33 := scalar * determinant3(ta11, ta12, ta14,
										ta21, ta22, ta24,
										ta41, ta42, ta44)
			a34 := scalar * -determinant3(ta11, ta12, ta14,
										ta21, ta22, ta24,
										ta31, ta32, ta34)

			a41 := scalar * -determinant3(ta21, ta22, ta23,
										ta31, ta32, ta33,
										ta41, ta42, ta43)
			a42 := scalar * determinant3(ta11, ta12, ta13,
										ta31, ta32, ta33,
										ta41, ta42, ta43)
			a43 := scalar * -determinant3(ta11, ta12, ta13,
										ta21, ta22, ta23,
										ta41, ta42, ta43)
			a44 := scalar * determinant3(ta11, ta12, ta13,
										ta21, ta22, ta23,
										ta31, ta32, ta33)
		end

	determinant3 (ta11, ta12, ta13,
				  ta21, ta22, ta23,
				  ta31, ta32, ta33: DOUBLE): DOUBLE is
			-- Determinant of the 3 x 3 matrix.
			--| Computed by Laplace expansion of the first row.
		do
			Result := ta11 * (ta22 * ta33 - ta23 * ta32)
					- ta21 * (ta12 * ta33 - ta13 * ta32)
					+ ta31 * (ta12 * ta23 - ta13 * ta22)
		end

	all_finite: BOOLEAN is
			-- Are all elements non-nan and not infinity or minus_infinity?
		do
			Result :=
				is_finite (a11) and then
				is_finite (a12) and then
				is_finite (a13) and then
				is_finite (a14) and then
				is_finite (a21) and then
				is_finite (a22) and then
				is_finite (a23) and then
				is_finite (a24) and then
				is_finite (a31) and then
				is_finite (a32) and then
				is_finite (a33) and then
				is_finite (a34) and then
				is_finite (a41) and then
				is_finite (a42) and then
				is_finite (a43) and then
				is_finite (a44)
		end

invariant

	both_dimensions_are_four: row_count = 4 and column_count = 4

end -- class MATRIX4

