

indexing
	description: "3d cartesian normals"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	NORMAL

inherit

	ANY
		undefine
			default_create, copy, out, is_equal
		end
		
	VECTOR3
		rename
			transform as vector3_transform,
			apply as vector3_apply
		export
			{NONE}
			   vector3_transform, vector3_apply
		redefine
			scale, infix "+", infix "-", prefix "+", prefix "-"
		end
	
creation
	make, default_create, make_from_array, make_from_c, make_from_vector

feature -- Initialization
	
	make_from_vector (v: VECTOR3) is
			-- Create a normal from `v'
		require 
			v_not_void: v /= Void
		do
			make (v.x, v.y, v.z)
			normalize
		ensure 
			is_unit: is_unit
		end 

feature -- Access
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion

	to_vector: VECTOR3 is
			-- Create a vector
		do
			create Result.make (x, y, z)
		end
		
feature -- Duplication
feature -- Basic operations
	
	transform (xform: TRANSFORM): NORMAL is
			-- Transform the point by `xform'
		require 
			xform_not_void: xform /= Void
		do
			Result := Current.twin
			Result.apply (xform)
		ensure 
			Result_not_void: Result /= Void
			Transformed: Result.transform (xform.inverse).is_equal (Current)
		end
	
	apply (xform: TRANSFORM) is
			-- Transform the point by `xform'
		local
			sx, sy, sz: DOUBLE
			inv_xform: TRANSFORM
		do
			inv_xform := xform.inverse
			sx := inv_xform.a11 * x + inv_xform.a21 * y + inv_xform.a31 * z
			sy := inv_xform.a12 * x + inv_xform.a22 * y + inv_xform.a32 * z
			sz := inv_xform.a13 * x + inv_xform.a23 * y + inv_xform.a33 * z

			make (sx, sy, sz)
		ensure 
			-- Bug (?) in 5.1.7; `old Current' = Current
--			Transformed: transform (xform.inverse).is_equal (old Current)
		end 

	scale (scalar: DOUBLE): NORMAL is
			-- Scale by the real `scalar'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			create Result
			
			if scalar /= 0.0 then

				if x /= 0.0 then
					result_x := x * scalar
				end

				if y /= 0.0 then
					result_y := y * scalar
				end

				if z /= 0.0 then
					result_z := z * scalar
				end

				Result.make (result_x, result_y, result_z)
			end
		end
	
	prefix "+": NORMAL is
			-- Positive of `Current'.
		do
			create Result.make (x, y, z)
		end

	prefix "-": NORMAL is
			-- Negative of `Current'.
		do
			create Result.make (-x, -y, -z)
		end

	infix "-" (other: NORMAL): NORMAL is
			-- Difference with `other'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			if x /= other.x then
				result_x := x - other.x
			end

			if y /= other.y then
				result_y := y - other.y
			end

			if z /= other.z then
				result_z := z - other.z
			end

			create Result.make (result_x, result_y, result_z)
		end

	infix "+" (other: NORMAL): NORMAL is
			-- Sum with `other'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			if x /= -other.x then
				result_x := x + other.x
			end

			if y /= -other.y then
				result_y := y + other.y
			end

			if z /= -other.z then
				result_z := z + other.z
			end

			create Result.make (result_x, result_y, result_z)
		end

	dot_with_vector (other: VECTOR3): DOUBLE is
			-- Inner-product with `other'.
		local
			the_product: DOUBLE
		do
			Result := product (x, other.x)
			the_product := product (y, other.y)

			Result := sum (Result, the_product)
			the_product := product (z, other.z)

			Result := sum (Result, the_product)
		ensure then
			result_correct:
				about_equal (Result,
							filter_nan (filter_nan (filter_nan (x * other.x) +
													filter_nan (y * other.y)) +
										filter_nan (z * other.z)))
		end
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

end -- NORMAL

