indexing

	description: "3D cartesian points."
	names: "3d", "cartesian", "point"
	representation: "expanded", "DOUBLE"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"

class

	POINT3

inherit

	POINT
		rename
			make as make_with_elements
		undefine
			default_create, two_norm, magnitude, to_real,
			magnitude_squared, unit, prefix "+"
		end

	LINEAR_TUPLE_3D
	
creation
	make, default_create, make_from_array, make_from_c
	
feature -- Basic operations

	transform (xform: TRANSFORM): POINT3 is
			-- Transform the point by `xform'
		require 
			xform_not_void: xform /= Void
		do
			Result := Current.twin
			Result.apply (xform)
		ensure 
			Result_not_void: Result /= Void
			Transformed: Result.transform (xform.inverse).is_equal (Current)
		end
	
	apply (xform: TRANSFORM) is
			-- Transform the point by `xform'
		require 
			xform_not_void: xform /= Void
		local
			sx, sy, sz, sw: DOUBLE
		do
			sx := xform.a11 * x + xform.a12 * y + xform.a13 * z + xform.a14
			sy := xform.a21 * x + xform.a22 * y + xform.a23 * z + xform.a24
			sz := xform.a31 * x + xform.a32 * y + xform.a33 * z + xform.a34

				-- For perspective and uniform scaling
			
			sw := xform.a41 * x + xform.a42 * y + xform.a43 * z + xform.a44

			if sw /= 1.0 and then sw /= 0.0 then
				sw := 1.0 / sw
				sx := sx * sw
				sy := sy * sw
				sz := sz * sw
			end

				check
					is_transformed: transform_components (sx, sy, sz, xform.inverse).is_equal (Current)
				end
			make (sx, sy, sz)
		ensure 
			-- Transformed: transform (xform.inverse).is_equal (old Current)
		end 

	infix "-" (other: like Current): VECTOR3 is
			-- Difference with `other'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			if x /= other.x then
				result_x := x - other.x
			end

			if y /= other.y then
				result_y := y - other.y
			end

			if z /= other.z then
				result_z := z - other.z
			end

			create Result.make (result_x, result_y, result_z)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_difference: Result.x = filter_nan (x - other.x)
			result_y_is_y_difference: Result.y = filter_nan (y - other.y)
			result_z_is_z_difference: Result.z = filter_nan (z - other.z)
		end

	infix "+" (other: VECTOR3): like Current is
			-- Sum with `other'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			if x /= -other.x then
				result_x := x + other.x
			end

			if y /= -other.y then
				result_y := y + other.y
			end

			if z /= -other.z then
				result_z := z + other.z
			end

			create Result.make (result_x, result_y, result_z)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_sum: Result.x = filter_nan (x + other.x)
			result_y_is_y_sum: Result.y = filter_nan (y + other.y)
			result_z_is_z_sum: Result.z = filter_nan (z + other.z)
		end

feature {NONE} -- Implementation

	transform_components (sx, sy, sz: DOUBLE; xform: TRANSFORM): POINT3 is
			-- 
		require 
			xform_not_void: xform /= Void
		local
			pt: POINT3
		do
			create pt.make (sx, sy, sz)
			Result := pt.transform (xform)
		end 

end -- class POINT3
