indexing

	description: "4D cartesian points."
	names: "4d", "cartesian", "point"
	representation: "expanded", "DOUBLE"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"

class

	POINT4

inherit

	POINT
		rename
			make as make_with_elements
		undefine
			default_create, two_norm, magnitude, to_real,
			magnitude_squared, unit, prefix "+"
		end

	LINEAR_TUPLE_4D
	
creation
	make, default_create, make_from_array, make_from_c
	
feature -- Basic operations

	infix "-" (other: like Current): VECTOR4 is
			-- Difference with `other'.
		local
			result_x, result_y, result_z, result_w: DOUBLE
		do
			if x /= other.x then
				result_x := x - other.x
			end

			if y /= other.y then
				result_y := y - other.y
			end

			if z /= other.z then
				result_z := z - other.z
			end

			if w /= other.w then
				result_w := w - other.w
			end

			create Result.make (result_x, result_y, result_z, result_w)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_difference: Result.x = filter_nan (x - other.x)
			result_y_is_y_difference: Result.y = filter_nan (y - other.y)
			result_z_is_z_difference: Result.z = filter_nan (z - other.z)
			result_w_is_w_difference: Result.w = filter_nan (w - other.w)
		end

	infix "+" (other: VECTOR4): like Current is
			-- Sum with `other'.
		local
			result_x, result_y, result_z, result_w: DOUBLE
		do
			if x /= -other.x then
				result_x := x + other.x
			end

			if y /= -other.y then
				result_y := y + other.y
			end

			if z /= -other.z then
				result_z := z + other.z
			end

			if w /= -other.w then
				result_w := w + other.w
			end

			create Result.make (result_x, result_y, result_z, result_w)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_sum: Result.x = filter_nan (x + other.x)
			result_y_is_y_sum: Result.y = filter_nan (y + other.y)
			result_z_is_z_sum: Result.z = filter_nan (z + other.z)
			result_w_is_w_sum: Result.w = filter_nan (w + other.w)
		end

	weighted_sum (weight1: DOUBLE; other: like Current; weight2: DOUBLE): like Current is
			-- Computed the weighted sum of `Current' and `other'
			do
				create Result.make (x * weight1 + other.x * weight2,
														y * weight1 + other.y * weight2,
														z * weight1 + other.z * weight2,
														w * weight1 + other.w * weight2)
			end
		
	dehomogenize: POINT3 is
			-- Convert to a `POINT3' by removing the homogenous coordinate
			do
				create Result.make (x/w, y/w, z/w)
			end
		
end -- class POINT4
