indexing

	description: "N x N real matricies."
	names: "matrix", "square", "N x N"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"
	date: "$Date: 2006-06-23 10:34:45 -0400 (Fri, 23 Jun 2006) $"
	revision: "$Revision: 226 $"
	author: "Mark Bolstad, US EPA/LM, bolstad.mark@epa.gov"
	status: "reviewed", "untested"

deferred class 

	SQUARE_MATRIX

inherit

	MATRIX

feature -- Initialization

	make_identity is
			-- Make `Current' the neutral element for "*".
		deferred
		end

feature -- Status report

	is_singular: BOOLEAN is
			-- Is the matrix non-invertable?
		deferred
		end

	inverted: BOOLEAN
			-- Did the last `invert' succeed?

feature -- Transformation

	inverse: like Current is
			-- Attempt to invert `Current'.
			-- If successful (matrix was non-singular), `inverted' is True,
			-- otherwise `inverted' is False and the matrix is unchanged.
		deferred
		ensure
			result_correct: Result.inverted implies 
							Current.one.is_equal (Current * Result)
		end

	invert is
			-- Attempt to invert `Current'.
			-- If successful (matrix was non-singular), `inverted' is True,
			-- otherwise `inverted' is False and the matrix is unchanged.
		deferred
		ensure
			correctly_inverted: inverted implies 
									Current.one.is_equal (Current.inverse * Current)
		end

feature -- Basic operations

	infix "^" (exponent: NUMERIC): like Current is
			-- Current raised to the power `exponent'.
		deferred
		ensure then
			result_exists: Result /= Void
		end

	to_real, determinant: DOUBLE is
			-- The determinant.
		deferred
		end
	
feature -- Conversion

	to_array_argument (a: DOUBLE_ARRAYED_LIST) is
			-- Replace the items of `a' with the row-major elements of `Current'.
		local
			i64: INTEGER_64
			i, j: INTEGER
		do
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + row_count
			variant
				i_increasing: 1 + row_count - i
			until
				i64 >= count
			loop
				from
					j := 1 
				invariant
					j_at_least_one: j >= 1
					j_small_enough: j <= 1 + column_count
				variant
					j_increasing: 1 + column_count - j
				until
					j >= column_count
				loop
					a.put (item (i, j), i64)
 
					j := j + 1
					i64 := i64 + 1
				end
				i := i + 1
			end
		end
	
feature {SQUARE_MATRIX} -- Implementation
	
	scaled_adjugate (scalar: DOUBLE) is
			-- Form the scaled transpose of the cofactor matrix.
		require
			non_zero_scalar: scalar /= 0.0
		deferred
		end
	
invariant

	same_dimensions: row_count = column_count

end -- deferred class SQUARE_MATRIX
