indexing

	description: "3D cartesian vectors."
	names: "3d", "cartesian", "vector", "line", "ray"
	representation: "expanded", "DOUBLE"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"

class

	VECTOR3

inherit

	VECTOR
		rename
			make as make_with_elements
		undefine
			default_create, two_norm, magnitude, to_real,
			magnitude_squared, unit, prefix "+"
		redefine
			cross, infix "*"
		end

	LINEAR_TUPLE_3D
	
creation
	make, default_create, make_from_array, make_from_c
	
feature -- Basic operations

	transform (xform: TRANSFORM): VECTOR3 is
			-- Transform the point by `xform'
		require 
			xform_not_void: xform /= Void
		do
			Result := Current.twin
			Result.apply (xform)
		ensure 
			Result_not_void: Result /= Void
			Transformed: Result.transform (xform.inverse).is_equal (Current)
		end
	
	apply (xform: TRANSFORM) is
			-- Transform the point by `xform'
		require 
			xform_not_void: xform /= Void
		local
			sx, sy, sz, sw: DOUBLE
		do
			sx := xform.a11 * x + xform.a12 * y + xform.a13 * z
			sy := xform.a21 * x + xform.a22 * y + xform.a23 * z
			sz := xform.a31 * x + xform.a32 * y + xform.a33 * z

				-- For perspective and uniform scaling
			
			sw := xform.a41 * x + xform.a42 * y + xform.a43 * z

			if sw /= 1.0 and then sw /= 0.0 then
				sw := 1.0 / sw
				sx := sx * sw
				sy := sy * sw
				sz := sz * sw
			end

			make (sx, sy, sz)
		ensure 
			Transformed: transform (xform.inverse).is_equal (old Current)
		end 

	infix "+" (other: like Current): like Current is
			-- Sum with `other'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			if x /= -other.x then
				result_x := x + other.x
			end

			if y /= -other.y then
				result_y := y + other.y
			end

			if z /= -other.z then
				result_z := z + other.z
			end

			create Result.make (result_x, result_y, result_z)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_sum: Result.x = filter_nan (x + other.x)
			result_y_is_y_sum: Result.y = filter_nan (y + other.y)
			result_z_is_z_sum: Result.z = filter_nan (z + other.z)
		end

	infix "-" (other: like Current): like Current is
			-- Difference with `other'.
		local
			result_x, result_y, result_z: DOUBLE
		do
			if x /= other.x then
				result_x := x - other.x
			end

			if y /= other.y then
				result_y := y - other.y
			end

			if z /= other.z then
				result_z := z - other.z
			end

			create Result.make (result_x, result_y, result_z)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_difference: Result.x = filter_nan (x - other.x)
			result_y_is_y_difference: Result.y = filter_nan (y - other.y)
			result_z_is_z_difference: Result.z = filter_nan (z - other.z)
		end

	cross, infix "*" (other: like Current): like Current is
			-- Cross Product by `other'.
		do
			create Result.make (filter_nan (filter_nan (y * other.z) - filter_nan (z * other.y)),
								filter_nan (filter_nan (z * other.x) - filter_nan (x * other.z)),
								filter_nan (filter_nan (x * other.y) - filter_nan (y * other.x)))
		ensure then
			result_x_correct:
							 Result.x = filter_nan (filter_nan (y * other.z) - filter_nan (z * other.y))
			result_y_correct:
							 Result.y = filter_nan (filter_nan (z * other.x) - filter_nan (x * other.z))
			result_z_correct:
							 Result.z = filter_nan (filter_nan (x * other.y) - filter_nan (y * other.x))
							 
--			orthogonal: Result.all_finite implies (about_equal (Result # Current, 0.0) and
--												   about_equal (Result # other, 0.0))
		end

end -- class VECTOR3
