indexing

	description: "4D cartesian vectors."
	names: "4d", "cartesian", "vector", "line", "ray"
	representation: "expanded", "DOUBLE"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"

class

	VECTOR4

inherit

	VECTOR
		rename
			make as make_with_elements
		undefine
			default_create, two_norm, magnitude, to_real,
			magnitude_squared, unit, prefix "+"
		end

	LINEAR_TUPLE_4D
	
creation
	make, default_create, make_from_array, make_from_c
	
feature -- Basic operations

	infix "-" (other: like Current): like Current is
			-- Difference with `other'.
		local
			result_x, result_y, result_z, result_w: DOUBLE
		do
			if x /= other.x then
				result_x := x - other.x
			end

			if y /= other.y then
				result_y := y - other.y
			end

			if z /= other.z then
				result_z := z - other.z
			end

			if w /= other.w then
				result_w := w - other.w
			end

			create Result.make (result_x, result_y, result_z, result_w)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_difference: Result.x = filter_nan (x - other.x)
			result_y_is_y_difference: Result.y = filter_nan (y - other.y)
			result_z_is_z_difference: Result.z = filter_nan (z - other.z)
			result_w_is_w_difference: Result.w = filter_nan (w - other.w)
		end

	infix "+" (other: like Current): like Current is
			-- Sum with `other'.
		local
			result_x, result_y, result_z, result_w: DOUBLE
		do
			if x /= -other.x then
				result_x := x + other.x
			end

			if y /= -other.y then
				result_y := y + other.y
			end

			if z /= -other.z then
				result_z := z + other.z
			end

			if w /= -other.w then
				result_w := w + other.w
			end

			create Result.make (result_x, result_y, result_z, result_w)
		ensure then
			Result_not_void: Result /= Void
			result_x_is_x_sum: Result.x = filter_nan (x + other.x)
			result_y_is_y_sum: Result.y = filter_nan (y + other.y)
			result_z_is_z_sum: Result.z = filter_nan (z + other.z)
			result_w_is_w_sum: Result.w = filter_nan (w + other.w)
		end

	dehomogenize: VECTOR3 is
			-- Convert to a `VECTOR3' by removing the homogenous coordinate
			do
				create Result.make (x/w, y/x, z/w)
			end
		
end -- class VECTOR4
