indexing

	description: "Collections of numeric values."
	names: "numeric", "data"
	patterns: "template method"
	representation: "array"
	access: "fixed"
	size: "fixed"
	contents: "COMPARABLE"

deferred class

	NUMERIC_COLLECTION

inherit

	BASIC_NUMERIC
		export
			{ANY}
				is_nan
		end

	HASHABLE
	
feature -- Initialization

	make_from_array (values: ARRAY [DOUBLE]) is
			-- Make with these component values.
			-- Default implementation uses `to_c' and so must be redefined by
			-- expanded heirs.
		require
			values_exist: values /= Void
			values_not_empty: values.count > 0
			argument_contents_are_nan_free:
				not is_nan (values.item (1)) and
					not is_nan (values.item (values.count))
		deferred
		end

	make_from_c (address: POINTER; the_count: INTEGER) is
			-- Re-construct from address of C storage of `the_count' DOUBLEs.
			-- Default implementation uses `to_c' and so must be redefined by
			-- expanded heirs.
		require
			address_not_default_pointer: address /= address.default
			count_strictly_positive: the_count > 0
			-- argument_contents_are_nan_free
		deferred		
		end

	fill (initial_value: DOUBLE) is
			-- Set each component to `initial_value'.
		require
			non_nan_argument: not is_nan (initial_value)
		deferred
		end

feature -- Status report

	count: INTEGER is
			-- Number of components/coordinates (e.g., 2 for COMPLEX/VECTOR2).
		deferred
		end

feature -- Conversion

	to_c: POINTER is
			-- Area of `count' doubles for passing to C routines.
			-- For passing the pointer to an instance of a reference
			-- NUMERIC_COMPARABLE object to a C routine:
			--	 Declare:
			--	   local the_area: ANY
			--	 Assign:
			--	   the_area := my_reference_numeric_comparable.to_c
			--	   the_count := my_reference_numeric_comparable.count
			--	 Call:
			--	   the_c_routine ($the_area, the_count)
			-- For achieving the same result for an instance of an expanded
			-- NUMERIC_COMPARABLE (containing only basic type (e.g., DOUBLE)
			-- attributes):
			--	 Call:
			--		the_c_routine ($my_expanded_numeric_comparable, the_count)
			-- For the above to work, expanded heirs must declare all of the
			-- appropriate expanded NUMERIC attributes at the start of the
			-- class, in order, so that the POINTER points at the contiguous
			-- block of writable basic (e.g., DOUBLE) attributes.
		require
--			has_area: has_area
		deferred
		ensure
			result_exists: Result /= default_pointer
		end

feature -- Basic operations

	to_real: DOUBLE is
			-- Project to a real. (E.g., to implement "<").
		deferred
		end

feature {NONE} -- Implementation

	copy_doubles (to_doubles, from_doubles: POINTER; the_count: INTEGER) is
			-- Copy `the_count' doubles `from_doubles' `to_doubles'.
		require
			to_doubles_exists: to_doubles /= default_pointer
			from_doubles_exists: from_doubles /= default_pointer
			the_count_strictly_positive: the_count > 0
		do
			copy_bytes (to_doubles, from_doubles, the_count * 8)
		end

	copy_bytes (to_bytes, from_bytes: POINTER; the_count: INTEGER) is
			-- Copy `the_count' bytes `from_bytes' `to_bytes'.
--		require
--			to_bytes_exists: to_bytes /= default_pointer
--			from_bytes_exists: from_bytes /= default_pointer
--			the_count_strictly_positive: the_count > 0
		external
			"C use <string.h>"
		alias
			"memcpy"
		end

invariant

	count_positive: count > 0
--	has_area_iff_reference_type:
--		has_area = (dynamic_type (Current) = Reference_type)

end -- deferred class NUMERIC_COLLECTION

