
indexing
	description: "Uniform random number generator"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"


class

	UNIFORM_RANDOM_GENERATOR

inherit

	RANDOM_GENERATOR

creation

	make, make_unit_distribution, make_with_parameters

feature -- Initialization

	make is
			-- 
		do
			make_unit_distribution
		end
	
	make_unit_distribution is
			-- A uniform RNG that creates variates distributed	
			-- between [0, 1)
		do
			make_with_parameters (0, 1.0)
			is_unit_distribution := true
		ensure
			in_range: minimum = 0 and maximum = 1.0
			is_unit: is_unit_distribution
		end -- unit_distribution
	
	make_with_parameters (min, max: DOUBLE) is
			-- 
		do
			initialize_rng 
			minimum := min
			maximum := max
			range := max - min
			forth
			is_unit_distribution := false
		end -- make_with_parameters

feature -- Access
	
	minimum: DOUBLE
			-- The `minimum' random variate generated
	
	maximum: DOUBLE
			-- The `maximum' random variate generated
	
feature -- Cursor movement
	
	forth is
			-- Move to the next random number
		do
			item := get_double * range + minimum
		end

feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	range: DOUBLE
	
	initialize_rng is
			-- 
		do
			init_rng (123456789)
--			init_rng (generate_seed)
		end


feature {NONE} -- Implementation
	
	init_rng (seed: INTEGER) is
		external
			"C use <mersenne_twister.h>"
		alias
			"init_genrand"
		end
	
	get_double : DOUBLE is
		external
			"C use <mersenne_twister.h>"
		alias
			"genrand_real1"
		end
	
	generate_seed: INTEGER is
			-- Generate a new seed
		external
			"C use <mersenne_twister.h>"
		alias
			"make_new_seed"
		end

end --	UNIFORM_RANDOM_GENERATOR

