

indexing
	description: "An oriented plane in 3-space"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"
	
	
	
	
class 
	
	PLANE
	
inherit
	
	INTERSECTABLE
		redefine
			intersect
		end
	
	BASIC_NUMERIC
		undefine
			default_create
		end
	
creation
	make_from_points, make_from_normal_and_distance, make_from_normal_and_point
	
feature -- Initialization
	
	make_from_points (pt1, pt2, pt3: POINT3) is
			-- Construct the plane from 3 points
		local
			v: VECTOR3
		do
			normal := (pt2 - pt1).cross (pt3 - pt1)
			create v.make (pt1.x, pt1.y, pt1.z)
			distance_from_origin := normal.dot (v)
			initialize
		end

	make_from_normal_and_distance (n: VECTOR3; d: DOUBLE) is
			-- Construct the plane from a normal and the distance 
			-- from the origin
		do
			normal := n
			distance_from_origin := d
			initialize
		end
	
	make_from_normal_and_point (n: VECTOR3; pt: POINT3) is
			-- Construct the plane from a normal and a point to pass 
			-- through.
		local
			v: VECTOR3
		do
			normal := n
			create v.make (pt.x, pt.y, pt.z)
			distance_from_origin := normal.dot (v)
			initialize
		end

feature -- Access
	
	normal: VECTOR3
			-- The normal direction of the plane
	
	distance_from_origin: DOUBLE
			-- the distance from the origin to the plane
	
	intersection: ARRAY [DOUBLE]
			-- Result of 'intersect' operation: ordered parametric coordinates
			-- along the ray of first, second, third, ... unique points of
			-- intersection.
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	point_is_in_half_space (pt: POINT3): BOOLEAN is
			-- Is `pt' in the half-space defined by the plane?
		local
			v: VECTOR3
		do
			create v.make (pt.x, pt.y, pt.z)
			Result := v.dot (normal) > distance_from_origin
		end 
	
	distance_from_point (pt: POINT3): DOUBLE is
			-- The signed distance from `pt' to the plane
		local
			v: VECTOR3
		do
			create v.make (pt.x, pt.y, pt.z)
			Result := v.dot (normal) - distance_from_origin
		end 

feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	intersect (ray: RAY) is
			-- Intersect with `ray'.
			-- If `ray' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		local
			t, denom: DOUBLE
			v: VECTOR3
		do
			intersection_count := intersection_count + 1
			
			-- Solve for t:	 n . (r.p + t * r.d) - d == 0
			denom := normal.dot (ray.direction)
			if about_equal (denom, 0) then
				intersected := false
			else
				create v.make (ray.origin.x, ray.origin.y, ray.origin.z)
				t := -(normal.dot (v) - distance_from_origin) / denom
				intersected := true
				intersection.put (t, 1)
				intersection_hit := intersection_hit + 1
			end
		end

	translate (d: DOUBLE) is
			-- translate the plane `d' along its normal
		do
			distance_from_origin := distance_from_origin + d
		end 
	
	apply (xform: TRANSFORM) is
			-- apply the transform to the plane
		local
			pt: VECTOR3
			t: TRANSFORM
		do
			pt := normal.scale (distance_from_origin)
			t := xform.inverse
			
			check
				inverse_succeeded: t.inverted
			end
			
			t.transpose
			normal := t.apply_as_direction (normal)
			normal.normalize
			
			pt := xform.apply_as_direction (pt)
			distance_from_origin := pt.dot (normal)
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	initialized: BOOLEAN
	
	initialize is
			-- Initialize features
		do
			normal.normalize
			create intersection.make (1, 1)
			initialized := true
		end 

invariant
	
	normal_is_unit: initialized implies normal.is_unit 
	intersection_not_void: initialized implies intersection /= Void
	only_one_intersection: initialized implies intersection.count = 1
	
end -- PLANE

