

indexing
	description: "A group of polygons (possibly 1) that share vertices"
	patterns: "none"
	status: "unreviewed"
	Date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	Revision: "$Revision: 286 $"




class 

	POLYGON_COLLECTION

creation
	make

feature -- Initialization

	make (npolys: INTEGER; nverticies: INTEGER_ARRAYED_LIST;
		  verticies: INTEGER_64_ARRAYED_LIST;
		  new_coordinates: POINT3_ARRAYED_LIST;
		  new_normals: NORMAL_ARRAYED_LIST;
		  new_colors: RGBA_COLOR_ARRAYED_LIST) is
			-- Create a collection of `npolys' polygons from a list 
			-- of verticies
		require
			npolys_positive: npolys >= 1
			nverticies_not_void: nverticies /= Void
			verticies_not_void: verticies /= Void
			correct_count: npolys = nverticies.count
			has_verticies: new_coordinates /= Void
		do
			coordinates := new_coordinates
			if new_normals /= Void then
				has_vertex_normals := True
				normals := new_normals
			end

			if new_colors /= Void then
				has_vertex_colors := True
				colors := new_colors
			end

			count := npolys
			vertex_count_per_polygon := nverticies
			vertex_indices_per_polygon := verticies
		end

feature -- Access

	coordinates: POINT3_ARRAYED_LIST
			-- The list of verticies shared by the collection

	vertex_count_per_polygon: INTEGER_ARRAYED_LIST
			-- The number of verticies in each polygon
	
	vertex_indices_per_polygon: INTEGER_64_ARRAYED_LIST
			-- The indices of the verticies in each polygon
	
	has_vertex_normals: BOOLEAN
			-- Do these polygons have per-vertex normals?

	has_vertex_colors: BOOLEAN
			-- Do these polygons have per-vertex colors?
	
	normals: NORMAL_ARRAYED_LIST
			-- The per-vertex normals
	
	colors: RGBA_COLOR_ARRAYED_LIST
			-- The per-vertex colors
	
feature -- Measurement

	count: INTEGER
			-- The number of polygons in the collection

feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	triangulate is
			-- Convert all polygons to triangles
		require 
			default: true
		local
			 j: INTEGER
		do
			from
				vertex_count_per_polygon.start
				create indices.make (3 * count)
				j := 1
			until
				vertex_count_per_polygon.after
			loop
				if vertex_count_per_polygon.item_for_iteration > 4 then
					triangulate_general_polygon (j)
				elseif vertex_count_per_polygon.item_for_iteration = 4 then
					triangulate_quad (j)
				else
					indices.force_last (vertex_indices_per_polygon.item (j))
					indices.force_last (vertex_indices_per_polygon.item (j + 1))
					indices.force_last (vertex_indices_per_polygon.item (j + 2))
				end
				
				j := j + vertex_count_per_polygon.item_for_iteration
			
				vertex_count_per_polygon.forth
			end

			if indices.count > 0 then
				vertex_indices_per_polygon := indices
			end
		ensure 
			default: true
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	indices: INTEGER_64_ARRAYED_LIST
	
	triangulate_general_polygon (vertex_base: INTEGER) is
			-- Convert a general polygon, e.g. number of sides > 4, to a 
			-- triangle
		require
			vertex_base_positive: vertex_base >= 1
		local
			i: INTEGER
			x, y, z, nx, ny, nz, r, g, b, a, weight: DOUBLE
			centroid: POINT3
			centroid_color: RGBA_COLOR
			centroid_normal: NORMAL
		do
			-- Compute the centroid
			from
				i := 0
				x := 0
				y := 0
				z := 0
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= vertex_count_per_polygon.item_for_iteration
			variant
				i_increasing: vertex_count_per_polygon.item_for_iteration - i
			until
				i >= vertex_count_per_polygon.item_for_iteration
			loop
				x := x + coordinates.item (vertex_indices_per_polygon.item (vertex_base + i)).x
				y := y + coordinates.item (vertex_indices_per_polygon.item (vertex_base + i)).y
				z := z + coordinates.item (vertex_indices_per_polygon.item (vertex_base + i)).z
				
				i := i + 1
			end
			
			create centroid.make (x/vertex_count_per_polygon.item_for_iteration,
								  y/vertex_count_per_polygon.item_for_iteration,
								  z/vertex_count_per_polygon.item_for_iteration)
			coordinates.force_last (centroid)
			
			-- Compute the interpolant weights
			from
				i := 0
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= vertex_count_per_polygon.item_for_iteration
			variant
				i_increasing: vertex_count_per_polygon.item_for_iteration - i
			until
				i >= vertex_count_per_polygon.item_for_iteration
			loop
				weight := weight + (centroid - coordinates.item (vertex_indices_per_polygon.item (vertex_base + i))).magnitude
				
				i := i + 1
			end
			
			if has_vertex_colors then
				from
					i := 0
					r := 0
					g := 0
					b := 0
					a := 0
				invariant
					i_at_least_one: i >= 0
					i_small_enough: i <= vertex_count_per_polygon.item_for_iteration
				variant
					i_increasing: vertex_count_per_polygon.item_for_iteration - i
				until
					i >= vertex_count_per_polygon.item_for_iteration
				loop
					x := (centroid - coordinates.item (vertex_indices_per_polygon.item (vertex_base + i))).magnitude
					centroid_color := (colors.item (vertex_indices_per_polygon.item (vertex_base + i))).scale (x / weight)
					r := r + centroid_color.red
					g := g + centroid_color.green
					b := b + centroid_color.blue
					a := a + centroid_color.alpha
					
					i := i + 1
				end
				
				-- Division included since `scale' doesn't 
				-- account for alpha. Should it?
				create centroid_color.make (r, g, b, a/vertex_count_per_polygon.item_for_iteration)
				colors.force_last (centroid_color)
			end
			
			if has_vertex_normals then
				-- Compute the centroid
				from
					i := 0
					x := 0
					y := 0
					z := 0
					nx := 0
					ny := 0
					nz := 0
				invariant
					i_at_least_one: i >= 0
					i_small_enough: i <= vertex_count_per_polygon.item_for_iteration
				variant
					i_increasing: vertex_count_per_polygon.item_for_iteration - i
				until
					i >= vertex_count_per_polygon.item_for_iteration
				loop
					x := x + coordinates.item (vertex_indices_per_polygon.item (vertex_base + i)).x
					y := y + coordinates.item (vertex_indices_per_polygon.item (vertex_base + i)).y
					z := z + coordinates.item (vertex_indices_per_polygon.item (vertex_base + i)).z
					
					nx := nx + normals.item (vertex_indices_per_polygon.item (vertex_base + i)).x
					ny := ny + normals.item (vertex_indices_per_polygon.item (vertex_base + i)).y
					nz := nz + normals.item (vertex_indices_per_polygon.item (vertex_base + i)).z
					
					i := i + 1
				end
				create centroid_normal.make (nx/vertex_count_per_polygon.item_for_iteration, 
											 ny/vertex_count_per_polygon.item_for_iteration, 
											 nz/vertex_count_per_polygon.item_for_iteration)
				centroid_normal.normalize
				normals.force_last (centroid_normal)
			end
			
			-- Create the indices
			from
				i := 0
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= vertex_count_per_polygon.item_for_iteration - 1
			variant
				i_increasing: vertex_count_per_polygon.item_for_iteration - i - 1
			until
				i >= vertex_count_per_polygon.item_for_iteration - 1
			loop
				indices.force_last (coordinates.count)
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + i))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + i + 1))
				
				i := i + 1
			end
			
			-- Pick up the triangle that wraps to the first vertex
			indices.force_last (coordinates.count)
			indices.force_last (vertex_indices_per_polygon.item (vertex_base + vertex_count_per_polygon.item_for_iteration - 1))
			indices.force_last (vertex_indices_per_polygon.item (vertex_base))
			
		ensure 
			default: true
		end 

	triangulate_quad (vertex_base: INTEGER) is
			-- Split a quad along the shortest diagonal into two 
			-- triangles
		require
			vertex_base_positive: vertex_base >= 1
			vertex_count_is_quad: vertex_count_per_polygon.item_for_iteration = 4
		local
			dist1, dist2: DOUBLE
			pt1, pt2: POINT3
		do
			pt1 := coordinates.item (vertex_indices_per_polygon.item (vertex_base))
			pt2 := coordinates.item (vertex_indices_per_polygon.item (vertex_base + 2))
			dist1 := pt2.distance_squared (pt1)

			pt1 := coordinates.item (vertex_indices_per_polygon.item (vertex_base + 1))
			pt2 := coordinates.item (vertex_indices_per_polygon.item (vertex_base + 3))
			dist2 := pt2.distance_squared (pt1)

			if dist1 <= dist2 then
				indices.force_last (vertex_indices_per_polygon.item (vertex_base))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 1))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 2))

				indices.force_last (vertex_indices_per_polygon.item (vertex_base))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 2))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 3))
			else
				indices.force_last (vertex_indices_per_polygon.item (vertex_base))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 1))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 3))

				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 1))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 2))
				indices.force_last (vertex_indices_per_polygon.item (vertex_base + 3))
			end
		end
	
invariant

	coordinates_not_void: coordinates /= Void
	has_vertex_normals: has_vertex_normals implies normals /= Void
	has_vertex_colors: has_vertex_colors implies colors /= Void
	color_count_correct: has_vertex_colors implies colors.count = coordinates.count
	vertex_count_per_polygon_matches_count: vertex_count_per_polygon.count = count
	
end -- POLYGON_COLLECTION

