

indexing
	description: "Abstract class for rendering NURBS curves and surfaces"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 

	NURBS_RENDERER
	
inherit
	
	MEMORY
		redefine
			dispose
		end

feature -- Initialization

	make (the_renderer: RENDERER) is
			-- Intialize the nurbs renderer
		do
			renderer := the_renderer
			
			new_renderer
			
			generate_normals := false
			normals_specified := false
		end

feature -- Access
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	generate_normals: BOOLEAN
			-- Should the renderer generate normals automatically
	
	normals_specified: BOOLEAN
			-- Were normals specified
	
feature -- Status setting
	
	set_property (property: INTEGER; value: REAL) is
			-- Set various properties for the nurbs
		deferred
		end
	
	set_knots (uknots, vknots: DOUBLE_ARRAYED_LIST) is
			-- Set the knots
		require
			uknots_not_void: uknots /= Void
		deferred
		end
	
	set_order (uorder, vorder: INTEGER) is
			-- Set the order
		require
			uorder_strictly_positive: uorder >=1
			vorder_positive: vorder >=0
		do
			u_order := uorder
			v_order := vorder
		ensure
			u_order_set: u_order = uorder
			v_order_set: v_order = vorder			
		end
	
	normals_generation (on: BOOLEAN) is
			-- Set whether normals should be automatically generated
		do
			generate_normals := on
		ensure
			generate_normals_set: generate_normals = on
		end
	
	use_domain_distance is
			-- Set the NURBS sampling method to use domain distance
		do
			set_property (sampling_method, domain_distance)
		end 

	set_u_step (ustep: REAL) is
			-- Set the NURBS u step property (used under domain 
			-- distance sampling)
		require
			u_step_positive: ustep > 0
		do
			set_property (u_step, ustep)
		end 

	set_v_step (vstep: REAL) is
			-- Set the NURBS v step property (used under domain 
			-- distance sampling)
		require
			v_step_positive: vstep > 0
		do
			set_property (v_step, vstep)
		end 

feature -- Cursor movement
feature -- Element change
feature -- Removal
	
	dispose is
			-- Delete the renderer
		do
			delete_renderer
		end
	
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	begin_surface is
			-- Begin the definition of a nurbs surface
		deferred
		end
	
	define_control_points (ucount, vcount: INTEGER;
						   control_points: BOUNDED_ARRAY2 [POINT4]) is
			-- Define the control points for the surface
		require
			ucount_strictly_positive: ucount >=1
			vcount_positive: vcount >=0
			control_points_not_void: control_points /= Void
		deferred
		end
	
	define_normals (ucount, vcount: INTEGER;
					normals: NORMAL_ARRAYED_LIST) is
			-- Define the normals for the surface
		require
			ucount_strictly_positive: ucount >=1
			vcount_positive: vcount >=0
			normals_not_void: normals /= Void
		deferred
		ensure
			normals_specified: normals_specified
		end
	
	define_colors (ucount, vcount: INTEGER;
				   colors: RGBA_COLOR_ARRAYED_LIST) is
			-- Define the colors for the surface
		require
			ucount_strictly_positive: ucount >=1
			vcount_positive: vcount >=0
			colors_not_void: colors /= Void
		deferred
		end
	
	define_texture_coordinates (ucount, vcount: INTEGER;
								texture_coords: POINT2_ARRAYED_LIST) is
			-- Define the texture coordinates for the surface
		require
			ucount_strictly_positive: ucount >=1
			vcount_positive: vcount >=0
			texture_coords_not_void: texture_coords /= Void
		deferred
		end
	
	end_surface is
			-- End the definition of a nurbs surface
		require
			normals_were_created: generate_normals or normals_specified
		deferred
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	renderer: RENDERER
	
	u_order, v_order: INTEGER	
	
	new_renderer is
			-- Define a new renderer
		deferred
		end

	delete_renderer is
			-- Define a new renderer
		deferred
		end

	sampling_method: INTEGER is
			-- Constants for `set_property'
		deferred
		end
	
	domain_distance: INTEGER is
			-- Constants for `set_property'
		deferred
		end
	
	u_step: INTEGER is
			-- Constants for `set_property'
		deferred
		end
	
	v_step: INTEGER is
			-- Constants for `set_property'
		deferred
		end
	
end -- NURBS_RENDERER

