

indexing
	description: "A 4-sided polygon, smaller than a pixel"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2006-09-04 15:55:08 -0400 (Mon, 04 Sep 2006) $"
	revision: "$Revision: 264 $"




class

	MICROPOLYGON

inherit

	COMPARABLE
		undefine
			is_equal
		end

creation
	make

feature -- Initialization

	make (pt1, pt2, pt3, pt4: SHADING_DATA) is
			-- Create the polygon from the four points
		require
			pt1_not_void: pt1 /= Void
			pt2_not_void: pt2 /= Void
			pt3_not_void: pt3 /= Void
			pt4_not_void: pt4 /= Void
			has_raster_points: pt1.raster_point /= Void and pt2.raster_point /= Void and
							   pt3.raster_point /= Void and pt4.raster_point /= Void
		local
			lower, upper: POINT3
		do
			lower_left := pt1
			lower_right := pt2
			upper_right := pt3
			upper_left := pt4

			update_references
			
			create lower.make (pt1.raster_point.x.min (pt2.raster_point.x.min (pt3.raster_point.x.min (pt4.raster_point.x))),
							   pt1.raster_point.y.min (pt2.raster_point.y.min (pt3.raster_point.y.min (pt4.raster_point.y))),
							   0)
			create upper.make (pt1.raster_point.x.max (pt2.raster_point.x.max (pt3.raster_point.x.max (pt4.raster_point.x))),
							   pt1.raster_point.y.max (pt2.raster_point.y.max (pt3.raster_point.y.max (pt4.raster_point.y))),
							   0)
			create extent.make (lower, upper)
		end

feature -- Access

	lower_left, lower_right, upper_right, upper_left: SHADING_DATA
			-- The four corners of the micropolygon

	depth: DOUBLE
			-- The depth of the micropolygon in camera space

	extent: AXIS_ALIGNED_BOX
			-- The raster space bound of the micropolygon

	item (index: INTEGER): SHADING_DATA is
			-- for iterators
		require
			index_in_range: index >= 1 and index <= 4
		do
			inspect
				index
			when 1 then
				Result := lower_left
			when 2 then
				Result := lower_right
			when 3 then
				Result := upper_right
			when 4 then
				Result := upper_left
			end
		ensure
			Result_not_void: Result /= Void
		end

feature -- Measurement
feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is `Current' less than `other'?
		do
			Result := depth < other.depth
		end

	is_equal (other: like Current): BOOLEAN is
			-- Do `Current' and other have equal-valued items?
		do
			Result := depth.is_equal (other.depth)
		end

feature -- Status report

	has (pt: POINT3): BOOLEAN is
			-- Shoot a test ray along +X axis.	The strategy, from MacMartin, is to
			-- compare vertex Y values to the testing point's Y and quickly discard
			-- edges which are entirely to one side of the test ray.
			--
			-- Samosky, Joseph, "SectionView: A system for interactively specifying 
			-- and visualizing sections through three-dimensional medical image
			-- data", M.S. Thesis, Department of Electrical Engineering and Computer 
			-- Science, Massachusetts Institute of Technology, 1993.
		local
			y_flag, y_flag1,crossing, line_flag, done: BOOLEAN
			i: INTEGER
			pt0, pt1: POINT3
		do
			if extent.has_2d (pt.x, pt.y) then
				pt0 := item (4).raster_point
				y_flag := pt0.y >= pt.y
				pt1 := item (1).raster_point
				Result := false
				line_flag := false
				
				from
					i := 2
					done := false
				invariant
					i_at_least_one: i >= 2
					i_small_enough: i <= 2 + 5
				variant
					i_increasing: 2 + 5 - i
				until
					i > 5 or done
				loop
					y_flag1 := pt1.y >= pt.y
					-- Check if endpoints straddle (are on opposite sides) of X
					-- axis (i.e. the Y's differ); if so, +X ray could intersect this
					-- edge.
					if y_flag /= y_flag1 then
						-- Check intersection of pgon segment with +X
						-- ray. Note if >= point's X; if so, the ray hits
						-- it.
						crossing := (pt1.y - pt.y) * (pt0.x - pt1.x) >= (pt1.x - pt.x) * (pt0.y - pt1.y)
						if crossing = y_flag1 then
							Result := not Result
						end
						if line_flag then
							done := true
						end
						line_flag := true
					end

					y_flag := y_flag1
					pt0 := pt1
					if i <= 4 then
						pt1 := item (i).raster_point
					end
					i := i + 1
				end
			end
		end
	
	is_backfacing (camera: SCENE_CAMERA): BOOLEAN is
			-- Is this micropolygon backfacing?
		require
			camera_not_void: camera /= Void
		do
			Result := (item (1).geometric_normal.transform (camera.world_to_camera).dot_with_vector (camera.projection_direction) < 0 and then
					   item (2).geometric_normal.transform (camera.world_to_camera).dot_with_vector (camera.projection_direction) < 0 and then
					   item (3).geometric_normal.transform (camera.world_to_camera).dot_with_vector (camera.projection_direction) < 0 and then
					   item (4).geometric_normal.transform (camera.world_to_camera).dot_with_vector (camera.projection_direction) < 0)
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_depth (new_depth: DOUBLE) is
			-- Set `depth' to `new_depth'
		do
			depth := new_depth
		ensure
			depth_set: depth = new_depth
		end

	release is
			-- Free the memory used by the micropolygon
		do
			lower_left.remove_reference (Current)
			lower_right.remove_reference (Current)
			upper_right.remove_reference (Current)
			upper_left.remove_reference (Current)
		end

	update_references is
			-- Free the memory used by the micropolygon
		do
			lower_left.add_reference (Current)
			lower_right.add_reference (Current)
			upper_right.add_reference (Current)
			upper_left.add_reference (Current)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

invariant

	lower_left_not_void: lower_left /= Void
	lower_right_not_void: lower_right /= Void
	upper_right_not_void: upper_right /= Void
	upper_left_not_void: upper_left /= Void

end -- MICROPOLYGON

