

indexing
	description: "A micropolygon renderer"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2007-02-07 15:02:18 -0500 (Wed, 07 Feb 2007) $"
	revision: "$Revision: 281 $"



class

	MICROPOLYGON_RENDERER

inherit

	NUMERIC_CONVERSION
		export
			{NONE} all
		end

	BASIC_NUMERIC

	SINGLETON_ACCESSOR
		rename
			singleton as statistics
		redefine
			statistics
		end

creation
	make

feature -- Initialization

	make (options: OPTIONS_TABLE) is
			-- Create the object from `options'
		require
			options_not_void: options /= Void
		local
			doubles: DOUBLE_ARRAYED_LIST
			p: PAIR [DOUBLE, DOUBLE]
		do
			create processed_list.make (1)

			doubles := options.get_doubles ("limits", "bucketsize")
			bucket_x_resolution := real_64_truncated_to_integer (doubles.item (1))
			bucket_y_resolution := real_64_truncated_to_integer (doubles.item (2))

			doubles := options.get_doubles ("limits", "gridsize")
			maximum_grid_size := real_64_truncated_to_integer (doubles.item (1))

			doubles := options.get_doubles ("standard", "pixelsamples")
			pixel_samples_in_x := real_64_truncated_to_integer (doubles.item (1))
			pixel_samples_in_y := real_64_truncated_to_integer (doubles.item (2))

			image_order := horizontal

			bucket_count_in_x := real_64_truncated_to_integer ((127.0 / bucket_x_resolution).ceiling)
			bucket_count_in_y := real_64_truncated_to_integer ((127.0 / bucket_y_resolution).ceiling)
			create pixels.make_with_extents (0, 0, bucket_x_resolution, bucket_y_resolution,
			pixel_samples_in_x, pixel_samples_in_y)
			create index_sorter.make (4)
			index_sorter.fill (0)
			create points_list.make (4)
			create p.make (0, 0)
			points_list.force_last (p)
			points_list.force_last (p.deep_twin)
			points_list.force_last (p.deep_twin)
			points_list.force_last (p.deep_twin)

			test_dice_rate := 8
			create test_grid.make (test_dice_rate, test_dice_rate)
		end

feature -- Access

	bucket_x_resolution: INTEGER
	bucket_y_resolution: INTEGER
			-- The resolution of an image bucket

	maximum_grid_size: INTEGER
			-- The maximum number of polygons in a grid

	camera: SCENE_CAMERA
			-- The camera for this renderer

	scene_graph: SCENE_GROUP
			-- The root of the scene graph

	image: IMAGE
			-- The rendered data

	horizontal, vertical, spiral: INTEGER is unique
			-- The possible order directions to process the buckets

feature -- Debug

	depth_image: IMAGE
			-- The rendered data

	debug_polys: DS_ARRAYED_LIST [DS_ARRAYED_LIST [MICROPOLYGON]]
			-- A list of micropolygons

	gather_polys, build_depth_image: BOOLEAN
			-- Are we keeping the generated micropolygons in
			-- `debug_polys'

feature -- Measurement
feature -- Comparison
feature -- Status report

	progress: REAL is
			-- What's the current progress of a render (in percent)?
		do
			if buckets /= Void then
				Result := ((current_bucket / buckets.count) * 1000.0).truncated_to_integer / 10.0
			end
		end

	is_first: BOOLEAN is
			-- Are we at the first bucket?
		do
			Result := col_index = 0 and row_index = 0
		end

	after: BOOLEAN is
			-- Are we `after' the last bucket?
		do
			if image_order = horizontal then
				Result := after_horizontal
			elseif image_order = vertical then
				Result := after_vertical
			else
					check
						implemented: false
					end
			end
		end

	image_order: INTEGER
			-- The order to process the buckets

feature -- Status setting

	set_image_order (new_order: INTEGER) is
			-- Set `image_order' to `new_order'
		require
			new_order_valid: new_order >= horizontal and new_order <= spiral
		do
			image_order := new_order
		ensure
			image_order_set: image_order = new_order
		end

	set_gather_polys_for_debug (on_or_off: BOOLEAN) is
			-- Should we gather polygons for debug
		do
			gather_polys := on_or_off
		end

	set_statistics_for_missed_samples (on_or_off: BOOLEAN) is
			-- Should we only collect statistics for samples that
			-- miss geometry
		local
			pixel: PIXEL
		do
			create pixel.make (0, 0)
			pixel.statistics_records_misses_only (on_or_off)
		end

	set_build_depth_image_for_debug (on_or_off: BOOLEAN) is
			-- Should we build the mpp complexity data for debug
		do
			build_depth_image := on_or_off
			if build_depth_image then
				create debug_polys.make (1)
			end
		end

feature -- Cursor movement

	start is
			-- Move to the first bucket
		require
			camera_set: camera /= Void
			scene_graph_set: scene_graph /= Void
		local
			bound: AXIS_ALIGNED_BOX
		do
			col_index := 0
			row_index := 0

			create bucket_culler.make (camera, bucket_count_in_x, bucket_count_in_y,
			bucket_x_resolution, bucket_y_resolution)

			bucket_culler.set_culling (true)
			-- Gather culled nodes in case we want to ray trace
			bucket_culler.set_gathering (true)

			scene_graph.traverse (bucket_culler)

			statistics.put (bucket_culler.count + bucket_culler.culled_pool.count,
			statistics.prim_count)

			create culler.make (camera)
			create bound_culler.make (camera)
			bound := camera.crop_bound
			bound_culler.set_test_bounds (bound.lower.x, bound.lower.y,
			bound.upper.x, bound.upper.y)
			buckets := bucket_culler.buckets
			buckets.set_resolution (bucket_x_resolution, bucket_y_resolution)

			if build_depth_image then
				create depth_image.make (camera.y_resolution, camera.x_resolution, 4)
			end

			if gather_polys then
				create debug_polys.make (1)
			end
		ensure
			is_first: is_first
			bucket_culler_not_void: bucket_culler /= Void
			culler_not_void: culler /= Void
			buckets_not_void: buckets /= Void
		end

	forth is
			-- Move forward one bucket
		do
			if image_order = horizontal then
				forth_horizontal
			elseif image_order = vertical then
				forth_vertical
			else
					check
						implemented: false
					end
			end
		end

feature -- Element change

	set_camera (new_camera: SCENE_CAMERA) is
			-- Set `camera' to `new_camera'
		require
			new_camera_not_void: new_camera /= Void
		do
			camera := new_camera
			bucket_count_in_x := real_64_truncated_to_integer (((camera.x_resolution - 1) / bucket_x_resolution).ceiling)
			bucket_count_in_y := real_64_truncated_to_integer (((camera.y_resolution - 1) / bucket_y_resolution).ceiling)
			create image.make (camera.y_resolution, camera.x_resolution, 4)
		ensure
			camera_set: camera = new_camera
			image_not_void: image /= Void
			image_width_set: image.width = camera.x_resolution
			image_height_set: image.height = camera.y_resolution
		end

	set_scene_graph (new_scene_graph: SCENE_GROUP) is
			-- Set `scene_graph' to `new_scene_graph'
		require
			new_scene_graph_not_void: new_scene_graph /= Void
		do
			scene_graph := new_scene_graph
		ensure
			scene_graph_set: scene_graph = new_scene_graph
		end

	set_bucket_resolution (new_x_resolution, new_y_resolution: INTEGER) is
			-- Set `bucket_x_resolution' to `new_x_resolution' and
			-- `bucket_y_resolution' to `new_y_resolution'
		require
			new_x_resolution: new_x_resolution > 0
			new_y_resolution: new_y_resolution > 0
		do
			bucket_x_resolution := new_x_resolution
			bucket_y_resolution := new_y_resolution

			pixels.make_with_extents (0, 0, bucket_x_resolution, bucket_y_resolution,
			pixel_samples_in_x, pixel_samples_in_y)
		ensure
			bucket_x_resolution_set: bucket_x_resolution = new_x_resolution
			bucket_y_resolution_set: bucket_y_resolution = new_y_resolution
		end

	set_maximum_grid_size (new_maximum_grid_size: INTEGER) is
			-- Set`maximum_grid_size' to `new_maximum_grid_size'
		require
			new_maximum_grid_size_positive: new_maximum_grid_size > 0
		do
			maximum_grid_size := new_maximum_grid_size
		ensure
			maximum_grid_size_set: maximum_grid_size = new_maximum_grid_size
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	render is
			-- Render the scene
		require
			camera_set: camera /= Void
			scene_graph_set: scene_graph /= Void
			image_not_void: image /= Void
		do
			io.put_string ("%N")

			from
				start
			until
				after
			loop
				render_bucket

				forth
			end
		end

	render_bucket is
			-- Render an individual bucket
		require
			camera_set: camera /= Void
			scene_graph_set: scene_graph /= Void
			image_not_void: image /= Void
		local
			bucket_list: SORTABLE_ARRAYED_LIST [BUCKET_ENTITY]
			the_bucket: BUCKET_TABLE_ITEM
			bucket_item: BUCKET_ENTITY
			element: SCENE_DRAWABLE_ELEMENT
			inserted: INTEGER
			mp: DS_ARRAYED_LIST [MICROPOLYGON]
		do
			current_bucket_imaged := false

			the_bucket := buckets.item (row_index, col_index)
			bucket_list := the_bucket.entity_list

			-- Process any micropolygons
			bound_box := buckets.bound (row_index, col_index)
			initialize_pixels

			current_bucket := buckets.bucket_index (row_index, col_index)

			debug ("render_progress")
				io.put_string ("Processing bucket #")
				io.put_string (current_bucket.out)
				io.put_string (" ( ")
				if bucket_list /= Void then
					io.put_string (bucket_list.count.out)
				else
					io.put_string ("0")
				end
				io.put_string (", ")
				io.put_string (the_bucket.mp_count.out)
				io.put_string (" ) - ")
				io.put_string ((buckets.bound (row_index, col_index).out))
				io.put_string ("%N")
			end

			debug ("bucket_counts")
				io.put_string ("%NPrimitives per Bucket%N")
				buckets.dump_buckets (true)
				io.put_string ("%NMicropolygons per Bucket%N")
				buckets.dump_buckets (false)
			end

			if the_bucket.mp_list /= Void then
				from
					mp := the_bucket.mp_list
					mp.start
				until
					mp.after
				loop
					--statistics.increment (statistics.mp_single_bucket)
					pixels.add_micropolygon (mp.item_for_iteration)
					mp.item_for_iteration.release

					mp.forth
				end
			end

			-- Now the geometry
			if bucket_list /= Void then
				culler.set_test_bounds (bound_box.lower.x, bound_box.lower.y,
				bound_box.upper.x - 1, bound_box.upper.y - 1)
				from
				until
					bucket_list.is_empty
				loop
					bucket_list.sort
					bucket_item := bucket_list.first
					bucket_list.remove_first

					element := bucket_item.drawable.geometry
					-- If this `element' was diced to satisfy the
					-- requirements of a neigboring grid, just add it
					-- to the list.
					if element.diced then
						if element.grid /= Void and then not element.grid.released then
							processed_list.force_last (element.grid)
						end
					else
						bound_culler.push_transform (bucket_item.drawable.transform)
						culler.push_transform (bucket_item.drawable.transform)

						element.test_diceability (Current)
						if element.diceable then
							processed_list.force_last (dice (bucket_item.drawable))
						else
							if not element.is_split then
								element.split
							end

							-- Increment by statistics; We're adding `split_items'.count
							-- primitives
							from
								element.split_items.start
							until
								element.split_items.after
							loop
								if element.split_items.item_for_iteration.is_split or else
									bound_culler.cull_state (element.split_items.item_for_iteration.extents.circumscribed_box) /= bound_culler.outside then
									inserted := find_bucket_for_element (element.split_items.item_for_iteration,
										bucket_item)
									statistics.increment (statistics.prim_generated)
										check
											bucket_index_is_greater: current_bucket <= inserted
										end
								else
									statistics.increment (statistics.prim_culled)
									if element.split_items.item_for_iteration.diced then
										element.split_items.item_for_iteration.release_grid
									end
									element.split_items.item_for_iteration.release
								end
 
								element.split_items.forth
							end
							
							-- Free Memory
							element.release
						end

						culler.pop_transform

						bound_culler.pop_transform
					end
				end
			end

			process_bucket

			buckets.mark_as_processed (row_index, col_index)
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SCENE_DRAWABLE_ELEMENT} -- Implementation

	test_grid: MICROPOLYGON_GRID
			-- A grid for test dicing

feature {NONE} -- Implementation

	row_index, col_index: INTEGER
			-- The current bucket index

	bucket_count_in_x, bucket_count_in_y: INTEGER
			-- The number of buckets in x and y respectively

	pixel_samples_in_x, pixel_samples_in_y: INTEGER
			-- The number of pixel samples in x and y respectively

	bucket_culler: BUCKETING_CULL_ACTION
			-- Culler that sorts into buckets

	culler, bound_culler: CULL_ACTION

	current_bucket_imaged: BOOLEAN
			-- Simply for debug

	current_bucket: INTEGER
			-- The index of the current bucket

	buckets: BUCKET_TABLE

	bound_box: AXIS_ALIGNED_BOX

	pixels: PIXEL_BUCKET

	index_sorter: SORTABLE_ARRAYED_LIST [INTEGER]
	points_list: DS_ARRAYED_LIST [PAIR [DOUBLE, DOUBLE]]
	processed_list: DS_ARRAYED_LIST [MICROPOLYGON_GRID]

	test_dice_rate: INTEGER
			-- The dicing rate for testing if the grid is small enough

	statistics: STATISTICS is
			-- The statistics
		once
			create Result
		end

feature {NONE} -- Implementation

	forth_vertical is
			-- Move forward one bucket incrementing in the vertical
			-- direction first
		do
			row_index := row_index + 1
			if row_index = bucket_count_in_y then
				row_index := 0
				col_index := col_index + 1
			end
		ensure
			moved: (row_index = 0 implies col_index = old col_index + 1) and
				   row_index /= 0 implies (row_index = old row_index + 1 and
														col_index = old col_index)
		end

	forth_horizontal is
			-- Move forward one bucket incrementing in the horizontal
			-- direction first
		do
			col_index := col_index + 1
			if col_index = bucket_count_in_x then
				col_index := 0
				row_index := row_index + 1
			end
		ensure
			moved: (col_index = 0 implies row_index = old row_index + 1) and
				   col_index /= 0 implies (col_index = old col_index + 1 and
														row_index = old row_index)
		end

	after_horizontal: BOOLEAN is
			-- Are we `after' the last bucket?
		do
			Result := col_index = 0 and row_index = bucket_count_in_y
		end

	after_vertical: BOOLEAN is
			-- Are we `after' the last bucket?
		do
			Result := row_index = 0 and col_index = bucket_count_in_x
		end

feature {NONE} -- Implementation

	dice (drawable: SCENE_DRAWABLE): MICROPOLYGON_GRID is
			-- Render a grid of micropolygons
		require
			drawable_not_void: drawable /= Void
		do
			create Result.make (drawable.geometry.requested_grid_x_dim,
							  drawable.geometry.requested_grid_y_dim)

			statistics.increment (statistics.grid_count)

			drawable.geometry.dice (Result, false)

			Result.set_parent (drawable)
		ensure
			Result_not_void: Result /= Void
		end

	render_grid (grid: MICROPOLYGON_GRID) is
			-- After all elements in the current bucket are diced, we
			-- bust, shade, and convert to pixels
		require
			grid_not_void: grid /= Void
		local
			mp, dbg_mp: DS_ARRAYED_LIST [MICROPOLYGON]
			micro: MICROPOLYGON
		do
			grid.trim
			if not grid.is_trimmed then
				grid.displace
				if not grid.shaded then
					grid.shade (camera)
				end
				mp := grid.bust (grid.parent_element.geometry, camera)

				-- Compute the statistics here instead of in
				-- `process_...' since the mp's may be double
				-- counted in that routine.
				if gather_polys then
					create dbg_mp.make (mp.count)
					from
						mp.start
					until
						mp.after
					loop
						if not cull (mp.item_for_iteration) then
							-- Twin the micropolygon so we can
							-- retain a reference to it.
							micro := mp.item_for_iteration.twin
							micro.update_references
							dbg_mp.force_last (micro)
						end

						mp.forth
					end

					if dbg_mp.count > 0 then
						debug_polys.force_last (dbg_mp)
					end
				end

				statistics.increase (mp.count, statistics.mp_count)

				process_or_forward_micropolygons (mp)
			end
		end

	process_or_forward_micropolygons (mp: DS_ARRAYED_LIST [MICROPOLYGON]) is
			-- process or forward a list of micropolygons
		local
			micro: MICROPOLYGON
			ll, lr, ur, ul, i: INTEGER
			last_bucket: INTEGER
			extent: AXIS_ALIGNED_BOX
		do
			from
				mp.start
			until
				mp.after
			loop
				micro := mp.item_for_iteration

				if not cull (micro) then
					extent := micro.extent
					ll := which_bucket (extent.lower.x, extent.lower.y)
					lr := which_bucket (extent.upper.x, extent.lower.y)
					ur := which_bucket (extent.upper.x, extent.upper.y)
					ul := which_bucket (extent.lower.x, extent.upper.y)

					if ll >= 0 and ul >= 0 and lr >= 0 and ur >= 0 then
						if ul = ll and ul = lr and ul = ur then
							-- All 4 corners project to the same
							-- bucket
							statistics.increment (statistics.mp_single_bucket)
							if ll = current_bucket then
								pixels.add_micropolygon (micro)
							else
								buckets.forward_micropolygon (micro,
															  extent.lower.x,
															  extent.lower.y)
								micro.update_references
								check
									correct_bucket: ll = buckets.last_bucket
								end
								increment_bucket_forwarding_statistics (buckets.last_bucket)
							end
						else
							statistics.increment (statistics.mp_multiple_bucket)

							points_list.item (1).make (extent.lower.x,
													   extent.lower.y)
							points_list.item (2).make (extent.upper.x,
													   extent.lower.y)
							points_list.item (3).make (extent.upper.x,
													   extent.upper.y)
							points_list.item (4).make (extent.lower.x,
													   extent.upper.y)

							index_sorter.replace (ll, 1)
							index_sorter.replace (lr, 2)
							index_sorter.replace (ur, 3)
							index_sorter.replace (ul, 4)

							index_sorter.sort_with_auxillary (points_list)

							from
								i := 1
								last_bucket := -1
							invariant
								i_at_least_one: i >= 1
								i_small_enough: i <= 5
							variant
								i_increasing: 5 - i
							until
								i > 4
							loop
								if index_sorter.item (i) > last_bucket then
									if index_sorter.item (i) = current_bucket then
										pixels.add_micropolygon (micro)
									elseif index_sorter.item (i) > current_bucket then

										buckets.forward_micropolygon (micro,
																	  points_list.item (i).first,
																	  points_list.item (i).second)
										micro.update_references
											check
												correct_bucket: index_sorter.item (i) = buckets.last_bucket
											end
										increment_bucket_forwarding_statistics (buckets.last_bucket)
									end
									last_bucket := index_sorter.item (i)
								end
								i := i + 1
							end
						end
					end
				else
					statistics.increment (statistics.mp_culled)
				end

				micro.release
				mp.forth
			end
		end

	increment_bucket_forwarding_statistics (last_bucket: INTEGER) is
			-- Increment statistics related to bucket forwarding
		require
			last_bucket_greater_than_current_bucket: last_bucket > current_bucket
		local
			diff, lb, cb: INTEGER
		do
			if image_order = horizontal then
				lb := real_64_truncated_to_integer (last_bucket/bucket_count_in_y)
				cb :=  real_64_truncated_to_integer (current_bucket/bucket_count_in_y)
			elseif image_order = vertical then
				lb := real_64_truncated_to_integer (last_bucket/bucket_count_in_x)
				cb :=  real_64_truncated_to_integer (current_bucket/bucket_count_in_x)
			else
					check
						implemented: false
					end
			end
			diff := lb - cb

			if diff = 0 then -- In the same scanline
				statistics.increment (statistics.mp_pushed_forward)
			elseif diff = 1 then -- In the next scanline
				statistics.increment (statistics.mp_pushed_down)
			else
				statistics.increment (statistics.mp_pushed_far)
			end
		end

	cull (micro: MICROPOLYGON): BOOLEAN is
			-- Can we cull this micropolygon?
		require
			micro_not_void: micro /= Void
		do
			Result := false
			if micro.extent.upper.x < 0 or micro.extent.lower.x > camera.x_resolution or
				micro.extent.upper.y < 0 or micro.extent.lower.y > camera.y_resolution then
				Result := true
			end
		end

	which_bucket (x, y: DOUBLE): INTEGER is
			-- Which bucket does the point`x', `y' fall in?
		local
			xx, yy: DOUBLE
		do
			-- has_2d checks the closed interval [lower, upper], we
			-- need to ensure that we in the open interval [lower, upper)
			if bound_box.has_2d (x, y) and
				x < bound_box.upper.x and
				y < bound_box.upper.y then
				Result := current_bucket
			else
				xx := x
				yy := y
				if xx < 0 then
					xx := 0
				elseif xx >= camera.x_resolution then
					xx := camera.x_resolution - 1
				end
				if yy < 0 then
					yy := 0
				elseif yy >= camera.y_resolution then
					yy := camera.y_resolution - 1
				end

				Result := buckets.to_bucket_index (xx, yy)
			end
		end

	initialize_pixels is
			-- Initialize the pixel structure
		local
			lower_y, lower_x, width, height: INTEGER
		do
			lower_y := real_64_truncated_to_integer (bound_box.lower.y.ceiling)
			lower_x := real_64_truncated_to_integer (bound_box.lower.x.ceiling)

			width := (camera.x_resolution - lower_x).min (bucket_x_resolution)
			height := (camera.y_resolution - lower_y).min (bucket_y_resolution)

			pixels.make_with_extents (lower_x, lower_y, width, height,
									  pixel_samples_in_x, pixel_samples_in_y)
		end

	process_pixels is
			-- Process the pixels to generate the appropriate color
		local
			color: RGBA_COLOR
			depth_complexity: INTEGER
		do
			pixels.sample

			from
				pixels.start
			until
				pixels.after
			loop
				debug ("pixels")
					io.put_string (pixels.row.out)
					io.put_string (" ")
					io.put_string (pixels.column.out)
					io.put_string (" ")
					io.put_string (pixels.item_for_iteration.color.out)
					io.new_line
				end

				image.put (pixels.item_for_iteration.color, 1.0,
				pixels.row, pixels.column)

				if build_depth_image then
					depth_complexity := pixels.item (pixels.row, pixels.column).count
					create color.make (depth_complexity, 0, 0, 1)
					depth_image.put (color, 1.0, pixels.row, pixels.column)
				end

				pixels.item_for_iteration.release

				pixels.forth
			end
		end

	process_bucket is
			-- After all elements in the current bucket are diced, we
			-- bust, shade, and convert to pixels
		local
			grid, new_grid: MICROPOLYGON_GRID
			drawable, new_drawable: SCENE_DRAWABLE
			elements: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
			element: SCENE_DRAWABLE_ELEMENT
			two_sided: BOOLEAN
		do
			from
				processed_list.start
			until
				processed_list.is_empty
			loop
				grid := processed_list.first
				processed_list.remove_first

				two_sided := grid.two_sided
				if two_sided or else (not two_sided and then
									  not grid.is_backfacing (camera)) then
					drawable := grid.parent_element

					culler.push_transform (drawable.transform)
					-- Assumption: All neighbors of the drawable geometry
					-- come from the same parent element
					from
					until
						grid.all_diced
					loop
						elements := drawable.geometry.all_neighbors
						from
							elements.start
						until
							elements.after
						loop
							element := elements.item_for_iteration
							if not element.diced then
								element.test_diceability (Current)
								if element.diceable then
									new_drawable := element.drawable.twin
									new_drawable.replace_first (element)
									new_grid := dice (new_drawable)
								else
									if not element.is_split then
										-- We split here, but let
										-- `render_bucket' take care of
										-- redistributing the pieces.
										element.split
									else
											check
												should_never_happen: false
											end
									end
								end
							end

							elements.forth
						end
					end

					-- Make sure that the grid and it's neighbors are patched
					elements := drawable.geometry.all_neighbors

					from
						elements.start
					until
						elements.after
					loop
						elements.item_for_iteration.patch_grid

						elements.forth
					end
					grid.parent_element.geometry.patch_grid

					debug ("print_neighbor_grids")
						io.put_string (grid.identifier.out)
						io.put_string (" - ")
						grid.print_neighbor_ids
						io.put_string ("%N")
					end

					render_grid (grid)
					culler.pop_transform
				else
					statistics.increment (statistics.grids_backfacing)
				end

				grid.release
			end

			debug ("print_active_grids")
				if grid /= Void then
					grid.print_active_grids
				end
			end

			process_pixels
		ensure
			processed_list_is_empty: processed_list.is_empty
		end

	find_bucket_for_element (element: SCENE_DRAWABLE_ELEMENT;
							 bucket_item: BUCKET_ENTITY): INTEGER is
			-- Find the bucket that overlaps `element's upper left corner
		require
			element_not_void: element /= Void
			bucket_item_not_void: bucket_item /= Void
		local
			new_bucket: BUCKET_ENTITY
			drawable: SCENE_DRAWABLE
			cull_state: INTEGER
			x, y: DOUBLE
		do
			cull_state := culler.cull_state (element.extents.circumscribed_box)

			if cull_state = culler.outside then
				x := culler.last_point.x.max (1).min (camera.x_resolution - 1)
				y := culler.last_point.y.max (1).min (camera.y_resolution - 1)

				create drawable
				drawable.put (element, bucket_item.drawable.elements,
				bucket_item.drawable.transform)
				element.set_drawable (drawable)
				create new_bucket.make (culler.last_point.z, drawable)
				if buckets.bucket_at_point_processed (x, y) then
					buckets.put_in_bucket (new_bucket, row_index, col_index)
					Result := current_bucket
				else
					buckets.put (new_bucket, x, y)
					Result := buckets.to_bucket_index (x, y)
						check
							in_bounds: buckets.bound_at_point (x, y).has (create {POINT3}.make (x, y, 0))
						end
				end
			else
				create drawable
				drawable.put (element, bucket_item.drawable.elements,
				bucket_item.drawable.transform)
				element.set_drawable (drawable)
				create new_bucket.make (culler.last_point.z, drawable)
				buckets.put_in_bucket (new_bucket, row_index, col_index)
				Result := current_bucket
				debug ("split")
					io.put_string ("  --> ")
				end
			end

			drawable.set_bucket (Result)

			debug ("split")
				io.put_string (culler.last_point.out)
				io.put_string (", ")
				io.put_string (Result.out)
				io.put_string ("%N")
			end
		ensure
			Result_in_range: Result >= 1 and Result <= buckets.count
		end

	has_four_neighbors (object: SCENE_DRAWABLE_ELEMENT): BOOLEAN is
			-- Does this patch have four neighbors? This is assuming
			-- that the master object has (u, v) bounds in the range
			-- of [0, 1]. Not useful in real situations, but good for debugging
		do
			if object.u_minimum > 0 and object.v_minimum > 0 and
				object.u_maximum < 1 and object.v_maximum < 1 then
				Result := object.neighbor (object.left).count > 0 and
					object.neighbor (object.right).count > 0 and
					object.neighbor (object.above).count > 0 and
					object.neighbor (object.below).count > 0
			else
				Result := true
			end
		end

invariant

	positive_bucket_resolution: bucket_x_resolution > 0 and bucket_y_resolution > 0
	maximum_grid_size_positive: maximum_grid_size > 0
	image_order_valid: image_order >= horizontal and image_order <= spiral
	pixels_not_void: pixels /= Void
	index_sorter_not_void: index_sorter /= Void
	pixel_samples_strictly_positive: pixel_samples_in_x >= 1 and  pixel_samples_in_y >= 1
	progress_in_range: progress >= 0 and progress <= 100
	test_grid_not_void: test_grid /= Void
	test_dice_rate_is_positive: test_dice_rate > 0

end -- MICROPOLYGON_RENDERER

