

indexing
	description: "The glue patch that joins two diced grids"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2006-09-04 11:14:06 -0400 (Mon, 04 Sep 2006) $"
	revision: "$Revision: 263 $"



class

	PATCH_OBJECT

creation
	make

feature -- Initialization

	make (new_patch: DS_ARRAYED_LIST [SHADING_DATA]) is
			-- Create the patch object from `new_patch'. We share the
		require
			new_patch_valid: new_patch /= Void and then new_patch.count > 0
		do
			patch := new_patch.twin
			busted := false
			create referers.make (2)

			update_references
		ensure
			patch_set: patch /= Void and patch.count = new_patch.count
			referers_not_void: referers /= Void
		end

feature -- Access

	referers: DS_ARRAYED_LIST [MICROPOLYGON_GRID]
			-- The list of grids that refer to this object

feature -- Measurement
feature -- Comparison
feature -- Status report

	busted: BOOLEAN
			-- Has the patch been busted into micropolygons

	count: INTEGER is
			-- The number of points
		do
			Result := patch.count
		ensure
			Result_positive: Result > 0
		end

	has (grid: MICROPOLYGON_GRID): BOOLEAN is
			-- Is this grid already refering to this object?
		require
			grid_not_void: grid /= Void
		do
			Result := referers.has (grid)
		end

	all_grids_shaded: BOOLEAN is
			-- Have all refering grids been shaded?
		do
			from
				Result := true
				referers.start
			until
				referers.after or not Result
			loop
				Result := referers.item_for_iteration.shaded
				referers.forth
			end
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change

	add_grid (grid: MICROPOLYGON_GRID) is
			-- Add a grid to the list of objects pointing to this
			-- object.
		require
			grid_not_void: grid /= Void
			not_already_a_refering_grid: not has (grid)
		do
			referers.force_last (grid)
		ensure
			has_grid: has (grid)
		end

feature -- Removal

	release is
			-- Free memory
		local
			grid: MICROPOLYGON_GRID
			i: INTEGER
		do
			busted := false

			-- The patch duplicates two vertices per triangle, so release
			-- up the first triangle
			patch.item (1).remove_reference (Current)
			patch.item (2).remove_reference (Current)
			patch.item (3).remove_reference (Current)

			-- and then skip over the duplicates, releasing only the
			-- unprocessed ones.
			from
				i := 6
			invariant
				i_at_least_one: i >= 6
				i_small_enough: i <= 6 + patch.count
			variant
				i_increasing: 6 + patch.count - i
			until
				i > patch.count
			loop
				patch.item (i).remove_reference (Current)
				i := i + 3
			end

			patch.wipe_out
			patch := Void

			from
				referers.start
			until
				referers.after
			loop
				grid := referers.item_for_iteration
				grid.remove (Current)

				referers.forth
			end

			referers.wipe_out
		end

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	bust (mp_list: DS_ARRAYED_LIST [MICROPOLYGON];
		  object_to_camera, object_to_raster: TRANSFORM) is
			-- Break the patch into micropolygons
		require
			mp_list_not_void: mp_list /= Void
		local
			mp: MICROPOLYGON
			i: INTEGER
			pt1, pt2, pt3: SHADING_DATA
			cpt1, cpt2, cpt3: POINT3
			depth: DOUBLE
		do
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + patch.count
			variant
				i_increasing: 1 + patch.count - i
			until
				i > patch.count
			loop
				pt1 := patch.item (i)
				pt1.set_raster_point (object_to_raster.apply (pt1.point))
				cpt1 := object_to_camera.apply (pt1.point)
				pt2 := patch.item (i+1)
				pt2.set_raster_point (object_to_raster.apply (pt2.point))
				cpt2 := object_to_camera.apply (pt2.point)
				pt3 := patch.item (i+2)
				pt3.set_raster_point (object_to_raster.apply (pt3.point))
				-- Twin pt3 so the reference count is correct
				--pt4 := pt3.twin
				cpt3 := object_to_camera.apply (pt3.point)

				-- Compute the depth in camera space; Use max
				-- since we're looking down -Z.
				depth := cpt1.z.max (cpt2.z.max (cpt3.z))

				create mp.make (pt1, pt2, pt3, pt3)
				mp.set_depth (depth)

				mp_list.force_last (mp)

				i := i + 3
			end

			busted := true
		ensure
			generated_micropolygons: mp_list.count > 0
			busted: busted
		end

	update_references is
			-- Update the reference count for the elements of the
			-- patch
		local
			i: INTEGER
		do
			-- The patch duplicates two vertices per triangle, so set
			-- up the first triangle
			patch.item (1).add_reference (Current)
			patch.item (2).add_reference (Current)
			patch.item (3).add_reference (Current)

			-- and then skip over the duplicates, processing only the
			-- new ones.
			from
				i := 6
			invariant
				i_at_least_one: i >= 6
				i_small_enough: i <= 6 + patch.count
			variant
				i_increasing: 6 + patch.count - i
			until
				i > patch.count
			loop
				patch.item (i).add_reference (Current)
				i := i + 3
			end
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	patch: DS_ARRAYED_LIST [SHADING_DATA]
			-- The patch points

invariant

	no_more_than_two_referers: referers.count <= 2

end -- PATCH_OBJECT

