
indexing
	description: "Renderer specific statistics"
	names: "Statistics"
	representation: "array"
	access: "index"
	size: "fixed"
	contents: "INTEGER", "REAL"
	patterns: "none"
	status: "unreviewed"




class 

	STATISTICS

inherit

	SINGLETON
		redefine
			default_create
		end
	
creation
	default_create

feature -- Initialization

	default_create is
			-- 
		do
		end

feature -- Access

	-- Integer statistics
	-- Primitive statistics
	prim_count, prim_culled, prim_generated,

	-- Grid statistics
	grid_count, grids_backfacing,
	
	-- Micropolygon statistics
	mp_count, mp_culled, mp_single_bucket, mp_multiple_bucket,
	mp_per_pixel_min, mp_per_pixel_avg, mp_per_pixel_max, mp_pixel_count,
	mp_pushed_forward, mp_pushed_down, mp_pushed_far

	: INTEGER is unique

	item (which_stat: INTEGER): INTEGER is
			-- The statistic corresponding to `which_stat'
		require
			which_stat_valid: which_stat >= prim_count and which_stat <= mp_pushed_far
		do
			Result := statistics.item (which_stat)
		end

	samples: DS_ARRAYED_LIST [POINT3] is
			-- A list of the sample positions
		once
			create Result.make (1)			
		end


feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	put (value: INTEGER; which_stat: INTEGER) is
			-- Set the statistic corresponding to `which_stat' to `value'
		require
			which_stat_valid: which_stat >= prim_count and which_stat <= mp_pushed_far
			value_positive: value >= 0
		do
			statistics.put (value, which_stat)
		ensure
			value_set: statistics.item (which_stat) = value
		end

	reset is
			-- Reset the statistics to zero
		local
			i: INTEGER
		do
			from
				i := prim_count 
			invariant
				i_at_least_one: i >= prim_count
				i_small_enough: i <= prim_count + mp_pushed_far
			variant
				i_increasing: prim_count + mp_pushed_far - i
			until
				i >= mp_pushed_far
			loop
				statistics.put (0, i)
				i := i + 1
			end
		end
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	increment (which_stat: INTEGER) is
			-- Increment the statistic corresponding to `which_stat'
		require
			which_stat_valid: which_stat >= prim_count and which_stat <= mp_pushed_far
		do
			statistics.put (statistics.item (which_stat) + 1, which_stat)
		ensure
			incremented: old statistics.item (which_stat) + 1 = statistics.item (which_stat)
		end
	
	increase (value: INTEGER; which_stat: INTEGER) is
			-- Increment the statistic corresponding to `which_stat'
		require
			which_stat_valid: which_stat >= prim_count and which_stat <= mp_pushed_far
			value_positive: value >= 0
		do
			statistics.put (statistics.item (which_stat) + value, which_stat)
		ensure
			incremented: old statistics.item (which_stat) + value = statistics.item (which_stat)
		end
	
	decrement (which_stat: INTEGER) is
			-- Increment the statistic corresponding to `which_stat'
		require
			which_stat_valid: which_stat >= prim_count and which_stat <= mp_pushed_far
		do
			statistics.put (statistics.item (which_stat) - 1, which_stat)
		ensure
			incremented: old statistics.item (which_stat) - 1 = statistics.item (which_stat)
			positive: statistics.item (which_stat) >= 0
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	statistics: ARRAY [INTEGER] is
			-- The actual statistics
		once
			create Result.make (prim_count, mp_pushed_far)			
		end

invariant
	statistics_not_void: statistics /= Void
	
end -- STATISTICS

