indexing
	description: "Interface to the OpenGL routines."
	note: "This only interfaces to the double variant of those %
			%routines that take multiple types. Converters are provided %
			%internally to those few routines that require reals"
	names: "opengl", "viewer", "scene graph", "3d"
	representation: "none"
	access: "fixed"
	size: "fixed"
	contents: "DOUBLE"
	patterns: "none"
	status: "unreviewed"

class

	GL_IMPLEMENTATION

inherit
	
	EWG_EXTERNAL_STRING_ROUTINES
		export
			{NONE} all
		redefine 
			default_create
		end
	
	BASIC_NUMERIC
		rename
			cube as cubed
		redefine 
			default_create
		end
	
	GL_CONSTANTS
		redefine 
			default_create
		end

	NUMERIC_CONVERSION
		redefine
			default_create
		end
		
creation
	default_create
	
feature -- Initialization
	
	default_create is
			-- Initialize the internals
		do
			quadric_normals := rd_smooth
			quadric_draw_style := rd_fill			
			quadric_orientation := rd_outside
		end
	
feature -- Quadric Routines
	
	normal_style (how: INTEGER) is
			-- Define whether or not to use normals
		do
			quadric_normals := how
		end
	
	set_draw_style (how: INTEGER) is
			-- Define how to draw quadrics
		do
			quadric_draw_style := how
		end
	
	orientation (how: INTEGER) is
			-- Define how to draw quadrics
		do
			quadric_orientation := how
		end
	
	use_textures (on_or_off: BOOLEAN) is
			-- Use Textures?
		do
			quadric_texturing := on_or_off
		end
	
feature {NONE}-- Implementation

	error_code: INTEGER
			-- The error code returned by glGetError
	
	quadric_normals: INTEGER
			-- Defines how normals are drawn
	
	quadric_draw_style: INTEGER
			-- Defines how quadrics are drawn
	
	quadric_orientation: INTEGER
			-- Defines how quadrics are drawn
	
	quadric_texturing: BOOLEAN
			-- Draw quadrics with textures?

	cache_size: INTEGER is 240
	
	sine_cache: ARRAY [DOUBLE] is
			-- Create a temporary object
		once
			create Result.make (0, cache_size)
		ensure then
			result_not_void: Result /= Void
		end
	
	cosine_cache: ARRAY [DOUBLE] is
			-- Create a temporary object
		once
			create Result.make (0, cache_size)
		ensure then
			result_not_void: Result /= Void
		end
	
	sine_cache_2: ARRAY [DOUBLE] is
			-- Create a temporary object
		once
			create Result.make (0, cache_size)
		ensure then
			result_not_void: Result /= Void
		end
	
	cosine_cache_2: ARRAY [DOUBLE] is
			-- Create a temporary object
		once
			create Result.make (0, cache_size)
		ensure then
			result_not_void: Result /= Void
		end
	
	sine_cache_3: ARRAY [DOUBLE] is
			-- Create a temporary object
		once
			create Result.make (0, cache_size)
		ensure then
			result_not_void: Result /= Void
		end
	
	cosine_cache_3: ARRAY [DOUBLE] is
			-- Create a temporary object
		once
			create Result.make (0, cache_size)
		ensure then
			result_not_void: Result /= Void
		end
	
feature {NONE}-- Glew queries Implementation

	has_buffer_object_support: BOOLEAN is
			-- Does this implementation support the shading language?
		do
			Result := glew_get_extension (string_to_pointer ("GL_ARB_vertex_buffer_object")) or
				glew_get_extension (string_to_pointer ("GL_VERSION_1_5"))
		end
	
	has_fragment_shader_support: BOOLEAN is
			-- Does this implementation support fragment shaders?
		do
			Result := glew_get_extension (string_to_pointer ("GL_ARB_fragment_shader"))
		end
	
	has_vertex_shader_support: BOOLEAN is
			-- Does this implementation support vertex shaders?
		do
			Result := glew_get_extension (string_to_pointer ("GL_ARB_vertex_shader"))
		end
	
	has_shader_object_support: BOOLEAN is
			-- Does this implementation support shader objects?
		do
			Result := glew_get_extension (string_to_pointer ("GL_ARB_shader_objects"))
		end
	
	has_shading_language_support: BOOLEAN is
			-- Does this implementation support the shading language?
		do
			Result := glew_get_extension (string_to_pointer ("GL_ARB_shading_language_100"))
		end
	
	check_shading_status (shader_id, which_mode: INTEGER): BOOLEAN is
			-- Has `shader_id' been compiled or linked successfully
		local
			status: EWG_INT_ARRAY
		do
			create status.make_new_unshared (1)
			gl_get_object_parameteriv (shader_id, which_mode,
									   status.array_address)
			Result := status.item (0) = 1
		end

feature {NONE}-- Primitive Implementation
	gl_sphere (radius, z_minimum, z_maximum, sweep_angle,
			   u_minimum, u_maximum, v_minimum, v_maximum: DOUBLE;
		slices, stacks: INTEGER) is
		local
			drho, dtheta, start_theta: DOUBLE
			max_rho, min_rho: DOUBLE
			max_r, min_r, max_radius, min_radius: DOUBLE
			nsign: DOUBLE
			normals, capped_zmax, capped_zmin: BOOLEAN
		do
			debug ("graphics")
				io.put_string ("gl_sphere (")
				io.put_double (radius)
				io.put_string (", ")
				io.put_double (z_minimum)
				io.put_string ("," )
				io.put_double (z_maximum)
				io.put_string ("," )
				io.put_double (sweep_angle)
				io.put_string (", ")
				io.put_double (u_minimum)
				io.put_string ("," )
				io.put_double (u_maximum)
				io.put_string (", ")
				io.put_double (v_minimum)
				io.put_string ("," )
				io.put_double (v_maximum)
				io.put_string (")")
				io.put_string ("%N")
			end
			
			if quadric_normals = rd_none then
				normals := false
			else
				normals := true
			end
			
			if quadric_orientation = rd_inside then
				nsign := -1.0
			else
				nsign := 1.0
			end
			
			max_radius := z_maximum
			if max_radius > radius then
				max_radius := radius
			end
			min_radius := z_minimum
			if min_radius < -radius then
				min_radius := -radius
			end
			
			max_rho := arc_sine( max_radius / radius )			
			min_rho := arc_sine( min_radius / radius )

			drho := max_rho - min_rho
			max_r := min_rho + v_maximum * drho
			min_r :=  min_rho + v_minimum * drho
			
			if absolute_value (max_r) < pi_over_two then
				capped_zmax := true
			end
			
			if absolute_value (min_r) < pi_over_two then
				capped_zmin := true
			end
			
			drho := (max_r - min_r) / stacks
			start_theta := u_minimum * sweep_angle
			dtheta := (sweep_angle * u_maximum - start_theta) / slices

			if quadric_draw_style =	 rd_line then
				gl_wire_sphere (radius, min_r, drho, dtheta, nsign, stacks, slices)
			elseif quadric_draw_style =	 rd_point then
				gl_point_sphere (radius, min_r, drho, dtheta, nsign, stacks, slices)
			elseif quadric_draw_style =	 rd_fill then
				gl_solid_sphere (radius, min_radius, max_radius,
						  max_r, min_r, drho, dtheta, start_theta, nsign,
						  capped_zmax, capped_zmin,
						  stacks, slices)

			end
		end
	
	gl_wire_sphere (radius, min_r, drho, dtheta, nsign: DOUBLE; 
		stacks, slices: INTEGER) is
			-- Draw a sphere as wire frame
		local
			i, j: INTEGER
			x, y, z: DOUBLE
			rho, theta: DOUBLE
		do
			-- Draw lines of latitude
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= stacks + 1
			variant
				i_increasing: stacks + 1 - i
			until
				i > stacks
			loop
				rho := min_r + i * drho
				gl_render_begin (rd_line_strip)
				from
					j := 0 
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= slices + 1
				variant
					j_increasing: slices + 1 - j
				until
					j > slices
				loop
					theta := j * dtheta
					x := cosine (theta) * cosine (rho)
					y := sine (theta) * cosine (rho)
					z := nsign * sine (rho)
					
					if quadric_normals /= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						gl_texture_coord_2 (j/slices, i/stacks)
					end
					
					gl_vertex3 (x * nsign * radius, y * nsign * radius, z * nsign * radius)
					
					j := j + 1
				end
				gl_render_end
				
				i := i + 1
			end
			
			from
				j := 0 
			invariant
				j_at_least_one: j >= 0
				j_small_enough: j <= slices + 1
			variant
				j_increasing: slices + 1 - j
			until
				j > slices
			loop
				theta := j * dtheta
				gl_render_begin (rd_line_strip)
				from
					i := 0 
				invariant
					i_at_least_one: i >= 0
					i_small_enough: i <= stacks + 1
				variant
					i_increasing: stacks + 1 - i
				until
					i > stacks
				loop
					rho := min_r + i * drho
					x := cosine (theta) * cosine (rho)
					y := sine (theta) * cosine (rho)
					z := nsign * sine (rho)
					
					if quadric_normals	/= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						gl_texture_coord_2 (j/slices, i/stacks)
					end
					
					gl_vertex3 (x * nsign * radius, y * nsign * radius, z * nsign * radius)
					
					i := i + 1
				end
				gl_render_end
				
				j := j + 1
			end
		end
	
	gl_point_sphere (radius, min_r, drho, dtheta, nsign: DOUBLE; 
		stacks, slices: INTEGER) is
			-- Draw a sphere as points
		local
			i, j: INTEGER
			x, y, z: DOUBLE
			rho, theta: DOUBLE
		do
			gl_render_begin (rd_points)
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= stacks + 1
			variant
				i_increasing: stacks + 1 - i
			until
				i > stacks
			loop
				rho := min_r + i * drho
				from
					j := 0 
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= slices + 1
				variant
					j_increasing: slices + 1 - j
				until
					j > slices
				loop
					theta := j * dtheta
					x := cosine (theta) * cosine (rho)
					y := sine (theta) * cosine (rho)
					z := nsign * sine (rho)
					
					if quadric_normals /= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						gl_texture_coord_2 (j/slices, i/stacks)
					end
					
					gl_vertex3 (x * nsign * radius, y * nsign * radius, z * nsign * radius)
					
					j := j + 1
				end
				
				i := i + 1
			end
			gl_render_end
		end
	
	gl_solid_sphere (radius, min_radius, max_radius: DOUBLE;
						  max_r, min_r, drho, dtheta, start_theta, nsign: DOUBLE; 
						  capped_zmax, capped_zmin: BOOLEAN;
						  stacks, slices: INTEGER) is
		local
			x, y, z: DOUBLE
			x_cap, y_cap, z_cap: DOUBLE
			i, j, imin, imax: INTEGER
			rho, theta: DOUBLE
		do
			if not capped_zmax then
				rho := max_r - drho
				
				x_cap := cosine (max_r)
				y_cap := 0.0
				z_cap := sine (max_r)

				gl_render_begin (rd_quad_strip)
				from
					j := slices
				invariant
					j_at_least_one: j >= -1
					j_small_enough: j <= slices + 1
				--variant
				--	j_increasing: slices + 1 - j
				until
					j < 0
				loop
					theta := j * dtheta + start_theta
					x := cosine (theta) * cosine (rho)
					y := sine (theta) * cosine (rho)
					z := sine (rho)
					
					if quadric_normals /= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						if quadric_orientation = rd_outside then
							gl_texture_coord_2 (j/slices, i/stacks)
						else
							gl_texture_coord_2 (1.0 - j/slices, i/stacks)
						end
					end
					
					gl_vertex3 (x * nsign * radius, 
								 y * nsign * radius, 
								 z * nsign * radius)
					
					if quadric_normals /= rd_none then
						gl_normal (x_cap * nsign, y_cap * nsign, z_cap * nsign)
					end
					
					if quadric_texturing then
						if quadric_orientation = rd_outside then
							gl_texture_coord_2 (j/slices, 1.0)
						else
							gl_texture_coord_2 (1.0 - j/slices, 1.0)
						end
					end
					
					gl_vertex3 (x_cap * nsign * radius, 
								y_cap * nsign * radius, 
								z_cap * nsign * radius)
					
					j := j - 1
				end
				gl_render_end

			end
			
			imax := stacks - 1
			if capped_zmin then
				imin := 0
			else
				imin := 1
			end
			
			from
				i := imin
			invariant
				i_at_least_one: i >= imin
				i_small_enough: i <= imax + 1
			variant
				i_increasing: imax + 1 - i
			until
				i > imax
			loop
				rho := min_r + i * drho
				gl_render_begin (rd_quad_strip)
				from
					j := 0 
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= slices + 1
				variant
					j_increasing: slices + 1 - j
				until
					j > slices
				loop
					theta := j * dtheta + start_theta
					x := cosine (theta) * cosine (rho)
					y := sine (theta) * cosine (rho)
					z := sine (rho)
					
					if quadric_normals /= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						if quadric_orientation = rd_outside then
							gl_texture_coord_2 (j/slices, i/stacks)
						else
							gl_texture_coord_2 (1.0 - j/slices, i/stacks)
						end
					end
					
					gl_vertex3 (x * nsign * radius, 
								 y * nsign * radius, 
								 z * nsign * radius)
					
					x := cosine (theta) * cosine (rho + drho)
					y := sine (theta) * cosine (rho + drho)
					z := sine (rho + drho)
					
					if quadric_normals /= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						if quadric_orientation = rd_outside then
							gl_texture_coord_2 (j/slices, (i + 1)/stacks)
						else
							gl_texture_coord_2 (1.0 - j/slices, (i + 1)/stacks)
						end
					end
					
					gl_vertex3 (x * nsign * radius, 
								 y * nsign * radius, 
								 z * nsign * radius)
					
					j := j + 1
				end
				gl_render_end
				
				i := i + 1
			end
			
			if not capped_zmin then
				rho := min_r + drho
				
				x_cap := cosine (min_r)
				y_cap := 0.0
				z_cap := sine (min_r)

				gl_render_begin (rd_quad_strip)
				from
					j := slices
				invariant
					j_at_least_one: j >= -1
					j_small_enough: j <= slices + 1
				--variant
					--j_increasing: slices + 1 - j
				until
					j < 0
				loop
					theta := j * dtheta + start_theta
					x := cosine (theta) * cosine (rho)
					y := sine (theta) * cosine (rho)
					z := sine (rho)
					
					if quadric_normals /= rd_none then
						gl_normal (x * nsign, y * nsign, z * nsign)
					end
					
					if quadric_texturing then
						if quadric_orientation = rd_outside then
							gl_texture_coord_2 (j/slices, i/stacks)
						else
							gl_texture_coord_2 (1.0 - j/slices, i/stacks)
						end
					end
					
					gl_vertex3 (x * nsign * radius, 
								 y * nsign * radius, 
								 z * nsign * radius)
					
					if quadric_normals /= rd_none then
						gl_normal (x_cap * nsign, y_cap * nsign, z_cap * nsign)
					end
					
					if quadric_texturing then
						if quadric_orientation = rd_outside then
							gl_texture_coord_2 (j/slices, 0.0)
						else
							gl_texture_coord_2 (1.0 - j/slices, 0.0)
						end
					end
					
					gl_vertex3 (x_cap * nsign * radius, 
								y_cap * nsign * radius, 
								z_cap * nsign * radius)
					
					j := j - 1
				end
				gl_render_end
			end
						
		end
	
	gl_cone (base_radius, height, sweep_angle,
			 u_minimum, u_maximum, v_minimum, v_maximum: DOUBLE;
			 slices, stacks: INTEGER) is
		do
			debug ("graphics")
				io.put_string ("gl_cone (base, height, sweep_angle: DOUBLE; slices, stacks: INTEGER)")
				io.put_string ("%N")
			end
			
			gl_cylinder (base_radius, 0, 0, height, sweep_angle,
						 u_minimum, u_maximum, v_minimum, v_maximum,
						 slices, stacks)
		end
	
	gl_cube (lower, upper: POINT3) is
		local
			type: INTEGER
		do
			if quadric_draw_style =	 rd_line or quadric_draw_style =  rd_point then
				type := rd_line_loop
			elseif quadric_draw_style =	 rd_fill then
				type := rd_quads
			end
			
			gl_render_begin (type)
			gl_normal (-1, 0, 0)
			gl_vertex3 (lower.x, lower.y, lower.z) -- 1
			gl_vertex3 (lower.x, lower.y, upper.z) -- 2
			gl_vertex3 (lower.x, upper.y, upper.z) -- 3
			gl_vertex3 (lower.x, upper.y, lower.z) -- 4
			gl_render_end
			
			gl_render_begin (type)
			gl_normal (0, 1, 0)
			gl_vertex3 (lower.x, upper.y, lower.z) -- 4
			gl_vertex3 (lower.x, upper.y, upper.z) -- 3
			gl_vertex3 (upper.x, upper.y, upper.z) -- 7
			gl_vertex3 (upper.x, upper.y, lower.z) -- 8
			gl_render_end
			
			gl_render_begin (type)
			gl_normal (1, 0, 0)
			gl_vertex3 (upper.x, upper.y, lower.z) -- 8
			gl_vertex3 (upper.x, upper.y, upper.z) -- 7
			gl_vertex3 (upper.x, lower.y, upper.z) -- 6
			gl_vertex3 (upper.x, lower.y, lower.z) -- 5
			gl_render_end
			
			gl_render_begin (type)
			gl_normal (0, -1, 0)
			gl_vertex3 (upper.x, lower.y, lower.z) -- 5
			gl_vertex3 (upper.x, lower.y, upper.z) -- 6
			gl_vertex3 (lower.x, lower.y, upper.z) -- 2
			gl_vertex3 (lower.x, lower.y, lower.z) -- 1
			gl_render_end
			
			gl_render_begin (type)
			gl_normal (0, 0, 1)
			gl_vertex3 (upper.x, lower.y, upper.z) -- 6
			gl_vertex3 (upper.x, upper.y, upper.z) -- 7
			gl_vertex3 (lower.x, upper.y, upper.z) -- 3
			gl_vertex3 (lower.x, lower.y, upper.z) -- 2
			gl_render_end
			
			gl_render_begin (type)
			gl_normal (0, 0, -1)
			gl_vertex3 (upper.x, upper.y, lower.z) -- 8
			gl_vertex3 (upper.x, lower.y, lower.z) -- 5
			gl_vertex3 (lower.x, lower.y, lower.z) -- 1
			gl_vertex3 (lower.x, upper.y, lower.z) -- 4
			gl_render_end
			
		end
	
	gl_cylinder (base_radius, top_radius, z_minimum, z_maximum, sweep_angle,
				 u_minimum, u_maximum, v_minimum, v_maximum: DOUBLE;
				 slices, stacks: INTEGER) is
		require
			slices_greater_than_3: slices > 3
			stacks_greater_than_2: stacks > 2
			base_radius_is_positive: base_radius >= 0
			top_radius_is_positive: top_radius >= 0
			height_in_proper_relation: z_minimum <= z_maximum
		local
			i: INTEGER
			new_base_radius, new_top_radius: DOUBLE
			angle, dtheta, start_theta: DOUBLE
			length, height, new_z_min, new_z_max: DOUBLE
			delta_radius, z_normal, xy_normal_ratio: DOUBLE 
			need_cache_2, need_cache_3: BOOLEAN
		do
			debug ("graphics")
				io.put_string ("gl_cylinder (")
				io.put_double (base_radius)
				io.put_string (", ")
				io.put_double (top_radius)
				io.put_string (", ")
				io.put_double (z_minimum)
				io.put_string (", ")
				io.put_double (z_maximum)
				io.put_string (", ")
				io.put_double (u_minimum)
				io.put_string ("," )
				io.put_double (u_maximum)
				io.put_string (", ")
				io.put_double (v_minimum)
				io.put_string ("," )
				io.put_double (v_maximum)
				io.put_string (", ")
				io.put_double (z_minimum)
				io.put_string ("," )
				io.put_double (z_maximum)
				io.put_string ("," )
				io.put_double (sweep_angle)
				io.put_string (")")
				io.put_string ("%N")
			end
			
			delta_radius := base_radius - top_radius
			height := z_maximum - z_minimum

			new_top_radius := base_radius - v_maximum * delta_radius
			new_base_radius := base_radius - v_minimum * delta_radius
			delta_radius := new_base_radius - new_top_radius
			
			new_z_min := v_minimum * height + z_minimum
			new_z_max := v_maximum * height + z_minimum
			height := new_z_max - new_z_min
			
			length := square_root (delta_radius*delta_radius + height*height)
			
			check
				length_not_zero: length /= 0.0
			end
			
			if quadric_normals = rd_smooth then
				need_cache_2 := true
			elseif quadric_normals = rd_flat then
				if quadric_draw_style = rd_point then
					need_cache_3 := true
				elseif quadric_draw_style = rd_line then
					need_cache_2 := true
				end
			end
			
			z_normal := delta_radius / length
			xy_normal_ratio := height / length

			start_theta := u_minimum * sweep_angle
			dtheta := (sweep_angle * u_maximum - start_theta) / slices
			
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + slices + 1
			variant
				i_increasing: 0 + slices + 1 - i
			until
				i >= slices + 1
			loop
				angle := start_theta + dtheta * i
				if need_cache_2 then
					if quadric_orientation = rd_outside then
						sine_cache_2.put (xy_normal_ratio * sine (angle), i)
						cosine_cache_2.put (xy_normal_ratio * cosine (angle), i)
					else
						sine_cache_2.put (-xy_normal_ratio * sine (angle), i)
						cosine_cache_2.put (-xy_normal_ratio * cosine (angle), i)
					end
				end
				
				sine_cache.put (sine (angle), i)
				cosine_cache.put (cosine (angle), i)
				
				i := i + 1
			end
			
			if need_cache_3 then
				from
					i := 0 
				invariant
					i_at_least_one: i >= 0
					i_small_enough: i <= 0 + slices + 1
				variant
					i_increasing: 0 + slices + 1 - i
				until
					i >= slices + 1
				loop
					angle := start_theta + dtheta * (i - 0.5)
					
					if quadric_orientation = rd_outside then
						sine_cache_3.put (xy_normal_ratio * sine (angle), i)
						cosine_cache_3.put (xy_normal_ratio * cosine (angle), i)
					else
						sine_cache_3.put (-xy_normal_ratio * sine (angle), i)
						cosine_cache_3.put (-xy_normal_ratio * cosine (angle), i)
					end
					
					i := i + 1
				end
			end
			
			if quadric_draw_style =	 rd_line then
				gl_wire_cylinder (new_base_radius, delta_radius, new_z_min,
											 z_normal, height, slices, stacks)
			elseif quadric_draw_style =	 rd_point then
				gl_point_cylinder (new_base_radius, delta_radius, new_z_min,
											 z_normal, height, slices, stacks)
			elseif quadric_draw_style =	 rd_fill then
				gl_solid_cylinder (new_base_radius, delta_radius, new_z_min,
											 z_normal, height, slices, stacks)
			end
		end
	
	gl_point_cylinder (base_radius, delta_radius, z_minimum, z_normal, height: DOUBLE;
		slices, stacks: INTEGER) is
		local
			radius_low: DOUBLE
			i, j: INTEGER
			z_low: DOUBLE
			sin_temp, cos_temp: DOUBLE			
		do
			gl_render_begin (rd_points)
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + slices + 1
			variant
				i_increasing: 0 + slices + 1 - i
			until
				i >= slices + 1
			loop
				if quadric_normals = rd_flat or quadric_normals = rd_smooth then
					gl_normal (cosine_cache_2 @ i, sine_cache_2 @ i, z_normal)
				end
				
				sin_temp := sine_cache @ i
				cos_temp := cosine_cache @ i
				from
					j := 0 
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= 0 + stacks + 1
				variant
					j_increasing: 0 + stacks + 1 - j
				until
					j >= stacks + 1
				loop
					z_low := z_minimum + j * height / stacks
					radius_low := base_radius - delta_radius * j / stacks
					
					if quadric_texturing then
						gl_texture_coord_2 (i/slices, j/stacks)
					end
					
					gl_vertex3 (radius_low * cos_temp, radius_low * sin_temp, z_low)
					
					j := j + 1
				end
				i := i + 1
			end
			gl_render_end
		end
	
	gl_wire_cylinder (base_radius, delta_radius, z_minimum, z_normal, height: DOUBLE;
		slices, stacks: INTEGER) is
		local
			radius_low: DOUBLE
			i, j: INTEGER
			z_low: DOUBLE
			sin_temp, cos_temp: DOUBLE			
		do
			from
				j := 0
			invariant
				j_at_least_one: j >= 0
				j_small_enough: j <= stacks + 1
			variant
				j_increasing: stacks + 1 - j
			until
				j >= stacks + 1
			loop
				z_low := z_minimum + j * height / stacks
				radius_low := base_radius - delta_radius * j / stacks
				
				gl_render_begin (rd_line_strip)
				
				from
					i := 0 
				invariant
					i_at_least_one: i >= 0
					i_small_enough: i <= 0 + slices + 1
				variant
					i_increasing: 0 + slices + 1 - i
				until
					i >= slices + 1
				loop
					if quadric_normals = rd_flat then
						gl_normal (cosine_cache_3 @ i, sine_cache_3 @ i, z_normal)
					elseif quadric_normals = rd_smooth then
						gl_normal (cosine_cache_2 @ i, sine_cache_2 @ i, z_normal)
					end
					
					if quadric_texturing then
						gl_texture_coord_2 (i/slices, j/stacks)
					end
					
					gl_vertex3 (radius_low * cosine_cache @ i,
								 radius_low * sine_cache @ i, z_low)
					
					i := i + 1
				end
				
				gl_render_end
				j := j + 1
			end
			
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + slices + 1
			variant
				i_increasing: 0 + slices + 1 - i
			until
				i >= slices + 1
			loop
				if quadric_normals = rd_flat or quadric_normals = rd_smooth then
					gl_normal (cosine_cache_2 @ i, sine_cache_2 @ i, z_normal)
				end
				
				sin_temp := sine_cache @ i
				cos_temp := cosine_cache @ i
				gl_render_begin (rd_line_strip)
			
				from
					j := 0 
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= 0 + stacks + 1
				variant
					j_increasing: 0 + stacks + 1 - j
				until
					j >= stacks + 1
				loop
					z_low := z_minimum + j * height / stacks
					radius_low := base_radius - delta_radius * j / stacks
					
					if quadric_texturing then
						gl_texture_coord_2 (i/slices, j/stacks)
					end
					
					gl_vertex3 (radius_low * cos_temp, radius_low * sin_temp, z_low)
					
					j := j + 1
				end
				i := i + 1
				gl_render_end
			end
		end
	
	gl_solid_cylinder (base_radius, delta_radius, z_minimum, z_normal, height: DOUBLE;
		slices, stacks: INTEGER) is
		local
			radius_low, radius_high: DOUBLE
			i, j: INTEGER
			z_low, z_high: DOUBLE
		do
			from
				j := 0 
			invariant
				j_at_least_one: j >= 0
				j_small_enough: j <= 0 + stacks
			variant
				j_increasing: 0 + stacks - j
			until
				j >= stacks
			loop
				z_low := z_minimum + j * height / stacks
				z_high := z_minimum + (j + 1) * height / stacks
				radius_low := base_radius - delta_radius * j / stacks
				radius_high := base_radius - delta_radius * (j + 1) / stacks
				
				gl_render_begin (rd_quad_strip)
				from
					i := 0 
				invariant
					i_at_least_one: i >= 0
					i_small_enough: i <= 0 + slices + 1
				variant
					i_increasing: 0 + slices + 1 - i
				until
					i >= slices + 1
				loop
					
					if quadric_normals = rd_flat then
						gl_normal (cosine_cache_3 @ i, sine_cache_3 @ i, z_normal)
					elseif quadric_normals = rd_smooth then
						gl_normal (cosine_cache_2 @ i, sine_cache_2 @ i, z_normal)
					end
					
					if quadric_orientation = rd_outside then
						if quadric_texturing then
							gl_texture_coord_2 (i/slices, (j + 1)/stacks)
						end
						
						gl_vertex3 (radius_high * cosine_cache @ i,
									 radius_high * sine_cache @ i, z_high)
						
						if quadric_texturing then
							gl_texture_coord_2 (i/slices, j/stacks)
						end
						
						gl_vertex3 (radius_low * cosine_cache @ i,
									radius_low * sine_cache @ i, z_low)
					else
						if quadric_texturing then
							gl_texture_coord_2 (1 - i/slices, j/stacks)
						end
						
						gl_vertex3 (radius_low * cosine_cache @ i,
									radius_low * sine_cache @ i, z_low)
						
						if quadric_texturing then
							gl_texture_coord_2 (1 - i/slices, (j + 1)/stacks)
						end
						
						gl_vertex3 (radius_high * cosine_cache @ i,
									 radius_high * sine_cache @ i, z_high)
					end
					
					i := i + 1
				end
				gl_render_end
				j := j + 1
			end
		end
	
	gl_disk (radius, height, sweep_angle: DOUBLE; slices, stacks: INTEGER) is
		do
			debug ("graphics")
				io.put_string ("gl_disk (radius, height, sweep_angle:: DOUBLE; slices, stacks: INTEGER)")
				io.put_string ("%N")
			end
			
		end
	
	gl_torus (inner_radius, outer_radius, phi_min, phi_max, sweep_angle,
			  u_minimum, u_maximum, v_minimum, v_maximum: DOUBLE; 
			  sides, rings: INTEGER) is
		local
			theta, phi, theta1, cos_theta, sin_theta,
			cos_theta1, sin_theta1, cos_phi, sin_phi, dist: DOUBLE
			phi_start, dphi, phi_end, start_theta, dtheta: DOUBLE
			i, j: INTEGER
		do
			debug ("graphics")
				io.put_string ("gl_torus (innerRadius, outerRadius, phi_min, phi_max, sweep_angle: DOUBLE; sides, rings: INTEGER)")
				io.put_string ("%N")
			end

			dphi := phi_max - phi_min
			phi_start := phi_min + v_minimum * dphi
			phi_end := phi_min + v_maximum * dphi
			dphi := (phi_end - phi_start) / sides
			
			start_theta := u_minimum * sweep_angle
			dtheta := (sweep_angle * u_maximum - start_theta) / rings
			
			if quadric_draw_style =	 rd_line then
				gl_push_attribute (gl_polygon_bit)
				gl_polygon_mode (rd_front_and_back, rd_line)
			elseif quadric_draw_style =	 rd_point then
				gl_push_attribute (gl_polygon_bit)
				gl_polygon_mode (rd_front_and_back, rd_point)
			end

			theta := start_theta
			cos_theta := cosine (theta)
			sin_theta := sine (theta)
			
			from
				i := 0
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= rings
			variant
				i_increasing: rings - i
			until
				i >= rings
			loop
				theta1 := theta + dtheta
				cos_theta1 := cosine (theta1)
				sin_theta1 := sine (theta1)
				phi := phi_start
				gl_render_begin (rd_quad_strip)
				
				from
					j := 0
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= sides + 1
				variant
					j_increasing: 1 + sides - j
				until
					j > sides
				loop
					phi := phi_start + j * dphi
					cos_phi := cosine (phi)
					sin_phi := sine (phi)
					dist := outer_radius + inner_radius * cos_phi
					
					if quadric_normals /= rd_none then
						gl_normal (cos_theta * cos_phi, sin_theta * cos_phi, sin_phi)
					end
					
					if quadric_texturing then
						gl_texture_coord_2 (i/rings, j/sides)
					end
					
					gl_vertex3 (cos_theta * dist, sin_theta * dist, inner_radius * sin_phi)
					
					if quadric_normals /= rd_none then
						gl_normal (cos_theta1 * cos_phi, sin_theta1 * cos_phi, sin_phi)
					end
					
					if quadric_texturing then
						gl_texture_coord_2 ((i + 1)/rings, j/sides)
					end
					
					gl_vertex3 (cos_theta1 * dist, sin_theta1 * dist, inner_radius * sin_phi)
					
					j := j + 1
				end
				
				gl_render_end
				
				theta := theta1
				cos_theta := cos_theta1
				sin_theta := sin_theta1
				i := i + 1
			end
			
			if quadric_draw_style =	 rd_line or
				quadric_draw_style =  rd_point then
				gl_pop_attribute
			end

		end
	
feature -- Shading language externals

	gl_attach_object (a4: INTEGER; a5: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glAttachObjectARB"
		end

	gl_compile_shader (a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glCompileShaderARB"
		end

	gl_create_program_object : INTEGER is
		external
			"C use <glew.h>"
		alias
			"glCreateProgramObjectARB"
		end
	
	gl_create_shader_object (a4: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glCreateShaderObjectARB"
		end
	
	gl_delete_object (a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glDeleteObjectARB"
		end
	
	gl_detach_object (a4: INTEGER; a5: INTEGER) is
	external
		"C use <glew.h>"
	alias
		"glDetachObjectARB"
	end

	gl_get_active_uniform (a4: INTEGER; a5: INTEGER; a6: INTEGER; a7: POINTER; a8: POINTER; a9: POINTER; a10: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetActiveUniformARB"
		end
	
	gl_get_attached_objects (a4: INTEGER; a5: INTEGER; a6: POINTER; a7: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetAttachedObjectsARB"
		end
	
	gl_get_handle (a4: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glGetHandleARB"
		end
	
	gl_get_info_log (a4: INTEGER; a5: INTEGER; a6: POINTER; a7: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetInfoLogARB"
		end
	
	gl_get_object_parameterfv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetObjectParameterfvARB"
		end
	
	gl_get_object_parameteriv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetObjectParameterivARB"
		end
	
	gl_get_shader_source (a4: INTEGER; a5: INTEGER; a6: POINTER; a7: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetShaderSourceARB"
		end
	
	gl_get_uniform_location (a4: INTEGER; a5: POINTER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glGetUniformLocationARB"
		end
	
	gl_get_uniformfv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetUniformfvARB"
		end
	
	gl_get_uniformiv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetUniformivARB"
		end
	
	gl_link_program (a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glLinkProgramARB"
		end
	
	gl_shader_source (a4: INTEGER; a5: INTEGER; a6: POINTER; a7: POINTER) is
		external
			"C macro use <GLSLWrapper.h>"
		alias
			"set_shader_source_external"
		end
	
	gl_uniform1f (a4: INTEGER; a5: REAL) is
		external
			"C use <glew.h>"
		alias
			"glUniform1fARB"
		end
	
	gl_uniform1fv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform1fvARB"
		end
	
	gl_uniform1i (a4: INTEGER; a5: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glUniform1iARB"
		end
	
	gl_uniform1iv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform1ivARB"
		end
	
	gl_uniform2f (a4: INTEGER; a5: REAL; a6: REAL) is
		external
			"C use <glew.h>"
		alias
			"glUniform2fARB"
		end
	
	gl_uniform2fv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform2fvARB"
		end
	
	gl_uniform2i (a4: INTEGER; a5: INTEGER; a6: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glUniform2iARB"
		end
	
	gl_uniform2iv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform2ivARB"
		end
	
	gl_uniform3f (a4: INTEGER; a5: REAL; a6: REAL; a7: REAL) is
		external
			"C use <glew.h>"
		alias
			"glUniform3fARB"
		end
	
	gl_uniform3fv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform3fvARB"
		end
	
	gl_uniform3i (a4: INTEGER; a5: INTEGER; a6: INTEGER; a7: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glUniform3iARB"
		end
	
	gl_uniform3iv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform3ivARB"
		end
	
	gl_uniform4f (a4: INTEGER; a5: REAL; a6: REAL; a7: REAL; a8: REAL) is
		external
			"C use <glew.h>"
		alias
			"glUniform4fARB"
		end
	
	gl_uniform4fv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform4fvARB"
		end
	
	gl_uniform4i (a4: INTEGER; a5: INTEGER; a6: INTEGER; a7: INTEGER; a8: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glUniform4iARB"
		end
	
	gl_uniform4iv (a4: INTEGER; a5: INTEGER; a6: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniform4ivARB"
		end
	
	gl_uniform_matrix2fv (a4: INTEGER; a5: INTEGER; a6: INTEGER; a7: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniformMatrix2fvARB"
		end
	
	gl_uniform_matrix3fv (a4: INTEGER; a5: INTEGER; a6: INTEGER; a7: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniformMatrix3fvARB"
		end
	
	gl_uniform_matrix4fv (a4: INTEGER; a5: INTEGER; a6: INTEGER; a7: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glUniformMatrix4fvARB"
		end
	
	gl_use_program_object (a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glUseProgramObjectARB"
		end
	
	gl_validate_program (a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glValidateProgramARB"
		end

feature -- Externals

	gl_get_error: INTEGER is
		external
			"C use <glew.h>"
		alias
			"glGetError"
		end
	
	gl_render_begin (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBegin"
		end
	
	gl_render_end is
		external
			"C use <glew.h>"
		alias
			"glEnd"
		end
	
	gl_rect (x1, y1, x2, y2: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glRectd"
		end
	
	gl_vertex2 (x, y: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glVertex2d"
		end
	
	gl_vertex3 (x, y, z: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glVertex3d"
		end
	
	gl_vertex4 (x, y, z, w: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glVertex4d"
		end
	
	gl_vertex_pointer (width, type, offset: INTEGER; p: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glVertexPointer"
		end
	
	gl_color_pointer (width, type, offset: INTEGER; p: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glColorPointer"
		end
	
	gl_rotate (angle, x, y, z: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glRotated"
		end
	
	gl_scale (x, y, z: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glScaled"
		end
	
	gl_translate (x, y, z: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glTranslated"
		end
	
	
	gl_ortho (left, right, bottom, top, near, far: DOUBLE) is
		external
			"C (GLdouble, GLdouble, GLdouble, %
			% GLdouble, GLdouble, GLdouble) | <glew.h>"
		alias
			"glOrtho"
		end
	
	gl_load_identity is
		external
			"C use <glew.h>"
		alias
			"glLoadIdentity"
		end
	
	gl_load_matrix (m: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glLoadMatrixd"
		end
	
	gl_matrix_mode (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glMatrixMode"
		end
	
	gl_pop_matrix is
		external
			"C use <glew.h>"
		alias
			"glPopMatrix"
		end
	
	gl_push_matrix is
		external
			"C use <glew.h>"
		alias
			"glPushMatrix"
		end
	
	gl_viewport (left, right, bottom, top: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glViewport"
		end
	
	gl_frustum (left, right, bottom, top, near_val, far_val: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glFrustum"
		end
	
	gl_color_3 (red, green, blue: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glColor3d"
		end
	
	gl_color_4 (red, green, blue, alpha: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glColor4d"
		end
	
	gl_color_material (face, mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glColorMaterial"
		end
	
	gl_color_index_imp (index: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glIndexd"
		end
	
	gl_normal (nx, ny, nz: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glNormal3d"
		end
	
	gl_shade_model (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glShadeModel"
		end
	
	gl_front_face (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glFrontFace"
		end
	
	gl_raster_position_2 (x, y: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glRasterPos2d"
		end
	
	gl_raster_position_3 (x, y, z: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glRasterPos3d"
		end
	
	gl_raster_position_4 (x, y, z, w: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glRasterPos4d"
		end
	
	gl_point_size (size: REAL) is
		external
			"C use <glew.h>"
		alias
			"glPointSize"
		end
	
	gl_line_width (width: REAL) is
		external
			"C use <glew.h>"
		alias
			"glLineWidth"
		end
	
	gl_polygon_mode (face, mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glPolygonMode"
		end
	
	gl_read_buffer (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glReadBuffer"
		end
	
	gl_pixel_store (pname, param: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glPixelStorei"
		end
	
	gl_pixel_transfer (pname: INTEGER; param: REAL_32) is
		external
			"C use <glew.h>"
		alias
			"glPixelTransferf"
		end
	
	gl_draw_pixels (width, height, format, type: INTEGER; pixels: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glDrawPixels"
		end
	
	gl_texture_coord_1 (s: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glTexCoord1d"
		end
	
	gl_texture_coord_2 (s, t: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glTexCoord2d"
		end
	
	gl_texture_coord_3 (s, t, r: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glTexCoord3d"
		end
	
	gl_texture_coord_4 (s, t, r, q: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glTexCoord4d"
		end
	
	gl_texture_env (target, pname, param: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glTexEnvi"
		end
	
	gl_texture_gen (coord, pname: INTEGER; param: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glTexGend"
		end
	
	gl_texture_parameter (target, pname, param: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glTexParameteri"
		end
	
	gl_scissor (x, y, width, height: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glScissor"
		end
	
	gl_stencil_func (func, ref, mask: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glStencilFunc"
		end
	
	gl_stencil_mask (mask: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glStencilMask"
		end
	
	gl_stencil_op (fail, zfail, zpass: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glStencilOp"
		end
	
	gl_clear (mask: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glClear"
		end
	
	gl_clear_color (red, green, blue, alpha: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glClearColor"
		end
	
	gl_clear_depth (depth: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glClearDepth"
		end
	
	gl_clear_index (index: DOUBLE) is
		external
			"C use <glew.h>"
		alias
			"glClearIndex"
		end
	
	gl_draw_buffer (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glDrawBuffer"
		end
	
	gl_draw_arrays (mode, offset, count: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glDrawArrays"
		end
	
	gl_draw_elements (mode, offset, count: INTEGER; indices: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glDrawElements"
		end
	
	gl_render_mode (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glRenderMode"
		end
	
	gl_new_list (list, mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glNewList"
		end
	
	gl_generate_lists (count: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glGenLists"
		end
	
	gl_delete_lists (list, count: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glDeleteLists"
		end
	
	gl_list_base (offset: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glListBase"
		end
	
	gl_end_list is
		external
			"C use <glew.h>"
		alias
			"glEndList"
		end
	
	gl_call_list (list: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glCallList"
		end
	
	gl_call_lists (n, type: INTEGER; lists: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glCallLists"
		end
	
	gl_disable (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glDisable"
		end
	
	gl_enable (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glEnable"
		end
	
	gl_enable_client_state (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glEnableClientState"
		end
	
	gl_disable_client_state (mode: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glDisableClientState"
		end
	
	gl_finish is
		external
			"C use <glew.h>"
		alias
			"glFinish"
		end
	
	gl_flush is
		external
			"C use <glew.h>"
		alias
			"glFlush"
		end
	
	gl_perspective (fovy, aspect, znear, zfar: DOUBLE) is
		external
			"C (GLdouble, GLdouble, GLdouble, GLdouble) | <glew.h>"
		alias
			"gluPerspective"
		end

	gl_look_at (eyex, eyey, eyez, centerx, centery, centerz, upx, upy, upz: DOUBLE) is
		external
			"C (GLdouble, GLdouble, GLdouble, GLdouble, %
			%GLdouble, GLdouble, GLdouble, GLdouble, GLdouble) | <glew.h>"
		alias
			"gluLookAt"
		end
	
	gl_clip_plane_imp (plane: INTEGER; equation: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glClipPlane"
		end
	
	gl_light_model_i (pname, parameter: INTEGER) is	
		external
			"C use <glew.h>"
		alias
			"glLightModeli"
		end
	
	gl_light_model_f (pname: INTEGER; parameter: POINTER) is	
		external
			"C use <glew.h>"
		alias
			"glLightModelfv"
		end
	
	gl_light_imp (light_number, pname: INTEGER; parameter: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glLightfv"
		end
	
	gl_lightf (light_number, pname: INTEGER; parameter: REAL) is
		external
			"C use <glew.h>"
		alias
			"glLightf"
		end
	
	gl_material_imp (light_number, pname: INTEGER; parameter: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glMaterialfv"
		end
	
	gl_mult_matrix (m: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glMultMatrixd"
		end
	
	gl_get_integer (query: INTEGER; parameter: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetIntegerv"
		end
	
	gl_push_attribute (mask: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glPushAttrib"
		end
	
	gl_pop_attribute is
		external
			"C use <glew.h>"
		alias
			"glPopAttrib"
		end
	
	glx_current_context: POINTER is
		external
			"C use <glew.h>"
		alias
			"glXGetCurrentContext"
		end
	
	glx_use_x_font (font, first, count, base: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glXUseXFont"
		end

	gl_map_1d (target: INTEGER; u1, u2: DOUBLE; stride, order: INTEGER; points: POINTER) is
		external
			"C use <glew.h>"
		alias
		"glMap1d"
		end
	
	gl_map_grid_1d (un: INTEGER; u1, u2: DOUBLE) is
		external
			"C use <glew.h>"
		alias
		"glMapGrid1d"
		end

	gl_map_2d (target: INTEGER; u1, u2: DOUBLE; ustride, uorder: INTEGER;
	v1, v2: DOUBLE; vstride, vorder: INTEGER; points: POINTER) is
		external
			"C use <glew.h>"
		alias
		"glMap2d"
		end

	gl_map_grid_2d (un: INTEGER; u1, u2: DOUBLE; vn: INTEGER; v1, v2: DOUBLE) is
		external
			"C use <glew.h>"
		alias
		"glMapGrid2d"
		end
		
	gl_eval_mesh_1d (mode, i1, i2: INTEGER) is
		external
			"C use <glew.h>"
		alias
		"glEvalMesh1"
		end
		
	gl_eval_mesh_2d (mode, i1, i2, j1, j2: INTEGER) is
		external
			"C use <glew.h>"
		alias
		"glEvalMesh2"
		end	
	
feature {NONE}-- GL 1.5 functions 

	gl_begin_query (a1: INTEGER; a2: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBeginQuery"
		end

	gl_begin_query_arb (a1: INTEGER; a2: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBeginQueryARB"
		end

	gl_bind_buffer (a1: INTEGER; a2: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBindBuffer"
		end

	gl_bind_buffer_arb (a1: INTEGER; a2: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBindBufferARB"
		end

	gl_buffer_data (a1: INTEGER; a2: POINTER; a3: POINTER; a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBufferData"
		end

	gl_buffer_data_arb (a1: INTEGER; a2: POINTER; a3: POINTER; a4: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glBufferDataARB"
		end

	gl_buffer_sub_data (a1: INTEGER; a2: POINTER; a3: POINTER; a4: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glBufferSubData"
		end

	gl_buffer_sub_data_arb (a1: INTEGER; a2: POINTER; a3: POINTER; a4: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glBufferSubDataARB"
		end

	gl_delete_buffers (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glDeleteBuffers"
		end

	gl_delete_buffers_arb (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glDeleteBuffersARB"
		end

	gl_delete_queries (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glDeleteQueries"
		end

	gl_delete_queries_arb (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glDeleteQueriesARB"
		end

	gl_end_query (a1: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glEndQuery"
		end

	gl_end_query_arb (a1: INTEGER) is
		external
			"C use <glew.h>"
		alias
			"glEndQueryARB"
		end

	gl_gen_buffers (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGenBuffers"
		end

	gl_gen_buffers_arb (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGenBuffersARB"
		end

	gl_gen_queries (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGenQueries"
		end

	gl_gen_queries_arb (a1: INTEGER; a2: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGenQueriesARB"
		end

	gl_get_buffer_parameteriv (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetBufferParameteriv"
		end

	gl_get_buffer_parameteriv_arb (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetBufferParameterivARB"
		end

	gl_get_buffer_pointerv (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetBufferPointerv"
		end

	gl_get_buffer_pointerv_arb (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetBufferPointervARB"
		end

	gl_get_buffer_sub_data (a1: INTEGER; a2: POINTER; a3: POINTER; a4: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetBufferSubData"
		end

	gl_get_buffer_sub_data_arb (a1: INTEGER; a2: POINTER; a3: POINTER; a4: POINTER)is
		external
			"C use <glew.h>"
		alias
			"glGetBufferSubDataARB"
		end

	gl_get_query_objectiv (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetQueryObjectiv"
		end

	gl_get_query_objectiv_arb (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetQueryObjectivARB"
		end

	gl_get_query_objectuiv (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetQueryObjectuiv"
		end

	gl_get_query_objectuiv_arb (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetQueryObjectuivARB"
		end

	gl_get_queryiv (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetQueryiv"
		end

	gl_get_queryiv_arb (a1: INTEGER; a2: INTEGER; a3: POINTER) is
		external
			"C use <glew.h>"
		alias
			"glGetQueryivARB"
		end

	gl_is_buffer (a1: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glIsBuffer"
		end

	gl_is_buffer_arb (a1: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glIsBufferARB"
		end

	gl_is_query (a1: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glIsQuery"
		end

	gl_is_query_arb (a1: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glIsQueryARB"
		end

	gl_map_buffer (a1: INTEGER; a2: INTEGER): POINTER is
		external
			"C use <glew.h>"
		alias
			"glMapBuffer"
		end

	gl_map_buffer_arb (a1: INTEGER; a2: INTEGER): POINTER is
		external
			"C use <glew.h>"
		alias
			"glMapBufferARB"
		end

	gl_unmap_buffer (a1: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glUnmapBuffer"
		end

	gl_unmap_buffer_arb (a1: INTEGER): INTEGER is
		external
			"C use <glew.h>"
		alias
			"glUnmapBufferARB"
		end


feature {NONE}-- Glew Implementation

	glew_get_extension (query: POINTER): BOOLEAN is
		external
			"C use <glew.h>"
		alias
			"glewGetExtension"
		end
	
	glew_init: INTEGER is
		external
			"C use <glew.h>"
		alias
			"glewInit"
		end
	
end -- class GL_IMPLEMENTATION

