

indexing
	description: "Renderer for NURBS curves and surfaces using GLU"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	GLU_NURBS_RENDERER
	
inherit
	
	NURBS_RENDERER

	NUMERIC_CONVERSION
	
creation
	make
	
feature -- Initialization
feature -- Access
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
	
	set_property (property: INTEGER; value: REAL) is
			-- Set various properties for the nurbs
		do
			glu_nurbs_property (nurbs_object, property, value)
		end
	
	set_knots (uknots, vknots: DOUBLE_ARRAYED_LIST) is
			-- Set the knots
		do		
			u_knots := double_array_to_real (uknots)
			
			if vknots /= Void then
				v_knots := double_array_to_real (vknots)
			end
		end
	
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	begin_surface is
			-- Begin the definition of a nurbs surface
		do
			if generate_normals then
				renderer.enable (renderer.rd_auto_normal)
				renderer.enable (renderer.rd_normalize)
			end
			
			glu_begin_surface (nurbs_object)
		end
	
	define_control_points (ucount, vcount: INTEGER;
						   control_points: BOUNDED_ARRAY2 [POINT4]) is
			-- Define the control points for the surface
		local
			points: REAL_ARRAYED_LIST
		do
			points := flatten_points (control_points)
			
			glu_nurbs_surface (nurbs_object,
							   u_knots.count.to_integer, u_knots.storage.item,
							   v_knots.count.to_integer, v_knots.storage.item,
							   vcount * 4, 4, points.storage.item,
							   u_order, v_order, renderer.rd_map2_vertex_4)
		end
	
	define_normals (ucount, vcount: INTEGER;
					normals: NORMAL_ARRAYED_LIST) is
			-- Define the normals for the surface
		local
			vectors: REAL_ARRAYED_LIST
		do		
			vectors := flatten_vectors (normals)
			
			glu_nurbs_surface (nurbs_object,
							   u_knots.count.to_integer, u_knots.storage.item,
							   v_knots.count.to_integer, v_knots.storage.item,
							   vcount * 3, 3, vectors.storage.item,
							   u_order, v_order, renderer.rd_map2_normal)
			
			normals_specified := true
		end
	
	define_colors (ucount, vcount: INTEGER;
				   colors: RGBA_COLOR_ARRAYED_LIST) is
			-- Define the colors for the surface
		do		
		end
	
	define_texture_coordinates (ucount, vcount: INTEGER;
								texture_coords: POINT2_ARRAYED_LIST) is
			-- Define the texture coordinates for the surface
		do		
		end
	
	end_surface is
			-- End the definition of a nurbs surface
		do		
			if generate_normals then
				renderer.disable (renderer.rd_auto_normal)
				renderer.disable (renderer.rd_normalize)
			end
			
			glu_end_surface (nurbs_object)
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	nurbs_object: POINTER
	
	u_knots, v_knots: REAL_ARRAYED_LIST
	
	new_renderer is
			-- Define a new renderer
		do
			nurbs_object := glu_new_nurbs_renderer
		end

	delete_renderer is
			-- Define a new renderer
		do
			glu_delete_nurbs_renderer (nurbs_object)
		end

	sampling_method: INTEGER is
			-- Constants for `set_property'
		do
			Result := glu_sampling_method
		end
	
	domain_distance: INTEGER is
			-- Constants for `set_property'
		do
			Result := glu_domain_distance
		end
	
	u_step: INTEGER is
			-- Constants for `set_property'
		do
			Result := glu_u_step
		end
	
	v_step: INTEGER is
			-- Constants for `set_property'
		do
			Result := glu_v_step
		end
	
	glu_new_nurbs_renderer: POINTER is
		external
			"C use <GLSLWrapper.h>"
		alias
			"gluNewNurbsRenderer"
		end
	
	glu_delete_nurbs_renderer (nobj: POINTER) is
		external
			"C use <GLSLWrapper.h>"
		alias
			"gluDeleteNurbsRenderer"
		end
	
	glu_nurbs_property (nobj: POINTER; property: INTEGER; value: REAL) is
		external
			"C use <GLSLWrapper.h>"
		alias
			"gluNurbsProperty"
		end
	
	glu_begin_surface (nobj: POINTER) is
		external
			"C use <GLSLWrapper.h>"
		alias
			"gluBeginSurface"
		end
	
	glu_end_surface (nobj: POINTER) is
		external
			"C use <GLSLWrapper.h>"
		alias
			"gluEndSurface"
		end
	
	glu_nurbs_surface (nobj: POINTER; uknot_count: INTEGER; uknot: POINTER;
					   vknot_count: INTEGER; vknot: POINTER; 
					   u_stride, v_stride: INTEGER; ctlarray: POINTER;
					   uorder, vorder, type: INTEGER) is
		external
			"C use <GLSLWrapper.h>"
		alias
			"gluNurbsSurface"
		end
	
	glu_sampling_method: INTEGER is
		external
			"C macro use <GLSLWrapper.h>"
		alias
			"GLU_SAMPLING_METHOD"
		end
		
	glu_domain_distance: INTEGER is
		external
			"C macro use <GLSLWrapper.h>"
		alias
			"GLU_DOMAIN_DISTANCE"
		end
		
	glu_u_step: INTEGER is
		external
			"C macro use <GLSLWrapper.h>"
		alias
			"GLU_U_STEP"
		end
		
	glu_v_step: INTEGER is
		external
			"C macro use <GLSLWrapper.h>"
		alias
			"GLU_V_STEP"
		end
		
feature {NONE}-- Implementation 
	
	double_array_to_real (values: DOUBLE_ARRAYED_LIST): REAL_ARRAYED_LIST is
			-- Convert a list of doubles to a list of reals
		require
			values_array_not_void: values /= Void
		do
			create Result.make (values.count)
			
			from
				values.start
			until
				values.after
			loop
				Result.force_last (real_64_truncated_to_real_32 (values.item_for_iteration))
				
				values.forth
			end
			
		ensure
			Result_not_void: Result /= Void
			same_size: Result.count = values.count
		end
	
	flatten_points (values: BOUNDED_ARRAY2 [POINT4]): REAL_ARRAYED_LIST is
			-- Convert a list of POINT4 to a list of reals
		require
			values_array_not_void: values /= Void
		local
			i, j: INTEGER
		do
			create Result.make (4 * values.count)
			
			from
				i := values.row_lower
			invariant
				i_at_least_one: i >= values.row_lower
				i_small_enough: i <= values.row_lower + values.row_upper + 1
			variant
				i_increasing: values.row_lower + values.row_upper - i + 1
			until
				i > values.row_upper
			loop
				from
					j := values.column_lower 
				invariant
					j_at_least_one: j >= values.column_lower
					j_small_enough: j <= values.column_lower + values.column_upper + 1
				variant
					j_increasing: values.column_lower + values.column_upper - j + 1
				until
					j > values.column_upper
				loop
					Result.force_last (real_64_truncated_to_real_32 (values.item (i, j).x))
					Result.force_last (real_64_truncated_to_real_32 (values.item (i, j).y))
					Result.force_last (real_64_truncated_to_real_32 (values.item (i, j).z))
					Result.force_last (real_64_truncated_to_real_32 (values.item (i, j).w))
					
					j := j + 1
				end
				i := i + 1
			end
			
		ensure
			Result_not_void: Result /= Void
			same_size: Result.count = 4 * values.count
		end
	
	flatten_vectors (values: NORMAL_ARRAYED_LIST): REAL_ARRAYED_LIST is
			-- Convert a list of VECTOR3 to a list of reals
		require
			values_not_void: values /= Void
		do
			create Result.make (3 * values.count)
			
			from
				values.start
			until
				values.after
			loop
				Result.force_last (real_64_truncated_to_real_32 (values.item_for_iteration.x))
				Result.force_last (real_64_truncated_to_real_32 (values.item_for_iteration.y))
				Result.force_last (real_64_truncated_to_real_32 (values.item_for_iteration.z))
				
				values.forth
			end
			
		ensure
			Result_not_void: Result /= Void
			same_size: Result.count = 3 * values.count
		end
	
end -- NURBS_RENDERER

