

indexing
	description: "Class for generating a DAG of variable dependancies"
	names: "action"
	contents: "generic"
	patterns: "strategy"


class 

	DAG_ACTION

inherit
	
	SL_ACTION
		redefine
			default_create, pre_action
		end

creation
	default_create
	
feature -- Initialization
	
	default_create is
			-- Initialize the action
		do
			Precursor

			create hash_table.make (10000)
			create parameters.make (20)
		end

feature -- Access

	root_node: DAG_NODE
			-- The root of the DAG
	
	parameters: DS_ARRAYED_LIST [SL_VARIABLE]
			-- All of the parameters to this shader
			
feature -- Comparison
feature -- Status report
	
	has_variable (variable_name: STRING): BOOLEAN is
			-- Have we seen this `variable_name' before
		require 
			variable_name_not_void: variable_name /= Void
		do
			Result := hash_table.has (variable_name)
		end 

feature -- Status setting
	
	push (node: PARSE_NODE) is
			-- Increase depth in tree
		do
		end 

	pop (node: PARSE_NODE) is
			-- Increase depth in tree
		local
			list: DS_LINKED_LIST [DAG_NODE]
		do
			if definition_node /= Void and then definition_node.definition.ewg_object_id = node.ewg_object_id then
				if has_variable (definition_node.variable.name) then
					list := hash_table.item (definition_node.variable.name)
				else
					create list.make
				end
				
				list.force_first (definition_node)
				hash_table.force (list, definition_node.variable.name)
				
				definition_node := Void
			end
		end 

	start_definition (node: VARIABLE_PARSE_NODE) is
			-- Begin a variable definition
		require 
			node_not_void: node /= Void
		do
			create definition_node
			definition_node.set_definition (node)
			definition_node.set_variable (node.variable)
			definition_node.set_line_number (node.line_number)
		end 

	set_item (node: VARIABLE_PARSE_NODE; name: STRING) is
			-- Set the `node' associated with `string'
		require
			node_not_void: node /= Void
			name_not_void: name /= Void
			name_not_empty: name.count > 0
		local
			dag: DAG_NODE
			list: DS_LINKED_LIST [DAG_NODE]
		do
			if has_variable (name) then
				dag := hash_table.item (name).first
			else
				create dag
				dag.set_definition (node)
				dag.set_variable (node.variable)
				if  node.variable.is_parameter then
					parameters.force_last (node.variable)					
				end
				create list.make
				list.force_first (dag)
				hash_table.force (list, name)
			end
			if definition_node /= Void then
				definition_node.add_last (dag)
			end
		end
	
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	pre_action (node: PARSE_NODE) is
			-- Apply this action before the traversal of children of `node'
		do
			node.dag_output (Current)
		end 

	build (output_variable: STRING) is
			-- Build the DAG for `output_variable'. When done, result will be in 
			-- `root_node'.
		require
			has_variable: has_variable (output_variable)
		do
			root_node := hash_table.item (output_variable).first
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	hash_table: DS_HASH_TABLE [DS_LINKED_LIST [DAG_NODE], STRING]
			-- A hash table to store object_ids

	definition_node: DAG_NODE
			-- A DAG node used for building variable definitions
	
invariant
	hash_table_not_void: hash_table /= Void
	
end -- DAG_ACTION

