

indexing
	description: "Class for ouputting the SL AST to dot format (graphvis)"
	names: "action"
	contents: "generic"
	patterns: "strategy"


class 

	DOT_ACTION

inherit
	
	SL_ACTION
		redefine
			finalize, pre_action
		end

creation
	make
	
feature -- Initialization
	
	make (output_file_name: STRING) is
			-- Initialize the action
		local
			file: KL_TEXT_OUTPUT_FILE
			std_out: KL_STDOUT_FILE
		do
			default_create

			create hash_table.make (10000)
			create seen_link.make (10000)
			
			if not output_file_name.is_equal ("-") then
				create file.make (output_file_name)
			end
			if file = Void then
				create std_out.make
				dot_file := std_out
			else
				file.open_write
				dot_file := file
			end

			dot_file.put_string ("digraph G {%N")
			dot_file.put_string ("%Tsize = %"8.5, 11%";%N")
			dot_file.put_string ("%Torientation = landscape;%N")
			dot_file.put_string ("%Tnode [shape = record, height = .1];%N")
			id_for_dot_output := 1
		end

feature -- Access

	dot_file: KI_CHARACTER_OUTPUT_STREAM
			-- The stream to output

	item (id: INTEGER): DOT_RECORD_INFO is
			-- The item for id
		require
			id_in_table: has_id (id)
		do
			Result := hash_table.item (id)
		end
	
feature -- Comparison
feature -- Status report

	has_id (id: INTEGER): BOOLEAN is
			-- Have we seen this `id' before
		require 
			id_positive: id > 0
		do
			Result := hash_table.has (id)
		end 

	has_link (start_link, end_link: INTEGER): BOOLEAN is
			-- Does a link from `start_link' to	 `end_link' exist
		require 
			start_link_positive: start_link > 0
			end_link_positive: end_link > 0
		local
			list: DS_ARRAYED_LIST [INTEGER]
		do
			Result := False
			if seen_link.has (end_link) then
				list := seen_link.item (end_link)
				from
					list.start
				until
					list.after or Result
				loop
					Result := list.item_for_iteration = start_link
					list.forth
				end
			end
		end 

	gathering: BOOLEAN
			-- Is this action just `gathering' information?
	
feature -- Status setting
	
	push (node: PARSE_NODE) is
			-- Increase depth in tree
		do
		end 

	pop (node: PARSE_NODE) is
			-- Increase depth in tree
		do
		end 

	set_visited_link (start_link, end_link: INTEGER) is
			-- Mark the link from `start_link' to `end_link' as 
			-- visited
		local
			list: DS_ARRAYED_LIST [INTEGER]
		do
			if not has_link (start_link, end_link) then
				if seen_link.has (end_link) then
					list := seen_link.item (end_link)
				else
					create list.make (2)
				end

				list.force_last (start_link)
				seen_link.force (list, end_link)
			end
		end
	
	set_item (id: INTEGER; format_string: STRING) is
			-- Set the `format_string' associated with `id'
		require
			id_positive: id > 0
			format_string_not_void: format_string /= Void
			format_string_not_empty: format_string.count > 0
		local
			dot_record: DOT_RECORD_INFO
			str: STRING
		do
			create str.make (30)
			str.append_string (identifier_prefix)
			str.append_string (id_for_dot_output.out)
			create dot_record.make (id_for_dot_output, str, format_string)
			hash_table.force (dot_record, id)
			id_for_dot_output := id_for_dot_output + 1
		end

	set_gathering (on_or_off: BOOLEAN) is
			-- Set `gathering' to `on_or_off'
		do
			gathering := on_or_off
		ensure 
			gathering_set: gathering = on_or_off
		end 

feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	pre_action (node: PARSE_NODE) is
			-- Apply this action to `node'
		do
			node.dot_output (Current)
		end 

	finalize is
			-- Do any necessary finalization
		do
			dot_file.put_string ("}%N")
			if dot_file.is_closable then
				dot_file.close
			end
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	hash_table: DS_HASH_TABLE [DOT_RECORD_INFO, INTEGER]
			-- A hash table to store object_ids

	seen_link: DS_HASH_TABLE [DS_ARRAYED_LIST [INTEGER], INTEGER]
			-- A hash table to store object_ids

	id_for_dot_output: INTEGER
			-- The next id

	identifier_prefix: STRING is "node"
			-- How we identify nodes
	
invariant
	id_for_dot_output_is_positive: id_for_dot_output > 0
	hash_table_not_void: hash_table /= Void
end -- DOT_ACTION

