

indexing
	description: "A node in the parse tree representing type casts"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	CAST_PARSE_NODE
	
inherit
	
	PARSE_NODE
		redefine
			default_create, type, stack_output, dot_label_string,
			glsl_pre_action, glsl_post_action
		end

	SL_TYPES
		undefine
			default_create
		end

creation
	default_create, make

feature -- Initialization
	
	default_create is
			-- Default Initialization
		do
			Precursor {PARSE_NODE}
			
			type := void_type
		end
	
	make (new_type: INTEGER) is
			-- Create a new node using `new_type'
		require
			valid_type: new_type >= float_type and new_type <= h_point_type
		do
			default_create
			type := new_type
		end
	
feature -- Access
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	type: INTEGER
			-- The type of the cast
	
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	stack_output (action: STACK_ACTION) is
			-- Output this node to the VM
		do
			action.shader_file.put_string ("%Tcast")
			action.shader_file.put_character (encoded_type (children.item (1).type))
			action.shader_file.put_character (encoded_type (type))
			action.shader_file.put_string ("%N")
		end
	
	glsl_pre_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		do
			initialized_cast := false
			inspect type
			when point_type, normal_type, vector_type then
				action.fragment_shader.append_string ("vec3( ")
				initialized_cast := true
			when matrix_type then
				action.fragment_shader.append_string ("mat4( ")
				initialized_cast := true
			when color_type, h_point_type then
				action.fragment_shader.append_string ("vec4( ")
				initialized_cast := true
			end
		end 

	glsl_post_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		do
			if initialized_cast then
				action.fragment_shader.append_string (" )")
			end
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	initialized_cast: BOOLEAN
			-- Flag for passing to the post-action if action is 
			-- rquired
	
	dot_label_string: STRING is
			-- The string to use as a label in the digraph
		do
			Result := dot_preamble
			Result.append_string (" | type\n")
			Result.append_string (type_to_string (type))
			Result.append_string (" } }")
			Result.append_character ('"')
		end
	
invariant	
	
	type_valid: type >= float_type and type <= h_point_type
	count_is_zero_or_one: children.count = 0 or children.count = 1
	
end -- CAST_PARSE_NODE

