

indexing
	description: "A node in the parse tree representing message passing functions"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	COMMUNICATION_FUNCTION_PARSE_NODE
	
inherit
	
	PARSE_NODE
		redefine
			default_create, type, resolve_type
		end

creation
	default_create, make, make_with_arguments

feature -- Initialization

	default_create is
			-- Initialize
		do
			Precursor
			create arguments.make_from_string ("")
		end
	
	make (new_type: INTEGER; new_variable: SL_VARIABLE) is
			-- Initialize with `new_type', and `new_variable'
		require
			new_variable_not_void: new_variable /= Void
			new_type_in_range: new_type >= atmosphere and new_type <= textureinfo
		do
			default_create
			variable := new_variable
			comm_type := new_type
		end
	
	make_with_arguments (new_type: INTEGER; new_arguments: STRING; new_variable: SL_VARIABLE) is
			-- Initialize with `new_type', `new_arguments', and `new_variable'
		require
			new_variable_not_void: new_variable /= Void
			new_arguments_not_void: new_arguments /= Void
			new_type_in_range: new_type >= atmosphere and new_type <= textureinfo
		do
			make (new_type, new_variable)
			arguments := new_arguments.twin
		end
	
feature -- Access
	
	variable: SL_VARIABLE
			-- The `variable' this node is refering to

	arguments: STRING
			-- The arguments for the function

	comm_type: INTEGER
			-- The communicator type
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	type: INTEGER is
			-- What is the type of this parse node?
		do
			Result := float_type
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	resolve_type (return_type: INTEGER) is
			-- Force type coercion as necessary to 
			-- ensure correct operation
		local
			cast_node: CAST_PARSE_NODE
		do
			if return_type = nil_type or else coercible (type, encoded_type (return_type)) then
				from
					children.start
					type_error := false
				until
					children.after
				loop
					children.item_for_iteration.resolve_type (nil_type)
					type_error := type_error or children.item_for_iteration.has_valid_type
					if not type_error and then
					   not (type = nil_type or else mutable (type, children.item_for_iteration.type)) then
					   -- Create a CAST_PARSE_NODE to cast `children.item_for_iteration.type' to `type'
					   create cast_node.make (type)
					   cast_node.add_last (children.item_for_iteration)
					   children.replace_at (cast_node)
					   cast_node.set_parent (Current)
					end
					
					children.forth
				end
			else
				error_string.append ("Unable to cast a ")
				error_string.append (type_to_string (type))
				error_string.append (" to a ")
				error_string.append (type_to_string (return_type))
				error_string.append ("%N%T")
				error_string.append ("Line	")
				error_string.append (line_number.out)
				error_string.append (": ")
				error_string.append (file_name)
				error_string.append ("%N%N")

				type_error := true
			end
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

invariant

	arguments_not_void: arguments /= Void
	comm_type_in_range: comm_type >= atmosphere and comm_type <= textureinfo
	
end -- COMMUNICATION_FUNCTION_PARSE_NODE

