

indexing
	description: "A node in the parse tree representing function calls"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	FUNCTION_CALL_PARSE_NODE
	
inherit
	
	PARSE_NODE
		redefine
			default_create, gather_usage_information, stack_output, type,
			resolve_type, has_valid_return_type, best_type_match, traverse_node,
			dot_label_string, glsl_pre_action, glsl_traverse_action, glsl_post_action
		end

	SL_TYPES
		undefine
			default_create
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize
		do
			Precursor {PARSE_NODE}
			function_index := 0
		end
	
	make (new_functions: DS_ARRAYED_LIST [SL_FUNCTION]) is
			-- Create a new node using `new_functions'
		require
			new_functions_not_void: new_functions /= Void
		do
			default_create
			functions := new_functions
		end
	
feature -- Access

	function: SL_FUNCTION is
			--
		require
			is_valid: not type_error
		do
			Result := functions.item (function_index)
		ensure
			Result_not_void: Result /= Void
		end
	
feature -- Measurement
feature -- Comparison
feature -- Status report

	best_type_match (a_type: INTEGER): INTEGER is
			-- What is the best type match to `a_type'?
		local
			index: INTEGER
		do
			index :=  find_best_type_match (a_type)

			check
				resolved_type: index >= 1 and index <= functions.count
			end
			
			Result := functions.item (index).type
		end 

	has_valid_return_type (return_type: INTEGER; use_exact_type: BOOLEAN): BOOLEAN is
			-- Does this node and children have a valid return type? 
			-- If `use_exact_match' then iff `type' = `return_type'; 
			-- if not `use_exact_match' then if `type' is coercible 
			-- to `return_type'
		do
			if use_exact_type then
				from
					functions.start
				until
					functions.after or Result
				loop
					Result := functions.item_for_iteration.type = return_type
					functions.forth
				end
			else
				from
					functions.start
				until
					functions.after or Result
				loop
					Result := coercible (functions.item_for_iteration.type, encoded_type (return_type))
					functions.forth
				end
			end
		end
	
	is_void: BOOLEAN is
			-- Is the return type of this `functions' void?
		do
			from
				functions.start
			until
				functions.after or Result
			loop
				if functions.item_for_iteration.type = void_type then
					Result := true
				end
				
				functions.forth
			end
		end 

	type: INTEGER is
			-- What is the type of this parse node?
		do
			Result := function.type
		end
	
feature -- Status setting
feature -- Cursor movement
feature -- Element change	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	resolve_type (return_type: INTEGER) is
			-- Resolve the types either directly or through coercion 
			-- to ensure a valid call. On error, sets `type_error' and `error_string'
		do
			function_index := find_best_type_match (return_type)

			if function_index >= 1 and function_index <= functions.count then
				type_error := false
				
				-- Now that we found the function, force type 
				-- resolution in the children
				
				resolve_types_in_children
			end
		ensure then
			no_error_implies_function_valid: not type_error implies
											 (function_index >= 1 and function_index <= functions.count)
		end
	
	stack_output (action: STACK_ACTION) is
			-- Output this node to the VM
		local
			val: FLOAT_CONSTANT_PARSE_NODE
		do
			if function.is_builtin then
				if children.count > 0 then
					-- Output the vararg list
					if function.has_variable_argument_length then
						-- Output the number of extra arguments
						create val.make (children.count - function.argument_types.count - 1)
						val.stack_output (action)
					end
				end

				action.shader_file.put_string ("%T")
				action.shader_file.put_string (function.internal_name)
				action.shader_file.put_string ("%N")
			else
				
			end
		end 

	glsl_pre_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		local
			c: CHARACTER
		do
			unary_operator := false
			
			if not function.name.has_substring ("operator") then
				if function.name.is_equal ("xcomp") or
					function.name.is_equal ("ycomp") or 
					function.name.is_equal ("zcomp") then
					component_operation := true
				else
					action.fragment_shader.append_string (function.name)
					component_operation := false
				end
				action.fragment_shader.append_string ("( ")
			else
				c := function.name.item (function.name.count)
				if (c = '-' or c = '+') and children.count = 1 then
					unary_operator := true
					action.fragment_shader.append_string (function.name.substring (function.name.count,
																																			function.name.count))
				else
					parenthesized := false
					if children.first.children.count > 0 then
						action.fragment_shader.append_string ("( ")
						parenthesized := true
					end
				end
			end
		end 

	glsl_traverse_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		do
			if function.name.has_substring ("operator") then
				if not unary_operator then
					if current_index = 1 then
						if parenthesized then
							action.fragment_shader.append_string (" )")
							parenthesized := false
						end
						action.fragment_shader.append_string (" ")
						if function.name.item (function.name.count) = '^' then
							action.fragment_shader.append_string ("*")
						else
							action.fragment_shader.append_string (function.name.substring (function.name.count, function.name.count))
						end
						action.fragment_shader.append_string (" ")

						if children.last.children.count > 0 then
							action.fragment_shader.append_string ("( ")
							parenthesized := true
						end
					end
				end
			else
				if current_index < children.count then
					action.fragment_shader.append_string (", ")
				end
			end
		end 

	glsl_post_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		do
			if not function.name.has_substring ("operator") then
				if function.name.is_equal ("faceforward") then
					action.fragment_shader.append_string (", ")
					children.first.traverse (action)
					action.fragment_shader.append_string (" )")
				elseif function.name.is_equal ("xcomp") then
					action.fragment_shader.append_string (" ).x")
				elseif function.name.is_equal ("ycomp") then
					action.fragment_shader.append_string (" ).y")
				elseif function.name.is_equal ("zcomp") then
					action.fragment_shader.append_string (" ).z")
				else
					action.fragment_shader.append_string (" )")
				end
			else
				if parenthesized then
					action.fragment_shader.append_string (" )")
					parenthesized := false
				end
			end
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {PARSE_NODE} -- Implementation

	gather_usage_information (variables: DS_ARRAYED_LIST [SL_VARIABLE]) is
			-- Traverse my children forcing updates of variable usage
		do
			from
				functions.start
			until
				functions.after
			loop
				if not functions.item_for_iteration.is_builtin then
					functions.item_for_iteration.function_arguments.gather_usage_information (variables)
					functions.item_for_iteration.function_definition.gather_usage_information (variables)
				end
				functions.forth
			end
			

			Precursor (variables)
		end 
	
feature {NONE} -- Implementation
	
	functions: DS_ARRAYED_LIST [SL_FUNCTION]
			-- The possible `functions' for this node

	function_index: INTEGER
			-- When the candidate is found, this refers to which 
			-- function in `functions'
	
	unary_operator: BOOLEAN
			-- Is the current operator a unary '+' or '-'

	component_operation: BOOLEAN
			-- Is the current operator a component selector
	
	parenthesized: BOOLEAN
			-- Has this operator function been parenthesized?
			
	is_coercible (candidate_function: SL_FUNCTION): BOOLEAN is
			-- Can `children' be coerced into `candidate_function.argument_types'
		require 
			candidate_function_not_void: candidate_function /= Void
		local
			i, arg_count: INTEGER
		do
			arg_count := candidate_function.arg_count
			if candidate_function.has_variable_argument_length then
				arg_count := arg_count - 2
			end
			
			check
				arg_count_less_than_arguments: arg_count <= children.count
			end

			Result := true
			if arg_count > 0 then
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + arg_count
				variant
					i_increasing: 1 + arg_count - i
				until
					i > arg_count
				loop
					Result := Result and
						children.item (i).has_valid_return_type (encoded_to_type (candidate_function.argument_types.item (i)),
																 false)
					
					i := i + 1
				end
			end
		end 

	score_candidates (candidates: DS_ARRAYED_LIST [SL_FUNCTION]): SL_FUNCTION is
			-- Score the `candidates' based on best match to actual 
			-- arguments. Return the best match
		require 
			candidates_not_void: candidates /= Void and then candidates.count >= 1
		local
			i, arg_count, arg_type: INTEGER
			score, max_score: INTEGER
		do
			from
				candidates.start
			until
				candidates.after
			loop
				arg_count := candidates.item_for_iteration.arg_count
				if candidates.item_for_iteration.has_variable_argument_length then
					arg_count := arg_count - 1
				end
				
				from
					i := 1
					score := 0
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + arg_count
				variant
					i_increasing: 1 + arg_count - i
				until
					i > arg_count
				loop
					arg_type := encoded_to_type (candidates.item_for_iteration.argument_types.item (i))
					score := score + type_priority (arg_type, 
											children.item (i).best_type_match (arg_type))
					i := i + 1
				end

				if score > max_score then
					max_score := score
					Result := candidates.item_for_iteration
				end
				
				candidates.forth
			end
			
		ensure 
			Result_not_void: Result /= Void
		end 

	resolve_types_in_children is
			-- Force type resolution in `children'
		require 
			function_index_set: function_index /= 0
		local
			i, arg_count, arg_type: INTEGER
			cast_node: CAST_PARSE_NODE
		do
			arg_count := function.arg_count
			if function.has_variable_argument_length then
				arg_count := arg_count - 1
			end

			debug ("parser")
				io.put_string (function.name)
				io.put_string ("%N")
			end
			
			from
				children.start
				i := 1
			until
				children.after
			loop
				if i <= arg_count then
					arg_type := encoded_to_type (function.argument_types.item (i))
				else
					arg_type := nil_type
				end

				children.item_for_iteration.resolve_type (arg_type)

				debug ("parser")
					io.put_string ("%T")
					io.put_integer (i)
					io.put_string (" - ")
					io.put_integer (arg_type)
					io.put_string (" ")
					io.put_integer (children.item_for_iteration.type)
					io.put_string ("%N")
				end
				
				if arg_type /= nil_type and then
					not mutable (arg_type, children.item_for_iteration.type) then
					-- Cast is needed
					create cast_node.make (arg_type)
					cast_node.add_last (children.item_for_iteration)
					children.replace_at (cast_node)
					cast_node.set_parent (Current)
				end

				i := i + 1
				children.forth
			end
		end 
		
	no_duplicate_functions: BOOLEAN is
			-- Ensure that all of the functions are uniquely defined
		local
			i, dup_count: INTEGER
			args: STRING
		do
			from
				functions.start
			until
				functions.after
			loop
				args := functions.item_for_iteration.argument_types
				
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + functions.count
				variant
					i_increasing: 1 + functions.count - i
				until
					i > functions.count
				loop
					if functions.item (i).argument_types.is_equal (args) and
						functions.item (i).type = functions.item_for_iteration.type then
						dup_count := dup_count + 1
					end
					i := i + 1
				end
				
				functions.forth
			end

			if dup_count = functions.count then
				Result := true
			end
			
		end 

	find_best_type_match (return_type: INTEGER): INTEGER is
			-- Resolve the types either directly or through coercion 
			-- to ensure a valid call. On error, sets `type_error' and `error_string'
		local
			candidate_functions: DS_ARRAYED_LIST [SL_FUNCTION]
			best_candidate: SL_FUNCTION
			found_return_type: BOOLEAN
			i: INTEGER
		do
			create candidate_functions.make (functions.count)
				
			-- Discard functions with the wrong number of arguments
			from
				functions.start
				until
				functions.after
			loop
				if (functions.item_for_iteration.has_variable_argument_length and
					functions.item_for_iteration.arg_count - 2 <= children.count) or else
					(not functions.item_for_iteration.has_variable_argument_length and
					 functions.item_for_iteration.arg_count = children.count) then
						
						candidate_functions.force_last (functions.item_for_iteration)
				end
				functions.forth
			end
				
			if candidate_functions.count = 0 then
				error_string.append ("Arguments to function `")
				error_string.append (functions.first.name)
				error_string.append ("' not valid.%N%T")
				error_string.append (file_name)
				error_string.append (": ")
				error_string.append (line_number.out)
				error_string.append ("%N")
			else
				from
					candidate_functions.start
				until
					candidate_functions.after
				loop
					if not is_coercible (candidate_functions.item_for_iteration) then
						candidate_functions.remove_at
					else
						candidate_functions.forth
					end
				end
				
				if candidate_functions.count = 0 then
					error_string.append ("Unable to find a match for function `")
					error_string.append (functions.first.name)
					error_string.append ("'.%N%T")
					error_string.append (file_name)
					error_string.append (": ")
					error_string.append (line_number.out)
					error_string.append ("%N")
				else
					-- Score the functions looking for the best match
					-- Note: This is a side-effect as the result 
					-- sets `function_index' so that `function' 
					-- will work in future calls
					
					if candidate_functions.count > 1 then
						-- Search for an exact match on the return type
						from
							candidate_functions.start
							found_return_type := false
						until
							candidate_functions.after or found_return_type
						loop
							if candidate_functions.item_for_iteration.type = return_type then
								found_return_type := true
							end
							candidate_functions.forth
						end
						
						if found_return_type then
							from
								candidate_functions.start
							until
								candidate_functions.after
							loop
								if candidate_functions.item_for_iteration.type /= return_type then
									candidate_functions.remove_at
								else
									candidate_functions.forth
								end
							end
						end
						
						if candidate_functions.count = 1 then
							best_candidate := candidate_functions.item (1)
						else
							best_candidate := score_candidates (candidate_functions)
						end
					else
						best_candidate := candidate_functions.item (1)
					end

					from
						functions.start
						i := 1
					until
						functions.after 
					loop
						if best_candidate = functions.item_for_iteration then
							Result := i
						else
							i := i + 1
						end
						functions.forth
					end
				end
			end
		end
	
	traverse_node (action: SL_ACTION) is
			-- Traverse this node
		do
			action.pre_action (Current)
			
			if not action.terminate_action then
				action.push (Current)
				
				-- This functions has arguments; So push them in 
				-- reverse order
				if action.reverse_order then
					from
						current_index := children.count 
					invariant
						current_index_at_least_one: current_index >= 0
						current_index_small_enough: current_index <= children.count + 1
					until
						current_index < 1
					loop
						if children.item (current_index).traversable then
							children.item (current_index).traverse (action)
							action.traversal_action (Current)
						end
						current_index := current_index - 1
					end
				else
					from
						current_index := 1 
					invariant
						current_index_at_least_one: current_index >= 0
						current_index_small_enough: current_index <= children.count + 1
					until
						current_index > children.count
					loop
						if children.item (current_index).traversable then
							children.item (current_index).traverse (action)
							action.traversal_action (Current)
						end
						current_index := current_index + 1
					end
				end
				action.pop (Current)
			end

			action.post_action (Current)
		end
	
	dot_label_string: STRING is
			-- The string to use as a label in the digraph
		do
			Result := dot_preamble
			Result.append_string (" | ")
			Result.append_string (function.name)
			Result.append_string (" } }")
			Result.append_character ('"')
		end
	
invariant

	all_functions_are_unique: no_duplicate_functions
	
end -- FUNCTION_CALL_PARSE_NODE

