

indexing
	description: "Shading language symbols"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	PARSE_NODE
	
inherit
	
	SL_TYPES
		redefine
			default_create
		end

	EWG_IDENTIFIED
		undefine
			default_create, copy, is_equal
		end
	
creation
	default_create

feature -- Initialization

	default_create is
			-- Initialize a node
		do
			can_be_duplicated := true
			create children.make
			create error_string.make (1)
			type_error := true
			traversable := true
			apply_action_first := false
			create parent.make (2)
		end

feature -- Access
	
	children: DS_BILINKED_LIST [PARSE_NODE]
			-- The children of this node
	
	parent: DS_ARRAYED_LIST [PARSE_NODE]
			-- This nodes parent(s)
	
	file_name: STRING
			-- Which file contains the information that defines this node
	
	line_number: INTEGER
			-- Which line in `file_number'
	
	first_child: PARSE_NODE is
			-- The first child
		require
			has_children: children.count > 0
		do
			Result := children.first
		ensure 
			Result_not_void: Result /= Void
			Result_is_first_child: Result = children.first
		end 

	last_child: PARSE_NODE is
			-- The last child
		require
			has_children: children.count > 0
		do
			Result := children.last
		ensure 
			Result_not_void: Result /= Void
			Result_is_last_child: Result = children.last
		end 

	type_error: BOOLEAN
			-- Does this node have a type error? Strange as it is, 
			-- this is true until `resolve_type' is called. This 
			-- prevents errors in calls in the tree that are of 
			-- indeterminent status until `resolve_type' is called.
	
feature -- Measurement
feature -- Comparison
feature -- Status report

	traversable: BOOLEAN
			-- Is this node traversable

	apply_action_first: BOOLEAN
			-- Should this node apply the action pre-traversal or 
			-- post-traversal?
	
	best_type_match (a_type: INTEGER): INTEGER is
			-- What is the best type match to `a_type'?
		require 
			a_type_in_range: a_type >= nil_type and a_type <= hex_tuple_type
			valid_type: has_valid_return_type (a_type, false)
		do
			Result := type
		ensure 
			Result_in_range: Result >= nil_type and Result <= h_point_type
		end 

	error: STRING is
			-- Return any errors in myself or my children
		do
			Result := error_string.twin

			from
				children.start
			until
				children.after
			loop
				Result.append (children.item_for_iteration.error)
				children.forth
			end
			
		ensure 
			Result_not_void: Result /= Void
		end 

	can_be_duplicated: BOOLEAN
			-- Should copies of this node be allowed?

	type: INTEGER is
			-- What is the type of this parse node?
		do
			Result := nil_type
		ensure
			Result_in_range: Result >= nil_type and Result <= hex_tuple_type
		end

	has_valid_return_type (return_type: INTEGER; use_exact_type: BOOLEAN): BOOLEAN is
			-- Does this node and children have a valid return type? 
			-- If `use_exact_match' then iff `type' = `return_type'; 
			-- if not `use_exact_match' then if `type' is coercible 
			-- to `return_type'
		require
			return_type_in_range: return_type >= nil_type and return_type <= hex_tuple_type
		do
			if use_exact_type then
				Result := type = return_type
			else
				Result := coercible (type, encoded_type (return_type))
			end
		end 

	has_valid_type: BOOLEAN is
			-- Does this node and children have a valid type?
		do
			Result := type_error
			if not type_error then
				from
					children.start
				until
					children.after or not Result
				loop
					Result := children.item_for_iteration.has_valid_type
					children.forth
				end
			end
		end 

feature -- Status setting

	disable_duplication is
		do
			can_be_duplicated := false
		end
	
	set_traversable (status: BOOLEAN) is
			-- set `traversable' to `status'
		do
			traversable := status
		ensure 
			traversable_set: traversable = status
		end -- set_traversable

	set_action_traversing (traverse_first: BOOLEAN) is
			-- set `apply_action_first' to `traverse_first'
		do
			apply_action_first := traverse_first
		ensure 
			apply_action_first_set: apply_action_first = traverse_first
		end -- set_action_traversing

feature -- Cursor movement
feature -- Element change
	
	add_first (node: PARSE_NODE) is
			-- Add `node' as the last child
		require 
			node_not_void: node /= Void
		do
			children.put_first (node)
			node.set_parent (Current)
		ensure 
			is_first: children.first = node
			parent_set: node.parent.last = Current
		end 

	add_last (node: PARSE_NODE) is
			-- Add `node' as the last child
		require 
			node_not_void: node /= Void
		do
			children.force_last (node)
			node.set_parent (Current)
		ensure 
			is_last: children.last = node
			parent_set: node.parent.last = Current
		end 
	
	copy_children (other: PARSE_NODE) is
			-- Copy the children from `other'
		require 
			other_not_void: other /= Void
		do
			from
				other.children.start
			until
				other.children.after
			loop
				add_last (other.children.item_for_iteration)
				other.children.forth
			end
		ensure 
			children_copied: children.count >= other.children.count
		end 

	set_parent (node: PARSE_NODE) is
			-- Set `parent'.first to `node'
		require 
			node_not_void: node /= Void
		do
			if parent.count = 0 then
				parent.force_first (node)
			else
				parent.replace (node, 1)
			end
		ensure 
			parent_set: parent.last = node
		end 
	
	set_file_name (new_file: STRING) is
			-- Set `file_name' to `new_file'
		require 
			new_file_not_void: new_file /= Void
			new_file_has_positive_count: new_file.count > 0
		do
			file_name := new_file.twin
		ensure 
			file_name_set: file_name.is_equal (new_file)
		end 
	
	set_line_number (new_line: INTEGER) is
			-- Set `line_number' to `new_line'
		require 
			new_line_positive: new_line > 0
		do
			line_number := new_line
		ensure 
			line_number_set: line_number = new_line
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	resolve_type (return_type: INTEGER) is
			-- Force type coercion as necessary to 
			-- ensure correct operation
		require
			return_type_in_range: return_type >= nil_type and return_type <= hex_tuple_type
		local
			cast_node: CAST_PARSE_NODE
		do
			if return_type = nil_type or else coercible (type, encoded_type (return_type)) then
				from
					children.start
					type_error := false
				until
					children.after
				loop
					children.item_for_iteration.resolve_type (type)
					type_error := type_error or children.item_for_iteration.has_valid_type
					if not type_error and then
					   not (type = nil_type or else mutable (type, children.item_for_iteration.type)) then
					   -- Create a CAST_PARSE_NODE to cast `children.item_for_iteration.type' to `type'
					   create cast_node.make (type)
					   cast_node.add_last (children.item_for_iteration)
					   children.replace_at (cast_node)
					   cast_node.set_parent (Current)
					end
					
					children.forth
				end
			else
				error_string.append ("Unable to cast a ")
				error_string.append (type_to_string (type))
				error_string.append (" to a ")
				error_string.append (type_to_string (return_type))
				error_string.append ("%N%T")
				error_string.append ("Line	")
				error_string.append (line_number.out)
				error_string.append (": ")
				error_string.append (file_name)
				error_string.append ("%N%N")

				type_error := true
			end
		end 

	traverse (action: SL_ACTION) is
			-- Traverse this node
		require 
			action_not_void: action /= Void
		do
			if traversable then
				traverse_node (action)
			end
		end
	
	stack_output (action: STACK_ACTION) is
			-- Output this node to the VM
		require 
			action_not_void: action /= Void
		do
			-- Default is a No-Op
		end 

	dag_output (action: DAG_ACTION) is
			-- Output this node to the VM
		require 
			action_not_void: action /= Void
		do
			-- Default is a No-Op
		end 

	dot_output (action: DOT_ACTION) is
			-- Output this node to in dot format
		require 
			action_not_void: action /= Void
		local
			dri: DOT_RECORD_INFO
			id: INTEGER
		do
			if action.gathering then
				if not action.has_id (ewg_object_id) then
					action.set_item (ewg_object_id, dot_label_string)
					dri := action.item (ewg_object_id)
					action.dot_file.put_string ("%T")
					action.dot_file.put_string (dri.identifier)
					action.dot_file.put_string ("[ label = ")
					action.dot_file.put_string (dri.format_string)
					action.dot_file.put_string (" ];%N")
				end
			else
				if parent.count > 0 then
					from
						parent.start
					until
						parent.after
					loop
						id := parent.item_for_iteration.ewg_object_id
						if action.has_id (id) and
							not action.has_link (ewg_object_id, id) then
							dri := action.item (id)
							
							action.dot_file.put_string ("%T")
							action.dot_file.put_string (dri.identifier)
							action.dot_file.put_string (" -> ")
							
							dri := action.item (ewg_object_id)
							action.dot_file.put_string (dri.identifier)
							action.dot_file.put_string (";%N")

							action.set_visited_link (ewg_object_id, id)
						end
						parent.forth
					end
				end
			end
		end 

	glsl_pre_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		require 
			action_not_void: action /= Void
		do
		ensure 
			default: true
		end 

	glsl_traverse_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		require 
			action_not_void: action /= Void
		do
		ensure 
			default: true
		end 

	glsl_post_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		require 
			action_not_void: action /= Void
		do
		ensure 
			default: true
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {PARSE_NODE} -- Implementation

	gather_usage_information (variables: DS_ARRAYED_LIST [SL_VARIABLE]) is
			-- Traverse my children forcing updates of variable usage
		require
			variables_not_void: variables /= Void
		do
			from
				children.start
			until
				children.after
			loop
				children.item_for_iteration.gather_usage_information (variables)
				children.forth
			end
		ensure
			variables_not_void: variables /= Void
			variables_count_monotonic: variables.count >= old variables.count
		end 

	check_children: BOOLEAN is
			-- See if the children have Current as their parent
		local
			index, pindex: INTEGER
			child: PARSE_NODE
		do
			-- Set and restore the current cursor
			index := children.index
			from
				children.start
				Result := true
			until
				children.after or not Result
			loop
				child := children.item_for_iteration
				from
					Result := false
					pindex := child.parent.index
					child.parent.start
				until
					child.parent.after or Result
				loop
					Result := child.parent.item_for_iteration = Current
 
					child.parent.forth
				end
				child.parent.go_i_th (pindex)
				
				children.forth
			end

			-- Set and restore the current cursor
			children.go_i_th (index)
		ensure
			children_has_not_moved: children.index = old children.index
		end 

feature {NONE} -- Implementation
	
	error_string: STRING

	current_index: INTEGER
			-- The index of the current iteration in the traversal loop
			
	dot_preamble: STRING is
			-- The string to use as a label in the digraph
		do
			create Result.make (40)
			Result.append_character ('"')
			Result.append_string ("{ ")
			Result.append_string (generator.out)
			Result.append_string (" | { line\n")
			Result.append_string (line_number.out)
		ensure
			Result_not_void: Result /= Void
		end
	
	dot_label_string: STRING is
			-- The string to use as a label in the digraph
		do
			Result := dot_preamble
			Result.append_string (" } }")
			Result.append_character ('"')
		ensure
			Result_not_void: Result /= Void
		end
	
	traverse_node (action: SL_ACTION) is
			-- Traverse this node
		require 
			is_traversable: traversable
			action_not_void: action /= Void
		do
			action.pre_action (Current)
			
			if not action.terminate_action then
				action.push (Current)
				
				from
					children.start
					current_index := 1
				until
					children.after or action.terminate_action
				loop
					if children.item_for_iteration.traversable then
						children.item_for_iteration.traverse (action)
						action.traversal_action (Current)
					end
					
					children.forth
					current_index := current_index + 1
				end
				
				action.pop (Current)
			end

			action.post_action (Current)
		end

invariant
	
	children_not_void: children /= Void
	children_count_positive: children.count >= 0
	error_string_not_void: error_string /= Void
	node_is_parent_of_childen: check_children
	
end -- PARSE_NODE

