

indexing
	description: "A node in the parse tree representing a shader"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SHADER_PARSE_NODE
	
inherit
	
	PARSE_NODE
		redefine
			default_create, stack_output, type
		end

creation
	make, default_create

feature -- Initialization
	
	default_create is
			-- Default Initialization
		do
			Precursor

			apply_action_first := true
			shader_type := "surface"
			type := nil_type
		end
	
	make (new_name, new_type: STRING) is
			-- Create a new shader of `new_type' with `new_name'
		require
			new_name_not_void: new_name /= Void
			new_type_not_void: new_type /= Void
		do
			default_create
			shader_name := new_name.twin
			shader_type := new_type.twin
		ensure
			name_set: shader_name.is_equal (new_name)
			type_set: shader_type.is_equal (new_type)
		end

feature -- Access
	
	shader_name: STRING
			-- The name used to call the shader
	
	shader_type: STRING
			-- The type of the shader
	
	type: INTEGER
			-- A shader has `nil_type' by default

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	stack_output (action: STACK_ACTION) is
			-- Output this node to the VM
		local
			variables: DS_ARRAYED_LIST [SL_VARIABLE]
		do
			action.shader_file.put_string (shader_type)
			action.shader_file.put_string ("%N")

			action.shader_file.put_string ("SLQ_V1.0")
			action.shader_file.put_string ("%N")
			action.shader_file.put_string ("%N")

			create variables.make (100)
			gather_usage_information (variables)

			action.shader_file.put_string ("Declaration")
			action.shader_file.put_string ("%N")
			action.shader_file.put_string ("%N")

			output_variable_declaration (variables, action)
			action.shader_file.put_string ("%N")
			action.shader_file.put_string ("%N")

			action.shader_file.put_string ("Initialization")
			action.shader_file.put_string ("%N")
			action.shader_file.put_string ("%N")

			output_variable_initializers (variables, action)
			action.shader_file.put_string ("%N")
			action.shader_file.put_string ("%N")

			action.shader_file.put_string ("Code")
			action.shader_file.put_string ("%N")
			action.shader_file.put_string ("%N")
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	output_variable_declaration (variables: DS_ARRAYED_LIST [SL_VARIABLE];
								 action: STACK_ACTION) is
			-- Output to `output_file' the declarations of `variables'
		require 
			variables_not_void: variables /= Void
			action_not_void: action /= Void
		do
			from
				variables.start
			until
				variables.after
			loop
				-- Reference count of one is a declared, but not used variable
				if variables.item_for_iteration.reference_count > 1 or variables.item_for_iteration.is_parameter or
				 (variables.item_for_iteration.reference_count > 0 and variables.item_for_iteration.is_builtin) then
					action.shader_file.put_string (variables.item_for_iteration.declaration)
					action.shader_file.put_string ("%N")
				end
				variables.forth
			end
			
		ensure 
			default: true
		end 

	output_variable_initializers (variables: DS_ARRAYED_LIST [SL_VARIABLE];
								 action: STACK_ACTION) is
			-- Output to `output_file' the initialization code of `variables'
		require 
			variables_not_void: variables /= Void
			action_not_void: action /= Void
		do
			from
				variables.start
			until
				variables.after
			loop
				-- Reference count of one is a declared, but not used variable
				if (variables.item_for_iteration.reference_count > 1 or variables.item_for_iteration.is_parameter or
					(variables.item_for_iteration.reference_count > 0 and variables.item_for_iteration.is_builtin)) and
					variables.item_for_iteration.default_value /= Void then
					variables.item_for_iteration.default_value.resolve_type (nil_type)
					variables.item_for_iteration.default_value.traverse (action)
				end
				variables.forth
			end
		ensure 
			default: true
		end 

invariant

	shader_type_not_void: shader_type /= Void
	type_is_invalid: type = nil_type
	
end -- SHADER_PARSE_NODE

