

indexing
	description: "SL Function description"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SL_FUNCTION
	
inherit
	
	SL_VARIABLE
		rename 
			make as var_make
		export
			{NONE} var_make, make_with_storage, is_array, count
		end

creation
	make, make_user_defined

feature -- Initialization

	make (new_type: INTEGER; new_name, new_internal_name, 
		  new_argument_types: STRING ) is
			-- Create a new function definition
		require
			new_type_in_range: new_type >= float_type and new_type <= h_point_type
			new_name_not_void:	new_name /= Void
			new_internal_name_not_void:	 new_internal_name /= Void
			new_argument_types_not_void: new_argument_types /= Void
		do
			type := new_type
			name := new_name
			internal_name := new_internal_name
			argument_types := new_argument_types
			is_builtin := true
			is_array := false
			count := 1
		end

	make_user_defined (new_type: INTEGER; new_name, new_internal_name, 
					   new_argument_types: STRING;	
					   new_definition, new_arguments: PARSE_NODE) is
			-- Create a new, user-defined function definition
		require
			new_type_in_range: new_type >= float_type and new_type <= h_point_type
			new_name_not_void:	new_name /= Void
			new_internal_name_not_void:	 new_internal_name /= Void
			new_argument_types_not_void: new_argument_types /= Void
			new_definition_not_void: new_definition /= Void
		do
			make (new_type, new_name, new_internal_name, new_argument_types)
			function_definition := new_definition
			function_arguments := new_arguments
			is_builtin := false
		end

feature -- Access
	
	internal_name: STRING
			-- The internal name
	
	argument_types: STRING
			-- The type and number of arguments coded as a string

	arg_count: INTEGER is
			-- The number of arguments
		do
			Result := argument_types.count
		ensure
			Result_correct: Result = argument_types.count
		end
	
	function_arguments: PARSE_NODE
			-- The arguments to a user-defined function
	
	function_definition: PARSE_NODE
			-- The definition for a user-defined function
	
feature -- Measurement
feature -- Comparison
feature -- Status report

	has_variable_argument_length: BOOLEAN is
			-- Are the arguments to this function of arbitrary 
			-- length? (in C, a vararg function)
		do
			if argument_types.count > 0 then
				Result := argument_types @ argument_types.count = '*'
			end
		end 

feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
invariant
	
	definition_set: not is_builtin implies function_definition /= Void
	argument_count_correct:	 arg_count = argument_types.count
	argument_count_consistent: not is_builtin implies arg_count = function_arguments.children.count
	
end -- SL_FUNCTION

