

indexing
	description: "SL Type description"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 
	
	SL_TYPES
	
feature -- Access

	translation_table: DS_ARRAYED_LIST [DS_ARRAYED_LIST [PAIR [SL_VARIABLE, SL_VARIABLE]]] is
		-- translate variables as function arguments to their 
			-- corresponding definitions
		once
			create Result.make (20)
		end
	
feature -- Constants
	
	uniform, vertex, varying: INTEGER is unique
			-- Possible storage classes
	
	nil_type, float_type, string_type, point_type, color_type, normal_type, vector_type,
	matrix_type, void_type, h_point_type,
		-- internal codes
	ri_integer_type, triple_type, hex_tuple_type: INTEGER is unique
		-- The codes for `type'
	
	atmosphere, displacement, lightsource, surface, attribute, option,
	rendererinfo, incident, opposite, textureinfo: INTEGER is unique
	
	greater_than_operator, greater_than_or_equal_operator,
	less_than_operator, less_than_or_equal_operator,
	equal_operator, not_equal_operator, and_operator, or_operator,
	negation_operator: INTEGER is unique

feature -- Status Report

	coercible (a_type: INTEGER; f_type: CHARACTER): BOOLEAN is
			-- Can `a_type' be converted to a `f_type'?
		require 
			a_type_in_range: a_type >= nil_type and a_type <= hex_tuple_type
		do
			inspect a_type
			when nil_type then 
				Result := f_type = 'q'
			when float_type then
				Result := f_type = 'f' or f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'm' or f_type = 'h' or f_type = 'i' or
					f_type = 't'
			when string_type then 
				Result := f_type = 's'
			when point_type then 
				Result := f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'h' or f_type = 't'
			when color_type then 
				Result := f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'h' or f_type = 't'
			when normal_type then 
				Result := f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'h' or f_type = 't'
			when vector_type then
				Result := f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'h' or f_type = 't'
			when matrix_type then 
				Result := f_type = 'm'
			when void_type then 
				Result := f_type = 'x'
			when h_point_type then
				Result := f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'h' or f_type = 't'
			when ri_integer_type then
				Result := f_type = 'f' or f_type = 'i'
			when triple_type then
				Result := f_type = 'p' or f_type = 'c' or f_type = 'n' or
					f_type = 'v' or f_type = 'h' or f_type = 't'
			when hex_tuple_type then
				Result := f_type = 'w' or f_type = 'm'
			end
		end 

	mutable (type1, type2: INTEGER): BOOLEAN is
			-- Are `type1' and `type2' mutable, i.e. can one be 
			-- coerced into the other without a cast?
		require 
			type1_in_range: type1 >= nil_type and type1 <= hex_tuple_type
			type2_in_range: type2 >= nil_type and type2 <= hex_tuple_type
		do
			Result := type1 = type2 or
				((type1 = point_type or type1 = normal_type or type1 = vector_type) and
				 (type2 = point_type or type2 = normal_type or type2 = vector_type))
		end 

	type_priority (type1, type2: INTEGER): INTEGER is
			-- The "fitness" to cast `type1' to `type2'
		require 
			type1_in_range: type1 >= nil_type and type1 <= hex_tuple_type
			type2_in_range: type2 >= nil_type and type2 <= hex_tuple_type
		do
			Result := type_score.item (type1, type2)
		ensure 
			Result_postive: Result >= 0
		end 

	type_count (type: INTEGER): INTEGER is
			-- The number of elements in `type'
		require 
			type_in_range: type >= nil_type and type <= hex_tuple_type
		do
			inspect type
			when nil_type then
				Result := 0
			when float_type, string_type, void_type, ri_integer_type then
				Result := 1
			when color_type, point_type, normal_type, vector_type,
				   triple_type, hex_tuple_type then 
				Result := 3
			when h_point_type then
				Result := 4
			when matrix_type then
				Result := 16
			end
		ensure 
			Result_in_range: Result >= 0 and Result <= 16
		end 

feature -- Conversion

	storage_class_to_string (storage_class: INTEGER): STRING is
			-- Convert storage class, `storage_class' to a string
		require 
			storage_class_in_range: storage_class >= uniform and storage_class <= varying
		do
			create Result.make (10)
			inspect storage_class
			when uniform then
				Result.append ("uniform")
			when vertex then
				Result.append ("vertex")
			when varying then
				Result.append ("varying")
			end
		ensure 
			Result_not_void: Result /= Void
		end 

	encoded_type (a_type: INTEGER): CHARACTER is
			-- `type' as a character
		require
			a_type_in_range: a_type >= nil_type and a_type <= hex_tuple_type
		do
			inspect a_type
			when nil_type then 
				Result := 'q'
			when float_type then 
				Result := 'f'
			when string_type then 
				Result := 's'
			when point_type then 
				Result := 'p'
			when color_type then 
				Result := 'c'
			when normal_type then 
				Result := 'n'
			when vector_type then
				Result := 'v'		
			when matrix_type then 
				Result := 'm'
			when void_type then 
				Result := 'x'
			when h_point_type then
				Result := 'h'
			when ri_integer_type then
				Result := 'i'
			when triple_type then
				Result := 't'
			when hex_tuple_type then
				Result := 'w'
			end
		end
	
	encoded_to_type (a_type: CHARACTER): INTEGER is
			-- `type' as a character
		do
			inspect a_type
			when  'q' then 
				Result := nil_type
			when  'f' then 
				Result := float_type
			when  's' then 
				Result := string_type
			when  'p' then 
				Result := point_type
			when  'c' then 
				Result := color_type
			when  'n' then 
				Result := normal_type
			when  'v' then
				Result :=	vector_type
			when  'm' then 
				Result := matrix_type
			when  'x' then 
				Result := void_type
			when  'h' then
				Result := h_point_type
			when  'i' then
				Result := ri_integer_type
			when  't' then
				Result := triple_type
			when  'w' then
				Result := hex_tuple_type
			end
			debug ("parser")
				io.put_string (type_to_string (Result))
				io.put_string ("%N")
			end
		end
	
	type_to_string (a_type: INTEGER): STRING is
			-- Convert a type specification to a string
		require
			a_type_in_range: a_type >= nil_type and a_type <= hex_tuple_type
		do
			create Result.make (10)
			
			inspect a_type
			when nil_type then
				Result.append ("invalid")
			when float_type then
				Result.append ("float")
			when string_type then
				Result.append ("string")
			when point_type then
				Result.append ("point")
			when color_type then
				Result.append ("color")
			when normal_type then
				Result.append ("normal")
			when vector_type then
				Result.append ("vector")
			when matrix_type then
				Result.append ("matrix")
			when void_type then
				Result.append ("void")
			when h_point_type then
				Result.append ("h_point")
			when ri_integer_type then
				Result.append ("integer")
			when triple_type then
				Result.append ("triple")
			when hex_tuple_type then
				Result.append ("hex_tuple")
			end
		ensure 
			Result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	has_translated_variable (variable: SL_VARIABLE): BOOLEAN is
			-- Is `variable' in `translation_table'?
		require 
			variable_not_void: variable /= Void
		local
			i: INTEGER
			table: DS_ARRAYED_LIST [PAIR [SL_VARIABLE, SL_VARIABLE]]
		do
			Result := false
			
			if translation_table.count > 0 then
				from
					translation_table.start
				until
					translation_table.after or Result
				loop
					if translation_table.item_for_iteration /= Void then
						from
							i := 1
							table := translation_table.item_for_iteration
						invariant
							i_at_least_one: i >= 1
							i_small_enough: i <= 1 + table.count
						variant
							i_increasing: 1 + table.count - i
						until
							i > table.count or Result
						loop
							if table.item_for_iteration.first = variable then
								Result := true
							end
							i := i + 1
						end
					end
					translation_table.forth
				end
			end
		end 

	translated_variable (variable: SL_VARIABLE): SL_VARIABLE is
			-- The translated `variable' from `translation_table'.
		require 
			variable_not_void: variable /= Void
			has_variable: has_translated_variable (variable)
		local
			i: INTEGER
			table: DS_ARRAYED_LIST [PAIR [SL_VARIABLE, SL_VARIABLE]]
			terminate: BOOLEAN
		do
			from
				translation_table.start
				terminate := false
			until
				translation_table.after or terminate
			loop
				if translation_table.item_for_iteration /= Void then
					from
						i := 1
						table := translation_table.item_for_iteration
					invariant
						i_at_least_one: i >= 1
						i_small_enough: i <= 1 + table.count
					variant
						i_increasing: 1 + table.count - i
					until
						i > table.count or terminate
					loop
						if table.item_for_iteration.first = variable then
							terminate := true
							Result := table.item_for_iteration.second
						end
						i := i + 1
					end
				end
				translation_table.forth
			end
		ensure
			Result_not_void: Result /= Void
		end 

	type_score: BOUNDED_ARRAY2 [INTEGER] is
			-- The score table
		once
			create Result.make (1, hex_tuple_type, 1, hex_tuple_type)

			-- nil
			Result.put (99, nil_type, nil_type)

			-- float
			Result.put (99, float_type, float_type)
			Result.put (98, float_type, ri_integer_type)
			Result.put ( 2, float_type, point_type)
			Result.put ( 1, float_type, color_type)
			Result.put ( 2, float_type, triple_type)
			Result.put ( 2, float_type, h_point_type)
			Result.put ( 2, float_type, normal_type)
			Result.put ( 2, float_type, vector_type)
			Result.put ( 1, float_type, matrix_type)
			
			-- integer
			Result.put (98, ri_integer_type, float_type)
			Result.put (99, ri_integer_type, ri_integer_type)
			
			-- point
			Result.put (99, point_type, point_type)
			Result.put (96, point_type, color_type)
			Result.put (98, point_type, triple_type)
			Result.put (98, point_type, h_point_type)
			Result.put (97, point_type, normal_type)
			Result.put (97, point_type, vector_type)
			
			-- string
			Result.put (99, string_type, string_type)

			-- color
			Result.put ( 1, color_type, point_type)
			Result.put (99, color_type, color_type)
			Result.put ( 2, color_type, triple_type)
			Result.put ( 1, color_type, h_point_type)
			Result.put ( 1, color_type, normal_type)
			Result.put ( 1, color_type, vector_type)
			
			-- triple
			Result.put (98, triple_type, point_type)
			Result.put (98, triple_type, color_type)
			Result.put (99, triple_type, triple_type)
			Result.put (97, triple_type, h_point_type)
			Result.put (98, triple_type, normal_type)
			Result.put (98, triple_type, vector_type)
			
			-- h_point
			Result.put (97, h_point_type, point_type)
			Result.put (96, h_point_type, color_type)
			Result.put (98, h_point_type, triple_type)
			Result.put (99, h_point_type, h_point_type)
			Result.put (98, h_point_type, normal_type)
			Result.put (98, h_point_type, vector_type)
			
			-- normal
			Result.put (98, normal_type, point_type)
			Result.put (96, normal_type, color_type)
			Result.put (98, normal_type, triple_type)
			Result.put (97, normal_type, h_point_type)
			Result.put (99, normal_type, normal_type)
			Result.put (98, normal_type, vector_type)
			
			-- vector
			Result.put (98, vector_type, point_type)
			Result.put (96, vector_type, color_type)
			Result.put (98, vector_type, triple_type)
			Result.put (97, vector_type, h_point_type)
			Result.put (98, vector_type, normal_type)
			Result.put (99, vector_type, vector_type)
			
			-- void
			Result.put (99, void_type, void_type)

			-- matrix
			Result.put (99, matrix_type, matrix_type)

			-- hex_tuple
			Result.put (98, hex_tuple_type, hex_tuple_type)
			Result.put (99, hex_tuple_type, matrix_type)

		ensure 
			Result_not_void: Result /= Void
		end 

end -- SL_TYPES

