

indexing
	description: "SL Variable description"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SL_VARIABLE
	
inherit
	
	SL_TYPES

creation
	make, make_with_storage, make_as_array

feature -- Initialization

	make (new_type: INTEGER; new_name: STRING) is
			-- Create a new variable definition
		require
			new_type_in_range: new_type >= float_type and new_type <= h_point_type
			new_name_not_void:	new_name /= Void
		do
			type := new_type
			name := new_name
			storage_class := uniform
			default_value := Void
			is_default_storage := true
			is_array := false
			is_parameter := false
			is_read_only := false
			count := 1
			reference_count := 0
			is_builtin := false
		end

	make_with_storage (new_type, new_storage_class: INTEGER; new_name: STRING) is
			-- Create a new variable definition
		require
			new_type_in_range: new_type >= float_type and new_type <= h_point_type
			new_storage_class_in_range: new_storage_class >= uniform and new_storage_class <= varying
			new_name_not_void:	new_name /= Void
		do
			make (new_type, new_name)
			storage_class := new_storage_class
			is_default_storage := false
		end

	make_as_array (new_type: INTEGER; new_name: STRING; array_length: INTEGER) is
			-- Create a new variable definition
		require
			new_type_in_range: new_type >= float_type and new_type <= h_point_type
			new_name_not_void:	new_name /= Void
			array_length_positive: array_length > 0
		do
			make (new_type, new_name)
			count := array_length
			is_array := true
		end
	
feature -- Access
	
	type: INTEGER
			-- The return type of the variable
	
	name: STRING
			-- The SL name of the variable
	
	storage_class: INTEGER
			-- The storage class of the variable
	
	default_value: PARSE_NODE
			-- The default value if any
	
	count: INTEGER
			-- The length of the variable
	
	reference_count: INTEGER
			-- How many times has this variable been referenced
	
feature -- Measurement
feature -- Comparison
feature -- Status report
   
	is_parameter: BOOLEAN
			-- Is this variable a parameter
   
	is_default_storage: BOOLEAN
			-- Is `storage_class' the default value?
   
	is_array: BOOLEAN
			-- Is the variable an array?

	is_builtin: BOOLEAN
			-- Is this a standard defined variable?

	is_read_only: BOOLEAN
			-- Is this variable read only?
   
feature -- Status setting

	set_builtin_status (status: BOOLEAN) is
			-- Set `is_builtin' to `status'
		do
			is_builtin := status
		ensure 
			is_builtin: is_builtin = status
		end 

feature -- Cursor movement
feature -- Element change
	
	set_name (new_name: STRING) is
			-- Set `name' to `new_name'
		require 
			new_name_not_void: new_name /= Void
			new_name_not_empty: new_name.count > 0
		do
			name := new_name.twin
		ensure 
			name_set: name.is_equal (new_name)
		end 
	
	set_type (new_type: INTEGER) is
			-- Set `type' to `new_type'
		require 
			valid_type: new_type >= float_type and new_type <= h_point_type
		do
			type := new_type
		ensure 
			type_set: type = new_type
		end 
	
	set_default_storage_class (new_storage_class: INTEGER) is
			-- Set `storage_class' to `new_storage_class' 
			-- ensuring `is_default_storage'
		require 
		   valid_storage_class: new_storage_class >= uniform and new_storage_class <= varying
		   default_storage: is_default_storage
		do
		   storage_class := new_storage_class
		ensure 
		   storage_class_set: storage_class = new_storage_class
		   default_storage: is_default_storage
		end 
	
	set_storage_class (new_storage_class: INTEGER) is
			-- Set `storage_class' to `new_storage_class'
		require 
			valid_storage_class: new_storage_class >= uniform and new_storage_class <= varying
		do
			storage_class := new_storage_class
			is_default_storage := false
		ensure 
		   storage_class_set: storage_class = new_storage_class
		   not_default_storage: not is_default_storage
		end 
	
	set_as_parameter is
			-- Set is_parameter to true
		do
			is_parameter := true
		end 
	
	set_read_only (val: BOOLEAN) is
			-- Set `is_read_only' to `val'
		do
			is_read_only := val
		ensure 
			is_read_only_set: is_read_only = val
		end 
	
	set_default_value (val: PARSE_NODE) is
			-- Set `default_value' to `val'
		do
			default_value := val
		ensure 
			default_value_set: default_value = val
		end 
	
	set_size (new_count: INTEGER) is
			-- Set `count' to `new_count'
		require 
			is_array: is_array
		do
			count := new_count
		ensure 
			count_set: count = new_count
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	add_reference is
			-- Indicate that this variable is being referenced
		do
			reference_count := reference_count + 1
		ensure 
			increased: reference_count = old reference_count + 1
		end 

	declaration: STRING is
			-- Generate a declaration for `Current'
		do
			create Result.make (30)
			if is_parameter then
				Result.append ("parameter ")
			end

			Result.append (storage_class_to_string (storage_class))
			Result.append (" ")

			Result.append (type_to_string (type))
			Result.append (" ")

			Result.append (name)

			if is_array then
				Result.append ("[")
				Result.append (count.out)
				Result.append ("]")
			end
		ensure 
			Result_not_void: Result /= Void
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
invariant
	
	count_positive: count >= 1
	count_implies_array: count > 1 implies is_array
	is_parameter_implies_not_is_read_only: is_parameter implies not is_read_only
	not_is_builtin_implies_not_is_read_only: not is_builtin implies not is_read_only
	is_parameter_implies_not_is_builtin: is_parameter implies not is_builtin
	
end -- SL_VARIABLE

