

indexing
	description: "A node in the parse tree representing variables"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	VARIABLE_PARSE_NODE
	
inherit
	
	PARSE_NODE
		redefine
			gather_usage_information, stack_output, type,
			dot_label_string, dag_output, glsl_pre_action
		end

creation
	default_create, make

feature -- Initialization
	
	make (new_variable: SL_VARIABLE) is
			-- Create a new node using `new_variable'
		require
			new_variable_not_void: new_variable /= Void
		do
			default_create
			variable := new_variable
		end
	
feature -- Access
	
	variable: SL_VARIABLE
			-- The `variable' this node is refering to
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	type: INTEGER is
			-- What is the type of this parse node?
		do
			Result := variable.type
		end
	
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_as_parameter is
			-- Set is_parameter to true
		do
			variable.set_as_parameter
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	stack_output (action: STACK_ACTION) is
			-- Output this node to the VM
		do
			action.shader_file.put_string ("%Tpushv ")

			if has_translated_variable (variable) then
				action.shader_file.put_string (translated_variable (variable).name)
				action.shader_file.put_string ("%N")
			else
				action.shader_file.put_string (variable.name)
				action.shader_file.put_string ("%N")				
			end				
		end 

	dag_output (action: DAG_ACTION) is
			-- Output this node to the DAG
		do
			action.set_item (Current, variable.name)
		end 

	glsl_pre_action (action: GLSL_OUTPUT) is
			-- Export the information corresponding to this node to a 
			-- GLSL shader
		do
			action.fragment_shader.append_string (translated_glsl_name (action))
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {PARSE_NODE} -- Implementation

	gather_usage_information (variables: DS_ARRAYED_LIST [SL_VARIABLE]) is
			-- Traverse my children forcing updates of variable usage
		do
			variable.add_reference
			if not variables.has (variable) then
				variables.force_last (variable)
			end

			Precursor (variables)
		end 

feature {NONE} -- Implementation

	correct_child_count: BOOLEAN is
			-- Does this node have the correct number of children
		do
			Result := true
		end
	
	dot_label_string: STRING is
			-- The string to use as a label in the digraph
		do
			Result := dot_preamble
			Result.append_string (" | ")
			Result.append_string (variable.name)
			Result.append_string (" | ")
			if variable.is_parameter then
				Result.append_string (" p ")
			elseif variable.is_builtin then
				Result.append_string (" b ")
			else 
				Result.append_string (" u ")
			end
			Result.append_string (" } }")
			Result.append_character ('"')
		end

	translated_glsl_name (action: GLSL_OUTPUT): STRING is
			-- Translate the variable name to a GLSL friendly one
		do
			if variable.name.is_equal ("N") then
				Result := action.glsl_normal_name
			elseif variable.name.is_equal ("I") then
				Result := action.glsl_eye_vector_name
			elseif variable.name.is_equal ("s") then
				Result := "gl_TexCoord[0].s"
			elseif variable.name.is_equal ("t") then
				Result := "gl_TexCoord[0].t"
			else
				Result := variable.name
			end
		ensure 
			Result_not_void: Result /= Void
		end 

	glsl_storage_class: STRING is
			-- Output the storage class for `variable'
		do
			create Result.make (20)
			if variable.storage_class = uniform then
				Result.append_string ("uniform")
			else
				Result.append_string ("attribute")
			end
		ensure 
			Result_not_void: Result /= Void
			Result_is_uniform_or_attribute: Result.is_equal ("uniform") or else
										  Result.is_equal ("attribute") 
		end 

	glsl_variable_type: STRING is
			-- Output the type for `variable'
		do
			create Result.make (20)
			inspect
				variable.type
			when point_type, normal_type, vector_type then
				Result.append_string ("vec3")
			when color_type then
				Result.append_string ("vec4")
			when float_type then
				Result.append_string ("float")
			when matrix_type then
				Result.append_string ("mat4")
			when h_point_type then
				Result.append_string ("vec4")
			else
				io.put_string ("Unconvertible type in shader: ")
				io.put_string (variable.type.out)
				io.put_string ("%N")
			end
		ensure 
			Result_not_void: Result /= Void
		end 

invariant
	correct_number_of_children: correct_child_count
	
end -- VARIABLE_PARSE_NODE

