

indexing
	description: "Class that contains information from a intersection event"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	INTERSECTION_DATA
	
inherit
	
	COMPARABLE
		redefine
			default_create
		end

creation
	default_create, make

feature -- Initialization
	
	default_create is
			-- Initialize to default values
		do
		end
	
	make (distance: DOUBLE; new_point: POINT3; new_normal: NORMAL;
		  uu, vv: DOUBLE; new_color: RGBA_COLOR; the_ray: RAY;
		  geom: SCENE_GEOMETRY) is
			-- Initialize the class with data
		require
			new_point_not_void: new_point /= Void
			new_normal_not_void: new_normal /= Void
			new_color_not_void: new_color /= Void
			geom_not_void: geom /= Void
			the_ray_not_void: the_ray /= Void
			distance_positive: distance >= 0
		do
			the_point := new_point
			the_normal := new_normal
			u := uu
			v := vv
			geometry := geom
			ray := the_ray
			the_color := new_color
			parametric_distance := distance
		ensure
			the_point_set: the_point = new_point
			the_normal_set: the_normal = new_normal
			color_set: the_color = new_color
			u_set: u = uu
			v_set: v = vv
			geometry_set: geometry = geom		
			parametric_distance_set: parametric_distance = distance
		end

feature -- Access
	
	parametric_distance: DOUBLE
			-- The parametric distance along the intersecting ray
	
	point: POINT3 is
			-- The intersection point
		do
			Result := the_point
		end
	
	normal: NORMAL is
			-- The normal at `point'
		do
			Result := the_normal
		end
	
	u, v: DOUBLE
			-- The parametric coordinates at `point'
	
	ray: RAY
			-- The ray that caused the intersection
	
	geometry: SCENE_GEOMETRY
			-- The object which generated this data
	
	color: RGBA_COLOR is
			-- Return the color at `u', `v'
		do
			Result := the_color
		end

feature -- Measurement
feature -- Comparison
	
	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
		do
			Result := parametric_distance < other.parametric_distance
		end

feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	the_point: POINT3
	
	the_normal: NORMAL
	
	the_color: RGBA_COLOR

invariant
	
	parametric_distance_positive: parametric_distance >= 0
	
end -- INTERSECTION_DATA

