
indexing
	description: "RenderMan API calls"
	patterns: "none"
	status: "unreviewed"
	Date: "$Date: 2007-04-07 10:37:11 -0400 (Sat, 07 Apr 2007) $"
	Revision: "$Revision: 287 $"


class 
	
	RI_API
	
inherit
	
	RI_CONSTANTS
		redefine
			default_create
		end	
	
	RIB_VALIDITY
		undefine
			default_create
		end	
	
	BASIC_NUMERIC
		export
			{ANY} about_equal
		undefine
			default_create
		end
	
	BASIC_STRING
		export
			{ANY} matches_word
		undefine
			default_create
		end
	
creation

	default_create, ri_begin

feature -- Initialization

	ri_begin, default_create is
			-- Initialize the RenderMan Interface
		local
			xf: SCENE_TRANSFORM
			light: SCENE_LIGHT_ELEMENT
			soe: SCENE_ORIENTATION_ELEMENT
			sse: SCENE_SIDES_ELEMENT
			ssre: SCENE_SHADING_RATE_ELEMENT
		do
			create shader_variables.make (1024)
			add_default_variables
			
			create options.make (128)
			add_options
			
			create attributes.make (128)
			add_attributes
			
			create {SCENE_ORTHOGRAPHIC_CAMERA} camera.make
			
			-- Initialize transform stacks
			create xf.make
			create transform_stack.make
			
			-- Initialize group stack
			current_group := xf
			create group_nodes.make
			group_nodes.put (xf)
			
			root_node := xf
			last_transform := xf
			
			-- Initialize attribute stack
			create current_attributes
			create soe
			create coordinate_system.make
			coordinate_system.put (soe)
			
			create soe
			create sse
			create ssre
			current_attributes.replace_element (soe)
			current_attributes.replace_element (sse)
			current_attributes.replace_element (ssre)
			create attribute_stack.make
			attribute_stack.put (current_attributes)
			
			create attribute_groups.make
			
			create transform.make
			
			create light_factory
			create active_lights.make
			create light
			active_lights.put (light)
			create inactive_lights.make
			create light
			inactive_lights.put (light)
			
			light_id := 1

			create render_output.make (1)

			create object_table.make (10)
		end
   
feature -- Access

	render_output: DS_ARRAYED_LIST [IMAGE_IO]
			-- What is the output of the renderer
	
	color_samples: INTEGER is 3
			-- The number of samples per color. Needs to be a 
			-- function if deeper color samples are allowed.
	
	root_node: SCENE_GROUP
			-- The root of the generated scene graph
	
	camera: SCENE_CAMERA
			-- The camera defined by the RI calls.
	
	options: OPTIONS_TABLE
			-- Table that contains all known options
	
	ri_a, ri_abort, ri_ambientlight, ri_amplitude, ri_az,
	ri_background, ri_beamdistribution, ri_bicubic, ri_bilinear, ri_black,
	ri_bumpy, ri_camera, ri_clamp, ri_comment, ri_coneangle,
	ri_conedeltaangle, ri_constant, ri_cs, ri_depthcue, ri_difference,
	ri_distance, ri_distantlight, ri_file, ri_flatness, ri_fog, ri_fov,
	ri_framebuffer, ri_from, ri_handler, ri_hidden, ri_identifier,
	ri_ignore, ri_inside, ri_intensity, ri_intersection, ri_ka, ri_kd,
	ri_kr, ri_ks, ri_lh, ri_lightcolor, ri_matte_tag, ri_maxdistance, ri_metal,
	ri_mindistance, ri_n, ri_name, ri_nonperiodic, ri_np, ri_object,
	ri_origin, ri_orthographic, ri_os, ri_outside, ri_p, ri_paint,
	ri_paintedplastic, ri_periodic, ri_perspective_tag, ri_plastic,
	ri_pointlight, ri_primitive, ri_print, ri_pw, ri_pz, ri_raster, ri_rgb,
	ri_rgba, ri_rgbaz, ri_rgbz, ri_rh, ri_roughness, ri_s, ri_screen,
	ri_shadinggroup, ri_shinymetal, ri_smooth, ri_specularcolor,
	ri_spotlight, ri_st, ri_structure, ri_t, ri_texturename, ri_to,
	ri_trimdeviation, ri_union, ri_world, ri_z: STRING
			-- Constants defined by the RenderMan Interface
	
feature -- Status report	
	
	has_coordinate_system (space: STRING): BOOLEAN is
			-- Is `space' a marked coordinate system?
		require 
			valid_space: space /= Void and then space.count > 0
		do
			Result := last_transform.has_coordinate_system (space)
		end

	valid_detail_range (minvisible, lowertransition,
						uppertransition, maxvisible: DOUBLE): BOOLEAN is
			-- Check it the detail range holds the proper relation
		do
			Result := minvisible >= 0 and then
				minvisible <= lowertransition and then
				lowertransition <= uppertransition and then
				uppertransition <= maxvisible
		end
	
	valid_handle (handle: INTEGER): BOOLEAN is
			-- Is `handle' valid?
		do
			Result := active_lights.item.has_handle (handle)
			if not Result then
				Result := inactive_lights.item.has_handle (handle)
			end
		end

	valid_texture_wrap (wrap: STRING): BOOLEAN is
			-- Is `wrap' a valid argument to be passed to the texture 
			-- making routines?
		require 
			wrap_valid: wrap /= Void and then wrap.count >= 1
		do
			Result := wrap.is_equal ("%"periodic%"") or 
				wrap.is_equal ("%"nonperiodic%"") or 
				wrap.is_equal ("%"clamp%"") or 
				wrap.is_equal ("%"black%"")
		end

	is_active (handle: INTEGER): BOOLEAN is
			-- Is light `handle' on?
		require 
			valid_handle: valid_handle (handle)
		do
			Result := active_lights.item.has_handle (handle)
		end

feature -- State operations
	
	ri_declare (name, declaration: STRING): STRING is
			-- Add `name' whose type is in `declaration' into the 
			-- global symbol dictionary
		require
			valid_name: name /= Void and then name.count > 0
			valid_declaration: declaration /= Void implies declaration.count > 0
		local
			sl: SL_INFO
		do
			create sl.make (name, declaration)
			shader_variables.force (sl, sl.name)
			if shader_variables.found then
				io.put_string ("%"")
				io.put_string (name)
				io.put_string ("%" redeclared from %"")
				io.put_string (shader_variables.found_item.out)
				io.put_string ("%" to %"")
				io.put_string (declaration)
				io.put_string ("%"%N")
			end
			
			Result := name.twin
		ensure
			Result_not_void: Result /= Void
			Result_not_empty: Result.count > 0
			Result_is_name: Result.is_equal (name)
		end
	
	ri_world_begin is
			-- Defines where options end, and the `world' begins. 
			-- Sets the world_to_camera matrix in SCENE_TRANSFORM
		local
			xform, scale_xform: TRANSFORM
			focal_point: POINT3
		do
			xform := accumulate_inverse_transform
			transform.identity
			
			create scale_xform
			-- Swap to right-handed coordinates
			scale_xform.scale (1, 1, -1)
			xform := scale_xform * xform
			
			camera.set_world_to_camera (xform)
			create focal_point
			focal_point := xform.apply (focal_point)
			if camera.position.is_equal (focal_point) then
				focal_point.make (0, 0, 1)
			end

			camera.set_focal_point (focal_point)
			
			create the_world.make
			
			current_group.add_child (the_world)
			group_nodes.put (the_world)
			current_group := the_world
			last_transform := the_world			
			root_node := the_world			
		end 

	ri_world_end is
			-- Terminate the world block
		do
			from
				active_lights.item.start
			until
				active_lights.item.after
			loop
				the_world.append_light (active_lights.item.item)
				active_lights.item.forth
			end
			
			from
				inactive_lights.item.start
			until
				inactive_lights.item.after
			loop
				the_world.append_light (inactive_lights.item.item)
				inactive_lights.item.forth
			end
		end
	
feature -- Camera operations
	
	ri_format (xresolution, yresolution: INTEGER; pixelaspect: DOUBLE) is
			-- Set the horizontal and vertical resolution of the image
		require 
			resolution_positive: xresolution > 0 and  yresolution > 0
			pixelaspect_positive: pixelaspect > 0
		do
			camera.display_format (xresolution, yresolution, pixelaspect)
		end
	
	ri_frame_aspect_ratio (ratio: DOUBLE) is
			-- Ratio of width to height of image
		require 
			positive: ratio > 0.0
		do
			camera.set_frame_aspect_ratio (ratio) 
		end
	
	ri_screen_window (left, right, bottom, top: DOUBLE) is
			-- Set the screen to raster mapping
		require 
			correct_horizontal_relation: left <= right
			correct_vertical_relation: bottom <= top
			horizontal_not_degenerate: not about_equal (right - left, 0.0)
			vertical_not_degenerate: not about_equal (top - bottom, 0.0)
		do
			camera.set_screen_window (left, right, bottom, top) 
		end
	
	ri_crop_window (xmin, xmax, ymin, ymax: DOUBLE) is
			-- Define a subwindow for rendering
		require 
			correct_horizontal_relation: xmin >= 0.0 and xmin <= xmax and xmax <= 1.0
			correct_vertical_relation: ymin >= 0.0 and ymin <= ymax and ymax <= 1.0
			horizontal_not_degenerate: not about_equal (xmax - xmin, 0.0)
			vertical_not_degenerate: not about_equal (ymax - ymin, 0.0)
		do
			camera.set_crop_window (xmin, xmax, ymin, ymax) 
		end
	
	ri_projection (name: STRING; parameters: PARAMETER_LIST) is
			-- Define the mapping from camera to screen coordinates
		require 
			valid_name: name /= Void and name.count > 0
			valid_projection: name.is_equal ("%"orthographic%"") or
							  name.is_equal ("%"perspective%"")
			orthographic_is_valid: (name.is_equal ("%"orthographic%"") implies
									parameters = Void or else (parameters /= Void and 
															   parameters.count = 0))
			perspective_is_valid: (name.is_equal ("%"perspective%"") implies	   
								   parameters = Void or else
								   parameters.count = 0 or else
								  (parameters.count = 1 and then
								  parameters.first.first.is_equal ("%"fov%"") and then
								  parameters.first.second.second /= Void and then
								  parameters.first.second.second.count = 1 and then
								  parameters.first.second.second.item (1) > 0.0))
		local
			c: SCENE_CAMERA
			fov: DOUBLE
		do
			if name.is_equal ("%"perspective%"") then
				fov := 90
				if parameters /= Void and then parameters.count = 1 then
					fov := parameters.first.second.second.item (1)
				end
				create {SCENE_PERSPECTIVE_CAMERA} c.make_with_fov (fov)
				c.copy_camera (camera)
				camera := c
			elseif name.is_equal ("%"orthographic%"") then
				create {SCENE_ORTHOGRAPHIC_CAMERA} c.make
				c.copy_camera (camera)
				camera := c 
			end
			
			transform.identity
			transform_was_last := true
		end

	ri_clipping (near, far: DOUBLE) is
			-- Set the clipping planes
		require 
			near_in_range: near >= epsilon and near <= infinity
			far_in_range: far >= epsilon and far <= infinity
			proper_relation: near < far
		do
			camera.set_clipping (near, far) 
		end
	
	ri_depth_of_field (fstop, focallength, focaldistance: DOUBLE) is
			-- 
		require 
			positive_distance: focaldistance >= 0.0
			positive_length: focallength >= 0.0
			positive_fstop: fstop > 0.0 and then fstop <= infinity
		do
			camera.set_depth_of_field (fstop, focallength, focaldistance) 
		end
	
	ri_shutter (open, close: DOUBLE) is
			-- Set the times for the shutter to open and close
		require 
			proper_relation: open <= close
		do
			camera.set_shutter (open, close) 
		end

feature -- Display operations
	
	ri_pixel_variance (variation: DOUBLE) is
			-- Set the upper bound on the acceptable estimated 
			-- variance of the pixel value from the true value
		require 
			variation_positive: variation >= 0
		do
		end
	
	ri_pixel_samples (xsamples, ysamples: DOUBLE) is
			-- Set the effective hider sampling rate in the 
			-- horizontal and vertical directions
		require 
			samples_greater_than_one: xsamples >= 1 and ysamples >= 1
		local
			doubles: DOUBLE_ARRAYED_LIST
		do
			doubles := options.get_doubles ("standard", "pixelsamples")
			doubles.replace (xsamples, 1)
			doubles.replace (ysamples, 2)
		end
	
	ri_pixel_filter (filterfunc: RI_FILTER_FUNCTION;
				   xwidth, ywidth: DOUBLE) is
			-- Set the parameters for the pixel filtering function
		require 
			support_positive: xwidth > 0 and ywidth > 0
		do
		end
	
	ri_exposure (the_gain, the_gamma: DOUBLE) is
			-- Controls the sensitivity and non-linearity of the 
			-- exposure process
		require 
			gamma_strictly_positive: the_gamma > 0
			gain_strictly_positive: the_gain > 0
		do
		end
	
	ri_imager (name: STRING; parameters: PARAMETER_LIST) is
			-- Select and Imager function programmed in the Shading Language
		require 
			valid_name: name /= Void and name.count > 0
		do
		end
	
	ri_quantize (type: STRING; one, min, max: INTEGER; ditheramplitude: DOUBLE) is
			-- Set the quantization parameters for colors or depth
		require 
			valid_type: matches_word (type, quantize_key_words)
			valid_order: one /= 0 implies min < max
			dither_positive: ditheramplitude >= 0
		do
			render_output.last.set_quantization (one, min, max, ditheramplitude)
		end

	ri_display (name, type, mode: STRING; parameters: PARAMETER_LIST) is
			-- Set the display parametes
		require 
			valid_name: name /= Void and name.count > 0
			valid_type: type /= Void and matches_word (type,
													   display_type_key_words)
			valid_mode: mode /= Void and matches_word (mode,
													   display_mode_key_words)
		local
			out_image: IMAGE_IO
		do
			if type.is_equal ("%"tiff%"") or type.is_equal ("%"file%"") then
				create {TIFF_IMAGE} out_image.make_open_write (name.substring (2, name.count -1))
			end

			render_output.force_last (out_image)
		end

feature -- Option operations
	
	ri_hider (type: STRING; parameters: PARAMETER_LIST) is
			-- Specify the type of `hider' used by the hidden surface algorithm
		require 
			valid_type: type /= Void and matches_word (type,
													   hider_key_words)
		do
		end
	
	ri_color_samples (n: INTEGER; n_rgb, rgb_n: DOUBLE_ARRAYED_LIST) is
			-- Set the number of color components or samples to be 
			-- used in specifing colors
		require 
			n_strictly_positive: n > 0
			correct_component_count: n_rgb.count = rgb_n.count and n_rgb.count = n
			component_count_is_multiple_of_three: rgb_n.count \\ 3 = 0
		do
		end
	
	ri_relative_detail (relativedetail: DOUBLE) is
			-- Scale the results of all level of detail calculations 
			-- by `relativedetail'
		require 
			strictly_positive: relativedetail > 0
		do
		end
	
	ri_option (name: STRING; parameters: PARAMETER_LIST) is
			-- Sets the named implementation-specific option
		require 
			valid_name: name /= Void and then name.count > 0
		local
			params: PARAMETER_LIST
			the_options: DS_ARRAYED_LIST [STRING]
			found_option: BOOLEAN
		do
			if options.has (name) then
				params := options.item (name)
				
			end
		end

feature -- Light operations
	
	ri_light_source (name: STRING; parameters: PARAMETER_LIST): INTEGER is
			-- Create a light, turn it on, and add it to the current 
			-- light source list
		require 
			valid_name: name /= Void and then name.count > 0
		local
			l: LIGHT_SOURCE
		do
			l := light_factory.new_light_source (light_id, name, 
												 parameters)
			l.set_light_to_world_transform (transform.current_transform)
			active_lights.item.add_item (l)
			Result := light_id
			light_id := light_id + 1
		ensure
			light_id_increased_by_one: light_id = old light_id + 1
		end

	ri_area_light_source (name: STRING; parameters: PARAMETER_LIST): INTEGER is
			-- Create an area light, turn it on, and add it to the current 
			-- light source list
		require 
			valid_name: name /= Void and then name.count > 0
		local
			a: AREA_LIGHT_SOURCE
		do
			create a.make (light_id, name, parameters) 
			a.set_light_to_world_transform (transform.current_transform)
			active_lights.item.add_item (a)
			Result := light_id
			light_id := light_id + 1
		ensure
			light_id_increased_by_one: light_id = old light_id + 1
		end

	ri_illuminate (handle: INTEGER; onoff: BOOLEAN) is
			-- Add or remove the light represented by `handle' to the 
			-- current active light list
		require 
			valid_handle: valid_handle (handle)
			valid_state: ((onoff implies not is_active (handle)) and
						  (not onoff implies is_active (handle)))
		local
			light: LIGHT_SOURCE
		do
			if onoff then
				light := inactive_lights.item.light (handle)
				inactive_lights.item.remove_item (light)
				active_lights.item.add_item (light)
			else
				light := active_lights.item.light (handle)
				active_lights.item.remove_item (light)
				inactive_lights.item.add_item (light)
			end
		end

feature -- Transform operations
	
	ri_transform_begin is
			-- Mark the current transformation
		local
			g: SCENE_GROUP
		do
			--Force the current transform onto the stack
			g := group
			
			transform_stack.put (current_group)
			
			group_nodes.put (g)
			coordinate_system.put (coordinate_system.item.twin)
		ensure 
			group_stack_depth_increased: group_nodes.count = 
										 old group_nodes.count + 1
		end 

	ri_transform_end is
			-- Insert a transformation that negates the accumulated 
			-- transformations between this call and the matching 
			-- `ri_transform_begin'. 
		local
			g: SCENE_GROUP
		do
			-- Force the transform onto the stack so we can pop it 
			-- off down below
			g := group
			
			from
			until
				group_nodes.item = transform_stack.item
			loop
				group_nodes.remove
			end
			
--			group_nodes.remove
			transform_stack.remove
			coordinate_system.remove
			current_group := group_nodes.item
		end 

	ri_identity is
			-- Set the current transformation to the identity
		local
			xform: TRANSFORM
			g: SCENE_GROUP
		do
			xform := world_to_object
			xform.invert
			 -- Not really necessary, but here for safety if I need 
			 -- to change the implementation later
			transform_was_last := true		
			transform.load_transform (xform)
			g := group
			
			transform.identity
			transform_was_last := true		
		end
	
	ri_transform (xform: TRANSFORM) is
			-- Set the current transform to `transform'
		local
			inv_xform: TRANSFORM
			g: SCENE_GROUP
		do
			inv_xform := world_to_object
			inv_xform.invert
			
			transform_was_last := true		
			transform.load_transform (inv_xform)
			g := group
			
			transform.load_transform (xform)
			transform_was_last := true	
		end
	
	ri_concat_transform (xform: TRANSFORM) is
			-- 
		do
			transform.concatonate_transform (xform)
			transform_was_last := true		
		end
	
	ri_perspective (fov: DOUBLE) is
			-- Concatenate a perspective divide onto the current transformation
		require 
			fov_is_positive: fov > 0
		do
			transform.perspective (fov)			
			transform_was_last := true		
		end
	
	ri_translate (dx, dy, dz: DOUBLE) is
			-- Concatenate a translation onto the current transformation
		do
			transform.translate (dx, dy, dz)		
			transform_was_last := true		
		end
	
	ri_rotate (angle, dx, dy, dz: DOUBLE) is
			-- Concatenate a rotation of `angle' degrees about the 
			-- given axis onto the current transformation
		require
			not_zero_axis: not (about_equal (dx, 0.0) and 
								about_equal (dy, 0.0) and
								about_equal (dz, 0.0))
		do
			transform.rotate (angle * pi/180.0, dx, dy, dz)
			transform_was_last := true		
		end

	ri_scale (sx, sy, sz: DOUBLE) is
			-- Concatenate a scaling onto the current transformation
		do
			transform.scale (sx, sy, sz)		
			transform_was_last := true		
		end
	
	ri_skew (angle, dx1, dy1, dz1, dx2, dy2, dz2: DOUBLE) is
			-- Concatenate a skew onto the current transformation
		require
			not_zero_axis1: not (about_equal (dx1, 0.0) and 
								 about_equal (dy1, 0.0) and
								 about_equal (dz1, 0.0))
			not_zero_axis2: not (about_equal (dx2, 0.0) and 
								 about_equal (dy2, 0.0) and
								 about_equal (dz2, 0.0))
			valid_angle: valid_skew_angle (angle, dx1, dy1, dz1, dx2, dy2, dz2)
		do
			transform.skew (angle, dx1, dy1, dz1, dx2, dy2, dz2)		
			transform_was_last := true		
		end
	
	ri_deformation (name: STRING; parameters: PARAMETER_LIST) is
			-- Set the current Deformation shader to `name'
		require 
			name_not_void: name /= Void and then name.count > 0
		do
			transform.deformation (name, parameters)		
			transform_was_last := true		
		end
	
	ri_coordinate_system (space: STRING) is
			-- Mark the current coordinate system with the name 
			-- `space' and save it
		require 
			valid_space: space /= Void and then space.count > 0
		do
			transform.coordinate_system (space)			
			transform_was_last := true		
		end
	
	ri_coord_sys_transform (space: STRING) is
			-- Replace the `current_transform' with `space'
		require 
			valid_space: space /= Void and then space.count > 0
		do
			transform.coordinate_system_transform (space)		
			transform_was_last := true		
		end

	ri_transform_points (fromspace, tospace: STRING;
					   n: INTEGER; points: ARRAY [POINT]): ARRAY [POINT] is
			-- Transform the `points' from `fromspace' to `tospace'
		require 
---			valid_fromspace: has_coordinate_system (fromspace)
---			valid_tospace: has_coordinate_system (tospace)
			valid_points: points /= Void and then points.count > 0
			consistent: n = points.count
		do
			Result := transform.transform_points(fromspace, tospace,
												 n, points)
			transform_was_last := true		
		end

feature -- Attribute operations
	
	ri_attribute (name: STRING; parameters: PARAMETER_LIST) is
			-- Set implementation specific attributes
		require 
			name_not_void: name /= Void and then name.count > 0
		local
			the_attributes: DS_ARRAYED_LIST [STRING]
		do
			if attributes.has (name) then
				the_attributes := attributes.item (name)
			end
		end

	ri_attribute_begin is
			-- Push the current set of attributes
		local
			g: SCENE_DEFAULT_STATE
		do
			-- Don't need to push the transform stack since updating 
			-- is handled directly in the Transformation operations
		
			create g.make
			g.merge (attribute_stack.item)
			group.add_child (g)
			current_group := g
			attribute_groups.put (g)
			group_nodes.put (g)
			
			current_attributes := attribute_stack.item.deep_twin
			attribute_stack.put (current_attributes)
			active_lights.put (active_lights.item.twin)
			inactive_lights.put (inactive_lights.item.twin)
		end 

	ri_attribute_end is
			-- Pop the current set of attributes
		local
			g: SCENE_GROUP
		do
			-- Force the transform onto the stack so we can pop it 
			-- off down below
			g := group
			
			from
			until
				group_nodes.item = attribute_groups.item
			loop
				group_nodes.remove
			end
			
			group_nodes.remove
			attribute_groups.remove
			current_group := group_nodes.item
			
			attribute_stack.remove
			current_attributes := attribute_stack.item
			active_lights.remove
			inactive_lights.remove
		end 

feature -- Object Instancing operations
	
	ri_object_begin (instance: INTEGER) is
			-- Begin an object definition
		local
			g: SCENE_GROUP
		do
			create g.make
			group.add_child (g)
			current_group := g
			group_nodes.put (g)

			object_table.force (g, instance)
			current_instance := instance
		end 

	ri_object_end is
			-- End an object definition
		local
			g: SCENE_GROUP
		do
			g := object_table.item (current_instance)
			
			from
			until
				group_nodes.item = g
			loop
				group_nodes.remove
			end
			
			group_nodes.remove
			current_group := group_nodes.item
			current_instance := 0
		end 

	ri_object_instance (instance: INTEGER) is
			-- Use an object definition
		local
			ca: CULL_ACTION
		do
			-- Force any transforms onto the stack
			create ca
			object_table.item (instance).traverse (ca)
			
			from
				ca.pool.start
			until
				ca.pool.after
			loop
				ca.pool.item_for_iteration.geometry.merge (current_attributes)
				--.set_world_to_object_transform (world_to_object.inverse)
				ca.pool.forth
			end
			
			group.add_child (object_table.item (instance))
		end
	
feature -- Geometry Attribute operations
	
	ri_bound (the_bound: AXIS_ALIGNED_BOX) is
			-- Set the current bounding box to `the_bound'
		do
		end

	ri_detail (the_bound: AXIS_ALIGNED_BOX) is
			-- Set the current detail box to `the_bound'
		do
		end
	
	ri_detail_range (minvisible, lowertransition,
				   uppertransition, maxvisible: DOUBLE) is
			-- Set the current detail range
		require 
			valid_parameters: valid_detail_range (minvisible, lowertransition,
												  uppertransition, maxvisible)
		do
		end

	ri_displacement (name: STRING; parameters: PARAMETER_LIST) is
			-- Set the current Displacement shader to `name'
		require 
			name_not_void: name /= Void and then name.count > 0
		do
		end
	
	ri_geometric_approximation (type: STRING; value: DOUBLE) is
			-- Set the chordal deviation
		require 
			type_not_void: type /= Void and then type.count > 0
			correct_type: matches_word (type, geometric_approximation_key_words)
			value_strictly_positive: value > 0
		do
		end
	
	ri_orientation (orientation: STRING) is
			-- Set the current orientation
		require 
			orientation_not_void: (orientation /= Void and then 
								   orientation.count > 0)
			valid_orientation: matches_word (orientation, orientation_key_words)
		local
			soe: SCENE_ORIENTATION_ELEMENT
			xform: TRANSFORM
		do
			xform := accumulate_inverse_transform
			soe ?= current_attributes.element_item ("SCENE_ORIENTATION_ELEMENT")
			
			check
				orientation_element_not_void: soe /= Void
			end
			
			if orientation.is_equal ("%"lh%"") then
				soe.set_orientation_from_handness (soe.lh, xform.determinant >= 0)
			elseif orientation.is_equal ("%"rh%"") then
				soe.set_orientation_from_handness (soe.rh, xform.determinant >= 0)
			elseif orientation.is_equal ("%"inside%"") then
				soe.set_orientation (soe.inside)
			else -- "outside"
				soe.set_orientation (soe.outside)
			end
		end
	
	ri_reverse_orientation is
			-- Flip the sense of the current orientation
		local
			soe: SCENE_ORIENTATION_ELEMENT
		do
			soe ?= current_attributes.element_item ("SCENE_ORIENTATION_ELEMENT")
			
			check
				orientation_element_not_void: soe /= Void
			end
			
			soe.flip_orientation
		end

	ri_sides (sides: INTEGER) is
			-- Set the number of `sides' for rendering
		require 
			sides_is_valid: sides = 1 or sides = 2
		local
			sse: SCENE_SIDES_ELEMENT
		do
			sse ?= current_attributes.element_item ("SCENE_SIDES_ELEMENT")
			
			check
				sides_element_not_void: sse /= Void
			end	
			
			sse.set_sides (sides)
		end

feature -- Shading Attribute operations

	ri_color (the_color: RGBA_COLOR) is
			-- Set the current color to `the_color'
		require 
			color_not_void: the_color /= Void
		local
			c: SCENE_COLOR_ELEMENT
		do
			create c.make (the_color)
			current_attributes.replace_element (c)
		end
	
	ri_opacity (the_opacity: RGB_COLOR) is
			-- Set the current opacity to color
		require 
			opacity_not_void: the_opacity /= Void
		local
			o: SCENE_OPACITY_ELEMENT
		do
			create o.make (the_opacity)
			current_attributes.replace_element (o)
		end
	
	ri_texture_coordinates (s1, t1, s2, t2, s3, t3, s4, t4: DOUBLE) is
			-- Set the current set of texture coordinates to the 
			-- values passed as arguments
		do
		end
	
	ri_surface (name: STRING; parameters: PARAMETER_LIST) is
			-- `name' is the name of the surface shader
		require 
			name_not_void: name /= Void and then name.count > 0
		local
			s: SCENE_SHADER_ELEMENT
			params: PARAMETER_LIST
			index: INTEGER
		do
			params := options.item ("searchpath")
			index := params.index_of_tag ("shader")
			parameters.force_last (params.item (index))
			-- If the shader has leading and trailing quotes, remove 
			-- them
			if name.item (1) = '%"' then
				name.remove_head (1)
				name.remove_tail (1)
			end
			create s.make (name, parameters)
			current_attributes.replace_element (s)
		end
	
	ri_atmosphere (name: STRING; parameters: PARAMETER_LIST) is
			-- Set the current atmosphere shader to `name'
		require 
			name_not_void: name /= Void and then name.count > 0
		do
		end

	ri_interior (name: STRING; parameters: PARAMETER_LIST) is
			-- Set the current interior shader to `name'
		require 
			name_not_void: name /= Void and then name.count > 0
		do
		end

	ri_exterior (name: STRING; parameters: PARAMETER_LIST) is
			-- Set the current exterior shader to `name'
		require 
			name_not_void: name /= Void and then name.count > 0
		do
		end
	
	ri_shading_rate (rate: DOUBLE) is
			-- Set the current shading rate to `rate'
		require 
			rate_valid: rate > 0 and rate < infinity
		local
			sse: SCENE_SHADING_RATE_ELEMENT
		do
			sse ?= current_attributes.element_item ("SCENE_SHADING_RATE_ELEMENT")
			
			check
				element_not_void: sse /= Void
			end	
			
			sse.set_shading_rate (rate)
		end
	
	ri_shading_interpolation (type: STRING) is
			-- Controls how values are interpolated between shading 
			-- samples (usually across a polygon)
		require 
			not_empty: type /= Void and then type.count > 0
			valid_type: matches_word (type, shading_interpolation_key_words)
		do
		end
	
	ri_matte (onoff: BOOLEAN) is
			-- Indicates whether subsequent primitives are matte objects
		do
		end

feature -- Geometry operations
	
	ri_polygon (nvertices: INTEGER; parameters: PARAMETER_LIST) is
		local
			nv: INTEGER_ARRAYED_LIST
			verticies: INTEGER_64_ARRAYED_LIST
			i: INTEGER_64
			per_vertex: INTEGER
		do
			if parameters.index_of_tag ("%"P%"") > 0 then
				per_vertex := 3
			elseif parameters.index_of_tag ("%"Pw%"") > 0 then
				per_vertex := 4
			elseif parameters.index_of_tag ("%"Pz%"") > 0 then
				per_vertex := 1
			end
			
			create nv.make (1)
			nv.force_last (nvertices // per_vertex)
			create verticies.make (nvertices // per_vertex)
			
			from
				i := 1 
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + nvertices // per_vertex
			variant
				i_increasing: 1 + nvertices // per_vertex - i
			until
				i > nvertices // per_vertex
			loop
				verticies.force_last (i)
				i := i + 1
			end
			
			ri_points_polygons (1, nv, verticies, parameters)
		end
	
	ri_points_polygons (npolys: INTEGER; nverticies: INTEGER_ARRAYED_LIST;
					  verticies: INTEGER_64_ARRAYED_LIST;
					 parameters: PARAMETER_LIST) is
			-- Create polygons with possibly shared verticies
		require
			npolys_positive: npolys >= 1
			nverticies_not_void: nverticies /= Void
			verticies_not_void: verticies /= Void
			correct_count: npolys = nverticies.count
			has_verticies: parameters.has_tag ("%"P%"") or 
						   parameters.has_tag ("%"Pw%"") or
						   parameters.has_tag ("%"Pz%"") 
		local
			sh: SCENE_SHAPE
			sp: SCENE_INDEXED_TRIANGLES
			ap: POINT3_ARRAYED_LIST
			av: NORMAL_ARRAYED_LIST
			ac: RGBA_COLOR_ARRAYED_LIST
			ine: SCENE_INDEXED_NORMAL_ELEMENT
			ice: SCENE_INDEXED_COLOR_ELEMENT
			sce: SCENE_COLOR_ELEMENT
			soe: SCENE_ORIENTATION_ELEMENT
			pc: POLYGON_COLLECTION
		do
			soe ?= current_attributes.element_item ("SCENE_ORIENTATION_ELEMENT")
			
			check
				orientation_element_not_void: soe /= Void
			end
			
			io.put_string ("orientation is same? - ")
			io.put_boolean (soe.orientation = coordinate_system.item.orientation)
			io.put_string ("%N")
			
			if parameters.index_of_tag ("%"P%"") > 0 then
				ap := extract_points (parameters, false)
			elseif parameters.index_of_tag ("%"Pw%"") > 0 then
				check
					implemented: false
				end
			elseif parameters.index_of_tag ("%"Pz%"") > 0 then
				check
					implemented: false
				end
			end
			
			-- Check for Colors
			if parameters.index_of_tag ("%"Cs%"") > 0 then
				ac := extract_colors (parameters, false)
			end
			
			-- Check for Normals
			if parameters.index_of_tag ("%"N%"") > 0 then
				av := extract_normals (parameters, false)
			end

			create pc.make (npolys, nverticies, verticies, ap, av, ac)
			pc.triangulate

			create sp		
			sp.set_coordinates (pc.coordinates)
			sp.add_indices (pc.vertex_indices_per_polygon)
			
			sp.merge (current_attributes)
			sce ?= sp.element_item ("SCENE_COLOR_ELEMENT")
			if sce /= Void then
				ice ?= sp.element_item ("SCENE_INDEXED_COLOR_ELEMENT")
				ice.make_from_color_element (sce)
				sp.remove_element (sce)
			end

			if pc.has_vertex_colors then
				create ice.make_from_coordinates_and_indices (pc.colors, pc.vertex_indices_per_polygon)
				sp.replace_element (ice)
				-- Remove the first strip which is created by the 
				-- default_create
				sp.set_color_mapping (sp.per_vertex)
			end
			
			if pc.has_vertex_normals then
				ine ?= sp.element_item ("SCENE_INDEXED_NORMAL_ELEMENT")
				if ine = Void then
					create ine
				else
					ine.remove_all_values
				end
				ine.make_from_coordinates_and_indices (pc.normals,
													  pc.vertex_indices_per_polygon)
				sp.set_normal_mapping (sp.per_vertex)
				sp.replace_element (ine)
			end
			
			sp.add_element (active_lights.item)
			sp.set_world_to_object_transform (world_to_object.inverse)
			create sh.make (sp)
			group.add_child (sh)
		end 

	ri_points_general_polygons (npolys: INTEGER; 
							 nloops, nverticies: INTEGER_ARRAYED_LIST;
							 verticies: INTEGER_64_ARRAYED_LIST;
							 parameters: PARAMETER_LIST) is
			-- Create polygons with possibly shared verticies
		require
			npolys_positive: npolys >= 1
			nloops_not_void: nloops /= Void
			nverticies_not_void: nverticies /= Void
			verticies_not_void: verticies /= Void
			loops_count_correct: npolys = nloops.count
			has_verticies: parameters.has_tag ("%"P%"") or 
						   parameters.has_tag ("%"Pw%"") or
						   parameters.has_tag ("%"Pz%"") 
		local
			single_loop_only: BOOLEAN
		do
			from
				nloops.start
				single_loop_only := true
			until
				nloops.after or not single_loop_only
			loop
				if nloops.item_for_iteration > 1 then
					single_loop_only := false
				end
				nloops.forth
			end
			
			if single_loop_only then
				ri_points_polygons (npolys, nverticies, verticies, parameters)
			else
				io.put_string ("Sorry, only PointsGeneralPolygons %
							   %with single loops are implemented.%N")
			end
		end
	
	ri_sphere (radius, zmin, zmax, theta: DOUBLE;
			  parameters: PARAMETER_LIST) is
			-- Create a sphere
		local
			sh: SCENE_SHAPE
			sphere: SCENE_SPHERE
		do
			create sphere.make_by_parameters (radius, zmin, zmax, theta)
			sphere.merge (current_attributes)
			sphere.add_element (active_lights.item.twin)
			sphere.set_world_to_object_transform (world_to_object.inverse)
			create sh.make (sphere)
			group.add_child (sh)
		end 

	ri_cone (height, radius, theta: DOUBLE;
				parameters: PARAMETER_LIST) is
			-- Create a cone
		local
			sh: SCENE_SHAPE
			cone: SCENE_CONE
		do
			create cone.make_by_parameters (radius, height, theta)
			cone.merge (current_attributes)
			cone.add_element (active_lights.item)
			cone.set_world_to_object_transform (world_to_object.inverse)
			create sh.make (cone)
			group.add_child (sh)
		end 

	ri_cylinder (radius, zmin, zmax, theta: DOUBLE;
				parameters: PARAMETER_LIST) is
			-- Create a Cylinder
		local
			sh: SCENE_SHAPE
			cylinder: SCENE_CYLINDER
		do
			create cylinder.make_by_parameters (radius, zmin, zmax, theta)
			cylinder.merge (current_attributes)
			cylinder.add_element (active_lights.item)
			cylinder.set_world_to_object_transform (world_to_object.inverse)
			create sh.make (cylinder)
			group.add_child (sh)
		end 

	ri_torus (majorradius, minorradius, phimin, phimax, thetamax: DOUBLE;
				parameters: PARAMETER_LIST) is
			-- Create a Torus
		local
			sh: SCENE_SHAPE
			torus: SCENE_TORUS
		do
			create torus.make_by_parameters (majorradius, minorradius, phimin, phimax, thetamax)
			torus.merge (current_attributes)
			torus.add_element (active_lights.item)
			torus.set_world_to_object_transform (world_to_object.inverse)
			create sh.make (torus)
			group.add_child (sh)
		end 

	ri_patch (type: STRING; parameters: PARAMETER_LIST) is
			-- Create a patch of `type'
		local
			sh: SCENE_SHAPE
			patch: SCENE_PATCH
			ap: POINT3_ARRAYED_LIST
			av: NORMAL_ARRAYED_LIST
			ac: RGBA_COLOR_ARRAYED_LIST
			sce: SCENE_COLOR_ELEMENT
			sne: SCENE_NORMAL_ELEMENT
		do
			if parameters.index_of_tag ("%"P%"") > 0 or else parameters.index_of_tag ("%"Pw%"") > 0 then
				ap := extract_points (parameters, false)
			elseif parameters.index_of_tag ("%"Pz%"") > 0 then
				check
					implemented: false
				end
			end
			
			create patch.make (ap)
			patch.merge (current_attributes)
			patch.add_element (active_lights.item)

			-- Check for Colors
			if parameters.index_of_tag ("%"Cs%"") > 0 then
				ac := extract_colors (parameters, false)
				sce ?= patch.element_item ("SCENE_COLOR_ELEMENT")
				if sce /= Void then
					sce.make_from_list (ac)
				else
					create sce.make_from_list (ac)
				end
				patch.replace_element (sce)
			end
			
			-- Check for Normals
			if parameters.index_of_tag ("%"N%"") > 0 then
				av := extract_normals (parameters, false)
				sne ?= patch.element_item ("SCENE_NORMAL_ELEMENT")
				if sne /= Void then
					sne.make_from_list (av)
				else
					create sne.make_from_list (av)
				end
				patch.replace_element (sne)
			end

			create sh.make (patch)
			group.add_child (sh)
		end
	
	ri_patch_mesh (type: STRING; nu: INTEGER; uwrap: STRING;
		nv: INTEGER; vwrap: STRING;
		parameters: PARAMETER_LIST) is
			-- Create a patch mesh
		local
			u_wrap, v_wrap: BOOLEAN
			sh: SCENE_SHAPE
			patch: SCENE_PATCH_MESH
			ap: POINT3_ARRAYED_LIST
			av: NORMAL_ARRAYED_LIST
			ac: RGBA_COLOR_ARRAYED_LIST
			sce: SCENE_COLOR_ELEMENT
			sne: SCENE_NORMAL_ELEMENT
		do
			if uwrap.is_equal ("%"nonperiodic%"") then
				u_wrap := false
			else
				u_wrap := true
			end

			if vwrap.is_equal ("%"nonperiodic%"") then
				v_wrap := false
			else
				v_wrap := true
			end
			
			if parameters.index_of_tag ("%"P%"") > 0 or else parameters.index_of_tag ("%"Pw%"") > 0 then
				ap := extract_points (parameters, false)
			elseif parameters.index_of_tag ("%"Pz%"") > 0 then
				check
					implemented: false
				end
			end
			
			create patch.make (nu, u_wrap, nv, v_wrap, ap)
			patch.merge (current_attributes)
			patch.add_element (active_lights.item)			

			-- Check for Colors
			if parameters.index_of_tag ("%"Cs%"") > 0 then
				ac := extract_colors (parameters, false)
				sce ?= patch.element_item ("SCENE_COLOR_ELEMENT")
				if sce /= Void then
					sce.make_from_list (ac)
				else
					create sce.make_from_list (ac)
				end
				patch.replace_element (sce)
			end
			
			-- Check for Normals
			if parameters.index_of_tag ("%"N%"") > 0 then
				av := extract_normals (parameters, false)
				sne ?= patch.element_item ("SCENE_NORMAL_ELEMENT")
				if sne /= Void then
					sne.make_from_list (av)
				else
					create sne.make_from_list (av)
				end
				patch.replace_element (sne)
			end

			create sh.make (patch)
			group.add_child (sh)
		end
	
	ri_nu_patch (nu, uorder: INTEGER; uknot: DOUBLE_ARRAYED_LIST; umin, umax: DOUBLE;
			   nv, vorder: INTEGER; vknot: DOUBLE_ARRAYED_LIST; vmin, vmax: DOUBLE;
			   parameters: PARAMETER_LIST) is
		local
			sh: SCENE_SHAPE
			nurbs: SCENE_NURBS
			ap: POINT4_ARRAYED_LIST
		do
			if parameters.index_of_tag ("%"P%"") > 0 or else parameters.index_of_tag ("%"Pw%"") > 0 then
				ap := extract_homogenous_points (parameters, false)
			elseif parameters.index_of_tag ("%"Pz%"") > 0 then
				check
					implemented: false
				end
			end
			
			create nurbs.make (nu, uorder, uknot, umin,umax,
							   nv, vorder, vknot, vmin, vmax,
							   ap)
			create sh.make (nurbs)
			group.add_child (sh)
			
			nurbs.clamp
			
			nurbs.u_knots.normalize
			nurbs.v_knots.normalize
			
			nurbs.merge (current_attributes)
			nurbs.add_element (active_lights.item)
			nurbs.set_world_to_object_transform (world_to_object.inverse)
		end
	
feature -- Texture operations
	
	ri_make_texture (picture_name, texture_name, swrap, twrap: STRING; 
				   filter: RI_FILTER_FUNCTION;
				   swidth, twidth: DOUBLE;
				   parameters: PARAMETER_LIST) is
			-- Convert an image in a standard picture file into a 
			-- texture file
		require 
			picture_name_not_void: (picture_name /= Void and then
									picture_name.count > 0)
			texture_name_not_void: (texture_name /= Void and then
									texture_name.count > 0)
			swrap_valid: valid_texture_wrap (swrap)
			twrap_valid: valid_texture_wrap (twrap)
			valid_filter: filter /= Void
		do
			
			debug
				io.put_string ("ri_make_texture: unimplemented%N")
			end
		end

	ri_make_bump (picture_name, texture_name, swrap, twrap: STRING; 
				filter: RI_FILTER_FUNCTION;
				swidth, twidth: DOUBLE;
				parameters: PARAMETER_LIST) is
			-- Convert a height-field image in a standard picture file into a 
			-- bump map file
		require 
			picture_name_not_void: (picture_name /= Void and then
									picture_name.count > 0)
			texture_name_not_void: (texture_name /= Void and then
									texture_name.count > 0)
			swrap_valid: valid_texture_wrap (swrap)
			twrap_valid: valid_texture_wrap (twrap)
			valid_filter: filter /= Void
		do

			debug
				io.put_string ("ri_make_bump: unimplemented%N")
			end
		end

	ri_make_lat_long_environment (picture_name, texture_name, swrap, twrap: STRING;
							  filter: RI_FILTER_FUNCTION;
							  swidth, twidth: DOUBLE;
							  parameters: PARAMETER_LIST) is
			-- Convert an image in a standard picture file
			-- representing latitude longitude map into an
			-- environment map
		require 
			picture_name_not_void: (picture_name /= Void and then
									picture_name.count > 0)
			texture_name_not_void: (texture_name /= Void and then
									texture_name.count > 0)
			swrap_valid: valid_texture_wrap (swrap)
			twrap_valid: valid_texture_wrap (twrap)
			valid_filter: filter /= Void
		do

			debug
				io.put_string ("ri_make_lat_long_environment: unimplemented%N")
			end
		end

	ri_make_cube_face_environment (px, nx, py, ny, pz, nz, texture_name: STRING;
							   fov: DOUBLE
							   filter: RI_FILTER_FUNCTION;
							   swidth, twidth: DOUBLE;
							   parameters: PARAMETER_LIST) is
			-- Convert six images in a standard picture file
			-- representing six viewing directions into an environment
			-- map
		require 
			px_not_void: (px /= Void and then px.count > 0)
			nx_not_void: (nx /= Void and then nx.count > 0)
			py_not_void: (py /= Void and then py.count > 0)
			ny_not_void: (ny /= Void and then ny.count > 0)
			pz_not_void: (pz /= Void and then pz.count > 0)
			nz_not_void: (nz /= Void and then nz.count > 0)
			texture_name_not_void: (texture_name /= Void and then
									texture_name.count > 0)
			valid_filter: filter /= Void
			fov_is_positive: fov > 0.0
		do

			debug
				io.put_string ("ri_make_cube_face_environment: unimplemented%N")
			end
		end

	ri_make_shadow (picture_name, texture_name: STRING
				   parameters: PARAMETER_LIST) is
			-- Convert a depth image into a shadow map
		require 
			picture_name_not_void: (picture_name /= Void and then
									picture_name.count > 0)
			texture_name_not_void: (texture_name /= Void and then
									texture_name.count > 0)
		do
			
			debug
				io.put_string ("ri_make_shadow: unimplemented%N")
			end
		end

feature {RIB_PARSER}-- Element change
	
feature {NONE} -- Implementation
	
	light_factory: LIGHT_SOURCE_FACTORY
			-- Factory for creating light sources
	
	light_id: INTEGER
			-- Next available Id for light handles
	
	active_lights: DS_LINKED_STACK [SCENE_LIGHT_ELEMENT]
			-- Currently active lights
	
	inactive_lights: DS_LINKED_STACK [SCENE_LIGHT_ELEMENT]
			-- Currently inactive lights
	
	the_world: SCENE_WORLD
			-- The root of the scene graph
	
	transform: SCENE_TRANSFORM
	transform_was_last: BOOLEAN
	
	group_nodes: DS_LINKED_STACK [SCENE_GROUP]
	current_group: SCENE_GROUP
	last_transform: SCENE_TRANSFORM
	transform_stack: DS_LINKED_STACK [SCENE_GROUP]
			-- This structure is used to mark `ri_transform_begin' / 
			-- `ri_transform_end' pairs
	
	attribute_stack: DS_LINKED_STACK [SCENE_STATE_ELEMENT_TABLE]
	current_attributes: SCENE_STATE_ELEMENT_TABLE
	attribute_groups: DS_LINKED_STACK [SCENE_DEFAULT_STATE]

	current_instance: INTEGER
			-- The current instance id
	
	coordinate_system: DS_LINKED_STACK [SCENE_ORIENTATION_ELEMENT]
			-- The orientation of the current coordinate system
	
	shader_variables: DS_HASH_TABLE [SL_INFO, STRING]
			-- Table that contains all of the variable declarations

	object_table: DS_HASH_TABLE [SCENE_GROUP, INTEGER]
			-- Table for storing instanced objects
	
	attributes: DS_HASH_TABLE [DS_ARRAYED_LIST [STRING], STRING]
			-- Table that contains all known attributes
	
	group: SCENE_GROUP is
			-- Interface routine to handle insertion of 
			-- transformation nodes when needed
		do
			if transform_was_last then
				current_group.add_child (transform)
				current_group := transform
				last_transform := transform
				
				transform_was_last := false
				
				create transform.make
			end
			
			Result := current_group
		end 

	add_default_variables is
			-- Add the default variables into the global symbol table
		do
			ri_a := ri_declare ("a", Void)
			ri_abort := ri_declare ("abort", Void)
			ri_ambientlight := ri_declare ("ambientlight", Void)
			ri_amplitude := ri_declare ("amplitude", Void)
			ri_az := ri_declare ("az", Void)
			ri_background := ri_declare ("background", Void)
			ri_beamdistribution := ri_declare ("beamdistribution", Void)
			ri_bicubic := ri_declare ("bicubic", Void)
			ri_bilinear := ri_declare ("bilinear", Void)
			ri_black := ri_declare ("black", Void)
			ri_bumpy := ri_declare ("bumpy", Void)
			ri_camera := ri_declare ("camera", Void)
			ri_clamp := ri_declare ("clamp", Void)
			ri_comment := ri_declare ("comment", Void)
			ri_coneangle := ri_declare ("coneangle", "float")
			ri_conedeltaangle := ri_declare ("conedeltaangle", "float")
			ri_constant := ri_declare ("constant", Void)
			ri_cs := ri_declare ("Cs", "vertex color")
			ri_depthcue := ri_declare ("depthcue", Void)
			ri_difference := ri_declare ("difference", Void)
			ri_distance := ri_declare ("distance", Void)
			ri_distantlight := ri_declare ("distantlight", Void)
			ri_file := ri_declare ("file", Void)
			ri_flatness := ri_declare ("flatness", Void)
			ri_fog := ri_declare ("fog", Void)
			ri_fov := ri_declare ("fov", "float")
			ri_framebuffer := ri_declare ("framebuffer", Void)
			ri_from := ri_declare ("from", "point")
			ri_handler := ri_declare ("handler", Void)
			ri_hidden := ri_declare ("hidden", Void)
			ri_identifier := ri_declare ("identifier", "string")
			ri_ignore := ri_declare ("ignore", Void)
			ri_inside := ri_declare ("inside", Void)
			ri_intensity := ri_declare ("intensity", "float")
			ri_intersection := ri_declare ("intersection", Void)
			ri_ka := ri_declare ("Ka", "float")
			ri_kd := ri_declare ("Kd", "float")
			ri_kr := ri_declare ("Kr", "float")
			ri_ks := ri_declare ("Ks", "float")
			ri_lh := ri_declare ("lh", Void)
			ri_lightcolor := ri_declare ("lightcolor", "color")
			ri_matte_tag := ri_declare ("matte", Void)
			ri_maxdistance := ri_declare ("maxdistance", "float")
			ri_metal := ri_declare ("metal", Void)
			ri_mindistance := ri_declare ("mindistance", "float")
			ri_n := ri_declare ("N", "vertex normal")
			ri_name := ri_declare ("name", "string")
			ri_nonperiodic := ri_declare ("nonperiodic", Void)
			ri_np := ri_declare ("Np", Void)
			ri_object := ri_declare ("object", Void)
			ri_origin := ri_declare ("origin", "point")
			ri_orthographic := ri_declare ("orthographic", Void)
			ri_os := ri_declare ("Os", "float")
			ri_outside := ri_declare ("outside", Void)
			ri_p := ri_declare ("P", "vertex point")
			ri_paint := ri_declare ("paint", Void)
			ri_paintedplastic := ri_declare ("paintedplastic", Void)
			ri_periodic := ri_declare ("periodic", Void)
			ri_perspective_tag := ri_declare ("perspective", Void)
			ri_plastic := ri_declare ("plastic", Void)
			ri_pointlight := ri_declare ("pointlight", Void)
			ri_primitive := ri_declare ("primitive", Void)
			ri_print := ri_declare ("print", Void)
			ri_pw := ri_declare ("Pw", "vertex hpoint")
			ri_pz := ri_declare ("Pz", "vertex float")
			ri_raster := ri_declare ("raster", Void)
			ri_rgb := ri_declare ("rgb", Void)
			ri_rgba := ri_declare ("rgba", Void)
			ri_rgbaz := ri_declare ("rgbaz", Void)
			ri_rgbz := ri_declare ("rgbz", Void)
			ri_rh := ri_declare ("rh", Void)
			ri_roughness := ri_declare ("roughness", "float")
			ri_s := ri_declare ("s", "vertex float")
			ri_screen := ri_declare ("screen", Void)
			ri_shadinggroup := ri_declare ("shadinggroup", Void)
			ri_shinymetal := ri_declare ("shinymetal", Void)
			ri_smooth := ri_declare ("smooth", Void)
			ri_specularcolor := ri_declare ("specularcolor", "color")
			ri_spotlight := ri_declare ("spotlight", Void)
			ri_st := ri_declare ("st", "vertex float[2]")
			ri_structure := ri_declare ("structure", Void)
			ri_t := ri_declare ("t", "vertex float")
			ri_texturename := ri_declare ("texturename", "string")
			ri_to := ri_declare ("to", "point")
			ri_trimdeviation := ri_declare ("trimdeviation", "float")
			ri_union := ri_declare ("union", Void)
			ri_world := ri_declare ("world", Void)
			ri_z := ri_declare ("z", Void)
		end
	
	add_attributes is
			-- Generate a table of all known attributes
		local
			strings: DS_ARRAYED_LIST [STRING]
		do
			create strings.make (1)
			strings.force_last ("%"samples%"")
			attributes.force (strings, "%"light%"")
		
			create strings.make (1)
			strings.force_last ("%"name%"")
			attributes.force (strings, "%"identifier%"")
		end 

	add_options is
			-- Generate a table of all known options
		local
			strings: DS_ARRAYED_LIST [STRING]
			values: DOUBLE_ARRAYED_LIST
			path, root_path: STRING
			parameters: PARAMETER_LIST
			env: KL_EXECUTION_ENVIRONMENT
		do
			create parameters.make

			-- Start statistics options
			create strings.make (1)
			strings.force_last ("false")
			parameters.put_string ("end of frame", strings)
			
			create strings.make (1)
			strings.force_last ("")
			parameters.put_string  ("filename", strings)
			
			create strings.make (1)
			strings.force_last ("false")
			parameters.put_string  ("verboseness", strings)
			
			options.force (parameters, "statistics")
			-- Start statistics options
			
			-- Start searchpath options
			create parameters.make
			create strings.make (1)
			create path.make (100)
			path.append (".:")
			create env
			root_path := env.variable_value ("PAPILLON_HOME")
			if root_path /= Void then
				path.append_string (root_path)
				path.append_string ("/src/internal/eiffel/test/sl_compiler/shaders")
			end
			strings.force_last (path)
			parameters.put_string  ("shader", strings)
			
			options.force (parameters, "searchpath")
			-- End searchpath options

			-- Start limits options
			create parameters.make
			create values.make (2)
			values.force_last (16)
			values.force_last (16)
			parameters.put_array  ("bucketsize", values)
			
			create values.make (1)
			values.force_last (256)
			parameters.put_array  ("gridsize", values)

			options.force (parameters, "limits")			
			-- End limits options
			
			-- Start standard options
			create parameters.make
			create values.make (2)
			values.force_last (2)
			values.force_last (2)
			parameters.put_array  ("pixelsamples", values)
			
			options.force (parameters, "standard")			
			-- End standard options
		end 

	world_to_object: TRANSFORM is
			-- Create a TRANSFORM for world to object transform
			--
			-- This routine will force any hanging transformations 
			-- into the scene graph
		local
			done: BOOLEAN
			t: SCENE_TRANSFORM
			g: SCENE_GROUP
			w: SCENE_WORLD
		do
			from
				create Result
				g := group
				t ?= g
				w ?= g
			until
				done
			loop
				if t /= Void then
					Result := t.current_transform * Result
				end
				
				if w /= Void then
					done := true
				end
				
				if g.parent_count > 0 then
					g := g.parent (1)
					t ?= g
					w ?= g
					if g = Void then
						done := true
					end
				else
					done := true
				end
			end
		end 
	
	accumulate_inverse_transform: TRANSFORM is
			-- Create a TRANSFORM that negates all the 
			-- transformations between this node and the last 
			-- non-accumulating transform.
			--
			-- This routine will force any hanging transformations 
			-- into the scene graph
		local
			done: BOOLEAN
			t: SCENE_TRANSFORM
			g: SCENE_GROUP
		do
			from
				create Result
				g := group
				t ?= g
			until
				done
			loop
				if t /= Void then
					Result := t.current_transform * Result
					if not t.accumulate then
						done := true
					end
				end
				
				if g.parent_count > 0 then
					g := g.parent (1)
					t ?= g
					if g = Void then
						done := true
					end
				else
					done := true
				end
			end
		end 
	
	extract_points (parameters: PARAMETER_LIST;
					swap_order: BOOLEAN): POINT3_ARRAYED_LIST is
			-- Extract the point data from a parameter list
		require 
			parameters_not_void: parameters /= Void
			parameters_has_point_data: parameters.has_tag ("%"P%"") or 
									   parameters.has_tag ("%"Pw%"") or
									   parameters.has_tag ("%"Pz%"")
		local
			pt: POINT3
			stride: INTEGER
			al: DOUBLE_ARRAYED_LIST
			i, j, index: INTEGER	
			divisor: DOUBLE
			homogeneous_points: BOOLEAN
		do
			if parameters.has_tag ("%"P%"") then
				index := parameters.index_of_tag ("%"P%"")
				homogeneous_points := false
				stride := 3
			elseif parameters.has_tag ("%"Pw%"") then
				index := parameters.index_of_tag ("%"Pw%"")
				homogeneous_points := true
				stride := 4
			else
			end
			
			if index > 0 then
				al := parameters.item (index).second.second
				
				check 
					vertex_count_is_multiple_of_stride: al.count \\ stride = 0
				end
				
				create Result.make (al.count // stride)
				
				from
					i := 1
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + al.count // stride
				variant
					i_increasing: 1 + al.count // stride - i
				until
					i > al.count // stride
				loop
					j := stride * (i - 1)
					if homogeneous_points then
						divisor := al.item (j + 4)
						check
							divisor_not_zero: divisor /= 0
						end
						
						create pt.make (al.item (j + 1) / divisor,
										al.item (j + 2) / divisor,
										al.item (j + 3) / divisor)
					else
						create pt.make (al.item (j + 1),
										al.item (j + 2),
										al.item (j + 3))
					end
					
					if swap_order then
						Result.put (pt, Result.count - i + 1)
					else
						Result.put (pt, i)
					end
					i := i + 1
				end
			end
		ensure 
			Result_not_void: Result /= Void
		end 
	
	extract_homogenous_points (parameters: PARAMETER_LIST;
							   swap_order: BOOLEAN): POINT4_ARRAYED_LIST is
			-- Extract the point data from a parameter list
		require 
			parameters_not_void: parameters /= Void
			parameters_has_point_data: parameters.has_tag ("%"P%"") or 
									   parameters.has_tag ("%"Pw%"") or
									   parameters.has_tag ("%"Pz%"")
		local
			pt: POINT4
			stride: INTEGER
			al: DOUBLE_ARRAYED_LIST
			i, j, index: INTEGER	
			homogeneous_points: BOOLEAN
		do
			if parameters.has_tag ("%"Pw%"") then
				index := parameters.index_of_tag ("%"Pw%"")
				al := parameters.item (index).second.second
				create Result.make_shared (al)
			elseif parameters.has_tag ("%"P%"") then
				index := parameters.index_of_tag ("%"P%"")
				homogeneous_points := false
				stride := 3
				
				if index > 0 then
					al := parameters.item (index).second.second
					
						check 
							vertex_count_is_multiple_of_stride: al.count \\ stride = 0
						end
					
					create Result.make (al.count // stride)
					
					from
						i := 1
					invariant
						i_at_least_one: i >= 1
						i_small_enough: i <= 1 + al.count // stride
					variant
						i_increasing: 1 + al.count // stride - i
					until
						i > al.count // stride
					loop
						j := stride * (i - 1)
						if homogeneous_points then
							create pt.make (al.item (j + 1),
							al.item (j + 2),
							al.item (j + 3),
							al.item (j + 4))
						else
							create pt.make (al.item (j + 1),
							al.item (j + 2),
							al.item (j + 3), 
							1)
						end
						if swap_order then
							Result.put (pt, Result.count - i + 1)
						else
							Result.put (pt, i)
						end
						i := i + 1
					end
				end
			end
		ensure 
			Result_not_void: Result /= Void
		end 
	
	extract_normals (parameters: PARAMETER_LIST;
					 swap_order: BOOLEAN): NORMAL_ARRAYED_LIST is
			-- Extract the point data from a parameter list
		require 
			parameters_not_void: parameters /= Void
			parameters_has_vector_data: parameters.has_tag ("%"N%"")
		local
			vec: NORMAL
			al: DOUBLE_ARRAYED_LIST
			i, j, index: INTEGER			
		do
			index := parameters.index_of_tag ("%"N%"")
			if index > 0 then
				al := parameters.item (index).second.second
				
				check 
					vertex_count_is_multiple_of_three: al.count \\ 3 = 0
				end
				
				create Result.make (al.count // 3)
				
				from
					i := 1
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + al.count // 3
				variant
					i_increasing: 1 + al.count // 3 - i
				until
					i > al.count // 3
				loop
					j := 3 * (i - 1)
					create vec.make (al.item (j + 1),
									al.item (j + 2),
									al.item (j + 3))
					
					if swap_order then
						Result.put (vec, Result.count - i + 1)
					else
						Result.put (vec, i)
					end
					
					i := i + 1
				end
			end
		ensure 
			Result_not_void: Result /= Void
		end 
	
	extract_colors (parameters: PARAMETER_LIST;
					swap_order: BOOLEAN): RGBA_COLOR_ARRAYED_LIST is
			-- Extract the color data from a parameter list
		require 
			parameters_not_void: parameters /= Void
			parameters_has_point_data: parameters.has_tag ("%"Cs%"")
		local
			color: RGBA_COLOR
			al: DOUBLE_ARRAYED_LIST
			i, j, index: INTEGER			
		do
			index := parameters.index_of_tag ("%"Cs%"")
			if index > 0 then
				al := parameters.item (index).second.second
				
				check 
					vertex_count_is_multiple_of_three: al.count \\ 3 = 0
				end
				
				create Result.make (al.count // 3)
				
				from
					i := 1
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + al.count // 3
				variant
					i_increasing: 1 + al.count // 3 - i
				until
					i > al.count // 3
				loop
					j := 3 * (i - 1)
					create color.make (al.item (j + 1),
									   al.item (j + 2),
									   al.item (j + 3),
									   1)
					if swap_order then
						Result.put (color, Result.count - i + 1)
					else
						Result.put (color, i)
					end
					
					i := i + 1
				end
			end
		ensure 
			Result_not_void: Result /= Void
		end 
	
invariant
	group_nodes_not_void: group_nodes /= Void
	group_stack_depth_positive: group_nodes.count >= 1
	shader_variables_not_void: shader_variables /= Void
	shader_variables_has_at_least_one_item: shader_variables.count >= 1
	transform_stack_not_void: transform_stack /= Void
	current_attributes_not_void: current_attributes /= Void
	attribute_groups_not_void: attribute_groups /= Void
	attribute_stack_not_void: attribute_stack /= Void
	light_id_is_positive: light_id >= 1
	active_lights_not_void: active_lights /= Void
	inactive_lights_not_void: inactive_lights /= Void
	options_not_void: options /= Void
	attributes_not_void: attributes /= Void
	render_output_not_void: render_output /= Void

end -- class RI_API
