

indexing
	description: "Provides a single class that defines all errors for the RenderMan API"
	patterns: "none"
	date: "$Date: 2007-02-07 15:02:18 -0500 (Wed, 07 Feb 2007) $"
	revision: "$Revision: 281 $"
	status: "unreviewed"




class 

	RIB_VALIDITY
	
inherit
	
	RI_CONSTANTS
	
	BASIC_NUMERIC
	
	BASIC_STRING

feature -- Access
	
	error_code: INTEGER
			-- The error code generated from a `is_rib_routine_valid' call.
	
	filter_key_words: STRING is " %"box%" %"catmull-rom%" %"gaussian%" %"sinc%" %"triangle%" "
	quantize_key_words: STRING is " %"rgba%" %"z%" "
	display_type_key_words: STRING is " %"framebuffer%" %"file%" %"tiff%" %"exr%" "
	display_mode_key_words: STRING is " %"a%" %"z%" %"az%" %"rgb%" %"rgba%" %"rgbaz%" "
	hider_key_words: STRING is " %"hidden%" %"paint%" %"null%" "
	geometric_approximation_key_words: STRING is " %"flatness%" "
	orientation_key_words: STRING is " %"outside%" %"inside%" %"rh%" %"lh%" "
	shading_interpolation_key_words: STRING is " %"constant%" %"smooth%" "
	patch_type_key_words: STRING is " %"bicubic%" %"bilinear%" "
	
	angle_range_error, axis_1_zero_length_error,
	axis_2_zero_length_error, bilinear_patch_count_error,
	bicubic_patch_count_error, clipping_relation_error,
	color_samples_array_count_error, color_samples_array_mismatch_error,
	color_samples_count_error, color_samples_multiple_of_three_error,
	detail_range_relation_error, display_mode_error,
	display_type_error, f_stop_positive_error,
	far_out_of_range_error, filter_xwidth_error,
	filter_ywidth_error, focal_distance_positive_error,
	focal_length_positive_error, gain_range_error,
	gamma_range_error, geometric_approximation_type_error,
	geometric_approximation_value_error, hider_type_error,
	horizontal_bounds_error, horizontal_degenerate_error,
	horizontal_relation_error, invalid_display_mode_error,
	invalid_display_name_error, invalid_display_type_error,
	invalid_filter_name_error, invalid_fov_parameter_count_error,
	invalid_fov_parameter_error, invalid_name_error,
	invalid_parameter_count_error, invalid_projection_type_error,
	missing_position_information_error, near_out_of_range_error, orientation_type_error,
	orthographic_projection_error, patch_type_error,
	patch_mesh_nu_count_error, patch_mesh_nv_count_error, patch_mesh_count_error,
	perspective_parameter_error,
	perspective_parameter_type_error, pixel_variance_error,
	pixel_xsamples_error, pixel_ysamples_error,
	pixelaspect_error, quantize_ditheramplitude_error,
	quantize_parameter_error, quantize_type_error,
	ratio_error, relative_detail_error,
	shading_interpolation_type_error, shading_rate_error,
	shutter_open_relation_error, sides_error,
	vertical_bounds_error, vertical_degenerate_error,
	vertical_relation_error, xresolution_error,
	yresolution_error, zero_rotation_axis_error: INTEGER is unique
			-- Error codes
	
	valid_skew_angle (angle, dx1, dy1, dz1, dx2, dy2, dz2: DOUBLE): BOOLEAN is
			-- Check if `angle' is less than the angle defined by the 
			-- two axis, or, if `angle' is negative, greater than 180 
			-- - the angle between the axes.
		local
			v1, v2: VECTOR3
			v_angle: DOUBLE
		do
			v1.make (dx1, dy1, dz1)
			v2.make (dx2, dy2, dz2)
			v_angle := arc_cosine (v1.dot (v2))
			
			Result := angle < v_angle and angle > v_angle - pi
		end
	
feature -- Camera checks
	
	is_frame_aspect_ratio_valid (ratio: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `frame_aspect_ratio' are valid
		do
			error_code := 0

			if ratio <= 0.0 then
				error_code := ratio_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_frame_aspect_ratio_error_message (ratio: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_frame_aspect_ratio_valid'
		do
			create Result.make (10)

			inspect error_code
			when ratio_error then
				Result.append ("===> Invalid ratio <===%N")
				Result.append ("`ratio' must be strictly positive. Found ")
				Result.append_string (ratio.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_screen_window_valid (left, right, bottom, top: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `screen_window' are valid
		do
			error_code := 0

			if right < left then
				error_code := horizontal_relation_error
			elseif bottom > top then
				error_code := vertical_relation_error
			elseif about_equal (right - left, 0.0) then
				error_code := horizontal_degenerate_error
			elseif about_equal (top - bottom, 0.0) then
				error_code := vertical_degenerate_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_screen_window_error_message (left, right, bottom, top: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_screen_window_valid'
		do
			create Result.make (10)

			inspect error_code
			when horizontal_relation_error then
				Result.append ("===> Incorrect horizontal relation <===%N")
				Result.append ("`left' is greater than `right'. The call is ignored.%N%N")
			when vertical_relation_error then
				Result.append ("===> Incorrect vertical relation <===%N")
				Result.append ("`bottom' is greater than `top'. The call is ignored.%N%N")
			when horizontal_degenerate_error then
				Result.append ("===> Horizontal parameters are degenerate <===%N")
				Result.append ("A window of zero-width would be %
								%generated which is not allowed. ")
				Result.append ("")
				Result.append ("The call is ignored.%N%N")
			when vertical_degenerate_error then
				Result.append ("===> Vertical parameters are degenerate <===%N")
				Result.append ("A window of zero-height would be %
								%generated which is not allowed. ")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_crop_window_valid (xmin, xmax, ymin, ymax: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `crop_window' are valid
		do
			error_code := 0

			if xmin < 0.0 or xmax > 1.0 then
				error_code := horizontal_bounds_error
			elseif ymin < 0.0 or ymax > 1.0 then
				error_code := vertical_bounds_error
			elseif xmax < xmin then
				error_code := horizontal_relation_error
			elseif ymin > ymax then
				error_code := vertical_relation_error
			elseif about_equal (xmax - xmin, 0.0) then
				error_code := horizontal_degenerate_error
			elseif about_equal (ymax - ymin, 0.0) then
				error_code := vertical_degenerate_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_crop_window_error_message (xmin, xmax, ymin, ymax: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_crop_window_valid'
		do
			create Result.make (10)

			inspect error_code
			when horizontal_bounds_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`xmin', `xmax'are not in the range [0, 1]. The call is ignored.%N%N")
			when vertical_bounds_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`xmin', `xmax'are not in the range [0, 1]. The call is ignored.%N%N")
			when horizontal_relation_error then
				Result.append ("===> Incorrect horizontal relation <===%N")
				Result.append ("`xmin' is greater than `xmax'. The call is ignored.%N%N")
			when vertical_relation_error then
				Result.append ("===> Incorrect vertical relation <===%N")
				Result.append ("`ymin' is greater than `ymax'. The call is ignored.%N%N")
			when horizontal_degenerate_error then
				Result.append ("===> Horizontal parameters are degenerate <===%N")
				Result.append ("A window of zero-width would be %
								%generated which is not allowed. ")
				Result.append ("")
				Result.append ("The call is ignored.%N%N")
			when vertical_degenerate_error then
				Result.append ("===> Vertical parameters are degenerate <===%N")
				Result.append ("A window of zero-height would be %
								%generated which is not allowed. ")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_projection_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `projection' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			elseif not (name.is_equal ("%"orthographic%"") or
						name.is_equal ("%"perspective%"")) then
				error_code := invalid_projection_type_error
			elseif name.is_equal ("%"orthographic%"") and then
				parameters /= Void and parameters.count /= 0 then
				error_code := orthographic_projection_error
			elseif name.is_equal ("%"perspective%"") then
				if parameters.count > 1 then
					error_code := invalid_parameter_count_error
				elseif parameters.count = 1 then
					if not parameters.first.first.is_equal ("%"fov%"") then
						error_code := perspective_parameter_type_error
					elseif parameters.first.second.second = Void or else
						parameters.first.second.second.count /= 1 then
						error_code := invalid_fov_parameter_count_error
					elseif parameters.first.second.second.item (1) <= 0.0 then
						error_code := invalid_fov_parameter_error
					end
				end
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_projection_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_projection_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when invalid_projection_type_error then
				Result.append ("===> Invalid Projection Type <===%N")
				Result.append ("Only %"orthographic%" or%
								% %"perspective%" is supported. Found ")
				Result.append (name)
				Result.append (" instead. The call is ignored.%N%N")
			when orthographic_projection_error then
				Result.append ("===> Invalid Parameter List <===%N")
				Result.append ("%"orthographic%" takes no parameters.")
				Result.append ("The call is ignored.%N%N")
			when invalid_parameter_count_error then
				Result.append ("===> Invalid Parameter List <===%N")
				Result.append ("%"perspective%" takes only one %
								%parameter, %"fov%". Found ")
				if parameters = Void then
					Result.append ("no")
				else
					Result.append_string (parameters.count.out)
				end
				Result.append (" parameters. The call is ignored.%N%N")
			when perspective_parameter_type_error then
				Result.append ("===> Invalid Parameter List <===%N")
				Result.append ("%"perspective%" takes only one %
								%parameter, %"fov%". Found ")
				Result.append (parameters.first.first)
				Result.append ("instead. The call is ignored.%N%N")
			when invalid_fov_parameter_count_error then
				Result.append ("===> Invalid Parameter List <===%N")
				Result.append ("%"fov%" requires a numeric parameter.")
				Result.append ("The call is ignored.%N%N")
			when invalid_fov_parameter_error then
				Result.append ("===> Invalid Parameter List <===%N")
				Result.append ("%"fov%" must be strictly positive. Found ")
				Result.append_string (parameters.first.second.second.item (1).out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_clipping_valid (near, far: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `clipping' are valid
		do
			error_code := 0

			if near < epsilon then
				error_code := near_out_of_range_error
			elseif far > infinity then
				error_code := far_out_of_range_error
			elseif far <= near then
				error_code := clipping_relation_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_clipping_error_message (near, far: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_clipping_valid'
		do
			create Result.make (10)

			inspect error_code
			when near_out_of_range_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`near' must be greater than `epsilon'. Found ")
				Result.append_string (near.out)
				Result.append (" instead. The call is ignored.%N%N")
			when far_out_of_range_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`far' must be less than `infinity'. Found ")
				Result.append_string (far.out)
				Result.append (" instead. The call is ignored.%N%N")
			when clipping_relation_error then
				Result.append ("===> Invalid Relation <===%N")
				Result.append ("`far' must be greater than `near'.")
				Result.append (" The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_depth_of_field_valid (fstop, focallength, focaldistance: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `depth_of_field' are valid
		do
			error_code := 0

			if fstop <= 0 or else fstop > infinity then
				error_code := f_stop_positive_error
			elseif focallength < 0 then
				error_code := focal_length_positive_error
			elseif focaldistance < 0 then
				error_code := focal_distance_positive_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_depth_of_field_error_message (fstop, focallength, focaldistance: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_depth_of_field_valid'
		do
			create Result.make (10)

			inspect error_code
			when f_stop_positive_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`fstop' must be strictly positive and less %
							   %than `infinity'. Found ")
				Result.append_string (fstop.out)
				Result.append (" instead. The call is ignored.%N%N")
			when focal_length_positive_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`focallength' must be strictly positive. Found ")
				Result.append_string (focallength.out)
				Result.append (" instead. The call is ignored.%N%N")
			when focal_distance_positive_error then
				Result.append ("===> Out of bounds <===%N")
				Result.append ("`focaldistance' must be strictly positive. Found ")
				Result.append_string (focaldistance.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_shutter_valid (open, close: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `shutter' are valid
		do
			error_code := 0

			if close > open then
				error_code := shutter_open_relation_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_shutter_error_message (open, close: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_shutter_valid'
		do
			create Result.make (10)

			inspect error_code
			when shutter_open_relation_error then
				Result.append ("===> Invalid Relation <===%N")
				Result.append ("`open' must be less than or equal to `close'. Found ")
				Result.append_string (open.out)
				Result.append (" and ")
				Result.append_string (close.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Display checks
	
	is_pixel_variance_valid (variation: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `pixel_variance' are valid
		do
			error_code := 0

			if variation < 0 then
				error_code := pixel_variance_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_pixel_variance_error_message (variation: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_pixel_variance_valid'
		do
			create Result.make (10)

			inspect error_code
			when pixel_variance_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`variation' must be positive. Found ")
				Result.append_string (variation.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_pixel_samples_valid (xsamples, ysamples: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `pixel_samples' are valid
		do
			error_code := 0

			if xsamples < 1 then
				error_code := pixel_xsamples_error
			elseif ysamples < 1 then
				error_code := pixel_ysamples_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_pixel_samples_error_message (xsamples, ysamples: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_pixel_samples_valid'
		do
			create Result.make (10)

			inspect error_code
			when pixel_xsamples_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`xsamples' must be positive. Found ")
				Result.append_string (xsamples.out)
				Result.append (" instead. The call is ignored.%N%N")
			when pixel_ysamples_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`ysamples' must be positive. Found ")
				Result.append_string (ysamples.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_pixel_filter_valid (filterfunc: STRING;
						   xwidth, ywidth: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `pixel_filter' are valid
		do
			error_code := 0
			
			if filterfunc = Void or else filterfunc.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (filterfunc, filter_key_words) then
				error_code := invalid_filter_name_error
			elseif xwidth <= 0 then
				error_code := filter_xwidth_error
			elseif ywidth <= 0 then
				error_code := filter_ywidth_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_pixel_filter_error_message (filterfunc: STRING;
										 xwidth, ywidth: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_pixel_filter_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`filterfunc' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when invalid_filter_name_error then
				Result.append ("===> Invalid Option <===%N")
				Result.append ("`filterfunc' must be one of the following: ")
				Result.append (filter_key_words)
				Result.append (". Found ")
				Result.append (filterfunc)
				Result.append (" instead. The call is ignored.%N%N")
			when filter_xwidth_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`xwidth' must be strictly positive. Found ")
				Result.append_string (xwidth.out)
				Result.append (" instead. The call is ignored.%N%N")
			when filter_ywidth_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`ywidth' must be strictly positive. Found ")
				Result.append_string (ywidth.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_exposure_valid (gain, gamma: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `exposure' are valid
		do
			error_code := 0

			if gain <= 0 then
				error_code := gain_range_error
			elseif gamma <= 0 then
				error_code := gamma_range_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_exposure_error_message (gain, gamma: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_exposure_valid'
		do
			create Result.make (10)

			inspect error_code
			when gain_range_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`gain' must be strictly positive. Found ")
				Result.append_string (gain.out)
				Result.append (" instead. The call is ignored.%N%N")
			when gamma_range_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`gamma' must be strictly positive. Found ")
				Result.append_string (gamma.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_imager_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `imager' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_imager_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_imager_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_quantize_valid (type: STRING; one, min, max: INTEGER; ditheramplitude: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `quantize' are valid
		do
			error_code := 0
			
			if type = Void or else type.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (type, quantize_key_words) then
				error_code := quantize_type_error
			elseif one /= 0 and then max < min then
				error_code := quantize_parameter_error
			elseif ditheramplitude < 0 then
				error_code := quantize_ditheramplitude_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_quantize_error_message (type: STRING; one, min, max: INTEGER; ditheramplitude: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_quantize_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when quantize_type_error then
				Result.append ("===> Invalid Option <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (quantize_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			when quantize_parameter_error then
				Result.append ("===> Invalid Relation <===%N")
				Result.append ("Since `one' is not equal to zero, `min' must %
							   % be less than 'max'. Found ")
				Result.append_string (min.out)
				Result.append (" > ")
				Result.append_string (max.out)
				Result.append (" instead. The call is ignored.%N%N")
			when quantize_ditheramplitude_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`ditheramplitude' must be positive. Found ")
				Result.append_string (ditheramplitude.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_display_valid (name, type, mode: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `display' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_display_name_error
			elseif type = Void or else type.count = 0 then
				error_code := invalid_display_type_error
			elseif mode = Void or else mode.count = 0 then
				error_code := invalid_display_mode_error
			elseif not matches_word (type, display_type_key_words) then
				error_code := display_type_error
			elseif not matches_word (mode, display_mode_key_words) then
				error_code := display_mode_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_display_error_message (name, type, mode: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_display_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_display_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when invalid_display_type_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when invalid_display_mode_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`mode' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when display_type_error then
				Result.append ("===> Invalid Option <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (display_type_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			when display_mode_error then
				Result.append ("===> Invalid Option <===%N")
				Result.append ("`mode' must be one of the following: ")
				Result.append (display_mode_key_words)
				Result.append (". Found ")
				Result.append (mode)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Option checks
	
	is_hider_valid (type: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `hider' are valid
		do
			error_code := 0

			if type = Void or else type.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (type, hider_key_words) then
				error_code := hider_type_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_hider_error_message (type: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_hider_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when hider_type_error then
				Result.append ("===> Invalid Option <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (hider_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_color_samples_valid (n: INTEGER; n_rgb, rgb_n: DOUBLE_ARRAYED_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `color_samples' are valid
		do
			error_code := 0

			if n <= 0 then
				error_code := color_samples_count_error
			elseif n_rgb.count /= n then
				error_code := color_samples_array_count_error
			elseif n_rgb.count \\ 3 /= 0 then
				error_code := color_samples_multiple_of_three_error
			elseif n_rgb.count /= rgb_n.count then
				error_code := color_samples_array_mismatch_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_color_samples_error_message (n: INTEGER; n_rgb, rgb_n: DOUBLE_ARRAYED_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_color_samples_valid'
		do
			create Result.make (10)

			inspect error_code
			when color_samples_count_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`n' must be strictly positive. Found ")
				Result.append_string (n.out)
				Result.append (" instead. The call is ignored.%N%N")
			when color_samples_array_count_error then
				Result.append ("===> Invalid Size <===%N")
				Result.append ("`n_rgb' is not `n' elements in length. Found `n' ")
				Result.append_string (n.out)
				Result.append (" and `n_rgb' size ")
				Result.append_string (n_rgb.count.out)
				Result.append (" instead. The call is ignored.%N%N")
			when color_samples_multiple_of_three_error then
				Result.append ("===> Invalid Size <===%N")
				Result.append ("`n_rgb' is not a multiple of 3 length. Found ")
				Result.append_string (n_rgb.count.out)
				Result.append (" instead. The call is ignored.%N%N")
			when color_samples_array_mismatch_error then
				Result.append ("===> Invalid Size <===%N")
				Result.append ("`n_rgb' and rgb_n are not the same length.")
				Result.append (" Found `n_rgb' size ")
				Result.append_string (n_rgb.count.out)
				Result.append (" and `rgb_n' size ")
				Result.append_string (rgb_n.count.out)
				Result.append (". The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_relative_detail_valid (relativedetail: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `relative_detail' are valid
		do
			error_code := 0

			if relativedetail <= 0 then
				error_code := relative_detail_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_relative_detail_error_message (relativedetail: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_relative_detail_valid'
		do
			create Result.make (10)

			inspect error_code
			when relative_detail_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`relativedetail' must be strictly positive. Found ")
				Result.append_string (relativedetail.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_option_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `option' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_option_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_option_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Light checks
	
	is_light_source_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `light_source' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_light_source_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_light_source_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Light checks
	
	is_perspective_valid (fov: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `perspective' are valid
		do
			error_code := 0

			if fov <= 0 then
				error_code := perspective_parameter_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_perspective_error_message (fov: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_perspective_valid'
		do
			create Result.make (10)

			inspect error_code
			when perspective_parameter_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`fov' must be strictly positive. Found ")
				Result.append_string (fov.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_rotate_valid (angle, dx, dy, dz: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `rotate' are valid
		do
			error_code := 0

			if about_equal (dx, 0.0) and about_equal (dy, 0.0) and
				about_equal (dz, 0.0) then
				error_code := zero_rotation_axis_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_rotate_error_message (angle, dx, dy, dz: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_rotate_valid'
		do
			create Result.make (10)

			inspect error_code
			when zero_rotation_axis_error then
				Result.append ("===> Zero-length Axis <===%N")
				Result.append ("Rotation about a zero-length axis is undefined. ")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_skew_valid (angle, dx1, dy1, dz1, dx2, dy2, dz2: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `skew' are valid
		do
			error_code := 0

			if about_equal (dx1, 0.0) and about_equal (dy1, 0.0) and
				about_equal (dz1, 0.0) then
				error_code := axis_1_zero_length_error
			elseif about_equal (dx2, 0.0) and about_equal (dy2, 0.0) and
				about_equal (dz2, 0.0) then
				error_code := axis_2_zero_length_error
			elseif not valid_skew_angle (angle, dx1, dy1, dz1, dx2, dy2, dz2) then
				error_code := angle_range_error
			end

			if true then
				error_code := 1
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_skew_error_message (angle, dx1, dy1, dz1, dx2, dy2, dz2: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_skew_valid'
		do
			create Result.make (10)

			inspect error_code
			when axis_1_zero_length_error then
				Result.append ("===> Zero-length Axis <===%N")
				Result.append ("The first axis is zero-length. ")
				Result.append ("The call is ignored.%N%N")
			when axis_2_zero_length_error then
				Result.append ("===> Zero-length Axis <===%N")
				Result.append ("The second axis is zero-length. ")
				Result.append ("The call is ignored.%N%N")
			when angle_range_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`angle' is either greater than the angle %
							   %between the two axis arguments or is less %
							   %than the angle between the two axis %
							   %arguments minus pi. Found ")
				Result.append_string (angle.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_deformation_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `deformation' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_deformation_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_deformation_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_coordinate_system_valid (space: STRING): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `coordinate_system' are valid
		do
			error_code := 0

			if space = Void or else space.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_coordinate_system_error_message (space: STRING): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_coordinate_system_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`space' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_coordinate_sys_transform_valid (space: STRING): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `coordinate_sys_transform' are valid
		do
			error_code := 0

			if space = Void or else space.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_coordinate_sys_transform_error_message (space: STRING): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_coordinate_sys_transform_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`space' is either void, zero-length, or doesn't exist.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Attribute check
	
	is_attribute_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `attribute' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_attribute_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_attribute_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Geometry Attribute check
	
	is_detail_range_valid (minvisible, lowertransition,
						   uppertransition, maxvisible: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `detail_range' are valid
		do
			error_code := 0

			if not (minvisible >= 0 and then
					minvisible <= lowertransition and then
					lowertransition <= uppertransition and then
					uppertransition <= maxvisible) then
				error_code := detail_range_relation_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_detail_range_error_message (minvisible, lowertransition,
										 uppertransition, maxvisible: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_detail_range_valid'
		do
			create Result.make (10)

			inspect error_code
			when detail_range_relation_error then
				Result.append ("===> Invalid Relation <===%N")
				Result.append ("DetailRange requires the following to be true:%N")
				Result.append ("%T0 <= `minvisible' <= `lowertransition' <= %
							   %`uppertransition' <= `maxvisible'%N")
				Result.append ("Found ")
				if minvisible < 0 then
					Result.append ("`minvisible' < 0")
				elseif minvisible > lowertransition then
					Result.append ("`minvisible'> `lowertransition'")
				elseif lowertransition > uppertransition then
					Result.append ("`lowertransition' > `uppertransition'")
				elseif uppertransition > maxvisible then
					Result.append ("`uppertransition' > `maxvisible'")
				end
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_displacement_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `displacement' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_displacement_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_displacement_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_geometric_approximation_valid (type: STRING; value: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `geometric_approximation' are valid
		do
			error_code := 0

			if type = Void or else type.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (type, geometric_approximation_key_words) then
				error_code := geometric_approximation_type_error
			elseif value <= 0 then
				error_code := geometric_approximation_value_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_geometric_approximation_error_message (type: STRING; value: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_geometric_approximation_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when geometric_approximation_type_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (geometric_approximation_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			when geometric_approximation_value_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`value' must be strictly positive. Found ")
				Result.append_string (value.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_orientation_valid (orientation: STRING): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `orientation' are valid
		do
			error_code := 0

			if orientation = Void or else orientation.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (orientation, orientation_key_words) then
				error_code := orientation_type_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_orientation_error_message (orientation: STRING): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_orientation_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`orientation' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when orientation_type_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`orientation' must be one of the following: ")
				Result.append (orientation_key_words)
				Result.append (". Found ")
				Result.append (orientation)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_sides_valid (sides: INTEGER): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `sides' are valid
		do
			error_code := 0

			if not (sides = 1 or sides = 2)	 then
				error_code := sides_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_sides_error_message (sides: INTEGER): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_sides_valid'
		do
			create Result.make (10)

			inspect error_code
			when sides_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`sides' must be either 1 or 2. Found ")
				Result.append_string (sides.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_surface_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `surface' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_surface_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_surface_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_atmosphere_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `atmosphere' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_atmosphere_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_atmosphere_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_interior_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `interior' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_interior_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_interior_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_exterior_valid (name: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `exterior' are valid
		do
			error_code := 0

			if name = Void or else name.count = 0 then
				error_code := invalid_name_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_exterior_error_message (name: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_exterior_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`name' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_shading_rate_valid (size: DOUBLE): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `shading_rate' are valid
		do
			error_code := 0

			if size < 0 or size > infinity then
				error_code := shading_rate_error
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_shading_rate_error_message (size: DOUBLE): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_shading_rate_valid'
		do
			create Result.make (10)

			inspect error_code
			when shading_rate_error then
				Result.append ("===> Out Of Bounds <===%N")
				Result.append ("`size' must be between 0 and `infinity'. Found ")
				Result.append_string (size.out)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_shading_interpolation_valid (type: STRING): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `shading_interpolation' are valid
		do
			error_code := 0

			if type = Void or else type.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (type, shading_interpolation_key_words) then
				error_code := shading_interpolation_type_error
			end


			if error_code = 0 then
				Result := true
			end
		end 

	generate_shading_interpolation_error_message (type: STRING): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_shading_interpolation_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when shading_interpolation_type_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (shading_interpolation_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature -- Geometry check
	
	is_patch_valid (type: STRING; parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `patch' are valid
		local
			tag_index, point_size: INTEGER
		do
			error_code := 0

			if type = Void or else type.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (type, patch_type_key_words) then
				error_code := patch_type_error
			elseif not (parameters.has_tag ("%"P%"") or else
				parameters.has_tag ("%"Pw%"") or else
				parameters.has_tag ("%"Pz%"")) then
					error_code := missing_position_information_error
			else
				if parameters.has_tag ("%"P%"") then
					tag_index := parameters.index_of_tag ("%"P%"")
					point_size := 3
				elseif parameters.has_tag ("%"Pw%"") then
					tag_index := parameters.index_of_tag ("%"Pw%"")
					point_size := 4
				else
					tag_index := parameters.index_of_tag ("%"Pz%"")
					point_size := 1
				end

				if type.is_equal ("%"bilinear%"") then
					if parameters.item (tag_index).second.second.count /= 4 * point_size then
						error_code := bilinear_patch_count_error
					end	
				else
					if parameters.item (tag_index).second.second.count /= 16 * point_size then
						error_code := bicubic_patch_count_error
					end	
				end
				
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_patch_error_message (type: STRING; parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_patch_valid'
		do
			create Result.make (10)

			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when patch_type_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (patch_type_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			when missing_position_information_error then
				Result.append ("===> Missing Parameter <===%N")
				Result.append ("%"P%", %"Pw%", or %"Pz%" must be one of the provided. ")
				Result.append (" The call is ignored.%N%N")
			when bilinear_patch_count_error then
				Result.append ("===> Invalid Count <===%N")
				Result.append ("The size of %"P%", %"Pw%", or %"Pz%" must be ")
				Result.append (" 4 for a bilinear patch. The call is ignored.%N%N")
			when bicubic_patch_count_error then
				Result.append ("===> Invalid Count <===%N")
				Result.append ("The size of %"P%", %"Pw%", or %"Pz%" must be ")
				Result.append (" 16 for a bicubic patch. The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
	is_patch_mesh_valid (type: STRING; nu: INTEGER; uwrap: STRING;
		nv: INTEGER; vwrap: STRING;
		parameters: PARAMETER_LIST): BOOLEAN is
			-- Check if the arguments to the RI call corresponding to 
			-- `patch_mesh' are valid
		local
			tag_index, point_size: INTEGER
		do
			error_code := 0

			if type = Void or else type.count = 0 then
				error_code := invalid_name_error
			elseif not matches_word (type, patch_type_key_words) then
				error_code := patch_type_error
			elseif nu <= 0 then
				error_code := patch_mesh_nu_count_error
			elseif nv <= 0 then
				error_code := patch_mesh_nv_count_error
			elseif not (parameters.has_tag ("%"P%"") or else
				parameters.has_tag ("%"Pw%"") or else
				parameters.has_tag ("%"Pz%"")) then
					error_code := missing_position_information_error
			else
				if parameters.has_tag ("%"P%"") then
					tag_index := parameters.index_of_tag ("%"P%"")
					point_size := 3
				elseif parameters.has_tag ("%"Pw%"") then
					tag_index := parameters.index_of_tag ("%"Pw%"")
					point_size := 4
				else
					tag_index := parameters.index_of_tag ("%"Pz%"")
					point_size := 1
				end

				if parameters.item (tag_index).second.second.count /= nu * nv * point_size then
					error_code := patch_mesh_count_error
				end	
			end

			if error_code = 0 then
				Result := true
			end
		end 

	generate_patch_mesh_error_message (type: STRING; nu: INTEGER; uwrap: STRING;
		nv: INTEGER; vwrap: STRING;
		parameters: PARAMETER_LIST): STRING is
			-- Generate the error message corresponding to an error 
			-- generated by `is_patch_mesh_valid'
		local
			tag_index, point_size: INTEGER
		do
			create Result.make (10)

			if parameters.has_tag ("%"P%"") then
				tag_index := parameters.index_of_tag ("%"P%"")
				point_size := 3
			elseif parameters.has_tag ("%"Pw%"") then
				tag_index := parameters.index_of_tag ("%"Pw%"")
				point_size := 4
			else
				tag_index := parameters.index_of_tag ("%"Pz%"")
				point_size := 1
			end
				
			inspect error_code
			when invalid_name_error then
				Result.append ("===> Invalid Name <===%N")
				Result.append ("`type' is either void or zero-length.")
				Result.append ("The call is ignored.%N%N")
			when patch_type_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`type' must be one of the following: ")
				Result.append (patch_type_key_words)
				Result.append (". Found ")
				Result.append (type)
				Result.append (" instead. The call is ignored.%N%N")
			when patch_mesh_nu_count_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`nu' must be positve. Found ")
				Result.append (nu.out)
				Result.append (" instead. The call is ignored.%N%N")
			when patch_mesh_nv_count_error then
				Result.append ("===> Invalid Parameter <===%N")
				Result.append ("`nv' must be positve. Found ")
				Result.append (nv.out)
				Result.append (" instead. The call is ignored.%N%N")
			when missing_position_information_error then
				Result.append ("===> Missing Parameter <===%N")
				Result.append ("%"P%", %"Pw%", or %"Pz%" must be one of the provided. ")
				Result.append (" The call is ignored.%N%N")
			when patch_mesh_count_error then
				Result.append ("===> Invalid Count <===%N")
				Result.append ("The size of %"P%", %"Pw%", or %"Pz%" must be ")
				Result.append (" nu*nv*<the size of a point>. Wanted ")
				Result.append ((nu * nv * point_size).out)
				Result.append (", found ")
				Result.append (parameters.item (tag_index).second.second.count.out)
				Result.append (". The call is ignored.%N%N")
			else
			end

		ensure 
			Result_not_void: Result /= Void
		end 
	
feature {NONE} -- Implementation
	
end -- RIB_VALIDITY

