
indexing
	description: "Holds information about a variable in the RenderMan Shading Language"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"


class 
	
	SL_INFO
	
inherit
	
	ANY
		redefine
			out
		end
	
creation
	make
   
feature -- Initialization
   
	make (variable_name, declaration: STRING) is
			-- Parse the SL declarations from the strings
		local
			validClass: BOOLEAN
			ind1, ind2: INTEGER
			str: STRING
			err: STRING
		do
			name := variable_name.twin
			variable_class := uniform
			variable_type := float_type
			count := 1
			is_array := false
	
			if declaration /= Void then
				create str.make (1)
				create err.make (1)
				-- Remove leading and trailing white space
				declaration.left_adjust
				declaration.right_adjust
				
				-- check the declaration's "class"
				if declaration.item (1) = 'u' then
					if declaration.substring_index ("uniform", 1) /= 0 then
						validClass := true
					end
				elseif declaration.item (1) = 'c' then
					if declaration.substring_index ("constant", 1) /= 0 then
						variable_class := constant
						validClass := true
					end
				elseif declaration.item (1) = 'v' then
					if declaration.substring_index ("varying", 1) /= 0 then
						variable_class := varying
						validClass := true
					elseif declaration.substring_index ("vertex", 1) /= 0 then
						variable_class := vertex
						validClass := true
					end
					if not validClass then
						err.append ("Error in declaration: Expected %"uniform%", ")
						err.append ("%"varying%", or %"vertex%", %"")
						err.append (declaration)
						err.append ("%" found.%N")
						io.put_string (err)
					end
				end
				
				-- check the declaration's "type"
				if declaration.substring_index ("float", 1) /= 0 then
					variable_type := float_type
				elseif declaration.substring_index ("integer", 1) /= 0 then
					variable_type := integer_type
				elseif declaration.substring_index ("string", 1) /= 0 then
					variable_type := string_type
				elseif declaration.substring_index ("point", 1) /= 0 then
					variable_type := point_type
				elseif declaration.substring_index ("hpoint", 1) /= 0 then
					variable_type := hpoint_type
				elseif declaration.substring_index ("color", 1) /= 0 then
					variable_type := color_type
				elseif declaration.substring_index ("normal", 1) /= 0 then
					variable_type := normal_type
				elseif declaration.substring_index ("vector", 1) /= 0 then
					variable_type := vector_type
				elseif declaration.substring_index ("matrix", 1) /= 0 then
					variable_type := matrix_type
				else
					err.wipe_out
					err.append ("Error in declaration: Expected %"float%", ")
					err.append ("%"integer%", %"string%", ")
					err.append ("%"point%", %"color%", %"hpoint%"")
					err.append ("%"normal%", %"vector%",  or %"matrix%", %"")
					err.append (declaration)
					err.append ("%" found.%N")
					io.put_string (err)
				end
				
				ind1 := declaration.index_of ('[', 1)
				if ind1 /= 0 then
					ind2 := declaration.index_of (']', ind1)
					if ind2 /= 0 and ind2 - ind1 > 1 then
						str := declaration.substring (ind1 + 1, ind2 - 1)
						if str.is_integer then
							count := str.to_integer
							is_array := true
						else
							err.wipe_out
							err.append ("Error in declaration: Expected array ")
							err.append ("must be an integer%N")
							io.put_string (err)
						end
					else
						err.wipe_out
						err.append ("Error in declaration: Missing ']' in array.%N")
						io.put_string (err)
					end
				end
			end
		end

feature -- Access

	variable_class: INTEGER
			-- What is the shading class ('constant', ...) of this variable
	
	variable_type: INTEGER
			-- What is the shading type ('float_type', ...) of this variable
   
	constant, uniform, varying, vertex: INTEGER is unique
			-- Valid `variable_class' types
	
	float_type, integer_type, string_type, point_type, color_type,
	hpoint_type, vector_type, normal_type, matrix_type: INTEGER is unique
			-- Valid `variable_type' types
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	name: STRING
			-- The name of this variable
   
	count: INTEGER
			-- The size of this variable
   
	is_array: BOOLEAN
			-- Is this variable an array?
   
	is_float: BOOLEAN is
			-- Is the current variable of type float?
		do
			Result := variable_type = float_type
		end
   
	is_integer: BOOLEAN is
			-- Is the current variable of type integer?
		do
			Result := variable_type = integer_type
		end

	is_string: BOOLEAN is
			-- Is the current variable of type string?
		do
			Result := variable_type = string_type
		end

	is_point: BOOLEAN is
			-- Is the current variable of type point?
		do
			Result := variable_type = point_type
		end

	is_hpoint: BOOLEAN is
			-- Is the current variable of type hpoint?
		do
			Result := variable_type = hpoint_type
		end
   
	is_color: BOOLEAN is
			-- Is the current variable of type color?
		do
			Result := variable_type = color_type
		end
   
	is_normal: BOOLEAN is
			-- Is the current variable of type normal?
		do
			Result := variable_type = normal_type
		end
   
	is_vector: BOOLEAN is
			-- Is the current variable of type vector?
		do
			Result := variable_type = vector_type
		end
   
	is_matrix: BOOLEAN is
			-- Is the current variable of type matrix?
		do
			Result := variable_type = matrix_type
		end
   
	is_constant: BOOLEAN is
			-- Is the current variable's class constant?
		do
			Result := variable_class = constant
		end
   
	is_uniform: BOOLEAN is
			-- Is the current variable's class uniform?
		do
			Result := variable_class = uniform
		end
   
	is_varying: BOOLEAN is
			-- Is the current variable's class varying?
		do
			Result := variable_class = varying
		end
   
	is_vertex: BOOLEAN is
			-- Is the current variable's class vertex?
		do
			Result := variable_class = vertex
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_count (n: INTEGER) is
			-- Set the number of elements
		require 
			positive: n > 0
			array: is_array
		do
			count := n
		ensure 
			correct_size: count = n
		end
	
	set_array is
			-- Change to an array
		do
			is_array := true
			set_count (1)
		ensure 
			array: is_array
			has_size: count = 1
		end
	
	set_float_type is
			-- Change the variable type to a float
		do
			variable_type := float_type
		ensure 
			correct_type: is_float
		end

	set_integer_type is
			-- Change the variable type to a integer
		do
			variable_type := integer_type
		ensure 
			correct_type: is_integer
		end

	set_string_type is
			-- Change the variable type to a string
		do
			variable_type := string_type
		ensure 
			correct_type: is_string
		end

	set_point_type is
			-- Change the variable type to a point
		do
			variable_type := point_type
		ensure 
			correct_type: is_point
		end

	set_hpoint_type is
			-- Change the variable type to a hpoint
		do
			variable_type := hpoint_type
		ensure 
			correct_type: is_hpoint
		end

	set_color_type is
			-- Change the variable type to a color
		do
			variable_type := color_type
		ensure 
			correct_type: is_color
		end

	set_normal_type is
			-- Change the variable type to a normal
		do
			variable_type := normal_type
		ensure 
			correct_type: is_normal
		end

	set_vector_type is
			-- Change the variable type to a vector
		do
			variable_type := vector_type
		ensure 
			correct_type: is_vector
		end

	set_matrix_type is
			-- Change the variable type to a matrix
		do
			variable_type := matrix_type
		ensure 
			correct_type: is_matrix
		end

	set_constant_class is
			-- Change the class type to be constant
		do
			variable_class := constant
		ensure 
			correct_class: is_constant
		end

	set_uniform_class is
			-- Change the class type to be uniform
		do
			variable_class := uniform
		ensure 
			correct_class: is_uniform
		end

	set_varying_class is
			-- Change the class type to be varying
		do
			variable_class := varying
		ensure 
			correct_class: is_varying
		end

	set_vertex_class is
			-- Change the class type to be vertex
		do
			variable_class := vertex
		ensure 
			correct_class: is_vertex
		end
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
	
	out: STRING is
			-- Printable version of self
		do
			create Result.make (15)
			
			inspect variable_class
			when uniform then
				Result.append ("uniform ")
			when constant then
				Result.append ("constant ")
			when varying then
				Result.append ("varying ")
			when vertex then
				Result.append ("vertex ")
			else
			end
			
			inspect variable_type
			when float_type then 
				Result.append ("float ")
			when integer_type then 
				Result.append ("integer ")
			when string_type then 
				Result.append ("string ")
			when point_type then 
				Result.append ("point ")
			when color_type then 
				Result.append ("color ")
			when hpoint_type then 
				Result.append ("hpoint ")
			when vector_type then 
				Result.append ("vector ")
			when normal_type then 
				Result.append ("normal ")
			when matrix_type then
				Result.append ("matrix ")
			else
			end
		end
	
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
invariant
	
	variable_class_correct: variable_class >= constant and variable_class <= vertex
	variable_type_correct: variable_type >= float_type and variable_type <= matrix_type
	
end -- SL_INFO

