

indexing
	description: "Abstract action for culling nodes"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"


deferred class

	CULLING_ACTION

inherit

	SCENE_ACTION
		redefine
			default_create
		end

feature -- Initialization

	default_create is
			--
		local
			set: SCENE_STATE_ELEMENT_TABLE
			t: TRANSFORM
		do
			Precursor

			culling := false
			culled := false
			gathering := false

			if current_state = Void then
				create current_state.make
			else
				current_state.wipe_out
			end

			if current_transform = Void then
				create current_transform.make
			else
				current_transform.wipe_out
			end

			create set.make_from_list (state_factory.element_list)
			current_state.put (set)

			create t
			current_transform.put (t)

			create test_bound
			create last_point
		end

	make (new_camera: SCENE_CAMERA) is
			-- Initialize with `new_camera'
		require
			new_camera_not_void: new_camera /= Void
		do
			default_create

			set_camera (new_camera)
		ensure
			camera_set: camera = new_camera
		end

feature -- Access

	global_lights: SCENE_LIGHTS
			-- Defines all of the  lights that are used for the entire
			-- scene.

	camera: SCENE_CAMERA
			-- The camera to be used for culling

	inside, outside, partial: INTEGER is unique
			-- Classification of the bounds w.r.t the view frustum

	test_bound: AXIS_ALIGNED_BOX
			-- Bounds to use for cull testing

	last_point: POINT3
			-- The nearest point of the last culled item in
			-- hybrid coordinates (x, y are in `camera's raster
			-- space, and z is in `camera' space

feature -- Measurement

	count: INTEGER
		-- The number of items added

feature -- Comparison
feature -- Status report

	culling: BOOLEAN
			-- Is this action frustum culling?

	culled: BOOLEAN
			-- Is the node culled?

	gathering: BOOLEAN
			-- Gather culled nodes?

feature -- Status setting

	push (node: SCENE_NODE) is
			-- Increase depth in tree		
		local
			t: SCENE_TRANSFORM
		do
			t ?= node
			if t /= Void then
				push_transform (t.current_transform)
			end
		end

	pop (node: SCENE_NODE) is
			-- Increase depth in tree
		local
			t: SCENE_TRANSFORM
		do
			t ?= node
			if t /= Void then
				pop_transform
			end
		end

	set_culling (on_or_off: BOOLEAN) is
			-- Set `culling' to `on_or_off'
		do
			culling := on_or_off
		ensure
			culling_set: culling = on_or_off
		end

	set_gathering (on_or_off: BOOLEAN) is
			-- Set `gathering' to `on_or_off'
		do
			gathering := on_or_off
		ensure
			gathering_set: gathering = on_or_off
		end

feature -- Cursor movement
feature -- Element change

	add_drawable (geometry: SCENE_GEOMETRY) is
			-- Add `geometry' to the drawable pool
		require
			geometry_not_void: geometry /= Void
		do
			count := count + 1
		ensure
			count_increased: old count + 1 = count
		end

	set_camera (new_camera: SCENE_CAMERA) is
			-- Set `camera' to `new_camera'
		require
			new_camera_not_void: new_camera /= Void
		local
			pt1, pt2: POINT3
		do
			camera := new_camera
			create pt1.make (0, 0, 0)
			create pt2.make (camera.x_resolution, camera.y_resolution, 1)
			test_bound.make (pt1, pt2)
		ensure
			camera_set: camera = new_camera
		end

	push_transform (new_transform: TRANSFORM) is
			-- Appends `new_transform' to the current state
		require
			new_transform_not_void: new_transform /= Void
		local
			t: TRANSFORM
		do
			t := current_transform.item * new_transform
			current_transform.put (t)
		ensure
			depth_increased: old current_transform.count + 1 =
							 current_transform.count
		end

	pop_transform is
			-- Removes the top element from the transform stack
		do
			current_transform.remove
		ensure
			depth_decreased: old current_transform.count - 1 =
							 current_transform.count
		end

	push_state (new_elements: SCENE_STATE_ELEMENT_TABLE) is
			-- Appends `new_elements' to the current state
		require
			new_elements_not_void: new_elements /= Void
		local
			elem: SCENE_STATE_ELEMENT_TABLE
		do
			elem := current_state.item.twin
			elem.merge (new_elements)
			current_state.put (elem)
		ensure
			depth_increased: old current_state.count + 1 =
							 current_state.count
		end

	pop_state is
			-- Removes the top element from the state stack
		do
			current_state.remove
		ensure
			depth_decreased: old current_state.count - 1 =
							 current_state.count
		end


	set_global_lights (new_lights: SCENE_LIGHTS) is
			-- Set `global_lights' to `new_lights'
		require
			new_lights_not_void: new_lights /= Void
		do
			global_lights := new_lights
		ensure
			global_lights_set: global_lights = new_lights
		end

	set_test_bounds (lower_x, lower_y, upper_x, upper_y: DOUBLE) is
			-- Set the extents of `test_bound'
		require
			ordered_in_x: lower_x <= upper_x
			ordered_in_y: lower_y <= upper_y
		local
			pt1, pt2: POINT3
		do
			create pt1.make (lower_x, lower_y, 0)
			create pt2.make (upper_x, upper_y, 1)
			test_bound.make (pt1, pt2)
		ensure
			test_bound_not_void: test_bound /= Void
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	apply (node: SCENE_NODE) is
			-- Apply the cull action to node
		do
			if culling then
				if cull_state (node.extents.circumscribed_box) = outside then
					culled := true
				else
					culled := false
				end
			else
				culled := false
			end

			node.cull (Current)
		end

	cull_state (bound: AXIS_ALIGNED_BOX): INTEGER is
			-- Is this `bound' not within the view frustum?
		require
			bound_not_void: bound /= Void
		local
			near_z, far_z: DOUBLE
		do
			Result := outside
			bound.transform (camera.world_to_camera * current_transform.item)

			-- Because we're looking down the -Z axis, upper and
			-- lower are swapped
			near_z := -bound.upper.z
			far_z := -bound.lower.z
			bound.transform (camera.camera_to_raster)
			last_point.make (bound.lower.x, bound.lower.y, near_z)

			if near_z >= camera.near_clipper and then
				far_z <= camera.far_clipper then
				-- Inside clipping planes
				if bound.lower.x >= test_bound.lower.x and then
					bound.upper.x <= test_bound.upper.x and then
					bound.lower.y  >= test_bound.lower.y and then
					bound.upper.y  <= test_bound.upper.y then
					Result := inside
				elseif bound.upper.x >= test_bound.lower.x and then
					bound.lower.x <= test_bound.upper.x and then
					bound.upper.y >= test_bound.lower.y and then
					bound.lower.y <= test_bound.upper.y then
					Result := partial
				end
			else
				Result := partial
				if near_z >= camera.far_clipper or
					far_z <= camera.near_clipper then
					Result := outside
				else
					if bound.lower.x >= test_bound.lower.x and then
						bound.upper.x <= test_bound.upper.x and then
						bound.lower.y  >= test_bound.lower.y and then
						bound.upper.y  <= test_bound.upper.y then
						Result := partial
					elseif not (((bound.lower.x <= test_bound.lower.x and
								  bound.upper.x > test_bound.lower.x) or
							 (bound.lower.x <= test_bound.upper.x and
							  bound.upper.x > test_bound.upper.x)) and
								((bound.lower.y <= test_bound.lower.y and
								  bound.upper.y > test_bound.lower.y) or
							 (bound.lower.y <= test_bound.upper.y and
							  bound.upper.y > test_bound.upper.y))) then
						Result := outside
					end
				end
			end
		ensure
			Result_valid: Result >= inside and Result <= partial
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	current_state: DS_LINKED_STACK [SCENE_STATE_ELEMENT_TABLE]
			-- The current state

	current_transform: DS_LINKED_STACK [TRANSFORM]
			-- The current modeling transform

	state_factory: SCENE_STATE_ELEMENT_FACTORY is
			-- The default state elements
		once
			create Result
		end

invariant

	state_factory_not_void: state_factory /= Void
	current_state_not_void: current_state /= Void
	current_transform_not_void: current_transform /= Void
	culling_implies_camera: culling implies camera /= Void
	test_bound_not_void: test_bound /= Void
	last_point_not_void: last_point /= Void

end -- CULLING_ACTION

