

indexing
	description: "Base action node for testing ray intersection"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 

	INTERSECT_ACTION
	
inherit
	
	SCENE_ACTION
		redefine
			default_create
		end

feature -- Initialization

	default_create is
			-- Initialize
		do
			Precursor
			
			intersected := false
			
			if color = Void then
				create color
			end
			color.fill (0) -- Initialize to black and transparent
			
			if ray_stack /= Void then
				ray_stack.wipe_out
			else
				create ray_stack.make
			end
			
			if intersection /= Void then
				intersection.wipe_out
			else
				create intersection.make (2)
			end
		ensure then
			ray_stack_is_empty: ray_stack.count = 0
		end

feature -- Access
	
	ray: RAY
			-- The ray to be used for intesection
	
	intersected: BOOLEAN
			-- Did the ray ever hit anything
	
	integrator: INTEGRATOR
			-- The light integrator
	
	color: RGBA_COLOR
			-- The accumulated color
	
	intersection: SORTABLE_ARRAYED_LIST [INTERSECTION_DATA] 
			-- Result of 'intersect' operation: ordered parametric coordinates
			-- along the ray of first, second, third, ... unique points of
			-- intersection.
			-- (Use RAY.project to convert a parametric coordinate into POINT3.)
	
	scene: SCENE_GROUP
			-- The root node of the scene graph
	
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
	
	push (node: SCENE_NODE) is
			-- Increase depth in tree		
		local
			t: SCENE_TRANSFORM
		do
			t ?= node
			if t /= Void then
				ray_stack.force (ray.twin)
				ray.apply (t.inverse_transform)
			end
		end 

	pop (node: SCENE_NODE) is
			-- Increase depth in tree
		local
			t: SCENE_TRANSFORM
		do
			t ?= node
			if t /= Void then
				ray := ray_stack.item
				ray_stack.remove
			end
		end 

	set_intersected (hit: BOOLEAN) is
			-- Set `intersected' to `hit'
		do
			intersected := hit
		ensure 
			intersected_set: intersected = hit
		end 
	
feature -- Cursor movement
feature -- Element change
	
	set_ray (new_ray: RAY) is
			-- Set `ray' to `new_ray'
		require 
			new_ray_not_void: new_ray /= Void
		do
			ray := new_ray
			terminate_action := false
			intersected := false
		ensure 
			ray_set: ray = new_ray
		end 
	
	set_scene (new_scene: SCENE_GROUP) is
			-- Set `scene' to `new_scene'
		require 
			new_scene_not_void: new_scene /= Void
		do
			scene := new_scene
			terminate_action := false
			intersected := false
		ensure 
			scene_set: scene = new_scene
		end 

	set_integrator (new_integrator: INTEGRATOR) is
			-- Set `integrator' to `new_integrator'
		require 
			new_integrator_not_void: new_integrator /= Void
		do
			integrator := new_integrator
		ensure 
			integrator_set: integrator = new_integrator
		end 
		
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	reset is
			-- Initialize
		do
			terminate_action := false
			intersected := false
			
			color.fill (0) -- Initialize to black and transparent
			
			ray_stack.wipe_out
			
			intersection.wipe_out
		ensure
			ray_stack_is_empty: ray_stack.count = 0
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	ray_stack: DS_LINKED_STACK [RAY]
			-- A stack of rays
	
invariant
	
	ray_stack_not_void: ray_stack /= Void
	intersection_not_void: intersection /= Void
	intersected_implies_intersection: intersected implies intersection.count > 0
	
end -- INTERSECT_ACTION

