

indexing
	description: "Base class for all nodes that can have children"
	names: "group", "children"
	representation: "DS_ARRAYED_LIST"
	access: "index"
	size: "resizable"
	contents: "SCENE_VOLUME"
	patterns: "none"
	status: "unreviewed"




deferred class 

	SCENE_CONTAINER [G-> SCENE_VOLUME]
	
inherit
	
	SCENE_COLLECTION [G]
		redefine
			print_object, print_features,
			default_create, cull
		end
	
feature -- Initialization

	default_create is
			-- Create the node.
		do
			Precursor
			
			create stop_child_traversal.make (16)
			
			all_intersections := true
		end

feature -- Access	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	all_intersections: BOOLEAN
			-- Generate all intersections with a ray, or any intersection?
	
feature -- Status setting
	
	generate_all_intersections (yes: BOOLEAN) is
			-- Set whether we want to generate all intersections with a ray, 
			-- or just the first intersection
		do
			all_intersections := yes
		ensure 
			all_intersections_set: all_intersections = yes
		end

feature -- Cursor movement
feature -- Element change
feature -- Removal	
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations	
	
	cull (action: CULLING_ACTION) is
			-- A traversal that culls geometry not in the viewing 
			-- frustrum
		do
			if not action.culled or (action.culled and action.gathering) then
				stop_child_traversal.force (false, action.identifier)
			else
				stop_child_traversal.force (true, action.identifier)
			end			
		end 

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of `r' against this volume.
		do
			stop_child_traversal.force (false, intersecter.identifier)
			
			extents.intersect_ray(intersecter.ray)
			
			if extents.outside then
				stop_child_traversal.replace (true, intersecter.identifier)
			end
		end
	
	print_object (action: PRINT_ACTION) is
			-- Create a representation of self
		do
			Precursor (action)
			
			print_children (action)
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	stop_child_traversal: DS_HASH_TABLE [BOOLEAN, INTEGER]
			-- Should the traversal continue on to other children, if 
			-- applicable. Using a hash table keyed off the 
			-- `object_id' of SCENE_ACTION allows the traversal to be 
			-- multi-thread safe.
			-- This is a negative query; the scene graph is relying 
			-- on the default_creation of BOOLEAN to be false
	
	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			action.io.put_string ("	 all_intersections = ")
			action.io.put_boolean (all_intersections)
			action.io.put_new_line
			
		end 
	
	print_children (action: PRINT_ACTION) is
			-- 
		require 
			action_not_void: action /= Void
		do
			action.io.put_string ("	 children (")
			action.io.put_integer (children.count)
			action.io.put_string (") = ")
			action.io.put_new_line
		end 

invariant
	
	intersected_always_false: intersected = false
	
end -- SCENE_CONTAINER

