

indexing
	description: "Concrete base class for all nodes that can have children"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_GROUP
	
inherit
	
	SCENE_CONTAINER [SCENE_NODE]
		redefine
			add_child, insert_child, replace_child,
			remove_child, remove_all_children
		end
	
creation
	default_create, make

feature -- Initialization

feature -- Access
	
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Element change
	
	add_child (node: SCENE_NODE) is
			-- add `node' as a new child
		require else
			not_an_ancestor: not ancestor (node)
		do
			Precursor (node)
			
			node.add_parent (Current)
			force_update
		end
	
	insert_child (node: SCENE_NODE; index: INTEGER) is
			-- insert `node' at `index'
		require else
			not_an_ancestor: not ancestor (node)
		do
			Precursor (node, index)
			
			node.add_parent (Current)
			force_update
		end
	
	replace_child (node: SCENE_NODE; index: INTEGER) is
			-- replace the child at `index' with `node'
		require else
			not_an_ancestor: not ancestor (node)
		do
			children.go_i_th (index)
			children.item_for_iteration.remove_parent (Current)
			children.replace_at (node)
			node.add_parent (Current)
			force_update
		end

feature -- Removal
	
	remove_child (node: SCENE_NODE) is
			-- remove the child corresponding to `node'
		do
			Precursor (node)
			
			node.remove_parent (Current)
			force_update
		end
	
	remove_all_children is
			-- remove all children
		do
			from
				children.start
			until
				children.off
			loop
				children.item_for_iteration.remove_parent (Current)
				
				children.forth
			end
			
			children.wipe_out
			force_update
		ensure then
			count_is_zero: count = 0
		end

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	traverse_node (action: SCENE_ACTION) is
			-- Traverse this node
		do
			action.apply (Current)
			
			if not action.terminate_action then
				if not stop_child_traversal.has (action.identifier) or else 
				(not stop_child_traversal.item (action.identifier) and count > 0) then
					action.push (Current)
					
					from
						children.start
					until
						children.after or 
							(stop_child_traversal.has (action.identifier) and then 
							stop_child_traversal.item (action.identifier)) or 
							action.terminate_action
					loop
						if children.item_for_iteration.traversable then
							children.item_for_iteration.traverse (action)
						end
						
						children.forth
					end
					
					action.pop (Current)
				end
			end
		end

end -- SCENE_GROUP

