

indexing
	description: "A container for a set of 3D geometry."
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_SHAPE
	
inherit
	
	SCENE_CONTAINER [SCENE_GEOMETRY]
		rename
			make as container_make,
			children as geometric_data,
			add_child as add_geometry,
			insert_child as insert_geometry,
			replace_child as replace_geometry,
			remove_child as remove_geometry,
			remove_all_children as remove_all_geometry
		redefine
			memory_usage, cull
		end

creation
	make

feature -- Initialization

	make (new_geometry: SCENE_GEOMETRY) is
			-- Create a new node with a single geometry item.
			-- This is simply reference assignment, no copying 
			-- is performed.
		require
			valid_geometry: new_geometry /= Void
		do
			container_make
			
			geometric_data.force_last (new_geometry)
		ensure
			geometric_data_has_one_item: geometric_data.count = 1
		end

feature -- Access
	
feature -- Measurement
	
	memory_usage: INTEGER is
			-- Compute the memory usage for `Current' using fields
		do
			Result := Precursor
			
			from
				geometric_data.start
			until
				geometric_data.after
			loop
				Result := Result + geometric_data.item_for_iteration.memory_usage
				geometric_data.forth
			end
		end
	
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	cull (action: CULLING_ACTION) is
			-- A traversal that culls geometry not in the viewing 
			-- frustrum
		do
			if not action.culled or (action.culled and action.gathering) then
				stop_child_traversal.force (false, action.identifier)
			
				from
					geometric_data.start
				until
					geometric_data.after
				loop
					geometric_data.item_for_iteration.cull (action)
				
					geometric_data.forth
				end
			end
			stop_child_traversal.replace (true, action.identifier)
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	traverse_node (action: SCENE_ACTION) is
			-- Traverse this node
		do
			action.apply (Current)
			
			if (not stop_child_traversal.has (action.identifier) or else
				not stop_child_traversal.item (action.identifier))	or
				not action.terminate_action then
				action.push (Current)
				
				from
					geometric_data.start
				until
					geometric_data.after or 
						(stop_child_traversal.has (action.identifier) and then
						stop_child_traversal.item (action.identifier)) or 
						action.terminate_action
				loop
					geometric_data.item_for_iteration.traverse (action)
					
					geometric_data.forth
				end
				
				action.pop (Current)
			end
		end

invariant
	
	at_least_one_item: count >= 1
	
end -- SCENE_SHAPE

