

indexing
	description: "Node for storing heirarchies of transformations."
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_TRANSFORM
	
inherit
	
	BASIC_NUMERIC
		undefine
			default_create, is_equal
		end
	
	SCENE_GROUP
		redefine
			default_create, gather_extents, print_features, 
			transform_to_parent
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize the features
		do
			Precursor {SCENE_GROUP}
			
			 create current_transform.make
			 create inverse.make
			
			updated := true
			accumulate := true
			
			create deformation_shader.make_from_string ("%"null%"")
			create coordinate_systems.make (5)	
		end

feature -- Access
	
	deformation_shader: STRING
			-- The current deformation shader
	
	current_transform: TRANSFORM
			-- The 4 x 4 `matrix' representation of all combined 
			-- transfomation operations
	
	inverse_transform: TRANSFORM is
			-- The inverse of `current_transform'
		do
			if not updated then
				inverse := current_transform.inverse
				updated := true
			end
			
			Result := inverse
		end
	
	get_coordinate_system (space: STRING): TRANSFORM is
			-- 
		require 
			has_space: has_coordinate_system (space)
		do
			Result := coordinate_systems.item (space)
		ensure 
			Result_not_void: Result /= Void
		end
	
	accumulate: BOOLEAN
			-- Should this transformation update or replace the 
			-- current transform during traversal
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	has_coordinate_system (space: STRING): BOOLEAN is
			-- Is `space' a marked coordinate system?
		require 
			valid_space: space /= Void and then space.count > 0
		do
			Result := coordinate_systems.has (space)
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	identity is
			-- Set the current transformation to the identity. Note 
			-- that this actually tromps over the current matrix and 
			-- stops accumulation of transforms.
		do
			current_transform.reset
			
			accumulate := false
			updated := false
			force_update
		ensure 
			is_identity: current_transform.is_equal (current_transform.one)
		end
	
	load_transform (xform: TRANSFORM) is
			-- Set the current transform to `transform'. Note 
			-- that this actually tromps over the current matrix and 
			-- stops accumulation of transforms.
		require 
			transform_not_void: xform /= Void
		do
			current_transform := xform.twin
			
			accumulate := false
			updated := false
			force_update
		ensure 
			current_transform_set: current_transform.is_equal (xform)
		end
	
	concatonate_transform (xform: TRANSFORM) is
			-- 
		require 
			transform_not_void: xform /= Void
		do
			current_transform.concatonate_transform (xform )
			
			updated := false
			force_update
		end
	
	perspective (fov: DOUBLE) is
			-- Concatenate a perspective divide onto the current transformation
		require 
			fov_is_positive: fov > 0
		do
			updated := false
			force_update
		ensure 
			default: true
		end
	
	translate (dx, dy, dz: DOUBLE) is
			-- Concatenate a translation onto the current transformation
		do
			current_transform.translate (dx, dy, dz)
			
			updated := false
			force_update
		end
	
	rotate (angle, dx, dy, dz: DOUBLE) is
			-- Concatenate a rotation of `angle' radians about the 
			-- given axis onto the current transformation
		local
			v: VECTOR3
		do
			create v.make (dx, dy, dz)
			v.normalize
			
			current_transform.rotate (angle, v)
			
			updated := false
			force_update
		end

	scale (sx, sy, sz: DOUBLE) is
			-- Concatenate a scaling onto the current transformation
		do
			current_transform.scale (sx, sy, sz)
			
			force_update
			updated := false
		end
	
	skew (angle, dx1, dy1, dz1, dx2, dy2, dz2: DOUBLE) is
			-- Concatenate a skew onto the current transformation
		require 
			default: true
		local
			d1, d2: VECTOR3
		do
			create d1.make (dx1, dy1, dz1)
			create d2.make (dx2, dy2, dz2)
			
			current_transform.skew (angle, d1, d2)
			
			updated := false
			force_update
		end
	
	deformation (name: STRING; parameters: PARAMETER_LIST) is
			-- Set the current Deformation shader to `name'
		require 
			name_not_void: name /= Void and then name.count > 0
		do
			deformation_shader := name.twin
		ensure 
			deformation_shader_set: deformation_shader.is_equal (name)
		end
	
	coordinate_system (space: STRING) is
			-- Mark the current coordinate system with the name 
			-- `space' and save it
		require 
			valid_space: space /= Void and then space.count > 0
		do
			coordinate_systems.put (current_transform.twin, space)
		ensure 
			space_inserted: has_coordinate_system (space)
		end
	
	coordinate_system_transform (space: STRING) is
			-- Replace the `current_transform' with `space'
		require 
			valid_space: space /= Void and then space.count > 0
		do
			current_transform := coordinate_systems.item (space)
		end

	transform_points (fromspace, tospace: STRING;
					  n: INTEGER; points: ARRAY [POINT]): ARRAY [POINT] is
			-- Transform the `points' from `fromspace' to `tospace'
		require 
			valid_fromspace: has_coordinate_system (fromspace)
			valid_tospace: has_coordinate_system (tospace)
			valid_points: points /= Void and then points.count > 0
			consistent: n = points.count
		do
			create Result.make (points.lower, points.upper)
		ensure 
			Result_consistent: (Result /= Void) implies Result.count = n
		end
	
	transform_to_parent (t: TRANSFORM) is
			-- Compute a transform to parent and append the transform
			-- to the parameter passed in.	This is the default
			-- implementation for all nodes that do not affect this
			-- matrix and therefore this function does nothing
		do
			t.concatonate_transform (inverse)
		end 
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	coordinate_systems: DS_HASH_TABLE [TRANSFORM, STRING]
	updated: BOOLEAN
	
	inverse: TRANSFORM
			-- The inverse of `current_transform'
	
	print_features (action: PRINT_ACTION) is
			-- print the group's features
		do
			Precursor (action)
			
			action.io.put_string ("	 accumulate = ")
			action.io.put_boolean (accumulate)
			action.io.put_new_line
			
			action.io.put_string ("	 current_transform = ")
			action.push (Current)
			action.io.put_new_line
			current_transform.print_using_io (action.io)
			action.pop (Current)
			action.io.put_new_line
		
			action.io.put_string ("	 deformation_shader = ")
			action.io.put_string (deformation_shader)
			action.io.put_new_line
		end 
	
	gather_extents is
			-- Gather together all bounding information.
		do
			update_needed := false
			if count > 0 then
				-- Initialize the bounding box to the first child
				children.start
				
				-- Transform the child extents in this node's space
				gathered_extents := children.item_for_iteration.extents.apply (current_transform)
				
				children.forth
				
				from
				until
					children.after
				loop
					gathered_extents.expand_to_include (children.item_for_iteration.extents.apply (current_transform))
					children.forth
				end
			end
		end
	
invariant
	current_transform_not_void: current_transform /= Void
	inverse_transform_not_void: inverse_transform /= Void
	inverse_really_is_inverse: (current_transform.one.is_equal
								(current_transform * inverse_transform))

end -- SCENE_TRANSFORM

