

indexing
	description: "Element to cache display lists"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_DISPLAY_LIST_ELEMENT
	
inherit
	
	SCENE_STATE_ELEMENT
		redefine
			default_create
		end

creation
	default_create, make

feature -- Initialization

	default_create, make is
			-- Initialize the table
		do
			if display_lists = Void then
				create display_lists.make (100)
			else
				display_lists.wipe_out
			end
		end

feature -- Access
	
	item (r: RENDERER): PAIR [INTEGER_64, BOOLEAN] is
			-- Return the display list and update flag corresponding to `r'
		require 
			renderer_not_void: r /= Void 
		do
			Result := display_lists.item (r.identifier)
		end 
	
feature -- Measurement
	
	count: INTEGER_64 is
			-- The number of display lists
		do
			Result := display_lists.count
		end
	
feature -- Comparison
feature -- Status report
	
	has_display_list (r: RENDERER): BOOLEAN is
			-- Has a display list been generated for r?
		require 
			renderer_not_void: r /= Void
			renderer_id_is_positive: r.identifier >= 0
		do
			Result := display_lists.has (r.identifier)
		end 

feature -- Status setting
	
	needs_updating is
			-- Indicate that the display list need updating
		do
			from
				display_lists.start 
			until
				display_lists.after
			loop
				display_lists.item_for_iteration.put_second (true)
				
				display_lists.forth
			end
		end
	
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	generate_display_list (r: RENDERER) is
			-- Generate a display list corresponding to `r'
		require 
			renderer_not_void: r /= Void
		local
			id: INTEGER_64
			p: PAIR [INTEGER_64, BOOLEAN]
		do
			id := r.generate_lists (1)
			create p.make (id, false)
			
			display_lists.force (p, r.identifier)
			check
				insertion_at_identifier: item (r) = p
			end
			
		ensure 
			count_increased_by_one: count = old count + 1
			element_added: has_display_list (r)
		end 

	remove_display_list (r: RENDERER) is
			-- Generate a display list corresponding to `r'
		require 
			renderer_not_void: r /= Void
		do
			r.delete_lists (item (r).first.to_integer, 1)
			display_lists.remove (r.identifier)
		ensure 
			display_list_is_removed: not has_display_list (r)
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	display_lists: DS_HASH_TABLE [PAIR [INTEGER_64, BOOLEAN], INTEGER]
			-- A hash table of display lists
	
invariant
	
	display_lists_not_void: display_lists /= Void
	
end -- SCENE_DISPLAY_LIST_ELEMENT

