

indexing
	description: "Abstract class for state elements with multiple elements accessed %
	%through indirect indexing"
	patterns: "none"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	status: "unreviewed"




deferred class 

	SCENE_INDEXED_MULTI_ELEMENT [G]
	
inherit
	
	SCENE_LARGE_MULTI_ELEMENT [G]
		redefine
			default_create, make_from_list, i_th, item,
			count, after, start, forth, add_item, insert_item,
			replace_item, remove_item, remove_all_values
		end

feature -- Initialization

	default_create is
			-- 
		local
			tmp_indices: INTEGER_64_ARRAYED_LIST
		do
			Precursor
			
			create tmp_indices.make (1)
			create indices.make (1)
			indices.force_last (tmp_indices)
			
			which_strip := 1
			indices.go_i_th (which_strip.to_integer)
		end

	make_from_coordinates_and_indices (vals: LARGE_ARRAYED_LIST [G]; 
									   new_indices: INTEGER_64_ARRAYED_LIST) is
			-- Initialize the element with `vals' and `new_indices'
		require
			vals_not_void: vals /= Void
			new_indices_not_void: new_indices /= Void
--			new_indices_at_least_as_big_as_vals: new_indices.count >= vals.count
		deferred
		ensure
			values_set: values = vals
			indices_set : indices.item_for_iteration.is_equal (new_indices)
		end
	
	make_from_list (vals: LARGE_ARRAYED_LIST [G]) is
			-- Initialize the element with `vals'
		local
			i: INTEGER_64
		do
			default_create
			
			from
				i := 1
				indices.start
				indices.item_for_iteration.wipe_out
				vals.start
				values.wipe_out
			until
				vals.after
			loop
				indices.item_for_iteration.force_last (i)
				values.force_last (vals.item_for_iteration)
				vals.forth
				i := i + 1
			end
		ensure then
			indices_as_big_as_values: indices.item_for_iteration.count = vals.count
		end
	
feature -- Access
	
	i_th (i: INTEGER_64): G is
			-- 
		do
			Result := values.item (indices.item_for_iteration.item (i))
		end 

	item: G is
			-- The item at the cursor
		do
			Result := values.item (indices.item_for_iteration.item_for_iteration)
		end 
	
	which_strip: INTEGER_64
			-- Which indexed strip do the current operations apply to?
	
feature -- Measurement
	
	count: INTEGER_64 is
			-- The number of values
		local
			index: INTEGER_64
		do
			if not indices.off then
				index := indices.item_for_iteration.index
			end
			
			from
				indices.start
			until
				indices.after
			loop
				Result := Result + indices.item_for_iteration.count
				indices.forth
			end
			indices.go_i_th (which_strip.to_integer)
			indices.item_for_iteration.go_i_th (index.to_integer)
		end
	
	
	strip_count: INTEGER_64 is
			-- The number of strips
		do
			Result := indices.count
		end
	
	
feature -- Comparison
feature -- Status report
	
	after: BOOLEAN is
			-- Is the cursor off the end?
		do
			Result := indices.item_for_iteration.after
		end
	
	strip_after: BOOLEAN is
			-- Is the cursor off the end?
		do
			Result := indices.after
		end
	
	in_range (potential_strip: INTEGER_64_ARRAYED_LIST): BOOLEAN is
			-- Are the indices in `potential_strip' in the correct range?
		require 
			potential_strip_not_void: potential_strip /= Void
		do
			Result := true
			from
				potential_strip.start
			until
				not Result or potential_strip.after
			loop
				if potential_strip.item_for_iteration <= 0 or
					potential_strip.item_for_iteration > values.count then
					Result := false
				end
				
				potential_strip.forth
			end
			
		end 

	shared_indices (test_indices: INTEGER_64_ARRAYED_LIST): BOOLEAN is
			-- Are `test_indices' shared with `indices'
		require 
			test_indices_not_void: test_indices /= Void
		do
			Result := test_indices = indices.item_for_iteration
		end 

feature -- Status setting
feature -- Cursor movement
	
	start is
			-- Move to the beginning of the elements
		do
			indices.item_for_iteration.start
		end
	
	forth is
			-- Move the cursor one element
		do
			indices.item_for_iteration.forth
		end
	
	strip_start is
			-- Move to the beginning of the elements
		do
			indices.start
			which_strip := indices.index
		end
	
	strip_forth is
			-- Move to the cursor one element forward
		do
			indices.forth
			if not indices.off then
				which_strip := indices.index
			end
		end
	
	go_i_th (i: INTEGER_64) is
			--
		do
			indices.item_for_iteration.go_i_th (i)
		end
	
feature -- Element change
	
	add_item (value: G) is
			-- add `value' as a new item
		do
			indices.item_for_iteration.force_last (values.count + 1)
			
			values.force_last (value)
		ensure then
			indices_references_value: i_th (indices.item_for_iteration.count).is_equal (value)
		end
	
	add_strip (new_strip: INTEGER_64_ARRAYED_LIST) is
			-- Add a new strip; Does not change `which_strip'
		require 
			new_strip_not_void: new_strip /= Void
			new_strip_indices_in_range: in_range (new_strip)
		do
			indices.force_last (new_strip)
		ensure 
			strip_added: indices.item (indices.count) = new_strip
		end 
	
	create_strip is
			-- Create a new strip; Move cursor to the new strip
		local
			tmp_indices: INTEGER_64_ARRAYED_LIST
		do
			create tmp_indices.make (1)
			indices.force_last (tmp_indices)
			
			which_strip := indices.count
			indices.go_i_th (which_strip.to_integer)
		end

	remove_strip is
			-- Remove a strip; Does not change `which_strip'
		do
			indices.go_i_th (which_strip.to_integer)
			indices.remove_at
			which_strip := which_strip - 1
			if which_strip = 0 and indices.count > 0 then
				which_strip := 1
			end
		end 
	
	insert_item (value: G; index: INTEGER_64) is
			-- insert `value' at `index'
		do
			indices.item_for_iteration.go_i_th (index)
			indices.item_for_iteration.put_left (values.count + 1)
			values.force_last (value)
		end
	
	replace_item (value: G; index: INTEGER_64) is
			-- replace the item at `index' with `value'
		do
			values.go_i_th (indices.item_for_iteration.item(index))
			values.replace_at (value)
		end
	
	set_strip_index (new_index: INTEGER_64) is
			-- Set `which_strip' to `new_index'
		require
			new_index_in_range: new_index >= 1 and new_index <= strip_count
		do
			which_strip := new_index
			indices.go_i_th (which_strip.to_integer)
		ensure
			which_strip_set: which_strip = new_index
		end
	
feature -- Removal
	
	remove_item (value: G) is
			-- remove the item corresponding to `value'
		local
			occurrences, index, index_of: INTEGER_64
		do
			occurrences := values.occurrences (value)
			check
				count_at_least_one: count >= 1
			end
			
			from
				index := 1 
			invariant
				index_at_least_one: index >= 1
				index_small_enough: index <= 1 + occurrences
			variant
				index_increasing: 1 + occurrences - index
			until
				index >= occurrences
			loop
				values.search_forth (value)
				index_of := values.index
				
				indices.item_for_iteration.delete (index_of)
				
				index := index + 1
			end
			
			values.start
			values.delete (value)
			
		end
	
	remove_all_values is
			-- remove all values
		do
			indices.item_for_iteration.wipe_out
			
			Precursor
		end

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	indices: DS_ARRAYED_LIST [INTEGER_64_ARRAYED_LIST]
			-- The indirect access into `values'
	
invariant
	
	indices_not_void: indices /= Void
	which_strip_in_range: which_strip >= 0 and which_strip <= strip_count
	which_strip_correct: ((indices.count > 0 and not indices.off) 
						  implies (which_strip > 0 and 
								   which_strip = indices.index))
	
end -- SCENE_INDEXED_MULTI_ELEMENT

