

indexing
	description: "State element for indexed color values"
	patterns: "none"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	status: "unreviewed"




class 

	SCENE_INDEXED_NORMAL_ELEMENT
	
inherit
	
	SCENE_NORMAL_ELEMENT
		undefine
			i_th, item,
			count, after, start, forth, remove_item, remove_all_values
		redefine
			default_create, add_item, insert_item, replace_item, make_from_list
		end
	
	SCENE_INDEXED_MULTI_ELEMENT [NORMAL]
		rename
			make as multi_make -- matches make from above
		undefine
			print_features
		redefine
			make_from_list, default_create, add_item, insert_item, replace_item
		end

creation
	default_create, make, make_from_list,
	make_from_coordinates_and_indices

feature -- Initialization

	default_create is
			-- 
		do
			Precursor {SCENE_INDEXED_MULTI_ELEMENT}
			
			is_normalized := false
		end
	
	make_from_list (vals: NORMAL_ARRAYED_LIST) is
			-- Initialize the element with `vals'
		local
			i: INTEGER_64
		do
			default_create
			values := vals
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + values.count
			variant
				i_increasing: 1 + values.count - i
			until
				i > values.count
			loop
				indices.item_for_iteration.force_last (i)
				i := i + 1
			end
		end
	
	make_from_coordinates_and_indices (vals: NORMAL_ARRAYED_LIST; 
									   new_indices: INTEGER_64_ARRAYED_LIST) is
			-- Initialize the element with `vals' and `new_indices'
		do
			default_create
			values := vals
			indices.start
			indices.replace_at (new_indices)
		end
	
feature -- Access
	
	is_normalized: BOOLEAN
			-- Have the normals been `normalized'?
	
feature -- Element change
	
	add_item (value: NORMAL) is
			-- add copy of `value' as a new item
		local
			v: NORMAL
		do
			v := value.twin
			
			Precursor {SCENE_INDEXED_MULTI_ELEMENT} (v)
			is_normalized := false
		end
	
	insert_item (value: NORMAL; index: INTEGER_64) is
			-- insert copy of `value' at `index'
		local
			v: NORMAL
		do
			v := value.twin
			
			Precursor {SCENE_INDEXED_MULTI_ELEMENT} (v, index)
			is_normalized := false
		end
	
	replace_item (value: NORMAL; index: INTEGER_64) is
			-- replace the item at `index' with a copy of `value'
		local
			v: NORMAL
		do
			v := value.twin
			
			Precursor {SCENE_INDEXED_MULTI_ELEMENT} (v, index)
			is_normalized := false
		end

feature -- Basic operations
	
	normalize is
			-- Normalize the elements to unit vectors; 0 length 
			-- normals are unmodified
		do
			from
				strip_start
			until
				strip_after
			loop
				
				from
					start
				until
					after
				loop
					if item.magnitude /= 0 then
						item.normalize
					end
					
					forth
				end
				
				strip_forth
			end
			
			is_normalized := true
		ensure 
			default: true
		end 

feature {NONE} -- Implementation
	
	normals_are_unit_length: BOOLEAN is
			-- Check if all the normals are unit length
		do
			Result := true
			from
				strip_start
			until
				strip_after or not Result
			loop
				
				from
					start
				until
					after or not Result
				loop
					Result := Result and item.is_unit
					
					forth
				end
				
				strip_forth
			end
		end 
	
invariant
	
	normals_are_unit: is_normalized implies normals_are_unit_length
	
end -- SCENE_INDEXED_NORMAL_ELEMENT

