

indexing
	description: "State element for lights. Describes which lights are currently active."
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_LIGHT_ELEMENT
	
inherit
	
	SCENE_SMALL_MULTI_ELEMENT [LIGHT_SOURCE]
		redefine
			apply, copy, is_equal, print_features, print_values
		end

creation
	default_create, make

feature -- Initialization
feature -- Access
	
	light (handle: INTEGER): LIGHT_SOURCE is
			-- Return the `light' at `handle'
		require 
			has_handle: has_handle (handle)
		do
			from
				values.start
			until
				values.after or Result /= Void
			loop
				if values.item_for_iteration.handle = handle then
					Result := values.item_for_iteration
				end
				values.forth
			end
			
		ensure 
			Result_not_void: Result /= Void
		end 

feature -- Measurement
feature -- Comparison
	
	is_equal (other: like Current): BOOLEAN is
			-- Are `Current' and `other' the same?
		do
			if count = other.count then
				Result := values.is_equal (other.values)
			end
		end 

feature -- Status report
	
	has_handle (handle: INTEGER): BOOLEAN is
			-- Is `handle' a member of this element?
		require 
			handle_is_positive: handle >= 0
		do
			Result := false
			
			from
				values.start
			until
				values.after or Result
			loop
				Result := values.item_for_iteration.handle = handle
				values.forth
			end
		end 

feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	copy (other: like Current) is
			-- Make `Current' a copy of `other'
		do
			default_create
			values.copy (other.values)
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	apply (renderer: RENDERER) is
			-- Enable the lights for this object
		do
			from
				start
			until
				after
			loop
				item.enable_light
				
				forth
			end
			
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			
			action.io.put_string ("	 lights = ")
			action.io.push
			
			print_values (action)
			
			action.io.pop
		end

	print_values (action: PRINT_ACTION) is
			-- Print the `values` feature
		local
			i: INTEGER_64
		do
			if action.print_all_values or count <= 2 * action.print_count then
				from
					values.start
				until
					values.after
				loop
					values.item_for_iteration.print_object (action)
					action.io.put_new_line
					
					values.forth
				end
			else
				-- The first `action.print_count` values
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + action.print_count
				variant
					i_increasing: 1 + action.print_count - i
				until
					i >= action.print_count
				loop
					values.item_for_iteration.print_object (action)
					action.io.put_new_line
 
					i := i + 1
				end
				
				-- The last `action.print_count` values
				from
					i := count - action.print_count 
				invariant
					i_at_least_one: i >= count - action.print_count
					i_small_enough: i <= count 
				variant
					i_increasing: count - i
				until
					i >= action.print_count
				loop
					values.item_for_iteration.print_object (action)
					action.io.put_new_line
 
					i := i + 1
				end
			end
		end 

end -- SCENE_LIGHT_ELEMENT

