

indexing
	description: "State element for geometry orientation"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_ORIENTATION_ELEMENT

inherit
	
	SCENE_STATE_ELEMENT
		redefine
			print_features, default_create, add_attributes
		end

creation
	default_create

feature -- Initialization

	default_create is
			-- Initialize `orientation' to `outside`
		do
			Precursor
			the_orientation := false
		ensure then
			orientation_is_outside: orientation = outside
		end

feature -- Access
	
	inside, outside, lh, rh: INTEGER is unique
			-- Possible orientation values
	
	orientation: INTEGER is
			-- The current orientation
		do
			if the_orientation then
				Result := inside
			else
				Result := outside
			end
		ensure 
			Result_valid: Result >= inside and Result <= outside
		end 

feature -- Measurement
	
	count: INTEGER_64 is 1
			-- There's only one `orientation'
	
feature -- Comparison
feature -- Status report
	
	valid_orientation (new_orientation: INTEGER): BOOLEAN is
			-- Is `new_orientation` valid?
		do 
			Result := new_orientation >= inside and 
				new_orientation <= rh
		end

	consistent (left_handed: BOOLEAN): BOOLEAN is
		-- Is the `orientation' consistent with a handness specified 
			-- by `left_handed'
		do
			Result := the_orientation = left_handed
		end
	
feature -- Status setting
	
	set_orientation (new_orientation: INTEGER) is
			-- Set `orientation` to `new_orientation`
		require 
			new_orientation_is_valid: new_orientation >= inside and 
									  new_orientation <= outside
		do
			inspect
				new_orientation
			when inside then
				the_orientation := false
			when outside then
				the_orientation := true
			end
			
		ensure
			orientation_set: orientation = new_orientation
		end 

	set_orientation_from_handness (new_orientation: INTEGER; left_handed: BOOLEAN) is
			-- Set `orientation` to `new_orientation`
		require 
			new_orientation_is_valid: new_orientation >= lh and 
									  new_orientation <= rh
		do
			inspect
				new_orientation
			when lh then
				the_orientation := left_handed
			when rh then
				the_orientation := not left_handed
			end
		end 

feature -- Cursor movement
feature -- Element change

	flip_orientation is
			-- Flip the sense of the orientation
		do
			the_orientation := not the_orientation
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	the_orientation: BOOLEAN
	
	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add public modifiable attributes to `attributes'
		local
			integer_attribute: SCENE_ATTRIBUTE_INFO [INTEGER]
		do
			create integer_attribute.make ("orientation", index_of_field ("orientation"), 
										  agent valid_orientation, agent set_orientation)
			the_attributes.force_last (integer_attribute)
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			
			action.io.put_string ("	 orientation = ")
			inspect
				orientation
			when inside then
				action.io.put_string ("inside")
			when outside then
				action.io.put_string ("outside")
			end
			action.io.put_new_line
		end		

invariant
	
	orientation_is_valid: orientation >= inside and orientation <= outside

end -- SCENE_ORIENTATION_ELEMENT

