

indexing
	description: "State element for shaders"
	names: "shader", "state element"
	representation: "fixed"
	access: "fixed"
	size: "fixed"
	contents: "STRING"
	patterns: "none"
	status: "unreviewed"




class 

	SCENE_SHADER_ELEMENT
	
inherit
	
	SCENE_STATE_ELEMENT
		redefine
			apply, print_features, default_create, add_attributes
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize 
		do
			Precursor
			create shader.make (1)
		end

	make (the_shader: STRING; the_parameters: PARAMETER_LIST) is
			-- Initialize the `shader' to `the_shader'
		require
			the_shader_not_void: the_shader /= Void and then the_shader.count > 0
		do
			default_create
			shader := the_shader
			parameters := the_parameters
		ensure
			shader_set: shader = the_shader
			parameters_set: parameters = the_parameters
		end

feature -- Access
	
	shader: STRING
			-- The name of the shader

	parameters: PARAMETER_LIST
			-- The parameters for the shader
	
feature -- Measurement
	
	count: INTEGER_64 is 1
			-- There's only one shader
	
feature -- Comparison
feature -- Status report
	
	valid_shader (the_shader: STRING): BOOLEAN is
			-- Is `the_shader' valid?
		do
			-- Later, add a check to see if we can find the shader
			Result := the_shader /= Void
		end
	
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_shader (the_shader: STRING) is
			-- Set `shader' to `the_shader'
		require 
			mode_is_valid: the_shader /= Void
		do
			shader := the_shader
		ensure
			shader_set: shader = the_shader
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	apply (renderer: RENDERER) is
			-- apply the `renderer' to `Current'. Default action is no-op.
		do
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add public modifiable attributes to `attributes'
		local
			string_attribute: SCENE_ATTRIBUTE_INFO [STRING]
		do
			create string_attribute.make ("shader", index_of_field ("shader"), 
										  agent valid_shader, agent set_shader)
			the_attributes.force_last (string_attribute)
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			
			action.io.put_string ("	 shader = ")
			action.io.put_string (shader)
			action.io.put_new_line
		end 
	
invariant
	valid_shader: shader /= Void
	
end -- SCENE_SHADER_ELEMENT

