

indexing
	description: "State element for shading rate"
	patterns: "none"
	status: "unreviewed"




class 

	SCENE_SHADING_RATE_ELEMENT
	
inherit
	
	SCENE_STATE_ELEMENT
		redefine
			print_features, default_create, add_attributes
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize to a shading rate of 1
		do
			Precursor
			shading_rate := 1
		ensure then
			once_per_pixel: shading_rate = 1
		end

	make (the_shading_rate: DOUBLE) is
			-- Initialize the `shading_rate' to `the_shading_rate'
		require
			strictly_positive: the_shading_rate > 0
		do
			default_create
			shading_rate := the_shading_rate
		ensure
			shading_rate_set: shading_rate = the_shading_rate
		end

feature -- Access
	
	shading_rate: DOUBLE
			-- How often do we shade per object? A shading rate of 1 
			-- is per_pixel, ri_infinity is once per object
	
feature -- Measurement
	
	count: INTEGER_64 is 1
			-- There's only one shading_rate
	
feature -- Comparison
feature -- Status report
	
	valid_shading_rate (the_shading_rate: DOUBLE): BOOLEAN is
			-- Is `the_shading_rate' valid?
		do 
			Result := the_shading_rate > 0
		end
	
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_shading_rate (the_shading_rate: DOUBLE) is
			-- Set `shading_rate' to `the_shading_rate'
		require 
			is_valid: valid_shading_rate (the_shading_rate)
		do
			shading_rate := the_shading_rate
		ensure
			shading_rate_set: shading_rate = the_shading_rate
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add public modifiable attributes to `attributes'
		local
			double_attribute: SCENE_ATTRIBUTE_INFO [DOUBLE]
		do
			create double_attribute.make ("shading_rate", index_of_field ("shading_rate"), 
										  agent valid_shading_rate, agent set_shading_rate)
			the_attributes.force_last (double_attribute)
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			
			action.io.put_string ("	 shading_rate = ")
			action.io.put_string (shading_rate.out)
			action.io.put_new_line
		end 
	
invariant
	shading_rate_strictly_positive: shading_rate > 0
	
end -- SCENE_SHADING_RATE_ELEMENT

