

indexing
	description: "State element for determining one vs. two-sided rendering"
	patterns: "none"
	status: "unreviewed"




class 

	SCENE_SIDES_ELEMENT
	
inherit
	
	SCENE_STATE_ELEMENT
		redefine
			apply, print_features, default_create, add_attributes
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize to double-sided
		do
			Precursor
			sides := 2
		ensure then
			double_sided: sides = 2
		end

	make (the_sides: INTEGER) is
			-- Initialize the `sides' to `the_sides'
		do
			default_create
			sides := the_sides
		ensure
			sides_set: sides = the_sides
		end

feature -- Access
	
	sides: INTEGER
			-- One or two-sided?
	
feature -- Measurement
	
	count: INTEGER_64 is 1
			-- There's only one sides
	
feature -- Comparison
feature -- Status report
	
	valid_sides (the_sides: INTEGER): BOOLEAN is
			-- Is `the_sides' valid?
		do 
			Result := the_sides = 1 or the_sides = 2
		end
	
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_sides (the_sides: INTEGER) is
			-- Set `sides' to `the_sides'
		require 
			mode_is_valid: the_sides = 1 or the_sides = 2
		do
			sides := the_sides
		ensure
			sides_set: sides = the_sides
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	apply (renderer: RENDERER) is
			-- apply the `renderer' to `Current'. Default action is no-op.
		do
			if sides = 2 then
				renderer.disable (renderer.rd_cull_face)
				renderer.enable_two_sided_lighting
			else
				renderer.enable (renderer.rd_cull_face)
				renderer.enable_one_sided_lighting
			end
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add public modifiable attributes to `attributes'
		local
			integer_attribute: SCENE_ATTRIBUTE_INFO [INTEGER]
		do
			create integer_attribute.make ("sides", index_of_field ("sides"), 
										  agent valid_sides, agent set_sides)
			the_attributes.force_last (integer_attribute)
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			
			action.io.put_string ("	 sides = ")
			action.io.put_integer (sides)
			action.io.put_new_line
		end 
	
invariant
	one_or_two_sided: sides = 1 or sides = 2
	
end -- SCENE_SIDES_ELEMENT

