

indexing
	description: "Abstract class for state elements with multiple elements"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 

	SCENE_SMALL_MULTI_ELEMENT [G]
	
inherit
	
	SCENE_MULTI_ELEMENT [G]
		redefine
			default_create, detailed_memory_usage
		end

feature -- Initialization

	default_create is
			-- 
		do
			Precursor
			create values.make (1)
		end
	
	make_from_list (vals: DS_ARRAYED_LIST [G]) is
			-- Initialize the element with `vals'
		require
			vals_not_void: vals /= Void
		do
			default_create
			values := vals
		ensure
			values_set: values = vals
		end
	
	make_from_element (element: like Current) is
			-- Initialize the element with `vals'
		do
			default_create
			values := element.values
		ensure then
			values_set: values = element.values
		end
	
feature -- Access
	
	i_th (i: INTEGER_64): G is
			-- 
		do
			Result := values.item (i.to_integer)
		end 

	has (v: G): BOOLEAN is
			-- Does `v' appear in array?
		do
			Result := values.has (v)
		end
	
	item: G is
			-- The item at the cursor
		do
			Result := values.item_for_iteration
		end 

	values: DS_ARRAYED_LIST [G]
			-- The element values
	
feature -- Measurement
	
	count: INTEGER_64 is
			-- The number of values
		do
			Result := values.count
		end

feature -- Comparison
feature -- Status report
	
	after: BOOLEAN is
			-- Is the cursor off the end?
		do
			Result := values.after
		end
	
feature -- Status setting
feature -- Cursor movement
	
	start is
			-- Move to the beginning of the elements
		do
			values.start
		end
	
	forth is
			-- Move to the	cursor one element
		do
			values.forth
		end
	
feature -- Element change
	
	add_item (value: G) is
			-- add `value' as a new item
		do
			values.force_last (value)
		end
	
	insert_item (value: G; index: INTEGER_64) is
			-- insert `value' at `index'
		do
			values.go_i_th (index.to_integer)
			values.put_left (value)
		end
	
	replace_item (value: G; index: INTEGER_64) is
			-- replace the item at `index' with `value'
		do
			values.go_i_th (index.to_integer)
			values.replace_at (value)
		end

feature -- Removal
	
	remove_item (value: G) is
			-- remove the item corresponding to `value'
		do
			values.start
			values.delete (value)
		end
	
	remove_all_values is
			-- remove all values
		do
			values.wipe_out
		end

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	detailed_memory_usage: INTEGER is
			-- Compute additional memory usage as required
		do
			from
				values.start
			until
				values.after
			loop
				Result := Result + object_memory_usage (values.item_for_iteration)
				values.forth
			end
		end 

	print_values (action: PRINT_ACTION) is
			-- Print the `values` feature
		local
			i: INTEGER_64
		do
			if action.print_all_values or count <= 2 * action.print_count then
				from
					values.start
				until
					values.after
				loop
					action.io.put_string (values.item_for_iteration.out)
					action.io.put_new_line
					
					values.forth
				end
			else
				-- The first `action.print_count` values
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + action.print_count
				variant
					i_increasing: 1 + action.print_count - i
				until
					i >= action.print_count
				loop
					action.io.put_string (values.item (i.to_integer).out)
					action.io.put_new_line
 
					i := i + 1
				end
				
				-- The last `action.print_count` values
				from
					i := count - action.print_count 
				invariant
					i_at_least_one: i >= count - action.print_count
					i_small_enough: i <= count 
				variant
					i_increasing: count - i
				until
					i >= action.print_count
				loop
					action.io.put_string (values.item (i.to_integer).out)
					action.io.put_new_line
 
					i := i + 1
				end
			end
		end 

invariant
	
	values_not_void: values /= Void
	
end -- SCENE_SMALL_MULTI_ELEMENT

