

indexing
	description: "Text that always faces the camera"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_BILLBOARD_TEXT
	
inherit
	EWG_EXTERNAL_STRING_ROUTINES
		undefine
			default_create, copy, is_equal
		end
	
	GDKFONT_FUNCTIONS
		undefine
			default_create, copy, is_equal
		end
	
	SCENE_GEOMETRY
		redefine
			default_create, print_features, draw
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize the object
		do
			Precursor {SCENE_GEOMETRY} 
			
			create {SCENE_SPHERE_BOUND} extents
			
			if not lists_generated then
				generate_lists := true
				display_list_base := -1
			else
				if generate_lists then
					display_list_base := -1
				else
					generate_lists := true
					-- Force creation of new display lists, but don't 
					-- change the list_base so we can delete the old ones.
				end
			end
			
			if text_string = Void then
				create text_string.make (10)
			else
				text_string.wipe_out
			end
			
			if position = Void then
				create position
			else
				position.default_create
			end
			
			draw_text_in_screenspace := false
			
			if camera_for_screenspace_rendering /= Void then
				camera_for_screenspace_rendering := Void
			end
			
			font_pointer := gdk_font_load ("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*")
		end
	
	make (new_string: STRING) is
			-- Initialize the object with `new_string'
		require
			new_string_not_void: new_string /= Void
		do
			default_create
			
			text_string := new_string.twin
		ensure
			text_string_is_set: text_string.is_equal (new_string)
		end

feature -- Access
	
	text_string: STRING
			-- The information to print
	
	extents: SCENE_BOUND
			-- The `extents' or bounds of the volume.
	
	position: POINT3
			-- The origin of the text string
	
	camera_for_screenspace_rendering: SCENE_CAMERA
			-- A camera to be used when rendering screenspace text
	
feature -- Measurement
	
	count: INTEGER_64 is
			-- The number of characters
		do
			Result := 1
		end
	
	mapping_count: INTEGER_64 is
			-- The number of element for mapping colors
		do
			inspect color_mapping
			when constant_color then
				Result := 1
			when per_object, per_element, per_vertex then
				Result := count
			end
		end 
	
feature -- Comparison
feature -- Status report
	
	has_extents: BOOLEAN is
			-- Does this volume have extents?
		do
			Result := extents /= Void
		end
	
	draw_text_in_screenspace: BOOLEAN
			-- Draw the text in screenspace, i.e., immune to transformations
	
feature -- Status setting
	
	set_screenspace_drawing (flag: BOOLEAN) is
			-- Set `draw_text_in_screenspace' to `flag'
		do
			draw_text_in_screenspace := flag
		ensure 
			draw_text_in_screenspace_set: draw_text_in_screenspace = flag
		end 

feature -- Cursor movement
feature -- Element change
	
	set_position (new_position: POINT3) is
			-- Set `position' to `new_position'
		require 
			new_position_not_void: new_position /= Void
		do
			position := new_position.twin
			
			update_display_lists
		ensure 
			position_set: position.is_equal (new_position)
		end 
	
	set_text_string (new_text_string: STRING) is
			-- Set `text_string' to `new_text_string'
		require 
			new_text_string_not_void: new_text_string /= Void
		do
			text_string := new_text_string.twin
			
			update_display_lists
		ensure 
			text_string_set: text_string.is_equal (new_text_string)
		end		
	
	set_screenspace_camera (new_camera: SCENE_CAMERA) is
			-- Set `camera_for_screenspace_rendering' to `new_camera'
		require 
			camera_not_void: new_camera /= Void
		do
			camera_for_screenspace_rendering := new_camera
		ensure 
			camera_for_screenspace_rendering_set: camera_for_screenspace_rendering = new_camera
		end 

	update_extents is
			-- Recompute the extents of the object
		do
		end
		
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		do
			check
				implemented: false
			end
		end
	
	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end -- tesselate

	draw (renderer: RENDERER) is
			-- Apply the renderer to this object
		do
			if draw_text_in_screenspace then
				draw_in_screenspace (renderer)
			else
				draw_immediate (renderer)
			end
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	font_pointer: POINTER
			-- Pointer to external font record
	
	display_list_base: INTEGER
	
	generate_lists: BOOLEAN
			-- Do I need to generate display lists?
	
	lists_generated: BOOLEAN
			-- Have display lists been generated?
	
	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			sce: SCENE_COLOR_ELEMENT
		do
			if generate_lists then
				if lists_generated then
					renderer.delete_lists (128, display_list_base)
				end
				display_list_base := renderer.generate_lists (128)
				
				renderer.load_font (font_structure (font_pointer), 0, 128, display_list_base)
				generate_lists := false
				lists_generated := true
			end
			
			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
			
			if color_mapping = constant_color then
				renderer.rgba_color (sce.i_th (1))
			end
			
			renderer.disable (renderer.rd_lighting)
			renderer.raster_position_3 (position.x, position.y, position.z)
			renderer.push_attribute (renderer.rd_list_bit)
			renderer.list_base (display_list_base)
			renderer.call_lists(text_string.count, renderer.rd_unsigned_byte,
								string_to_pointer (text_string))
			renderer.pop_attribute
			renderer.enable (renderer.rd_lighting)
		end
	
	draw_in_screenspace (renderer: RENDERER) is
			-- Output an annotation string to the display
		do
			renderer.disable (renderer.rd_depth_test)
			renderer.matrix_mode (renderer.rd_projection)
			renderer.push_matrix
			renderer.load_identity
			renderer.ortho (0, camera_for_screenspace_rendering.x_resolution, 
							0, camera_for_screenspace_rendering.y_resolution, -1, 1)
			renderer.matrix_mode (renderer.rd_modelview)
			renderer.load_identity
			renderer.enable (renderer.rd_blend)
			
			draw_immediate (renderer)
			
			renderer.disable (renderer.rd_blend)
			renderer.matrix_mode (renderer.rd_projection)
			renderer.pop_matrix
			renderer.matrix_mode (renderer.rd_modelview)
			renderer.enable (renderer.rd_depth_test)
		end
	
	font_structure (font: POINTER): INTEGER is
			--
		external
			"C use <gdk/gdk.h>"
		alias
			"gdk_font_id"
		end
	
	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray', 
			-- against this volume. It is assumed that the ray has 
			-- been transformed into this object's coordinate system
		do
			check
				implemented: false
			end
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the group's features
		do
			Precursor (action)
			
			action.io.put_string ("	 text_string = ")
			action.io.put_string (text_string)
			action.io.put_new_line
		end
	
	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information 
			-- into `intersection'
		do
			check
				never_called: false
			end
		end

invariant
	
	position_not_void: position /= Void
	text_string_not_void: text_string /= Void
	camera_set_if_drawing_in_screenspace: (draw_text_in_screenspace implies
										   camera_for_screenspace_rendering /= Void)
	
end -- SCENE_BILLBOARD_TEXT

