

indexing
	description: "Cone Primitive"
	patterns: "none"
	date: "$Date: 2006-12-19 16:00:24 -0500 (Tue, 19 Dec 2006) $"
	revision: "$Revision: 275 $"
	status: "unreviewed"




class

	SCENE_CONE

inherit

	SCENE_QUADRIC
		rename
			make_by_parameters as quadric_make_by_parameters,
			z_maximum as height,
			set_z_maximum as set_height,
			valid_z_maximum as valid_height
		export
			{NONE} set_z_minimum, quadric_make_by_parameters
		redefine
			default_create, compute_intermediates, quadric_make_by_parameters,
			add_attributes
		end

creation
	default_create, make_by_parameters

feature -- Initialization

	default_create is
			-- Create a cone of radius 1 and height 1
		do
			Precursor

			height := 1
			z_minimum := 0

			compute_intermediates
		end

	quadric_make_by_parameters (r, zmin, zmax, theta: DOUBLE) is
			-- Create a cone of radius `r', with height
			-- `zmax' - `zmin', and sweep angle `theta'
			-- (closed quadric = 360)
		do
			make_by_parameters (r, zmax - zmin, theta)
		end

	make_by_parameters (r, the_height, theta: DOUBLE) is
			-- Create a quadric of radius `r', with bottom
			-- `zmin', top	`zmax', and sweep angle `theta'
			-- (closed quadric = 360)
		do
			default_create

			radius := r
			height := the_height
			sweep_angle := theta * pi / 180.0

			compute_intermediates

			set_wrapping
		end

feature -- Element change

	update_extents is
			-- Recompute the extents of the object
		local
			local_max: DOUBLE
			rmin, rmax, tmin, tmax: DOUBLE
			b: SCENE_BOX_BOUND
		do
			local_max := radius
			if local_max < height.abs/2 then
				local_max := height.abs/2
			end

			local_max := local_max * local_max
			radius_squared := radius * radius

			rmin := radius * (1 - v_maximum)
			rmax := radius * (1 - v_minimum)

			tmin := u_minimum * sweep_angle
			tmax := u_maximum * sweep_angle

			create {SCENE_BOX_BOUND} extents
			sweep_for_bounds (rmin, rmax, tmin, tmax)

			b ?= extents

			b.lower.put_z (v_minimum * height)
			b.upper.put_z (v_maximum * height)
		end

feature -- Basic operations

	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		local
			pt: POINT3
			dpdu, dpdv: VECTOR3
			normal: NORMAL
			theta: DOUBLE
			sce: SCENE_COLOR_ELEMENT
		do
			theta := u * sweep_angle
			pt := result_data.point
			pt.make (cosine (theta) * radius * (1 - v),
					 sine (theta) * radius * (1 - v),
					 v * height)
			pt.apply (object_to_world)
			result_data.set_two_sided (two_sided)

			if not points_only then
				create dpdu.make (sweep_angle * -sine (theta) * radius * (1 - v),
								  sweep_angle * cosine (theta) * radius * (1 - v),
								  0)
				create dpdv.make (-cosine (theta) * radius,
								  -sine (theta) * radius,
								  height)

				result_data.set_derivatives (dpdu, dpdv)

				create normal.make (cosine (theta) * height,
									sine (theta) * height,
									radius)
				normal.apply (object_to_world)
				normal.normalize
				result_data.set_geometric_normal (normal)

				sce := get_colors
					check
						color_element_not_void: sce /= Void
					end
				result_data.set_color (sce.i_th (1))
			end
		end

	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end -- tesselate

feature {NONE} -- Implementation

	radius_squared: DOUBLE

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			sce: SCENE_COLOR_ELEMENT
		do
			renderer.push_matrix
			renderer.translate (origin.x, origin.y, origin.z)

			if	is_left_handed then
				renderer.front_face (renderer.rd_cw)
			else
				renderer.front_face (renderer.rd_cw)
			end

			if get_orientation.consistent (is_left_handed) then
				renderer.orientation (renderer.rd_outside)
			else
				renderer.orientation (renderer.rd_inside)
			end

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
			renderer.enable (renderer.rd_color_material_constant)
			renderer.color_material (renderer.rd_front_and_back,
							   renderer.rd_ambient_and_diffuse)

			renderer.rgba_color (sce.i_th (1))

			renderer.cone (radius, height, sweep_angle,
							u_minimum, u_maximum, v_minimum, v_maximum)
			renderer.front_face (renderer.rd_ccw)
			renderer.pop_matrix
		end

	compute_intermediates is
			--
		do
			Precursor

			update_extents
		end

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray',
			-- against this volume. It is assumed that the ray has
			-- been transformed into this object's coordinate system
		local
			A, B, C, inv2A, discrim, root_discrim: DOUBLE
			t0, t1, min_t, max_t: DOUBLE
			hit_pt: POINT3
			u, v, k: DOUBLE
		do
			intersected := false

			-- Eventually, min_t and max_t should be a parameter to
			-- this routine.
			min_t := 0
			max_t := maximum_double

			k := radius_squared / (height * height)
			A := intersecter.ray.direction.x * intersecter.ray.direction.x +
				intersecter.ray.direction.y * intersecter.ray.direction.y -
				k * intersecter.ray.direction.z * intersecter.ray.direction.z
			B := 2.0 * (intersecter.ray.direction.x * intersecter.ray.origin.x +
						intersecter.ray.direction.y * intersecter.ray.origin.y -
						k * intersecter.ray.direction.z * (intersecter.ray.origin.z - height) )
			C := intersecter.ray.origin.x * intersecter.ray.origin.x +
				intersecter.ray.origin.y * intersecter.ray.origin.y -
				k * (intersecter.ray.origin.z - height) * (intersecter.ray.origin.z - height)

			discrim := B * B - 4 * A * C
			if discrim >= 0 then
				root_discrim := square_root (discrim)
				inv2A := 1.0 / (2.0 * A)
				t0 := (-B - root_discrim) * inv2A
				if t0 <= max_t then
					t1 := (-B + root_discrim) * inv2A
					if t1 >= min_t then
						-- At least one intersection is in range
						-- So, Check the first potential intersection

						if t0 >= min_t then
							hit_pt := intersecter.ray.project (t0)
							if hit_pt.z >= z_minimum and hit_pt.z <= height then
								u := 1 - (arc_tangent2 (hit_pt.y, -hit_pt.x) + pi) / sweep_angle
								v := (hit_pt.z - z_minimum) / (height - z_minimum)
								if u <= 1 then
									insert_intersection_information (intersecter, u, v, t0,
																	 hit_pt, intersecter.ray)
									intersected := true
								end
							end
						end

						-- Now, check the second potential intersection
						if t1 <= max_t then
							hit_pt := intersecter.ray.project (t1)
							if hit_pt.z >= z_minimum and hit_pt.z <= height then
								u := 1 - (arc_tangent2 (hit_pt.y, -hit_pt.x) + pi) / sweep_angle
								v := (hit_pt.z - z_minimum) / (height - z_minimum)
								if u <= 1 then
									insert_intersection_information (intersecter, u, v, t1,
																	 hit_pt, intersecter.ray)
									intersected := true
								end
							end
						end
					end
				end
			end
		end

	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		local
			normal: NORMAL
			info: INTERSECTION_DATA
			theta: DOUBLE
			sce: SCENE_COLOR_ELEMENT
		do
			theta := u * sweep_angle
			create normal.make (cosine (theta) * height,
								sine (theta) * height,
								radius)
			normal.normalize

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end

			create info.make (t, point, normal,
							  u, v, sce.i_th (1), ray,
							  Current)
			action.intersection.force_last (info)
		end

	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add attributes for the GUI_ACTION
		local
			attribute: SCENE_ATTRIBUTE_INFO [DOUBLE]
		do
			create attribute.make ("radius", index_of_field ("radius"),
									agent valid_radius, agent set_radius)
			the_attributes.force_last (attribute)
			create attribute.make ("height", index_of_field ("height"),
									agent valid_height, agent set_height)
			the_attributes.force_last (attribute)
			create attribute.make ("sweep_angle", index_of_field ("sweep_angle"),
									agent valid_sweep_angle, agent set_sweep_angle)
			the_attributes.force_last (attribute)
		end

end -- SCENE_CONE

