

indexing
	description: "Base for all classes needing indexed coordinates"
	patterns: "none"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	status: "unreviewed"




class 

	SCENE_COORDINATES

inherit

	ANY
		redefine
			default_create
		end

feature -- Initialization

	default_create is
			-- Initialize the coordinate arrays
		do
			create coordinates.make (10)
			create coordinate_indices.make (1)
		end

	make_with_coordinates (new_coordinates: POINT3_ARRAYED_LIST; 
												 new_coordinate_indices: INTEGER_64_ARRAYED_LIST) is
			-- 
		require 
			new_coordinates_not_void: new_coordinates /= Void
			new_coordinate_indices_not_void: new_coordinate_indices /= Void
		do
			default_create
			
			update_coordinates (new_coordinates, new_coordinate_indices)
		end

feature -- Access

	coordinates: POINT3_ARRAYED_LIST
	coordinate_indices: DS_ARRAYED_LIST [INTEGER_64_ARRAYED_LIST]
	
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_coordinates_and_indices (new_coordinates: POINT3_ARRAYED_LIST; 
					 new_coordinate_indices: INTEGER_64_ARRAYED_LIST) is
			-- Update `coordinates' and `coordinate_indices' to 
			-- `new_coordinates' and `new_coordinate_indices'.
		require 
			new_coordinates_not_void: new_coordinates /= Void
			new_coordinate_indices_not_void: new_coordinate_indices /= Void
		do
			update_coordinates (new_coordinates, new_coordinate_indices)	
		end

	set_coordinates (new_coordinates: POINT3_ARRAYED_LIST) is
			-- Update `coordinates'	 to `new_coordinates'.
		require 
			new_coordinates_not_void: new_coordinates /= Void
		do
			coordinates := new_coordinates
		end

	add_indices (new_coordinate_indices: INTEGER_64_ARRAYED_LIST) is
			-- Update `coordinate_indices' to `new_coordinate_indices'.
		require 
			new_coordinate_indices_not_void: new_coordinate_indices /= Void
		do
			coordinate_indices.force_last (new_coordinate_indices)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	update_coordinates (new_coordinates: POINT3_ARRAYED_LIST; 
						new_coordinate_indices: INTEGER_64_ARRAYED_LIST) is
			-- Update the coordinate lists
		require 
			new_coordinates_not_void: new_coordinates /= Void
			new_coordinate_indices_not_void: new_coordinate_indices /= Void
		do
			coordinates := new_coordinates
			if coordinate_indices.count = 0 then
				coordinate_indices.force_last (new_coordinate_indices)
			else
				coordinate_indices.wipe_out
				coordinate_indices.put_last (new_coordinate_indices)
			end
		end 

invariant
	
	coordinates_not_void: coordinates /= Void
	coordinate_indices_not_void: coordinate_indices /= Void

end -- SCENE_COORDINATES

