

indexing
	description: "Cylinder Primitive"
	patterns: "none"
	date: "$Date: 2006-12-10 07:10:42 -0500 (Sun, 10 Dec 2006) $"
	revision: "$Revision: 274 $"
	status: "unreviewed"




class

	SCENE_CYLINDER

inherit

	SCENE_QUADRIC
		redefine
			compute_intermediates
		end

creation
	default_create, make_by_parameters

feature -- Basic operations

	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		local
			pt: POINT3
			dpdu, dpdv: VECTOR3
			normal: NORMAL
			theta: DOUBLE
			sce: SCENE_COLOR_ELEMENT
		do
			theta := u * sweep_angle
			pt := result_data.point
			pt.make (cosine (theta) * radius,
								sine (theta) * radius,
								z_minimum + v * (z_maximum - z_minimum))
			result_data.set_two_sided (two_sided)

			if not points_only then
				create dpdu.make (sweep_angle * -sine (theta) * radius,
								  sweep_angle * cosine (theta) * radius,
								  0)
				create dpdv.make (0, 0, z_maximum - z_minimum)

				result_data.set_derivatives (dpdu, dpdv)

				create normal.make_from_vector (pt - pt.zero)
				normal.put_z (0)
				normal.apply (object_to_world)
				normal.normalize
				result_data.set_geometric_normal (normal)

				sce := get_colors
					check
						color_element_not_void: sce /= Void
					end
				result_data.set_color (sce.i_th (1))
			end

			pt.apply (object_to_world)
			result_data.set_point (pt)
		end

	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end -- tesselate

feature -- Element change

	update_extents is
			--
		local
			local_max, tmin, tmax: DOUBLE
			b: SCENE_BOX_BOUND
		do
			local_max := radius
			if local_max < z_minimum.abs then
				local_max := z_minimum.abs
			end
			if local_max < z_maximum.abs then
				local_max := z_maximum.abs
			end

			local_max := local_max * local_max
			radius_squared := radius * radius

			tmin := u_minimum * sweep_angle
			tmax := u_maximum * sweep_angle

			create {SCENE_BOX_BOUND} extents
			sweep_for_bounds (radius, radius, tmin, tmax)

			b ?= extents

			b.lower.put_z (z_minimum + v_minimum * (z_maximum - z_minimum))
			b.upper.put_z (z_minimum + v_maximum * (z_maximum - z_minimum))
		end

feature {NONE} -- Implementation

	radius_squared: DOUBLE

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			sce: SCENE_COLOR_ELEMENT
		do
			renderer.push_matrix
			renderer.translate (origin.x, origin.y, origin.z)

			if not is_left_handed then
				renderer.front_face (renderer.rd_cw)
			else
				renderer.front_face (renderer.rd_cw)
			end

			if get_orientation.consistent (is_left_handed) then
				renderer.orientation (renderer.rd_outside)
			else
				renderer.orientation (renderer.rd_inside)
			end

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
			renderer.enable (renderer.rd_color_material_constant)
			renderer.color_material (renderer.rd_front_and_back,
							   renderer.rd_ambient_and_diffuse)

			renderer.rgba_color (sce.i_th (1))

			renderer.cylinder (radius, radius, z_minimum, z_maximum, sweep_angle,
								u_minimum, u_maximum, v_minimum, v_maximum)
			renderer.front_face (renderer.rd_ccw)
			renderer.pop_matrix
		end

	compute_intermediates is
			--
		do
			Precursor

			update_extents
		end

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray',
			-- against this volume. It is assumed that the ray has
			-- been transformed into this object's coordinate system
		local
			A, B, C, inv2A, discrim, root_discrim: DOUBLE
			t0, t1, min_t, max_t: DOUBLE
			hit_pt: POINT3
			u, v: DOUBLE
		do
			intersected := false

			-- Eventually, min_t and max_t should be a parameter to
			-- this routine.
			min_t := 0
			max_t := maximum_double

			A := intersecter.ray.direction.x * intersecter.ray.direction.x + intersecter.ray.direction.y * intersecter.ray.direction.y
			B := 2.0 * (intersecter.ray.direction.x * intersecter.ray.origin.x + intersecter.ray.direction.y * intersecter.ray.origin.y)
			C := intersecter.ray.origin.x * intersecter.ray.origin.x + intersecter.ray.origin.y * intersecter.ray.origin.y - radius_squared

			discrim := B * B - 4 * A * C
			if discrim >= 0 then
				root_discrim := square_root (discrim)
				inv2A := 1.0 / (2.0 * A)
				t0 := (-B - root_discrim) * inv2A
				if t0 <= max_t then
					t1 := (-B + root_discrim) * inv2A
					if t1 >= min_t then
						-- At least one intersection is in range
						-- So, Check the first potential intersection

						if t0 >= min_t then
							hit_pt := intersecter.ray.project (t0)
							if hit_pt.z >= z_minimum and hit_pt.z <= z_maximum then
								u := 1 - (arc_tangent2 (hit_pt.y, -hit_pt.x) + pi) / sweep_angle
								v := (hit_pt.z - z_minimum) / (z_maximum - z_minimum)
								if u <= 1 then
									insert_intersection_information (intersecter, u, v, t0,
																	 hit_pt, intersecter.ray)
									intersected := true
								end
							end
						end

						-- Now, check the second potential intersection
						if t1 <= max_t then
							hit_pt := intersecter.ray.project (t1)
							if hit_pt.z >= z_minimum and hit_pt.z <= z_maximum then
								u := 1 - (arc_tangent2 (hit_pt.y, -hit_pt.x) + pi) / sweep_angle
								v := (hit_pt.z - z_minimum) / (z_maximum - z_minimum)
								if u <= 1 then
									insert_intersection_information (intersecter, u, v, t1,
																	 hit_pt, intersecter.ray)
									intersected := true
								end
							end
						end
					end
				end
			end
		end

	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		local
			normal: NORMAL
			info: INTERSECTION_DATA
			sce: SCENE_COLOR_ELEMENT
		do
			create normal.make_from_vector (point - point.zero)

			normal.put_z (0)

			normal.normalize

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end

			create info.make (t, point, normal,
							  u, v, sce.i_th (1), ray,
							  Current)
			action.intersection.force_last (info)
		end

end -- SCENE_CYLINDER

