

indexing
	description: "A collection of geometry elements, typically the result of a split operation"
	names: "collection", "geometry"
	representation: "list"
	access: "iteration"
	size: "resizable"
	contents: "SCENE_GEOMETRY"
	patterns: "none"
	status: "unreviewed"




class

	SCENE_GEOMETRIC_COLLECTION

inherit

	SCENE_DRAWABLE_ELEMENT
		rename
			item as scene_element_item
		redefine
			default_create, intersect
		end

	SCENE_COLLECTION [SCENE_DRAWABLE_ELEMENT]
		undefine
			print_features, is_equal, copy, detailed_memory_usage
		redefine
			default_create, intersect
		end

creation
	default_create

feature -- Initialization

	default_create is
			-- Initialize the collection
		do
			Precursor {SCENE_DRAWABLE_ELEMENT}
			Precursor {SCENE_COLLECTION}

			create object_to_world
		end

feature -- Access

	object_to_world: TRANSFORM

	two_sided: BOOLEAN is
			-- Are all of the element of the object two-sided?
		do
			from
				children.start
				Result := children.item_for_iteration.two_sided
				children.forth
			until
				children.after or not Result
			loop
				Result := children.item_for_iteration.two_sided
				children.forth
			end
		end

feature -- Measurement
feature -- Comparison
feature -- Status report

	test_diceability (the_renderer: MICROPOLYGON_RENDERER) is
			-- Is the resulting of dicing this object "small enough"?
		do
			check
				implemented: false
			end
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	cull (action: CULLING_ACTION) is
			-- A traversal that culls geometry not in the viewing
			-- frustrum
		do
			from
				children.start
			until
				children.after
			loop
				children.item_for_iteration.cull (action)
				children.forth
			end
		end

	dice (the_grid: MICROPOLYGON_GRID; points_only: BOOLEAN) is
			-- Dice this object onto `the_grid'
		do
			check
				implemented: false
			end
		end

	patch_grid is
			-- For the grid associatied with this object, create a
			-- filler to deal with displacement cracks. If the
			-- neighbors haven't been diced, wait and patch from the
			-- other side.
		do
			from
				children.start
			until
				children.after
			loop
				children.item_for_iteration.patch_grid
				children.forth
			end
		end

	draw (renderer: RENDERER) is
			-- Apply the renderer to this object.
		do
			from
				children.start
			until
				children.after
			loop
				children.item_for_iteration.draw (renderer)
				children.forth
			end
		end

	intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		do
			intersected := false

			extents.intersect_ray (intersecter.ray)
			if not extents.outside then
				from
					children.start
				until
					children.after
				loop
					children.item_for_iteration.intersect (intersecter)
					children.forth
				end
			end
		end

	split is
			-- Split the object into smaller pieces
		do
			check
				implemented: false
			end
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		do
			check
				implemented: false
			end
		end

	traverse_node (action: SCENE_ACTION) is
			-- Traverse this node
		do
			check
				implemented: false
			end
		end

end -- SCENE_GEOMETRIC_COLLECTION

