

indexing
	description: "Abstract class for geometry objects"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2007-04-18 16:11:07 -0400 (Wed, 18 Apr 2007) $"
	revision: "$Revision: 289 $"




deferred class

	SCENE_GEOMETRY

inherit

	SCENE_DRAWABLE_ELEMENT
		redefine
			invalidate, print_object, default_create
		end

	NUMERIC_CONVERSION
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end

feature -- Initialization

	default_create is
			-- Initialize common geometry features
		do
			Precursor {SCENE_DRAWABLE_ELEMENT}

			create_color_element
			create_orientation_element

			create world_to_object
			create object_to_world

			color_mapping := constant_color

			create patch_points.make (1)
		end

feature -- Access

	parameters: PARAMETER_LIST
			-- Additional parameters for the geometry

	patch_points: DS_ARRAYED_LIST [SHADING_DATA]
			-- List of points to fill gap

	world_to_object: TRANSFORM
			-- Conversion from world coordinates to object coordinates

	object_to_world: TRANSFORM
			-- Conversion from object coordinates to world coordinates

	i_th_bound (index: INTEGER): AXIS_ALIGNED_BOX is
			-- Return a bound around the `index'th element
		require
--			index_correct: index >= 1 and index <= count
		do
			Result := extents.circumscribed_box
		ensure
			bound_not_void: Result /= Void
		end

	orientation: SCENE_ORIENTATION_ELEMENT is
			-- Return the orientation
		do
			if cached_orientation = Void then
				cached_orientation ?=
				element_item ("SCENE_ORIENTATION_ELEMENT")
			end

			Result := cached_orientation
		ensure
			Result_not_void: Result /= Void
		end

	two_sided: BOOLEAN is
			-- Is the object one-sided or two-sided?
		do
			if cached_sides = Void then
				cached_sides ?=
				element_item ("SCENE_SIDES_ELEMENT")
			end

			Result := cached_sides = Void or else cached_sides.sides = 2
		end

	extents_in_world_space: SCENE_BOUND is
			-- The `extents' in world space
			do
				Result := extents.apply (object_to_world)
			end

feature -- Measurement

	count: INTEGER_64 is
			-- The number of items
		deferred
		end

	mapping_count: INTEGER_64 is
			-- The number of elements with respect to mapping colors,
			-- normals, ...
		deferred
		end

feature -- Comparison

feature -- Status report

	constant_color, per_object, per_element, per_vertex: INTEGER is unique

	color_mapping: INTEGER
			-- How are colors mapped to this object

	normal_mapping: INTEGER
			-- How are normals mapped to this object

	is_consistent: BOOLEAN is
			-- Is the mapping parameters consistent with the
			-- appropriate state elements?
		do
			Result := element_item ("SCENE_COLOR_ELEMENT").count = mapping_count
			if valid_key ("SCENE_OPACITY_ELEMENT") then
				Result := Result and
					element_item ("SCENE_OPACITY_ELEMENT").count = mapping_count
			end
		end

	has_color_element: BOOLEAN is
			-- Does this geometry have an associated
			-- SCENE_COLOR_ELEMENT (or its descendants?
			-- (The answer better be yes!)
		do
			Result := valid_key ("SCENE_COLOR_ELEMENT") or valid_key ("SCENE_INDEXED_COLOR_ELEMENT")
		end

	has_lights: BOOLEAN is
			-- Does this geometry have any associated lights?
		do
			Result := valid_key ("SCENE_LIGHT_ELEMENT")
		end

	test_diceability (the_renderer: MICROPOLYGON_RENDERER) is
			-- Is the resulting of dicing this object "small enough"?
		local
			pt1, pt2, pt3: POINT3
			i, j: INTEGER
			xform: TRANSFORM
			point_list: DS_ARRAYED_LIST [POINT3]
			test_grid: MICROPOLYGON_GRID
		do
			test_grid := the_renderer.test_grid

			dice (test_grid, true)
			diced := false
			-- The output from `dice' is in world space'
			xform := the_renderer.camera.world_to_raster
			pt1 := xform.apply (test_grid.item (1, 1).point)
			pt2 := xform.apply (test_grid.item (2, 1).point)
			pt3 := xform.apply (test_grid.item (1, 2).point)
			max_u_dist := square_root (square(pt2.x - pt1.x) + square(pt2.y - pt1.y))
			max_v_dist := square_root (square(pt3.x - pt1.x) + square(pt3.y - pt1.y))

			requested_grid_x_dim := real_64_truncated_to_integer ((test_grid.width * max_u_dist) / shading_rate)
			requested_grid_y_dim := real_64_truncated_to_integer ((test_grid.height * max_v_dist) / shading_rate)

			requested_grid_x_dim := requested_grid_x_dim.max (4)
			requested_grid_y_dim := requested_grid_y_dim.max (4)

			if requested_grid_x_dim < the_renderer.maximum_grid_size and
				requested_grid_y_dim < the_renderer.maximum_grid_size and
				requested_grid_x_dim * requested_grid_y_dim < the_renderer.maximum_grid_size then
				create point_list.make (test_grid.width)
				
				-- Compute and store the points and u distances in the
				-- first row of points
				from
					i := 2
					pt1 := xform.apply (test_grid.item (1, 1).point)
					point_list.force (pt1, 1)
					max_u_dist := -1
				invariant
					i_at_least_one: i >= 2
					i_small_enough: i <= 2 + test_grid.width
				variant
					i_increasing: 2 + test_grid.width - i
				until
					i > test_grid.width
				loop
					pt2 := xform.apply (test_grid.item (i, 1).point)
					point_list.force (pt2, i)
					max_u_dist := max_u_dist.max (square(pt2.x - pt1.x) + square(pt2.y - pt1.y))
					pt1 := pt2
					i := i + 1
				end
				
				-- Compute the v distances in the
				-- last column of points
				from
					i := 2
					pt1 := xform.apply (test_grid.item (test_grid.width, 1).point)
					max_v_dist := -1
				invariant
					i_at_least_one: i >= 2
					i_small_enough: i <= 2 + test_grid.height
				variant
					i_increasing: 2 + test_grid.height - i
				until
					i > test_grid.height
				loop
					pt2 := xform.apply (test_grid.item (test_grid.width, i).point)
					max_v_dist := max_v_dist.max (square(pt2.x - pt1.x) + square(pt2.y - pt1.y))
					pt1 := pt2
					i := i + 1
				end
				
				-- For each remaining row, compute the max distances in u
				-- and v
				from
					j := 2
				invariant
					j_at_least_one: j >= 2
					j_small_enough: j <= 2 + test_grid.height
				variant
					j_increasing: 2 + test_grid.height - j
				until
					j > test_grid.height
				loop
					from
						i := 1
						pt2 := xform.apply (test_grid.item (i, j).point)
					invariant
						i_at_least_one: i >= 1
						i_small_enough: i <= 1 + test_grid.width
					variant
						i_increasing: 1 + test_grid.width - i
					until
						i >= test_grid.width
					loop
						pt1 := point_list.item (i)
						pt3 := xform.apply (test_grid.item (i + 1, j).point)
						max_v_dist := max_v_dist.max (square(pt2.x - pt1.x) + square(pt2.y - pt1.y))
						max_u_dist := max_u_dist.max (square(pt3.x - pt2.x) + square(pt3.y - pt2.y))
						
						point_list.replace (pt3, i)
						pt2 := pt3
						
						i := i + 1
					end
					j := j + 1
				end
				
				max_u_dist := square_root (max_u_dist)
				max_v_dist := square_root (max_v_dist)
				
				requested_grid_x_dim := real_64_truncated_to_integer ((test_grid.width * max_u_dist) / shading_rate)
				requested_grid_y_dim := real_64_truncated_to_integer ((test_grid.height * max_v_dist) / shading_rate)
				
				requested_grid_x_dim := requested_grid_x_dim.max (4)
				requested_grid_y_dim := requested_grid_y_dim.max (4)
				
				if requested_grid_x_dim < the_renderer.maximum_grid_size and
					requested_grid_y_dim < the_renderer.maximum_grid_size and
					requested_grid_x_dim * requested_grid_y_dim < the_renderer.maximum_grid_size then
					diceable := true
				else
					diceable := false
				end
			else
				diceable := false
			end
		ensure then
			dimensions_in_bounds: diceable implies (requested_grid_x_dim > 0 and
												  requested_grid_y_dim > 0)
		end

feature -- Status setting

	set_color_mapping (how: INTEGER) is
			-- Set how to interpret the color mapping
		require
			how_is_in_range: how >= constant_color and how <= per_vertex
		do
			color_mapping := how
			update_display_lists
		ensure
			color_mapping_is_set: color_mapping = how
		end

	set_normal_mapping (how: INTEGER) is
			-- Set how to interpret the normal mapping
		require
			how_is_in_range: how >= per_object and how <= per_vertex
		do
			normal_mapping := how
			update_display_lists
		ensure
			normal_mapping_is_set: normal_mapping = how
		end

	set_parameter (params: PARAMETER_LIST) is
			-- Set `parameters' to `params'. `params' may be void, so
			-- `parameters' references `params'. No copying is performed
		do
			parameters := params
		ensure
			parameters_set: parameters = params
		end

feature -- Cursor movement
feature -- Element change

	set_world_to_object_transform (new_transform: TRANSFORM) is
			-- Set `world_to_object' to `new_transform'
		require
			new_transform_not_void: new_transform /= Void
		do
			world_to_object := new_transform
			object_to_world := world_to_object.inverse
		ensure
			world_to_object_set: world_to_object = new_transform
			inverse_set: equal (object_to_world.inverse, world_to_object)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	cull (action: CULLING_ACTION) is
			-- A traversal that culls geometry not in the viewing
			-- frustrum
		do
			action.push_state (Current)
			action.add_drawable (Current)
			action.pop_state
		end

	dice (the_grid: MICROPOLYGON_GRID; points_only: BOOLEAN) is
			-- Dice this object onto `the_grid'
		local
			i, j: INTEGER
			u,v, du, dv: DOUBLE
			u_min, u_max, v_min, v_max: DOUBLE
		do
			du := (u_maximum - u_minimum) / (the_grid.width - 1)
			dv := (v_maximum - v_minimum) / (the_grid.height - 1)
			u_min := u_minimum + 0.15 * du
			u_max := u_maximum - 0.15 * du
			v_min := v_minimum + 0.15 * dv
			v_max := v_maximum - 0.15 * dv
			du := (u_max - u_min) / (the_grid.width - 1)
			dv := (v_max - v_min) / (the_grid.height - 1)
			from
				i := 1
				u := u_min
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + the_grid.width
				u_greater_than_minimum: u >= u_min
				u_less_than_maximum: u <= u_max + du + Tolerance
			variant
				i_increasing: 1 + the_grid.width - i
			until
				i > the_grid.width
			loop
				from
					j := 1
					v := v_min
				invariant
					j_at_least_one: j >= 1
					j_small_enough: j <= 1 + the_grid.height
					v_greater_than_minimum: v >= v_min
					v_less_than_maximum: v <= v_max + dv + Tolerance
				variant
					j_increasing: 1 + the_grid.height - j
				until
					j > the_grid.height
				loop
					evaluate (u.min (u_max), v.min (v_max), points_only, the_grid.item (i, j))
					v := v + dv
					j := j + 1
				end
				u := u + du
				i := i + 1
			end

			set_grid (the_grid)
			diced := true
		end

	patch_grid is
			-- For the grid associatied with this object, create a
			-- filler to deal with displacement cracks. If the
			-- neighbors haven't been diced, wait and patch from the
			-- other side.
		local
			i, j, the_count: INTEGER
			u_min, u_max, v_min, v_max: DOUBLE
			p_u_min, p_u_max, p_v_min, p_v_max: DOUBLE
			tmp: SCENE_DRAWABLE_ELEMENT
			tmp_grid: MICROPOLYGON_GRID
		do
			from
				i := left
				p_u_min := grid.parent_element.geometry.u_minimum
				p_v_min := grid.parent_element.geometry.v_minimum
				p_u_max := grid.parent_element.geometry.u_maximum
				p_v_max := grid.parent_element.geometry.v_maximum
			invariant
				i_at_least_one: i >= left
				i_small_enough: i <= left + below + 1
			variant
				i_increasing: left + below + 1 - i
			until
				i > below
			loop
				the_count := neighbors.item (i).count

				from
					j := 1
				invariant
					j_at_least_one: j >= 1
					j_small_enough: j <= 1 + the_count
				variant
					j_increasing: 1 + the_count - j
				until
					j > the_count
				loop
					tmp := neighbors.item (i).item (j)
					if tmp.diced then
						tmp_grid := tmp.grid

						-- Want the max of the mins, and the min of
						-- the maximums
						u_min := p_u_min.max (tmp_grid.parent_element.geometry.u_minimum)
						v_min := p_v_min.max (tmp_grid.parent_element.geometry.v_minimum)
						u_max := p_u_max.min (tmp_grid.parent_element.geometry.u_maximum)
						v_max := p_v_max.min (tmp_grid.parent_element.geometry.v_maximum)
						inspect i
						when left then
							if not (grid.has_left_patch and tmp_grid.has_right_patch) then
								generate_patch (grid,
												grid.left_start (v_min),
												grid.left_after (v_max), grid.left_stride,
												tmp_grid,
												tmp_grid.right_start (v_min),
												tmp_grid.right_after (v_max),
												tmp_grid.right_stride)
								grid.set_left_patch (patch_points)
								tmp_grid.attach_right_patch (grid.left_patch.last)
							end
						when right then
							if not (grid.has_right_patch and tmp_grid.has_left_patch) then
								generate_patch (grid,
												grid.right_start (v_min),
												grid.right_after (v_max),
												grid.right_stride,
												tmp_grid,
												tmp_grid.left_start (v_min),
												tmp_grid.left_after (v_max),
												tmp_grid.left_stride)
								grid.set_right_patch (patch_points)
								tmp_grid.attach_left_patch (grid.right_patch.last)
							end
						when below then
							if not (grid.has_below_patch and tmp_grid.has_above_patch) then
								generate_patch (grid,
												grid.below_start (u_min),
												grid.below_after (u_max),
												grid.below_stride,
												tmp_grid,
												tmp_grid.above_start (u_min),
												tmp_grid.above_after (u_max),
												tmp_grid.above_stride)
								grid.set_below_patch (patch_points)
								tmp_grid.attach_above_patch (grid.below_patch.last)
							end
						when above then
							if not (grid.has_above_patch and tmp_grid.has_below_patch) then
								generate_patch (grid,
												grid.above_start (u_min),
												grid.above_after (u_max),
												grid.above_stride,
												tmp_grid,
												tmp_grid.below_start (u_min),
												tmp_grid.below_after (u_max),
												tmp_grid.below_stride)
								grid.set_above_patch (patch_points)
								tmp_grid.attach_below_patch (grid.above_patch.last)
							end
						end
					end

					j := j + 1
				end

				i := i + 1
			end
		end

	draw (renderer: RENDERER) is
			-- Apply the renderer to this object.
		local
			dlist: SCENE_DISPLAY_LIST_ELEMENT
		do
			debug ("graphics")
				if get_orientation.consistent (is_left_handed) then
					io.put_string ("%Tis consistent%N")
				else
					io.put_string ("%Tis not consistent%N")
				end
			end

			if renderer.use_display_lists then
				dlist := get_display_list
				if dlist = Void then
					create dlist
					dlist.generate_display_list (renderer)
					renderer.new_list (dlist.item (renderer).first.to_integer,
									   renderer.rd_compile_and_execute)
					draw_immediate (renderer)
					renderer.end_list
					add_element (dlist)
				else
					if dlist.has_display_list (renderer) then
						if dlist.item (renderer).second then
							renderer.new_list (dlist.item (renderer).first.to_integer,
											   renderer.rd_compile_and_execute)
							draw_immediate (renderer)
							renderer.end_list
						else
							renderer.call_list (dlist.item (renderer).first.to_integer)
						end
					else
						dlist.generate_display_list (renderer)

						renderer.new_list (dlist.item (renderer).first.to_integer,
										   renderer.rd_compile_and_execute)
						draw_immediate (renderer)
						renderer.end_list
					end
				end
				dlist.item (renderer).put_second (false)
			else
				if renderer.show_extents then
					extents.draw_immediate (renderer)
				end

				draw_immediate (renderer)
			end
		end

	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		require
			u_in_range: u >= u_minimum and u <= u_maximum
			v_in_range: v >= v_minimum and v <= v_maximum
			result_data_not_void: result_data /= Void
		deferred
		end

	split is
			-- Split the object into smaller pieces
			-- This is the generic routine for when the object is 
			-- split into two pieces. Override if need more parts
		local
			du, dv: DOUBLE
			i, split_count: INTEGER
			split_part: like Current
		do
			split_count := 2
			update_split_directions
			du := (u_maximum - u_minimum) / split_count
			dv := (v_maximum - v_minimum) / split_count
			updating_neighbors := true
			-- Set the flag here so it's propogated to the children

			if split_in_u then
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + split_count
				variant
					i_increasing: 1 + split_count - i
				until
					i > split_count
				loop
					split_part := Current.twin
					split_part.reset_split
					split_part.allocate_local_features
					split_part.set_parameter_bounds (u_minimum + du * (i - 1),
						u_minimum + du * i,
						v_minimum, v_maximum)
					split_part.update_extents

					split_items.put_last (split_part)

					i := i + 1
				end
				
				process_u_split
			else
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + split_count
				variant
					i_increasing: 1 + split_count - i
				until
					i > split_count
				loop
					split_part := Current.twin
					split_part.reset_split
					split_part.allocate_local_features
					split_part.set_parameter_bounds (u_minimum, u_maximum,
						v_minimum + dv * (i - 1), v_minimum + dv * i)
					split_part.update_extents

					split_items.put_last (split_part)

					i := i + 1
				end

				process_v_split
			end

				-- Clear the neighbors from Current since they're no
				-- longer valid
			from
				i := left
			invariant
				i_at_least_one: i >= left
				i_small_enough: i <= left + below + 1
			variant
				i_increasing: left + below + 1 - i
			until
				i > below
			loop
				neighbors.item (i).wipe_out
				i := i + 1
			end

			updating_neighbors := false

			from
				split_items.start
			until
				split_items.after
			loop
				split_items.item_for_iteration.toggle_updating_neighbors
 
				split_items.forth
			end

			is_split := true
		ensure then
			extents_not_shared: extents /= split_items.first.extents and extents /= split_items.last.extents
		end

	tesselate is
			-- Transform the object into a polygonal structure
		deferred
		end -- tesselate

	traverse (action: SCENE_ACTION) is
			-- Traverse this node
		require
			action_not_void: action /= Void
		do
			action.apply (Current)
		end

	print_object (action: PRINT_ACTION) is
			-- Create a representation of self
		do
			Precursor {SCENE_DRAWABLE_ELEMENT} (action)
			print_elements (action)
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SCENE_GEOMETRY} -- Implementation

	bound_distance: VECTOR3
			-- The distance of the bounds in screen space (w.r.t a specific
			-- camera)

	max_u_dist, max_v_dist: DOUBLE
			-- The maximum distance in u and v on the MP grid

	set_parameter_bounds (u_min, u_max, v_min, v_max: DOUBLE) is
			-- Set the bounds for `u' and `v'
		require
			u_order_correct: u_min <= u_max
			v_order_correct: v_min <= v_max
		do
			u_minimum := u_min
			u_maximum := u_max
			v_minimum := v_min
			v_maximum := v_max
		ensure
			default: true
		end

	get_colors: SCENE_COLOR_ELEMENT is
			-- Return the appropriate colors, possibly Void if
			-- it doesn't exist
		do
			if cached_colors = Void then
				cached_colors ?=
				element_item ("SCENE_COLOR_ELEMENT")
			end

			Result := cached_colors
		end

	update_split_directions is
			-- Update the split direction from the information
			-- computed in `diceable'
		local
			u_split: BOOLEAN
			ratio: DOUBLE
		do
			if requested_grid_x_dim > requested_grid_y_dim then
				u_split := true
			else
				u_split := false
			end

			ratio := (u_maximum - u_minimum) / (v_maximum - v_minimum)

			if ratio > 10 then
				split_in_u := false
			elseif ratio < 0.01 then
				split_in_u := true
			else
				if u_split and split_in_u then
					split_in_u := false
				elseif not u_split and not split_in_u then
					split_in_u := true
				else
					split_in_u := u_split
				end
			end
		end

	reset_split is
			-- Remove split status
		do
			is_split := false
		end

	allocate_local_features is
			-- Allocate local memory for splits
		do
			allocate_neighbors
			
			create split_items.make (2)
		end

	allocate_neighbors is
			-- Allocate and copy neighbors to avoid aliasing
		local
			tmp: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
			tmp_neighbors: ARRAY [DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]]
			i: INTEGER
		do
			from
				create tmp_neighbors.make (0, 3)
				i := left
			invariant
				i_at_least_one: i >= left
				i_small_enough: i <= left + below + 1
			variant
				i_increasing: left + below + 1 - i
			until
				i > below
			loop
				tmp := neighbors.item (i).twin
				tmp_neighbors.put (tmp, i)
				i := i + 1
			end
			neighbors := tmp_neighbors
		end

feature {NONE} -- Implementation

	cached_display_list: SCENE_DISPLAY_LIST_ELEMENT
	cached_colors: SCENE_COLOR_ELEMENT
	cached_normals: SCENE_NORMAL_ELEMENT
	cached_orientation: SCENE_ORIENTATION_ELEMENT
	cached_sides: SCENE_SIDES_ELEMENT

	invalidate is
			-- Elements have been changed, so invalidate state as
			-- necessary
		do
			Precursor
			cached_display_list := Void
			cached_colors := Void
			cached_normals := Void
			cached_orientation := Void
			cached_sides := Void
		ensure then
			cache_cleared: (cached_display_list = Void and cached_colors = Void and
							cached_normals = Void and cached_orientation = Void and
							cached_sides = Void)
		end

	get_display_list: SCENE_DISPLAY_LIST_ELEMENT is
			-- Return the appropriate display list, possibly Void if
			-- it doesn't exist
		do
			if cached_display_list = Void then
				cached_display_list ?=
				element_item ("SCENE_DISPLAY_LIST_ELEMENT")
			end

			Result := cached_display_list
		end

	get_normals: SCENE_NORMAL_ELEMENT is
			-- Return the appropriate normals, possibly Void if
			-- it doesn't exist
		do
			if cached_normals = Void then
				cached_normals ?=
				element_item ("SCENE_NORMAL_ELEMENT")
			end

			Result := cached_normals
		end

	get_orientation: SCENE_ORIENTATION_ELEMENT is
			-- Return the appropriate orientation, possibly Void if
			-- it doesn't exist
		do
			if cached_orientation = Void then
				cached_orientation ?=
				element_item ("SCENE_ORIENTATION_ELEMENT")
			end

			Result := cached_orientation
		end

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object.
		require
			renderer_not_void: renderer /= Void
		deferred
		end

	update_display_lists is
			-- Update the display list if geometry has changed
		local
			dlist: SCENE_DISPLAY_LIST_ELEMENT
		do
			dlist := get_display_list
			if dlist /= Void then
				dlist.needs_updating
			end
		end

	create_color_element is
			-- Initialize common geometry features
		local
			sce: SCENE_COLOR_ELEMENT
		do
			create sce

			add_element (sce)
		end

	create_orientation_element is
			-- Initialize common geometry features
		local
			soe: SCENE_ORIENTATION_ELEMENT
		do
			create soe

			add_element (soe)
		end

	print_elements (action: PRINT_ACTION) is
			-- print the node's features
		do
			action.io.put_string ("	 state_elements = ")
			action.io.push

			action_print (action)

			action.io.pop

			if parameters /= Void then
				action.io.put_string ("	 parameters = ")
				action.io.put_string (parameters.out)
				action.io.put_new_line
			end
		end

	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		require
			t_positive: t >= 0
			point_not_void: point /= Void
			action_not_void: action /= Void
			ray_not_void: ray /= Void
		deferred
		ensure
			inserted: action.intersection.count = old action.intersection.count + 1
		end

	process_v_split is
			-- Set the neighbors for a split in v
		local
			i: INTEGER
			first, second: SCENE_DRAWABLE_ELEMENT
		do
			from
				i := 1 
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + split_items.count
			variant
				i_increasing: 1 + split_items.count - i
			until
				i >= split_items.count
			loop
				first := split_items.item (i)
				second := split_items.item (i + 1)
				second.set_neighbor (first, below)
				update_v_neighbors (first, below)
				update_u_neighbors (first, left)
				update_u_neighbors (first, right)
			
				-- Only `set' one of the neighbors, since the other is 
				-- automatically set
				update_v_neighbors (second, above)
				update_u_neighbors (second, left)
				update_u_neighbors (second, right)
 
				i := i + 1
			end
		end

	process_u_split is
			-- Set the neighbors for a split in u
		local
			i: INTEGER
			first, second: SCENE_DRAWABLE_ELEMENT
		do
			from
				i := 1 
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + split_items.count
			variant
				i_increasing: 1 + split_items.count - i
			until
				i >= split_items.count
			loop
				first := split_items.item (i)
				second := split_items.item (i + 1)
				second.set_neighbor (first, left)
				update_u_neighbors (first, left)
				update_v_neighbors (first, above)
				update_v_neighbors (first, below)

				-- Only `set' one of the neighbors, since the other is 
				-- automatically set
				update_u_neighbors (second, right)
				update_v_neighbors (second, above)
				update_v_neighbors (second, below)
 
				i := i + 1
			end
		end

	update_u_neighbors (part: SCENE_DRAWABLE_ELEMENT; index: INTEGER) is
			-- Update the neighbors
		require
			index_valid: index >= left and index <= below
		local
			tmp: SCENE_DRAWABLE_ELEMENT
			i, the_count, other: INTEGER
		do
			the_count := part.neighbors.item (index).count
			other := (index + 2 ) \\ 4

			if the_count = 1 and then
				part.neighbors.item (index).item (1) = Current and
				part.neighbors.item (index).item (1) = part.neighbors.item (other).item (1) then
					part.neighbors.item (index).replace (part, 1)
					part.neighbors.item (other).replace (part, 1)
			else
				from
					i := 1
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + the_count
				variant
					i_increasing: 1 + the_count - i
				until
					i > the_count
				loop
					tmp := part.neighbors.item (index).item (i)
					tmp.neighbors.item (other).start
					tmp.neighbors.item (other).search_forth (Current)
					if not tmp.neighbors.item (other).after then
						tmp.neighbors.item (other).remove_at
					end

					if (not (part.v_minimum = tmp.v_maximum or
						tmp.v_minimum = part.v_maximum)) and
						(within_range (part.v_minimum, tmp.v_minimum, tmp.v_maximum) or else
						within_range (part.v_maximum, tmp.v_minimum, tmp.v_maximum) or else
						within_range (tmp.v_minimum, part.v_minimum, part.v_maximum) or else
						within_range (tmp.v_maximum, part.v_minimum, part.v_maximum)) then
						if not tmp.neighbors.item (other).has (part) then
							tmp.add_neighbor (part, other)
						end
					else
						part.neighbors.item (index).remove (i)
						i := i - 1
						the_count := the_count - 1
					end

					i := i + 1
				end
			end
		end

	update_v_neighbors (part: SCENE_DRAWABLE_ELEMENT; index: INTEGER) is
			-- Update the neighbors
		require
			index_valid: index >= left and index <= below
		local
			tmp: SCENE_DRAWABLE_ELEMENT
			i, the_count, other: INTEGER
		do
			the_count := part.neighbors.item (index).count
			other := (index + 2 ) \\ 4

			if the_count = 1 and then
				part.neighbors.item (index).item (1) = Current and
				part.neighbors.item (index).item (1) = part.neighbors.item (other).item (1) then
					part.neighbors.item (index).replace (part, 1)
					part.neighbors.item (other).replace (part, 1)
			else
				from
					i := 1
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + the_count
				variant
					i_increasing: 1 + the_count - i
				until
					i > the_count
				loop
					tmp := part.neighbors.item (index).item (i)
					tmp.neighbors.item (other).start
					tmp.neighbors.item (other).search_forth (Current)
					if not tmp.neighbors.item (other).after then
						tmp.neighbors.item (other).remove_at
					end

					if (not (part.u_minimum	 = tmp.u_maximum or
						part.u_maximum = tmp.u_minimum)) and
						(within_range (part.u_minimum, tmp.u_minimum, tmp.u_maximum) or else
						within_range (part.u_maximum, tmp.u_minimum, tmp.u_maximum) or else
						within_range (tmp.u_minimum, part.u_minimum, part.u_maximum) or else
						within_range (tmp.u_maximum, part.u_minimum, part.u_maximum)) then
						if not tmp.neighbors.item (other).has (part) then
							tmp.add_neighbor (part, other)
						end
					else
						part.neighbors.item (index).remove (i)
						i := i - 1
						the_count := the_count - 1
					end

					i := i + 1
				end
			end
		end

	generate_patch (grid1: MICROPOLYGON_GRID;
				grid1_start, grid1_after, grid1_stride: INTEGER;
				grid2: MICROPOLYGON_GRID;
				grid2_start, grid2_after, grid2_stride: INTEGER) is
			-- Create a patch between `grid1' and `grid2'
		require
			grid1_not_void: grid1 /= Void
			grid2_not_void: grid2 /= Void
		local
			lower_bound, upper_bound: INTEGER
			l, r, p: INTEGER
			p1, p2: POINT3
			p3l, p3r: SHADING_DATA
			skew_l, skew_r: DOUBLE
		do
			lower_bound := grid1_start.max (grid2_start)
			upper_bound := grid1_after.min (grid2_after)

			from
				l := grid1_start
				r := grid2_start
				patch_points.wipe_out
			until
				l >= grid1_after and r >= grid2_after
			loop
				p1 := grid1.indexed_item (l).point
				p2 := grid2.indexed_item (r).point
				patch_points.force_last (grid1.indexed_item (l))
				patch_points.force_last (grid2.indexed_item (r))
				if l < grid1_after then
					p := l + grid1_stride
					p3l := grid1.indexed_item (p)
					skew_l := compute_skew (p1, p2, p3l.point)
				else
					skew_l := maximum_double
				end

				if r < grid2_after then
					p := r + grid2_stride
					p3r := grid2.indexed_item (p)
					skew_r := compute_skew (p1, p2, p3r.point)
				else
					skew_r := maximum_double
				end

				if skew_l < skew_r then
					patch_points.force_last (p3l)
					l := l + grid1_stride
				else
					patch_points.force_last (p3r)
					r := r + grid2_stride
				end
			end

		ensure
			default: true
		end

	compute_skew (p1, p2, p3: POINT3): DOUBLE is
			-- Compute the skew of the triangle <`p1', `p2', `p3' >
		require
			not_void: p1 /= Void and p2 /= Void and p3 /= Void
		local
			a, b, c, a1, a2, a3: DOUBLE
			a_min, a_max: DOUBLE
		do
			a := (p2 - p1).magnitude
			b := (p3 - p2).magnitude
			c := (p1 - p3).magnitude

			a1 := arc_cosine ((a*a + b*b - c*c) / (2.0*a*b)) * 180.0 / pi
			a2 := arc_cosine ((c*c + a*a - b*b) / (2.0*a*c)) * 180.0 / pi
			a3 := arc_cosine ((b*b + c*c - a*a) / (2.0*b*c)) * 180.0 / pi

				check
					sum_is_180: about_equal (180.0, a1 + a2 + a3)
				end

			a_min := a1.min (a2.min (a3))
			a_max := a1.max (a2.max (a3))
			Result := ((a_max - 60) / 120.0 ).max( ( 60.0 - a_min ) / 60.0 )
		ensure
			skew_positive: Result > 0
		end

invariant

	count_is_positive: count >= 0
	element_count_is_positive: element_count >= 1
	table_holds_color_element: has_color_element
	mapping_count_is_positive: mapping_count >= 1
	color_mapping_in_range: (color_mapping >= constant_color and
							 color_mapping <= per_vertex)
	world_to_object_not_void: world_to_object /= Void
	object_to_world_is_inverse: equal (object_to_world.inverse, world_to_object)
	has_orientation: valid_key ("SCENE_ORIENTATION_ELEMENT")
--	normal_mapping_in_range: (normal_mapping >= constant_color and
--							  normal_mapping <= per_vertex)

end -- SCENE_GEOMETRY

