

indexing
	description: "Line geometry"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_LINES
	
inherit
	
	BASIC_NUMERIC
		undefine
			copy, is_equal, default_create
		end
	
	SCENE_GEOMETRY
		redefine
			default_create, detailed_memory_usage, print_features
		end
	
creation
	default_create, make, make_simple, make_from_verticies

feature -- Initialization

	default_create is
			-- Make degenerate polyline with two zero vertices.
		local
			p: POLYLINE
		do
			Precursor {SCENE_GEOMETRY}
			
			create lines.make (1, 1)
			create p
			lines.put (p, 1)
			
			line_width := 1.0
			create tmp_array.make (1, 10)
			
			color_mapping := constant_color
			
		end
			
	make (the_lines: DS_ARRAYED_LIST [POLYLINE]) is
			-- Intialize the scene lines with these polylines
		require
			lines_not_void: the_lines /= Void
			at_least_one_line: the_lines.count >= 1
		local
			i: INTEGER
		do
			default_create
			create lines.make (1, the_lines.count)

			from
				the_lines.start
				i := 1
			until
				the_lines.after
			loop
				lines.put (the_lines.item_for_iteration.standard_twin, i)

				i := i + 1
				the_lines.forth
			end
			
			update_extents
		end
	
	
	make_simple (line: POLYLINE) is
			-- Intialize the scene lines with `line'
		do
			default_create
			create lines.make (1, 1)
			lines.put (line.deep_twin, 1)
			
			create {SCENE_SPHERE_BOUND} extents.make_with_point_and_radius	(line.extent.centroid,
																			 (line.extent.upper - line.extent.centroid).magnitude)
		ensure
			lines_not_void: lines /= Void
			lines_has_one_polyline: lines.count = 1
			first_vertex_set: lines.item (1).item (1).is_equal (line.item (1))
			second_vertex_set: lines.item (1).item (2).is_equal (line.item (2))
		end
	
	
	make_from_verticies (initial_first_vertex, initial_second_vertex: POINT3) is
			-- Intialize the scene lines with these verticies
		local
			p: POLYLINE
		do
			default_create
			create lines.make (1, 1)
			create p.make_simple (initial_first_vertex, initial_second_vertex)
			lines.put (p, 1)
			
			create {SCENE_SPHERE_BOUND} extents.make_with_point_and_radius	(p.extent.centroid,
																			 (p.extent.upper - p.extent.centroid).magnitude)
		ensure
			lines_not_void: lines /= Void
			lines_has_one_polyline: lines.count = 1
			first_vertex_set: lines.item (1).item (1).is_equal (initial_first_vertex)
			second_vertex_set: lines.item (1).item (2).is_equal (initial_second_vertex)
		end
	
	
feature -- Access
	
	extents: SCENE_BOUND
			-- The `extents' or bounds of the volume.
	
	line_width: DOUBLE
			-- The width of the lines
	
feature -- Measurement
	
	count: INTEGER_64 is
			-- The number of lines
		do
			Result := lines.count
		end
	
	edge_count: INTEGER is
			-- The number of edges
		local
			i: INTEGER
		do
			Result := 0
			
			from
				i := lines.lower 
			invariant
				i_at_least_one: i >= lines.lower
				i_small_enough: i <= lines.lower + lines.upper
			variant
				i_increasing: lines.lower + lines.upper - i
			until
				i > lines.count
			loop
				Result := Result + lines.item (i).edge_count
				i := i + 1
			end
		end
	
	vertex_count: INTEGER is
			-- The number of edges
		local
			i: INTEGER
		do
			Result := 0
			
			from
				i := lines.lower 
			invariant
				i_at_least_one: i >= lines.lower
				i_small_enough: i <= lines.lower + lines.upper
			variant
				i_increasing: lines.lower + lines.upper - i
			until
				i > lines.count
			loop
				Result := Result + lines.item (i).vertex_count
				i := i + 1
			end
		end
	
	mapping_count: INTEGER_64 is
			-- The number of element for mapping colors
		do
			inspect color_mapping
			when constant_color then
				Result := 1
			when per_object then
				Result := lines.count
			when per_element then
				Result := edge_count
			when per_vertex then
				Result := vertex_count
			end
		end 
	
feature -- Comparison
feature -- Status report
	
	has_extents: BOOLEAN is
			-- Does this volume have extents?
		do
			Result := extents /= Void
		end
		
feature -- Cursor movement
feature -- Element change
	
	set_line_width (new_size: DOUBLE) is
			-- Set `line_width' to `new_size'
		require 
			new_size_is_positive: new_size >= 0
		do
			line_width := new_size
			
			update_display_lists
		ensure 
			line_width_is_set: line_width = new_size
		end 

	update_extents is
			-- Recompute the extents of the pbject
		local
			i: INTEGER
			tmp_bound: SCENE_SPHERE_BOUND
			v: VECTOR3
		do
			create {SCENE_SPHERE_BOUND} extents
			create tmp_bound									  
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + lines.count
			variant
				i_increasing: 1 + lines.count - i
			until
				i >= lines.count
			loop
				v := lines.item (i).extent.upper - lines.item (i).extent.centroid
				tmp_bound.make_with_point_and_radius  (lines.item (i).extent.centroid,
													   v.magnitude)
				extents.expand_to_include (tmp_bound)
				i := i + 1
			end
		end
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		do
			check
				implemented: false
			end
		end
	
	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end -- tesselate

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	lines: ARRAY [POLYLINE]
			-- An array of polylines
	
	tmp_array: TRIMMABLE_ARRAY [DOUBLE]
			-- Internal array for shoving data to GL
	
	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray', 
			-- against this volume. It is assumed that the ray has 
			-- been transformed into this object's coordinate system
		do
			check
				implemented: false
			end
		end
	
	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			i: INTEGER
			sce: SCENE_COLOR_ELEMENT
		do
			renderer.disable (renderer.rd_lighting)
			renderer.line_width (line_width)
			
			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
				
			if color_mapping = constant_color then
				renderer.rgba_color (sce.i_th (1))
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + lines.count
				variant
					i_increasing: 1 + lines.count - i
				until
					i > lines.count
				loop
					draw_line_strip (renderer, i)
					i := i + 1
				end
			elseif color_mapping = per_object then
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + lines.count
				variant
					i_increasing: 1 + lines.count - i
				until
					i > lines.count
				loop
					renderer.rgba_color (sce.i_th (i))
					draw_line_strip (renderer, i)
					i := i + 1
				end
			elseif color_mapping = per_element then
				renderer.shade_model (renderer.rd_flat)
				
				draw_element_mapped (renderer, sce)
				
				renderer.shade_model (renderer.rd_smooth)
			else -- per_vertex
				draw_vertex_mapped (renderer, sce)
			end
			
			renderer.enable (renderer.rd_lighting)
		end
		
	detailed_memory_usage: INTEGER is
			-- Compute additional memory usage as required
		local
			i: INTEGER
		do
			Result := Precursor
			
			from
				i := lines.lower
			until
				i > lines.upper
			loop
				Result := Result + object_memory_usage (lines.item (i))
				i := i + 1
			end
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the group's features
		do
			Precursor (action)
			
			action.io.put_string ("	 line_width = ")
			action.io.put_double (line_width)
			action.io.put_new_line
		end
	
	draw_line_strip (r: RENDERER; index: INTEGER) is
			-- Take the `i'_th polyline and flatten the verticies 
			-- into `tmp_array'
		local
			p: POLYLINE
			i: INTEGER
		do
			p := lines.item (index)
			
			r.render_begin (r.rd_line_strip)
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + p.vertex_count
			variant
				i_increasing: 1 + p.vertex_count - i
			until
				i > p.vertex_count
			loop
				r.vertex3 (p.item (i).x, p.item (i).y, p.item (i).z)
				
				i := i + 1
			end
			
			r.render_end
		end 

	draw_element_mapped (r: RENDERER; sce: SCENE_COLOR_ELEMENT) is
			-- Apply the renderer to this object
		local
			i, j, color_index: INTEGER
			pl: POLYLINE
		do
			color_index := 1
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + lines.count
			variant
				i_increasing: 1 + lines.count - i
			until
				i > lines.count
			loop
				pl := lines.item (i)
				
				r.render_begin (r.rd_line_strip)
				
				from
					j := 1 
				invariant
					j_at_least_one: j >= 1
					j_small_enough: j <= 1 + pl.vertex_count
				variant
					j_increasing: 1 + pl.vertex_count - j
				until
					j > pl.vertex_count
				loop
					if j > 1 then
						r.rgba_color (sce.i_th (color_index))
						color_index := color_index + 1
					end
					r.set_vertex (pl.item (j))
					j := j + 1
				end
				
				r.render_end
				i := i + 1
			end
		end
		
	draw_vertex_mapped (r: RENDERER; sce: SCENE_COLOR_ELEMENT) is
			-- Apply the renderer to this object
		local
			i, j, color_index: INTEGER
			pl: POLYLINE
		do
			color_index := 1
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + lines.count
			variant
				i_increasing: 1 + lines.count - i
			until
				i > lines.count
			loop
				pl := lines.item (i)
				
				r.render_begin (r.rd_line_strip)
				
				from
					j := 1 
				invariant
					j_at_least_one: j >= 1
					j_small_enough: j <= 1 + pl.vertex_count
				variant
					j_increasing: 1 + pl.vertex_count - j
				until
					j > pl.vertex_count
				loop
					r.rgba_color (sce.i_th (color_index))
					r.set_vertex (pl.item (j))
					color_index := color_index + 1
					j := j + 1
				end
				
				r.render_end
				i := i + 1
			end
		end
	
	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information 
			-- into `intersection'
		do
			check
				never_called: false
			end
		end

invariant
	
	lines_not_void: lines /= Void
	at_least_one_line: lines.count >= 1
	line_width_is_positive: line_width >= 1
	
end -- SCENE_LINES

