

indexing
	description: "NURBS primitive"
	patterns: "none"
	date: "$Date: 2007-04-18 16:11:07 -0400 (Wed, 18 Apr 2007) $"
	revision: "$Revision: 289 $"
	status: "unreviewed"




class 

	SCENE_NURBS
	
inherit
	
	SCENE_GEOMETRY
		rename
			insert_intersection_information as geometry_insert_intersection_information
		undefine
			i_th_bound, intersect
		redefine
			default_create, print_features, out, object_intersect, split
		end
	
	SCENE_TESSELATOR
		undefine
			is_equal, copy, out, add_indices, 
			set_coordinates_and_indices, set_coordinates, make_with_coordinates
		redefine
			default_create
		end
	
	TRIANGLE_INTERSECTOR
		rename
			compute_normals as triangle_compute_normals
		undefine
			intersectable_intersect, out, invalidate, is_equal
		redefine
			default_create,
			object_intersect,
			update_extents, split
		end

	NUMERIC_CONVERSION
		export
			{NONE} all
		undefine
			default_create, is_equal, copy, out
		end
		
creation
	default_create, make, make_from_control_points

feature -- Initialization

	default_create is
			-- Create a simple patch
		local
			pt: POINT4
		do
			Precursor {SCENE_TESSELATOR}
			Precursor {SCENE_GEOMETRY}
			Precursor {TRIANGLE_INTERSECTOR}
			
			u_order := 4
			v_order := 4
			
			create u_knots.make_from_array (<<0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0>>, 
											  u_order)
			create v_knots.make_from_array (<<0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0>>, 
											  v_order)
			
			create control_points.make (0, 3, 0, 3)
			create pt.make_from_array (<<0.0, 0, 0, 1>>)
			control_points.put (pt.twin, 0, 0)
			create pt.make_from_array (<<0.25, 0, 0, 1>>)
			control_points.put (pt.twin, 0, 1)
			create pt.make_from_array (<<0.5, 0, 0, 1>>)
			control_points.put (pt.twin, 0, 2)
			create pt.make_from_array (<<0.75, 0, 0, 1>>)
			control_points.put (pt.twin, 0, 3)
			create pt.make_from_array (<<0, 0.25, 0, 1>>)
			control_points.put (pt.twin, 1, 0)
			create pt.make_from_array (<<0.25, 0.25, -0.5, 1>>)
			control_points.put (pt.twin, 1, 1)
			create pt.make_from_array (<<0.5, 0.25, -0.5, 1>>)
			control_points.put (pt.twin, 1, 2)
			create pt.make_from_array (<<0.75, 0.25, 0, 1>>)
			control_points.put (pt.twin, 1, 3)
			create pt.make_from_array (<<0, 0.5, 0, 1>>)
			control_points.put (pt.twin, 2, 0)
			create pt.make_from_array (<<0.25, 0.5, -0.5, 1>>)
			control_points.put (pt.twin, 2, 1)
			create pt.make_from_array (<<0.5, 0.5, -0.5, 1>>)
			control_points.put (pt.twin, 2, 2)
			create pt.make_from_array (<<0.75, 0.5, 0, 1>>)
			control_points.put (pt.twin, 2, 3)
			create pt.make_from_array (<<0, 0.75, 0, 1>>)
			control_points.put (pt.twin, 3, 0)
			create pt.make_from_array (<<0.25, 0.75, 0, 1>>)
			control_points.put (pt.twin, 3, 1)
			create pt.make_from_array (<<0.5, 0.75, 0, 1>>)
			control_points.put (pt.twin, 3, 2)
			create pt.make_from_array (<<0.75, 0.75, 0, 1>>)
			control_points.put (pt.twin, 3, 3)
			
			initialize
			
			update_extents
		end

	make (nu, uorder: INTEGER; uknot: DOUBLE_ARRAYED_LIST; umin, umax: DOUBLE;
		  nv, vorder: INTEGER; vknot: DOUBLE_ARRAYED_LIST; vmin, vmax: DOUBLE;
		  points: POINT4_ARRAYED_LIST) is
			-- Initialize the nurbs with parameters
		require
			nu_strictly_positive: nu >= 1
			nu_correct: nu + uorder = uknot.count
			uorder_strictly_positive: uorder >= 1
			uknot_not_void: uknot /= Void
			umin_less_than_umax: umin < umax
			nv_strictly_positive: nv >= 1
			nu_correct: nv + vorder = vknot.count
			vorder_strictly_positive: vorder >= 1
			vknot_not_void: vknot /= Void
			vmin_less_than_vmax: vmin < vmax
			point_not_void: points /= Void
			correct_size: points.count = nu * nv
		do
			default_create
			
			u_order := uorder
			create u_knots.make_from_arrayed_list  (uknot, u_order)
			v_order := vorder
			create v_knots.make_from_arrayed_list (vknot, v_order)
			create control_points.make_from_large_list (nu, nv, points)
			
			initialize
			
			u_minimum := umin
			u_maximum := umax
			v_minimum := vmin
			v_maximum := vmax
			
			update_extents
		end

	make_from_control_points (nu, uorder: INTEGER; uknot: DOUBLE_ARRAYED_LIST; umin, umax: DOUBLE;
		  nv, vorder: INTEGER; vknot: DOUBLE_ARRAYED_LIST; vmin, vmax: DOUBLE;
		  points: BOUNDED_ARRAY2 [POINT4]) is
			-- Initialize the nurbs with parameters
		require
			nu_strictly_positive: nu >= 1
			nu_correct: nu + uorder = uknot.count
			uorder_strictly_positive: uorder >= 1
			uknot_not_void: uknot /= Void
			umin_less_than_umax: umin < umax
			nv_strictly_positive: nv >= 1
			nu_correct: nv + vorder = vknot.count
			vorder_strictly_positive: vorder >= 1
			vknot_not_void: vknot /= Void
			vmin_less_than_vmax: vmin < vmax
			point_not_void: points /= Void
			correct_size: points.count = nu * nv
		do
			default_create
			
			u_order := uorder
			create u_knots.make_from_arrayed_list  (uknot, u_order)
			v_order := vorder
			create v_knots.make_from_arrayed_list (vknot, v_order)
			control_points := points
			
			initialize
			
			u_minimum := umin
			u_maximum := umax
			v_minimum := vmin
			v_maximum := vmax
			
			update_extents
		end

feature -- Access
	
	display_smoothness: REAL
			-- How "smooth" the nurbs appears in interactive 
			-- displays; the larger the number, the smoother the surface
	
	u_knots: KNOT_VECTOR
	v_knots: KNOT_VECTOR
			-- The knot vectors

feature -- Measurement
	
	count: INTEGER_64 is 1
			-- There is only one NURBS
	
	mapping_count: INTEGER_64 is
			-- There is only one element
		do
			Result := control_points.count
		end
	
	v_count: INTEGER is
			-- The number of control points in the `v' parametric directions
		do
			Result := control_points.width
		ensure
			Result_positive: Result > 0
			Result_correct: Result = control_points.width
		end
	
	u_count: INTEGER is
			-- The number of control points in the `u' parametric directions
		do
			Result := control_points.height
		ensure
			Result_positive: Result > 0
			Result_correct: Result = control_points.height
		end
	
	u_order, v_order: INTEGER
			-- The order of the polynomial in the parametric directions
	
	u_degree: INTEGER is
			-- The degree in the u direction
		do
			Result := u_order - 1
		ensure 
			Result_positive: Result > 0
		end 

	v_degree: INTEGER is
			-- The degree in the u direction
		do
			Result := v_order - 1
		ensure 
			Result_positive: Result > 0
		end 

feature -- Comparison
feature -- Status report

	has_extents: BOOLEAN is
			-- Does this volume have extents?
		do
			Result := extents /= Void
		end
		
feature -- Status setting

	set_intersection_status (status: BOOLEAN) is
			-- Set the intersection status
		do
			intersected := status
		end
	
feature -- Cursor movement
feature -- Element change
	
	set_display_smoothness (new_smooth: REAL) is
			-- Set `display_smoothness' to `new_smooth'
		require 
			new_smooth_positive: new_smooth > 0
		do
			display_smoothness := new_smooth
		ensure 
			display_smoothness_set: display_smoothness = new_smooth
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
	
	out: STRING is
			-- Yield a printable representation.
		local
			i, j: INTEGER
		do
			create Result.make (80)
			Result.append_character ('%N')
			Result.append_string ("u_knots: ")
			Result.append_string (u_knots.out)
			Result.append_character ('%N')
			Result.append_string ("v_knots: ")
			Result.append_string (v_knots.out)
			Result.append_character ('%N')
			
			from
				i := control_points.row_lower
				Result.append_string ("control points: %N")
			invariant
				i_at_least_one: i >= control_points.row_lower
				i_small_enough: i <= control_points.row_lower + control_points.row_upper + 1
			variant
				i_increasing: control_points.row_lower + control_points.row_upper - i + 1
			until
				i > control_points.row_upper
			loop
				from
					j := control_points.column_lower 
				invariant
					j_at_least_one: j >= control_points.column_lower
					j_small_enough: j <= control_points.column_lower + control_points.column_upper + 1
				variant
					j_increasing: control_points.column_lower + control_points.column_upper - j + 1
				until
					j > control_points.column_upper
				loop
					Result.append_string ("	 (")
					Result.append_string (i.out)
					Result.append_string (", ")
					Result.append_string (j.out)
					Result.append_string (") - ")
					Result.append_string (control_points.item (i, j).out)
					Result.append_character ('%N')
					j := j + 1
				end
				i := i + 1
			end
		ensure then
			result_not_void: Result /= Void
		end

feature -- Duplication
feature -- Basic operations
	
	clamp is
			-- Ensure a nonperiodic (clamped) surface
		do
			clamp_in_u
			clamp_in_v
		ensure 
			u_knots_clamped: u_knots.clamped
			v_knots_clamped: v_knots.clamped
		end 

	insert_knots (u_knot_val: DOUBLE; u_knot_multiplicity: INTEGER;
				 v_knot_val: DOUBLE; v_knot_multiplicity: INTEGER)	is
			-- Insert `u_knot_val' into the surface `u_knot_multiplicity' 
			-- times and `v_knot_val'  `v_knot_multiplicity'  times 
			-- updating the control points as necessary
		require 
			u_knot_val_in_range: u_knot_val >= u_knots.minimum and 
								 u_knot_val <= u_knots.maximum
			u_knot_multiplicity_strictly_positive: u_knot_multiplicity >= 1
			v_knot_val_in_range: v_knot_val >= v_knots.minimum and 
								 v_knot_val <= v_knots.maximum
			v_knot_multiplicity_strictly_positive: v_knot_multiplicity >= 1
		do
			insert_knot_in_u (u_knot_val, u_knot_multiplicity)
			insert_knot_in_v (v_knot_val, v_knot_multiplicity)			
		end
	
	insert_knot_in_u (u_knot_val: DOUBLE; u_knot_multiplicity: INTEGER)	 is
			-- Insert `u_knot_val' into the surface `u_knot_multiplicity' 
			-- times updating the control points as necessary
		require 
			u_knot_val_in_range: u_knot_val >= u_knots.minimum and 
								 u_knot_val <= u_knots.maximum
			u_knot_multiplicity_strictly_positive: u_knot_multiplicity >= 1
		local
			i, j, row, L: INTEGER_64
			multiplicity, insertion_point: INTEGER_64
			alpha: DOUBLE
			old_u: KNOT_VECTOR
			new_control_points: BOUNDED_ARRAY2 [POINT4]
			tmp_cp: ARRAY [POINT4]
			p: POINT4
		do
			create old_u
			old_u.copy (u_knots)
			
			u_knots.insert_knot (u_knot_val, u_knot_multiplicity)
			if u_knots.insertion_count > 0 then
				-- Process the U-direction control points
				create new_control_points.make (control_points.row_lower,
												control_points.row_upper + u_knots.insertion_count,
												control_points.column_lower,
					control_points.column_upper)
				if about_equal (old_u.item (old_u.find_interval (u_knot_val)), u_knot_val ) then
					multiplicity := old_u.multiplicity (old_u.find_interval (u_knot_val))
				else
					multiplicity := 0
				end
				
				-- Subtract one from the insertion point 
				-- since its one-based, and the array is zero-based
				insertion_point := u_knots.insertion_point - 1
				
				from
					row := control_points.column_lower 
				invariant
					column_at_least_one: row >= control_points.column_lower
					column_small_enough: row <= control_points.column_lower + control_points.column_upper + 1
				variant
					column_increasing: control_points.column_lower + control_points.column_upper  + 1 - row
				until
					row > control_points.column_upper
				loop
					-- Copy up to the insertion point
					from
						i := control_points.row_lower 
					invariant
						i_at_least_one: i >= control_points.row_lower
						i_small_enough: i <= control_points.row_lower + control_points.row_lower + insertion_point - u_degree + 1
					variant
						i_increasing: control_points.row_lower + control_points.row_lower + insertion_point - u_degree - i + 1
					until
						i > control_points.row_lower + insertion_point - u_degree
					loop
						new_control_points.put (control_points.item (i.to_integer, row.to_integer), i.to_integer, row.to_integer)
						i := i + 1
					end
					
					-- Copy from the insertion point to the end
					from
						i := control_points.row_lower + insertion_point - multiplicity 
					invariant
						i_at_least_one: i >= control_points.row_lower + insertion_point - multiplicity
						i_small_enough: i <= control_points.row_lower + insertion_point - multiplicity + control_points.row_upper +1
					variant
						i_increasing: control_points.row_lower + insertion_point - multiplicity + control_points.row_upper - i + 1
					until
						i > control_points.row_upper
					loop
						new_control_points.put (control_points.item (i.to_integer, row.to_integer), (i + u_knots.insertion_count).to_integer, row.to_integer)
						
						i := i + 1
					end
					
					create tmp_cp.make (0, u_degree)
					
					from
						i := 0 
					invariant
						i_at_least_one: i >= 0
						i_small_enough: i <= 0 + u_degree - multiplicity + 1
					variant
						i_increasing: 0 + u_degree - multiplicity  - i + 1
					until
						i > u_degree - multiplicity 
					loop
						tmp_cp.put (control_points.item ((control_points.row_lower + insertion_point - u_degree + i).to_integer, row.to_integer),
									i.to_integer)
						i := i + 1
					end
					
					from
						j := 1 
					invariant
						j_at_least_one: j >= 1
						j_small_enough: j <= 1 + u_knots.insertion_count +1
					variant
						j_increasing: 1 + u_knots.insertion_count - j + 1
					until
						j > u_knots.insertion_count
					loop
						L := insertion_point - u_degree + j
						
						from
							i := 0 
						invariant
							i_at_least_one: i >= 0
							i_small_enough: i <= 0 + u_degree - j - multiplicity + 1
						variant
							i_increasing: 0 + u_degree - j - multiplicity - i + 1
						until
							i > u_degree - j - multiplicity
						loop
							alpha := (u_knot_val - old_u.item (L + 1 + i)) /
								(old_u.item (i + u_knots.insertion_point + 1) - old_u.item (L + 1 + i))
							p := tmp_cp.item ((i + 1).to_integer).weighted_sum (alpha, tmp_cp.item (i.to_integer), 1 - alpha)
							tmp_cp.put (p, i.to_integer)
							i := i + 1
						end
						
						new_control_points.put (tmp_cp.item (0), L.to_integer, row.to_integer)
						if u_degree - j - multiplicity > 0 then
							new_control_points.put (tmp_cp.item ((u_degree - j - multiplicity).to_integer), 
													(insertion_point + u_knots.insertion_count - j - multiplicity).to_integer, row.to_integer)
						end
						j := j + 1
					end

					from
						i := L + 1 
					invariant
						i_at_least_one: i >= L + 1
						i_small_enough: i <= L + 1 + insertion_point - multiplicity
					variant
						i_increasing: L + 1 + insertion_point - multiplicity - i
					until
						i >= insertion_point - multiplicity
					loop
						new_control_points.put (tmp_cp.item ((i - L).to_integer), i.to_integer, row.to_integer)
						i := i + 1
					end
					row := row + 1
				end
				control_points := new_control_points
				
			end
		ensure 
			control_points_not_void: valid_control_points
		end 
	
	insert_knot_in_v (v_knot_val: DOUBLE; v_knot_multiplicity: INTEGER)	 is
			-- Insert `v_knot_val' into the surface `v_knot_multiplicity' 
			-- times updating the control points as necessary
		require 
			v_knot_val_in_range: v_knot_val >= v_knots.minimum and 
								 v_knot_val <= v_knots.maximum
			v_knot_multiplicity_strictly_positive: v_knot_multiplicity >= 1
		local
			i, j, col, L: INTEGER_64
			multiplicity, insertion_point: INTEGER_64
			alpha: DOUBLE
			old_v: KNOT_VECTOR
			new_control_points: BOUNDED_ARRAY2 [POINT4]
			tmp_cp: ARRAY [POINT4]
		do
			create old_v
			old_v.copy (v_knots)
			
			v_knots.insert_knot (v_knot_val, v_knot_multiplicity)
			if v_knots.insertion_count > 0 then
				-- Process the V-direction control points
				create new_control_points.make (control_points.row_lower,
												control_points.row_upper,
												control_points.column_lower,
												control_points.column_upper + v_knots.insertion_count)
				if about_equal (old_v.item (old_v.find_interval (v_knot_val)), v_knot_val ) then
					multiplicity := old_v.multiplicity (old_v.find_interval (v_knot_val))
				else
					multiplicity := 0
				end
				
				-- Subtract one from the insertion point 
				-- since its one-based, and the array is zero-based
				insertion_point := v_knots.insertion_point - 1
				
				from
					col := control_points.row_lower 
				invariant
					column_at_least_one: col >= control_points.row_lower
					column_small_enough: col <= control_points.row_lower + control_points.row_upper + 1
				variant
					column_increasing: control_points.row_lower + control_points.row_upper	+ 1 - col
				until
					col > control_points.row_upper
				loop
					-- Copy up to the insertion point
					from
						i := control_points.column_lower 
					invariant
						i_at_least_one: i >= control_points.column_lower
						i_small_enough: i <= control_points.column_lower + control_points.column_lower + insertion_point - v_degree + 1
					variant
						i_increasing: control_points.column_lower + control_points.column_lower + insertion_point - v_degree - i + 1
					until
						i > control_points.column_lower + insertion_point - v_degree
					loop
						new_control_points.put (control_points.item (col.to_integer, i.to_integer), col.to_integer, i.to_integer)
						i := i + 1
					end
					
					-- Copy from the insertion point to the end
					from
						i := control_points.column_lower + insertion_point - multiplicity 
					invariant
						i_at_least_one: i >= control_points.column_lower + insertion_point - multiplicity
						i_small_enough: i <= control_points.column_lower + insertion_point - multiplicity + control_points.column_upper
					variant
						i_increasing: control_points.column_lower + insertion_point - multiplicity + control_points.column_upper - i
					until
						i > control_points.column_upper
					loop
						new_control_points.put (control_points.item (col.to_integer, i.to_integer), col.to_integer, (i + v_knots.insertion_count).to_integer)
						
						i := i + 1
					end
					
					create tmp_cp.make (0, v_degree)
					
					from
						i := 0 
					invariant
						i_at_least_one: i >= 0
						i_small_enough: i <= 0 + v_degree - multiplicity + 1
					variant
						i_increasing: 0 + v_degree - multiplicity  - i + 1
					until
						i > v_degree - multiplicity 
					loop
						tmp_cp.put (control_points.item (col.to_integer, (control_points.column_lower + insertion_point - v_degree + i).to_integer),
									i.to_integer)
						i := i + 1
					end
					
					from
						j := 1 
					invariant
						j_at_least_one: j >= 1
						j_small_enough: j <= 1 + v_knots.insertion_count +1
					variant
						j_increasing: 1 + v_knots.insertion_count - j + 1
					until
						j > v_knots.insertion_count
					loop
						L := insertion_point - v_degree + j
						
						from
							i := 0 
						invariant
							i_at_least_one: i >= 0
							i_small_enough: i <= 0 + v_degree - j - multiplicity + 1
						variant
							i_increasing: 0 + v_degree - j - multiplicity - i + 1
						until
							i > v_degree - j - multiplicity
						loop
							alpha := (v_knot_val - old_v.item (L + 1 + i)) /
								(old_v.item (i + v_knots.insertion_point + 1) - old_v.item (L + 1 + i))
							tmp_cp.put (tmp_cp.item ((i + 1).to_integer).weighted_sum (alpha,  tmp_cp.item (i.to_integer), 1 - alpha),
										i.to_integer)
							i := i + 1
						end
						
						new_control_points.put (tmp_cp.item (0), col.to_integer, L.to_integer)
						if v_degree - j - multiplicity > 0 then
							new_control_points.put (tmp_cp.item ((v_degree - j - multiplicity).to_integer), 
													col.to_integer, (insertion_point + v_knots.insertion_count - j - multiplicity).to_integer)
						end
						j := j + 1
					end

					from
						i := L + 1 
					invariant
						i_at_least_one: i >= L + 1
						i_small_enough: i <= L + 1 + insertion_point - multiplicity
					variant
						i_increasing: L + 1 + insertion_point - multiplicity - i
					until
						i >= insertion_point - multiplicity
					loop
						new_control_points.put (tmp_cp.item ((i - L).to_integer), col.to_integer, i.to_integer)
						i := i + 1
					end
					col := col + 1
				end
				control_points := new_control_points
			end
		end
	
	normal (u, v: DOUBLE): NORMAL is
			-- Compute the normal at the parametric coordinates 
			-- (`u',  `v') 
		require else
			u_in_range: u >= 0 and u <= 1
			v_in_range: v >= 0 and v <= 1
		local
			uu, vv: DOUBLE
			normal_vector: VECTOR3
		do
			uu := u
			vv := v
			compute_derivatives (uu, vv, 1)
			create Result
			
			normal_vector := derivatives.item (0, 1) * derivatives.item (1, 0)
			
			if about_equal (normal_vector.magnitude_squared, 0) then
				uu := u + Tolerance
				if uu > 1 then
					uu := u - Tolerance
				end
				
				vv := v + Tolerance
				if vv > 1 then
					vv := v - Tolerance
				end
				
				compute_derivatives (uu, vv, 1)
				
				normal_vector := derivatives.item (0, 1) * derivatives.item (1, 0)
			end
			
			check
				normal_vector_not_zero: normal_vector.magnitude_squared /= 0
			end
			
			Result.make_from_vector (normal_vector)
			Result.normalize
			
			debug ("nurb")
				io.put_string ("( ")
				io.put_string (formatter.formatted (uu))
				io.put_string (", ")
				io.put_string (formatter.formatted (vv))
				io.put_string (" ) - ")
				io.put_string (Result.out)
				io.put_string ("%N")
			end			
		ensure then
			Result_not_void: Result /= Void
			Result_is_unit: Result.is_unit
		end 
	
	split is
			-- Split the object into smaller pieces
		local
			mid_knot_val, mid_val: DOUBLE
			i, nu, nv, order, span: INTEGER
		do
			update_split_directions

			updating_neighbors := true
			-- Set the flag here so it's propogated to the children
			if split_in_u then
				mid_knot_val := (u_knots.maximum + u_knots.minimum) / 2
				mid_val := (u_maximum + u_minimum) / 2
				order := u_knots.order
				insert_knot_in_u (mid_knot_val, order)
				
				span := u_knots.find_span (mid_knot_val)
				nu := span - order
				
				split_along_u (1, span, control_points.row_lower, nu, u_minimum, mid_val)
				split_along_u (span - order + 1, span, control_points.row_lower+nu, nu, mid_val, u_maximum)

				process_u_split
			else
				mid_knot_val := (v_knots.maximum + v_knots.minimum) / 2
				mid_val := (v_maximum + v_minimum) / 2
				order := v_knots.order
				insert_knot_in_v (mid_knot_val, order)
				
				span := v_knots.find_span (mid_knot_val)
				nv := span - order
				
				split_along_v (1, span, control_points.column_lower,
					nv, v_minimum, mid_val)
				split_along_v (span - order + 1, span, control_points.column_lower+nv,
					nv, mid_val, v_maximum)
				
				process_v_split
			end

				-- Clear the neighbors from Current since they're no
				-- longer valid
			from
				i := left
			invariant
				i_at_least_one: i >= left
				i_small_enough: i <= left + below + 1
			variant
				i_increasing: left + below + 1 - i
			until
				i > below
			loop
				neighbors.item (i).wipe_out
				i := i + 1
			end

			updating_neighbors := false

			from
				split_items.start
			until
				split_items.after
			loop
				split_items.item_for_iteration.toggle_updating_neighbors
				-- Set the direction here, 'cause for nurbs it's 
				-- always reset in the split_along_[uv] routines
				split_items.item_for_iteration.set_split_direction (split_in_u)
				
				split_items.forth
			end

			is_split := true
		end

	split_edge (pt1, pt2: POINT2): BOOLEAN is
			-- determine whether an edge should be split
		do
			Result := true
			edge_length := pt1.distance_squared (pt2)
		end
	
	split_center (pts: ARRAY [POINT2]): BOOLEAN is
			-- determine whether a tile should be split in the center
		do
		end
	
	evaluate_pt (pt: POINT2): POINT3 is
			-- The value at `pt'
		do
			compute_derivatives (pt.x, pt.y, 1)
			create Result.make (derivatives.item (0, 0).x, 
								derivatives.item (0, 0).y, 
								derivatives.item (0, 0).z)
		end
	
	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		local
			pt: POINT3
		do
			pt := result_data.point
			compute_derivatives (u, v, 1)
			pt.make (derivatives.item (0, 0).x, 
				derivatives.item (0, 0).y, 
				derivatives.item (0, 0).z)
			if not points_only then
					check
						implemented: false
					end
			end
			
			pt.apply (object_to_world)
		end

	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end 

	update_extents is
			-- Compute new `extents' information
		local
			sb: SCENE_BOUND
			i, j: INTEGER
			old_control_points: BOUNDED_ARRAY2 [POINT4]
			old_u_knots, old_v_knots: KNOT_VECTOR
			new_u_knots, new_v_knots: DOUBLE_ARRAYED_LIST
		do
			old_control_points := control_points.deep_twin
			create old_u_knots
			old_u_knots.copy (u_knots)
			create old_v_knots
			old_v_knots.copy (v_knots)
			
			create new_u_knots.make (2)
			create new_v_knots.make (2)

			clamp
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + u_knots.count
			variant
				i_increasing: 1 + u_knots.count - i
			until
				i >= u_knots.count
			loop
				if u_knots @ i /= u_knots @ (i + 1) then
					new_u_knots.force_last (0.75 * u_knots @ i + 0.25 * u_knots @ (i + 1) )
					new_u_knots.force_last ((u_knots @ i + u_knots @ (i + 1) ) / 2)
					new_u_knots.force_last (0.25 * u_knots @ i + 0.75 * u_knots @ (i + 1) )
				end
				
				i := i + 1
			end
			
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + v_knots.count
			variant
				i_increasing: 1 + v_knots.count - i
			until
				i >= v_knots.count
			loop
				if v_knots @ i /= v_knots @ (i + 1) then
					new_v_knots.force_last (0.75 * v_knots @ i + 0.25 * v_knots @ (i + 1) )
					new_v_knots.force_last ((v_knots @ i + v_knots @ (i + 1) ) / 2)
					new_v_knots.force_last (0.25 * v_knots @ i + 0.75 * v_knots @ (i + 1) )
				end
				
				i := i + 1
			end
			
			refine_in_u (new_u_knots)
			refine_in_v (new_v_knots)
			
			sb := extents
			
			sb.make (control_points.item (0, 0).dehomogenize,
					 control_points.item (0,1).dehomogenize)
			
			from
				i := control_points.row_lower
			invariant
				i_at_least_one: i >= control_points.row_lower
				i_small_enough: i <= control_points.row_lower + control_points.row_upper + 1
			variant
				i_increasing: control_points.row_lower + control_points.row_upper - i + 1
			until
				i > control_points.row_upper
			loop
				from
					j := control_points.column_lower 
				invariant
					j_at_least_one: j >= control_points.column_lower
					j_small_enough: j <= control_points.column_lower + control_points.column_upper + 1
				variant
					j_increasing: control_points.column_lower + control_points.column_upper - j + 1
				until
					j > control_points.column_upper
				loop
					sb.expand_to_include_point (control_points.item (i, j).dehomogenize)
					j := j + 1
				end
				i := i + 1
			end
			
			control_points := old_control_points
			u_knots := old_u_knots
			v_knots := old_v_knots
		end 
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SCENE_NURBS} -- Implementation
	
	derivatives: BOUNDED_ARRAY2 [VECTOR3]
			-- The derivatives `evaluate'd at (u, v)
	
	homogeneous_derivatives: BOUNDED_ARRAY2 [VECTOR4]
			-- The derivatives `evaluate'd at (u, v)
	
	control_points: BOUNDED_ARRAY2 [POINT4]
			-- The mesh of control points

	copy_neighbors (other: like Current) is
			-- 
		require 
			other_not_void: other /= Void
		do
			neighbors := other.neighbors.twin
			allocate_neighbors
			if other.handness_set then
				set_handness (other.is_left_handed)
			end
		ensure 
			default: true
		end 

feature {NONE} -- Implementation

	split_along_u (knot_start, knot_count, cp_start, cp_count: INTEGER;
		new_min, new_max: DOUBLE) is
			-- Split the object in the U dimension
		local
			i, ii, j: INTEGER
			split_part: like Current
			new_knots: DOUBLE_ARRAYED_LIST
			points: BOUNDED_ARRAY2 [POINT4]
		do
			create points.make (0, cp_count - 1,
				control_points.column_lower, control_points.column_upper)

			from
				i := 1
				j := knot_start
				create new_knots.make (knot_count)
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + knot_count
			variant
				i_increasing: 1 + knot_count - i
			until
				i > knot_count
			loop
				new_knots.put (u_knots.item (j), i)
				j := j + 1
				i := i + 1
			end
			
			from
				i := cp_start
				ii := 0
			invariant
				i_at_least_one: i >= cp_start
				i_small_enough: i <= cp_start + cp_start + cp_count
			variant
				i_increasing: cp_start + cp_start + cp_count - i
			until
				i > cp_start + cp_count - 1
			loop
				from
					j := control_points.column_lower 
				invariant
					j_at_least_one: j >= control_points.column_lower
					j_small_enough: j <= control_points.column_lower + control_points.column_upper + 1
				variant
					j_increasing: control_points.column_lower + control_points.column_upper - j + 1
				until
					j > control_points.column_upper
				loop
					points.put (control_points.item (i, j), ii, j)
					
					j := j + 1
				end
				i := i + 1
				ii := ii + 1
			end

			create split_part.make_from_control_points (cp_count, u_knots.order, new_knots, new_min, new_max,
				v_count, v_knots.order, v_knots.knots, v_minimum, v_maximum,
				points)
			split_part.u_knots.normalize

			split_part.toggle_updating_neighbors
			
			split_part.copy_neighbors (Current)
			
			split_items.put_last (split_part)
		end
	
	split_along_v (knot_start, knot_count, cp_start, cp_count: INTEGER;
		new_min, new_max: DOUBLE) is
			-- Split the object in the V dimension
		local
			i, ii, j: INTEGER
			split_part: like Current
			new_knots: DOUBLE_ARRAYED_LIST
			points: BOUNDED_ARRAY2 [POINT4]
		do
			create points.make (control_points.row_lower, control_points.row_upper,
				0, cp_count - 1)

			from
				i := 1
				j := knot_start
				create new_knots.make (knot_count)
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + knot_count
			variant
				i_increasing: 1 + knot_count - i
			until
				i > knot_count
			loop
				new_knots.put (v_knots.item (j), i)
				j := j + 1
				i := i + 1
			end
			
			from
				i := cp_start
				ii := 0
			invariant
				i_at_least_one: i >= cp_start
				i_small_enough: i <= cp_start + cp_start + cp_count
			variant
				i_increasing: cp_start + cp_start + cp_count - i
			until
				i > cp_start + cp_count - 1
			loop
				from
					j := control_points.row_lower 
				invariant
					j_at_least_one: j >= control_points.row_lower
					j_small_enough: j <= control_points.row_lower + control_points.row_upper + 1
				variant
					j_increasing: control_points.row_lower + control_points.row_upper - j + 1
				until
					j > control_points.row_upper
				loop
					points.put (control_points.item (j, i), j, ii)
					
					j := j + 1
				end
				i := i + 1
				ii := ii + 1
			end

			create split_part.make_from_control_points (u_count, u_knots.order, u_knots.knots, u_minimum, u_maximum,
				cp_count, v_knots.order, new_knots, new_min, new_max,
				points)
			split_part.v_knots.normalize

			split_part.toggle_updating_neighbors
			
			split_part.copy_neighbors (Current)
			
			split_items.put_last (split_part)
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the sphere's features
		do
			Precursor (action)
			
			action.io.put_string ("	 u count = ")
			action.io.put_double (u_count)
			action.io.put_new_line
		end
	
	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			sce: SCENE_COLOR_ELEMENT
			sne: SCENE_NORMAL_ELEMENT
			nurber: NURBS_RENDERER
		do
			
			if is_left_handed then
				renderer.front_face (renderer.rd_cw)
			else
				renderer.front_face (renderer.rd_ccw)				
			end
			
			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
			
			sne := get_normals
			if sne = Void then
				compute_normals
				sne := get_normals
			end
			
			check
				color_element_not_void: sce /= Void
			end
			
			renderer.color_material (renderer.rd_front_and_back,
							   renderer.rd_ambient_and_diffuse)
			renderer.enable (renderer.rd_color_material_constant)
			
			renderer.rgba_color (sce.i_th (1))
			
			nurber := renderer.new_nurbs_renderer
			nurber.set_knots (u_knots.knots, v_knots.knots)
			nurber.set_order (u_order, v_order)
			nurber.use_domain_distance
			nurber.set_u_step (display_smoothness)
			nurber.set_v_step (display_smoothness)
			
			nurber.begin_surface
			nurber.define_control_points (u_count, v_count, control_points)
			nurber.define_normals (u_count, v_count, sne.values)
			nurber.end_surface
		end 
	
	compute_normals is
			-- Compute the normals at the mesh points
		local
			i, j: INTEGER
			stride_u, stride_v, u, v: DOUBLE
			sne: SCENE_NORMAL_ELEMENT
		do
			stride_u := 1.0 / (u_count - 1)
			stride_v := 1.0 / (v_count - 1)
			
			sne := get_normals
			if sne = Void then
				create sne
			else
				sne.remove_all_values
			end
			
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + u_count
			variant
				i_increasing: 0 + u_count - i
			until
				i > u_count - 1
			loop
				u := i * stride_u
				
				from
					j := 0
					v := 0
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= 0 + v_count
				variant
					j_increasing: 0 + v_count - j
				until
					j > v_count - 1
				loop
					v := j * stride_v
					
					sne.add_item (normal (u, v))
					
					j := j + 1
				end 
				
				i := i + 1
			end
			
			replace_element (sne)
		ensure 
			has_normals: get_normals /= Void
		end 
	
	compute_derivatives_new (u, v: DOUBLE; degree: INTEGER) is
			-- Compute the array of derivatives up to `degree' at (`u', `v') 
		require 
			u_in_range: u >= 0 and u <= 1
			v_in_range: v >= 0 and v <= 1
			degree_strictly_positive: degree >= 1
		local
			pv: VECTOR3
			vv: VECTOR4
			k, l: INTEGER
		do
			derivatives.resize (0, degree, 0, degree)
			compute_homogeneous_derivatives (u, v, degree)
			
			create pv
			
			from
				k := 0 
			invariant
				k_at_least_one: k >= 0
				k_small_enough: k <= 0 + degree + 1
			variant
				k_increasing: 0 + degree - k + 1
			until
				k > degree
			loop
				from
					l := 0 
				invariant
					l_at_least_one: l >= 0
					l_small_enough: l <= 0 + degree - k + 1
				variant
					l_increasing: 0 + degree - k - l + 1
				until
					l > degree - k
				loop
					vv := homogeneous_derivatives.item (k, l)
					pv.make (vv.x, vv.y, vv.z)
					
					derivatives.put (pv.scale (1.0/homogeneous_derivatives.item (0, 0).w), k, l)
					l := l + 1
				end
				k := k + 1
			end
		end
	
	compute_derivatives (u, v: DOUBLE; degree: INTEGER) is
			-- Compute the array of derivatives up to `degree' at (`u', `v') 
		require 
			u_in_range: u >= 0 and u <= 1
			v_in_range: v >= 0 and v <= 1
			degree_strictly_positive: degree >= 1
		local
			pv, pv2: VECTOR3
			vv: VECTOR4
			i, j, k, l: INTEGER
			coef: BOUNDED_ARRAY2 [DOUBLE]
		do
			derivatives.resize (0, degree, 0, degree)
			compute_homogeneous_derivatives (u, v, degree)
			
			coef := binomial_coefficients (degree + 1, degree + 1)
			
			create pv
			create pv2
			
			from
				k := 0 
			invariant
				k_at_least_one: k >= 0
				k_small_enough: k <= 0 + degree + 1
			variant
				k_increasing: 0 + degree - k + 1
			until
				k > degree
			loop
				from
					l := 0 
				invariant
					l_at_least_one: l >= 0
					l_small_enough: l <= 0 + degree - k + 1
				variant
					l_increasing: 0 + degree - k - l + 1
				until
					l > degree - k
				loop
					vv := homogeneous_derivatives.item (k, l)
					pv.make (vv.x, vv.y, vv.z)
					
					from
						j := 1 
					invariant
						j_at_least_one: j >= 1
						j_small_enough: j <= 1 + l
					variant
						j_increasing: 1 + l - j
					until
						j > l
					loop
						pv := pv - derivatives.item (k, l - j).scale (coef.item (l, j) * 
														homogeneous_derivatives.item (0, j).w)
						j := j + 1
					end
					
					from
						i := 1 
					invariant
						i_at_least_one: i >= 1
						i_small_enough: i <= 1 + k
					variant
						i_increasing: 1 + k - i
					until
						i > k
					loop
						pv := pv - derivatives.item (k - i, l).scale (coef.item (k, i) * 
														homogeneous_derivatives.item (i, 0).w)
						pv2.fill (0)
						from
							j := 1 
						invariant
							j_at_least_one: j >= 1
							j_small_enough: j <= 1 + l
						variant
							j_increasing: 1 + l - j
						until
							j > l
						loop
							pv2 := pv2 + derivatives.item (k - i, l - j).scale (coef.item  (l, j) * homogeneous_derivatives.item (i, j).w)
							
							j := j + 1
						end
						pv := pv - pv2.scale (coef.item (k, i))
						i := i + 1
					end
					
					derivatives.put (pv.scale (1.0/homogeneous_derivatives.item (0, 0).w), k, l)
					l := l + 1
				end
				k := k + 1
			end
		end 
	
	binomial_coefficients (n, k: INTEGER): BOUNDED_ARRAY2 [DOUBLE] is
			-- Create an array of binomial coefficients up to (n k)
		local
			i, j: INTEGER
		do
			create Result.make (0, n, 0, k)
			Result.put (1, 0, 0)
			
			from
				i := 0 
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + n
			variant
				i_increasing: 0 + n - i
			until
				i > n - 1
			loop
				Result.put (1, i + 1, 0)
				from
					j := 1 
				invariant
					j_at_least_one: j >= 1
					j_small_enough: j <= 1 + k
				variant
					j_increasing: 1 + k - j
				until
					j > k
				loop
					if i + 1 >= j then
						Result.put (Result.item (i, j) + Result.item (i, j - 1), i + 1, k)
					end
					j := j + 1
				end
				i := i + 1
			end
		end 

	compute_homogeneous_derivatives (u, v: DOUBLE; degree: INTEGER) is
			-- Compute the array of derivatives up to `degree' at (`u', `v') 
		require 
			u_in_range: u >= 0 and u <= 1
			v_in_range: v >= 0 and v <= 1
			degree_strictly_positive: degree >= 1
		local
			dd, du, dv, k, l, r, s: INTEGER
			u_span, v_span: INTEGER
			nu, nv: BOUNDED_ARRAY2 [DOUBLE]
			tmp_pt, pt: POINT4
			vec: VECTOR4
			pts: ARRAY [POINT4]
		do
			du := minimum (degree, u_degree)
			dv := minimum (degree, v_degree)
			
			homogeneous_derivatives.resize (0, degree, 0, degree)
			
			initialize_derivatives
			
			u_span := u_knots.find_span (u)
			v_span := v_knots.find_span (v)
			
			nu := derivative_basis_functions (du, u, u_span, u_degree, u_knots)
			nv := derivative_basis_functions (dv, v, v_span, v_degree, v_knots)
			
			create pts.make (0, v_degree + 1)
			
			from
				k := 0 
			invariant
				k_at_least_one: k >= 0
				k_small_enough: k <= 0 + du + 1
			variant
				k_increasing: 0 + du - k + 1
			until
				k > du
			loop
				from
					s := 0 
				invariant
					s_at_least_one: s >= 0
					s_small_enough: s <= 0 + v_degree + 1
				variant
					s_increasing: 0 + v_degree - s + 1
				until
					s > v_degree
				loop
					create tmp_pt
					
					from
						r := 0 
					invariant
						r_at_least_one: r >= 0
						r_small_enough: r <= 0 + u_degree + 1
					variant
						r_increasing: 0 + u_degree - r + 1
					until
						r > u_degree
					loop
						pt := control_points.item (u_span - 1 - u_degree + r, v_span - 1 - v_degree + s) * nu.item (k, r)
						tmp_pt := tmp_pt.weighted_sum (1.0, pt, 1.0)
						r := r + 1
					end
					pts.put (tmp_pt, s)
					
					s := s + 1
				end
				dd := minimum (degree - k, dv)
				
				from
					l := 0 
				invariant
					l_at_least_one: l >= 0
					l_small_enough: l <= 0 + dd + 1
				variant
					l_increasing: 0 + dd - l + 1
				until
					l > dd
				loop
					create tmp_pt
										
					from
						s := 0 
					invariant
						s_at_least_one: s >= 0
						s_small_enough: s <= 0 + v_degree + 1
					variant
						s_increasing: 0 + v_degree - s + 1
					until
						s > v_degree
					loop
						tmp_pt := tmp_pt.weighted_sum (1.0, pts.item (s), nv.item (l, s))
						
						s := s + 1
					end
					create vec.make (tmp_pt.x, tmp_pt.y, tmp_pt.z, tmp_pt.w)
					homogeneous_derivatives.put (vec, k, l)
					l := l + 1
				end
				k := k + 1
			end
		end 
	
	minimum (i1, i2: INTEGER): INTEGER is
			-- The minimum of `i1' and `i2'
		do
			if i1 < i2 then
				Result := i1
			else
				Result := i2
			end
		end
	
	derivative_basis_functions (n: INTEGER; u: DOUBLE; span, deg: INTEGER; 
								knots: KNOT_VECTOR): BOUNDED_ARRAY2 [DOUBLE] is
			-- Compute the derivatives functions at `u' of a NURBS curve
		require 
			n_positive: n >= 0
			deg_positive: deg >= 1
			knots_not_void: knots /= Void
		local
			r, j, k, s1, s2, rk, pk, j1, j2: INTEGER
			left_side, right_side: ARRAY [DOUBLE]
			a, ndu: BOUNDED_ARRAY2 [DOUBLE]
			d, saved, temp: DOUBLE
		do
			create Result.make (0, n, 0, deg)
			create ndu.make (0, deg, 0, deg)
			create right_side.make (0, deg)
			create left_side.make (0, deg)
			
			ndu.put (1, 0, 0)
			
			from
				j := 1 
			invariant
				j_at_least_one: j >= 1
				j_small_enough: j <= 1 + deg
			variant
				j_increasing: 1 + deg - j
			until
				j > deg
			loop
				left_side.put (u - knots @ (span + 1 - j), j)
				right_side.put (knots @ (span + j) - u, j)
				saved := 0
				
				from
					r := 0 
				invariant
					r_at_least_one: r >= 0
					r_small_enough: r <= 0 + j
				variant
					r_increasing: 0 + j - r
				until
					r >= j
				loop
					-- Lower triangle
					ndu.put (right_side.item (r + 1) + left_side.item (j - r), j, r)
					temp := ndu.item (r, j - 1) / ndu.item (j, r)
					
					-- Upper triangle
					ndu.put (saved + right_side.item (r + 1) * temp, r, j)
					saved := left_side.item (j - r) * temp
					r := r + 1
				end
				
				ndu.put (saved, j, j)
				
				j := j + 1
			end
			
			from
				j := 0 
			invariant
				j_at_least_one: j >= 0
				j_small_enough: j <= 0 + deg + 1
			variant
				j_increasing: 0 + deg - j + 1
			until
				j > deg
			loop
				Result.put (ndu.item (j, deg), 0, j)
				j := j + 1
			end
			
			-- Compute the derivatives
			create a.make (0, deg + 1, 0, deg + 1)
			
			from
				r := 0 
			invariant
				r_at_least_one: r >= 0
				r_small_enough: r <= 0 + deg + 1
			variant
				r_increasing: 0 + deg - r + 1
			until
				r > deg
			loop
				s1 := 0
				s2 := 1
				a.put (1, 0, 0)
				
				from
					k := 1
				invariant
					k_at_least_one: k >= 1
					k_small_enough: k <= 1 + n
				variant
					k_increasing: 1 + n - k
				until
					k > n
				loop
					d := 0
					rk := r - k
					pk := deg -k
					
					if r >= k then
						a.put (a.item (s1, 0) / ndu.item (pk + 1, rk), s2, 0)
						d := a.item (s2, 0) * ndu.item (rk, pk)
					end
					
					if rk >= -1 then
						j1 := 1
					else
						j1 := -rk
					end
					
					if r -1 <= pk then
						j2 := k - 1
					else
						j2 := deg - r
					end
					
					from
						j := j1 
					invariant
						j_at_least_one: j >= j1
						j_small_enough: j <= j1 + j2
					variant
						j_increasing: j1 + j2 - j
					until
						j > j2
					loop
						a.put ((a.item (s1, j) - a.item (s1, j - 1))/ ndu.item (pk + 1, rk + j), 
							   s2, j)
						d := d + a.item (s2, j) * ndu.item (rk + j, pk)
						j := j + 1
					end
					
					if r <= pk then
						a.put (-a.item (s1, k - 1) / ndu.item (pk + 1, r), s2, k)
						d := d + a.item (s2, k) * ndu.item (r, pk)
					end
					
					Result.put (d, k, r)
					-- Switch rows
					j := s1
					s1 := s2
					s2 := j
					
					k := k + 1
				end
				r := r + 1
			end
			
			r := deg
			from
				k := 1 
			invariant
				k_at_least_one: k >= 1
				k_small_enough: k <= 1 + n
			variant
				k_increasing: 1 + n - k
			until
				k > n
			loop
				from
					j := 0 
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= 0 + deg + 1
				variant
					j_increasing: 0 + deg - j + 1
				until
					j > deg
				loop
					Result.put (Result.item (k, j) * r, k, j)
					j := j + 1
				end
				r := r * (deg - k)
				k := k + 1
			end
		ensure 
			Result_not_void: Result /= Void
		end 

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray', 
			-- against this volume. It is assumed that the ray has 
			-- been transformed into this object's coordinate system
		local
			ice: SCENE_INDEXED_COLOR_ELEMENT
			ce: SCENE_COLOR_ELEMENT
		do
			if not tesselated then
				u_knots.normalize
				v_knots.normalize
				
				u_minimum := 0
				u_maximum := 1
				v_minimum := 0
				v_maximum := 1
				
				begin_tesselation (Current)
				
				ice := get_indexed_colors
				if ice = Void then
					ce := get_colors
					check
						color_element_not_void: ce /= Void
					end
					create ice.make_from_list (ce.values)
					add_element (ice)
				end
			end
			
			Precursor (intersecter)
		end 

	refine_in_u (new_knots: DOUBLE_ARRAYED_LIST) is
			-- Refine the surface by adding `new_knots'
		require 
			new_knots_not_void: new_knots /= Void
		local
			a, b, i, ind, j, k, l, row: INTEGER_64
			alpha: DOUBLE
			pt: POINT4
			new_control_points: BOUNDED_ARRAY2 [POINT4]
			old_knots: KNOT_VECTOR
		do
			a := u_knots.find_span (new_knots.item (1))
			b := u_knots.find_span (new_knots.item (new_knots.count))
			-- Adjust a & b based on knot-vector being one-based
			a := a - 1
			
			i := b + u_degree - 1
			k := b + u_degree + new_knots.count - 1
			
			create old_knots
			old_knots.copy (u_knots)
			u_knots.refine (new_knots)
			
			create new_control_points.make (integer_64_to_integer (control_points.row_lower),
										integer_64_to_integer ((control_points.row_upper + new_knots.count)),
										integer_64_to_integer (control_points.column_lower),
										integer_64_to_integer (control_points.column_upper))
			
			from
				row := control_points.column_lower 
			invariant
				row_at_least_one: row >= control_points.column_lower
				row_small_enough: row <= control_points.column_lower + 
														   control_points.column_upper + 1
			variant
				row_increasing: control_points.column_lower + 
										 control_points.column_upper - row +1
			until
				row > control_points.column_upper
			loop
				from
					j := control_points.row_lower
				invariant
					j_at_least_one: j >= control_points.row_lower
					j_small_enough: j <= control_points.row_lower + 
													  control_points.row_lower + a - u_degree + 1
				variant
					j_increasing: control_points.row_lower + 
										 control_points.row_lower + a - u_degree - j + 1
				until
					j > control_points.row_lower + a - u_degree
				loop
					new_control_points.put (control_points.item (j.to_integer, row.to_integer), j.to_integer, row.to_integer)
					j := j + 1
				end
				
				from
					j := control_points.row_lower + b - 1
				invariant
					j_at_least_one: j >= control_points.row_lower + b - 1
					j_small_enough: j <= control_points.row_lower + b - 1 + 
													  control_points.row_upper
				variant
					j_increasing: control_points.row_lower + b - 1 + 
										 control_points.row_upper - j
				until
					j > control_points.row_upper
				loop
					new_control_points.put (control_points.item (j.to_integer, row.to_integer), (j + new_knots.count).to_integer, row.to_integer)
					j := j + 1
				end
				row := row + 1
			end
			
			from
				j := new_knots.count
			until
				j = 0
			loop
				from
				until
					not (new_knots.item (j) <= old_knots.item (i + 1) and i > a)
				loop
					from
						row := control_points.column_lower 
					invariant
						row_at_least_one: row >= control_points.column_lower
						row_small_enough: row <= control_points.column_lower + 
																   control_points.column_upper +1
					variant
						row_increasing: control_points.column_lower + 
												 control_points.column_upper - row + 1
					until
						row > control_points.column_upper
					loop
						new_control_points.put (control_points.item ((i - u_degree - 1).to_integer, row.to_integer), (k - u_degree - 1).to_integer, row.to_integer)
						row := row + 1
					end
					
					k := k - 1
					i := i - 1
				end
				
				from
					row := control_points.column_lower 
				invariant
					row_at_least_one: row >= control_points.column_lower
					row_small_enough: row <= control_points.column_lower + 
															   control_points.column_upper +1 
				variant
					row_increasing: control_points.column_lower + 
											  control_points.column_upper - row +1
				until
					row > control_points.column_upper
				loop
					new_control_points.put (new_control_points.item ((k - u_degree).to_integer, row.to_integer), (k - u_degree - 1).to_integer, row.to_integer)
					row := row + 1
				end
				
				from
					l := 1 
				invariant
					l_at_least_one: l >= 1
					l_small_enough: l <= 1 + u_degree
				variant
					l_increasing: 1 + u_degree - l
				until
					not (l <= u_degree)
				loop
					ind := k - u_degree + l
					alpha := u_knots.item (k+l + 1) - new_knots.item (j)
					if alpha = 0 then
						from
							row := control_points.column_lower 
						invariant
							row_at_least_one: row >= control_points.column_lower
							row_small_enough: row <= control_points.column_lower + 
																	   control_points.column_upper + 1
						variant
							row_increasing: control_points.column_lower + 
													 control_points.column_upper - row + 1
						until
							row > control_points.column_upper
						loop
							new_control_points.put (control_points.item (ind.to_integer, row.to_integer), (ind - 1).to_integer, row.to_integer)
							row := row + 1
						end
					else
						alpha := alpha / (u_knots.item (k + l + 1) - old_knots.item (i - u_degree + l + 1))
						from
							row := control_points.column_lower 
						invariant
							row_at_least_one: row >= control_points.column_lower
							row_small_enough: row <= control_points.column_lower + 
																	   control_points.column_upper + 1
						variant
							row_increasing: control_points.column_lower + 
													  control_points.column_upper - row + 1
						until
							row > control_points.column_upper
						loop
							pt := new_control_points.item ((ind - 1).to_integer, row.to_integer).weighted_sum (alpha, 
																	new_control_points.item (ind.to_integer, row.to_integer), 1 - alpha)
							new_control_points.put (pt, (ind - 1).to_integer, row.to_integer)
							row := row + 1
						end
					end
					
					l := l + 1
				end
				j := j - 1
				k := k - 1
			end
			
			control_points := new_control_points
		ensure
			u_count_increased: u_count = old u_count + new_knots.count
		end 

	refine_in_v (new_knots: DOUBLE_ARRAYED_LIST) is
			-- Refine the surface by adding `new_knots'
		require 
			new_knots_not_void: new_knots /= Void
		local
			a, b, i, ind, j, k, l, col: INTEGER_64
			alpha: DOUBLE
			pt: POINT4
			new_control_points: BOUNDED_ARRAY2 [POINT4]
			old_knots: KNOT_VECTOR
		do
			a := v_knots.find_span (new_knots.item (1))
			b := v_knots.find_span (new_knots.item (new_knots.count))
			-- Adjust a & b based on knot-vector being one-based
			a := a - 1
			
			i := b + v_degree - 1
			k := b + v_degree + new_knots.count - 1
			
			create old_knots
			old_knots.copy (v_knots)
			
			v_knots.refine (new_knots)
			
			create new_control_points.make (integer_64_to_integer (control_points.row_lower),
										integer_64_to_integer (control_points.row_upper),
										integer_64_to_integer (control_points.column_lower),
										integer_64_to_integer ((control_points.column_upper + new_knots.count)))
			
			from
				col := control_points.row_lower 
			invariant
				col_at_least_one: col >= control_points.row_lower
				col_small_enough: col <= control_points.row_lower + 
														   control_points.row_upper + 1
			variant
				col_increasing: control_points.row_lower + 
										 control_points.row_upper - col +1
			until
				col > control_points.row_upper
			loop
				from
					j := control_points.column_lower
				invariant
					j_at_least_one: j >= control_points.column_lower
					j_small_enough: j <= control_points.column_lower + 
													  control_points.column_lower + a - v_degree + 1
				variant
					j_increasing: control_points.column_lower + 
										 control_points.column_lower + a - v_degree - j + 1
				until
					j > control_points.column_lower + a - v_degree
				loop
					new_control_points.put (control_points.item (col.to_integer, j.to_integer), col.to_integer, j.to_integer)
					j := j + 1
				end
				
				from
					j := control_points.column_lower + b - 1
				invariant
					j_at_least_one: j >= control_points.column_lower + b - 1
					j_small_enough: j <= control_points.column_lower + b - 1 + 
													  control_points.column_upper
				variant
					j_increasing: control_points.column_lower + b - 1 + 
										 control_points.column_upper - j
				until
					j > control_points.column_upper
				loop
					new_control_points.put (control_points.item (col.to_integer, j.to_integer), col.to_integer, (j + new_knots.count).to_integer)
					j := j + 1
				end
				col := col + 1
			end
			
			from
				j := new_knots.count
			until
				j = 0
			loop
				from
				until
					not (new_knots.item (j) <= old_knots.item (i + 1) and i > a)
				loop
					from
						col := control_points.row_lower 
					invariant
						col_at_least_one: col >= control_points.row_lower
						col_small_enough: col <= control_points.row_lower + 
																   control_points.row_upper +1
					variant
						col_increasing: control_points.row_lower + 
												 control_points.row_upper - col + 1
					until
						col > control_points.row_upper
					loop
						new_control_points.put (control_points.item (col.to_integer, (i - v_degree - 1).to_integer), col.to_integer, (k - v_degree - 1).to_integer)
						col := col + 1
					end
					
					k := k - 1
					i := i - 1
				end
				
				from
					col := control_points.row_lower 
				invariant
					col_at_least_one: col >= control_points.row_lower
					col_small_enough: col <= control_points.row_lower + 
															   control_points.row_upper +1 
				variant
					col_increasing: control_points.row_lower + 
											  control_points.row_upper - col +1
				until
					col > control_points.row_upper
				loop
					new_control_points.put (new_control_points.item ( col.to_integer, (k - v_degree).to_integer), col.to_integer, (k - v_degree - 1).to_integer)
					col := col + 1
				end
				
				from
					l := 1 
				invariant
					l_at_least_one: l >= 1
					l_small_enough: l <= 1 + v_degree
				variant
					l_increasing: 1 + v_degree - l
				until
					not (l <= v_degree)
				loop
					ind := k - v_degree + l
					alpha := v_knots.item (k+l + 1) - new_knots.item (j)
					if alpha = 0 then
						from
							col := control_points.row_lower 
						invariant
							col_at_least_one: col >= control_points.row_lower
							col_small_enough: col <= control_points.row_lower + 
																	   control_points.row_upper + 1
						variant
							col_increasing: control_points.row_lower + 
													 control_points.row_upper - col + 1
						until
							col > control_points.row_upper
						loop
							new_control_points.put (control_points.item (col.to_integer, ind.to_integer), col.to_integer, (ind - 1).to_integer)
							col := col + 1
						end
					else
						alpha := alpha / (v_knots.item (k + l + 1) - old_knots.item (i - v_degree + l + 1))
						from
							col := control_points.row_lower 
						invariant
							col_at_least_one: col >= control_points.row_lower
							col_small_enough: col <= control_points.row_lower + 
																	   control_points.row_upper + 1
						variant
							col_increasing: control_points.row_lower + 
													  control_points.row_upper - col + 1
						until
							col > control_points.row_upper
						loop
							pt := new_control_points.item (col.to_integer, (ind - 1).to_integer).weighted_sum (alpha,
								new_control_points.item (col.to_integer, ind.to_integer), 1 - alpha)
							new_control_points.put (pt, col.to_integer, (ind - 1).to_integer)
							col := col + 1
						end
					end
					
					l := l + 1
				end
				j := j - 1
				k := k - 1
			end
			
			control_points := new_control_points
		ensure
			v_count_increased: v_count = old v_count + new_knots.count
		end 

	initialize is
			-- Initialize the class
		do
			create {SCENE_BOX_BOUND} extents
			
			display_smoothness := 15
			
			u_minimum := 0
			u_maximum := 1
			v_minimum := 0
			v_maximum := 1
			
			create derivatives.make (0, 1, 0, 1)
			create homogeneous_derivatives.make (0, 1, 0, 1)
			
			debug ("nurb")
				create formatter.make (6, 5)
			end
				
		end 
	
	initialize_derivatives is
			-- Initialize the derivititives
		local
			i, j: INTEGER
			v3: VECTOR3
			v4: VECTOR4
		do
			create v3
			create v4
			
			from
				i := derivatives.row_lower 
			invariant
				i_at_least_one: i >= derivatives.row_lower
				i_small_enough: i <= derivatives.row_lower + derivatives.row_upper + 1
			variant
				i_increasing: derivatives.row_lower + derivatives.row_upper - i + 1
			until
				i > derivatives.row_upper
			loop
				from
					j := derivatives.column_lower 
				invariant
					j_at_least_one: j >= derivatives.column_lower
					j_small_enough: j <= derivatives.column_lower + derivatives.column_upper + 1
				variant
					j_increasing: derivatives.column_lower + derivatives.column_upper - j + 1
				until
					j > derivatives.column_upper
				loop
					if derivatives.item (i, j) = Void then
						derivatives.put (v3.twin, i, j)
					else
						derivatives.item (i, j).default_create
					end
					
					j := j + 1
				end
				
				i := i + 1
			end
		
			from
				i := homogeneous_derivatives.row_lower 
			invariant
				i_at_least_one: i >= homogeneous_derivatives.row_lower
				i_small_enough: i <= homogeneous_derivatives.row_lower + homogeneous_derivatives.row_upper + 1
			variant
				i_increasing: homogeneous_derivatives.row_lower + homogeneous_derivatives.row_upper - i + 1
			until
				i > homogeneous_derivatives.row_upper
			loop
				from
					j := homogeneous_derivatives.column_lower 
				invariant
					j_at_least_one: j >= homogeneous_derivatives.column_lower
					j_small_enough: j <= homogeneous_derivatives.column_lower + homogeneous_derivatives.column_upper + 1
				variant
					j_increasing: homogeneous_derivatives.column_lower + homogeneous_derivatives.column_upper - j + 1
				until
					j > homogeneous_derivatives.column_upper
				loop
					if homogeneous_derivatives.item (i, j) = Void then
						homogeneous_derivatives.put (v4.twin, i, j)
					else
						homogeneous_derivatives.item (i, j).default_create
					end
					
					j := j + 1
				end
				
				i := i + 1
			end
		end 
	
	clamp_in_u is
			-- Ensure a nonperiodic (clamped) surface in the u-direction
		local
			n1, n2, i, row: INTEGER
			new_control_points: BOUNDED_ARRAY2 [POINT4]
		do
			insert_knot_in_u (u_knots.item (u_order), u_degree)
			n1 := u_knots.insertion_count
			insert_knot_in_u (u_knots.item (u_knots.count - u_degree), u_degree)
			n2 := u_knots.insertion_count
			
			if n1 > 0 or n2 > 0 then
				u_knots.trim (u_minimum, u_maximum)
				create new_control_points.make (control_points.row_lower,
												control_points.row_upper - n1 - n2,
												control_points.column_lower,
												control_points.column_upper)
				
				from
					row := control_points.column_lower 
				invariant
					column_at_least_one: row >= control_points.column_lower
					column_small_enough: row <= control_points.column_lower + control_points.column_upper + 1
				variant
					column_increasing: control_points.column_lower + control_points.column_upper  + 1 - row
				until
					row > control_points.column_upper
				loop
					from
						i := new_control_points.row_lower 
					invariant
						i_at_least_one: i >= new_control_points.row_lower
						i_small_enough: i <= new_control_points.row_lower + new_control_points.row_upper + 1
						i_in_range: control_points.row_lower + i + n1 <= control_points.row_upper - n2 + 1
					variant
						i_increasing: new_control_points.row_lower + new_control_points.row_upper - i + 1
					until
						i > new_control_points.row_upper
					loop
						new_control_points.put (control_points.item (i + n1, row),
												i, row)
						i := i + 1
					end
					row := row + 1
				end
				control_points := new_control_points
			end
		
		ensure 
			u_knots_clamped: u_knots.clamped
		end 

	clamp_in_v is
			-- Ensure a nonperiodic (clamped) surface in the v-direction
		local
			n1, n2, i, col: INTEGER
			new_control_points: BOUNDED_ARRAY2 [POINT4]
		do
			insert_knot_in_v (v_knots.item (v_order), v_degree)
			n1 := v_knots.insertion_count
			insert_knot_in_v (v_knots.item (v_knots.count - v_degree), v_degree)
			n2 := v_knots.insertion_count
			
			if n1 > 0 or n2 > 0 then
				v_knots.trim (v_minimum, v_maximum)
				create new_control_points.make (control_points.row_lower,
												control_points.row_upper,
												control_points.column_lower,
												control_points.column_upper - n1 - n2)
				
				from
					col := control_points.row_lower 
				invariant
					column_at_least_one: col >= control_points.row_lower
					column_small_enough: col <= control_points.row_lower + control_points.row_upper + 1
				variant
					column_increasing: control_points.row_lower + control_points.row_upper	+ 1 - col
				until
					col > control_points.row_upper
				loop
					from
						i := new_control_points.column_lower 
					invariant
						i_at_least_one: i >= new_control_points.column_lower
						i_small_enough: i <= new_control_points.column_lower + new_control_points.column_upper + 1
						i_in_range: control_points.column_lower + i + n1 <= control_points.column_upper - n2 + 1
					variant
						i_increasing: new_control_points.column_lower + new_control_points.column_upper - i + 1
					until
						i > new_control_points.column_upper
					loop
						new_control_points.put (control_points.item (col, i + n1),
												col, i)
						i := i + 1
					end
					col := col + 1
				end
				control_points := new_control_points
			end
		
		ensure 
			v_knots_clamped: v_knots.clamped
		end 

	valid_control_points: BOOLEAN is
			-- Check if the control points are valid
		local
			i, j: INTEGER
		do
			Result := true
			from
				i := control_points.row_lower 
			invariant
				i_at_least_one: i >= control_points.row_lower
				i_small_enough: i <= control_points.row_lower + control_points.row_upper + 1
			variant
				i_increasing: control_points.row_lower + control_points.row_upper - i
			until
				i > control_points.row_upper or not Result
			loop
				from
					j := control_points.column_lower 
				invariant
					j_at_least_one: j >= control_points.column_lower
					j_small_enough: j <= control_points.column_lower + control_points.column_upper + 1
				variant
					j_increasing: control_points.column_lower + control_points.column_upper - j
				until
					j > control_points.column_upper or not Result
				loop
					Result := not (control_points.item (i, j) = Void)
					j := j + 1
				end
				i := i + 1
			end
		ensure 
			default: true
		end 

invariant
	
	derivatives_not_void: derivatives /= Void
	homogeneous_derivatives_not_void: homogeneous_derivatives /= Void
	u_knot_vector_not_void: u_knots /= Void
	v_knot_vector_not_void: v_knots /= Void
	control_points_not_void: control_points /= Void
	order_is_strictly_positive: u_order >= 1 and v_order >= 1
	u_degree_less_than_order: u_order = u_degree + 1
	v_degree_less_than_order: v_order = v_degree + 1
	display_smoothness_positive: display_smoothness > 0
	u_count_positive: u_count > 0
	v_count_positive: v_count > 0
	u_count_correct_relation: u_count + u_order = u_knots.count 
	v_count_correct_relation: v_count + v_order = v_knots.count 
	control_points_correct_for_u: (control_points.height > u_degree and 
								   control_points.height+ u_order = u_knots.count )
	control_points_correct_for_v: (control_points.width > v_degree and 
								   control_points.width + v_order = v_knots.count)
	control_point_valid: valid_control_points
	
end -- SCENE_NURBS

