

indexing
	description: "Bilinear and Bicubic Patches"
	patterns: "none"
	status: "unreviewed"
	Date: "$Date: 2007-01-25 13:00:51 -0500 (Thu, 25 Jan 2007) $"
	Revision: "$Revision: 280 $"



class

	SCENE_PATCH

inherit

	SCENE_GEOMETRY

creation
	make

feature -- Initialization

	make (points: POINT3_ARRAYED_LIST) is
			-- Create a patch, bilinear if `points'.count = 4,
			-- bicubic if `points'.count = 16
		require
			points_not_void: points /= Void
			points_count_correct: points.count = 4 or points.count = 16
		do
			default_create
			coordinates := points

			create {SCENE_BOX_BOUND} extents
			update_extents
		end

feature -- Access
feature -- Measurement

	coordinates: POINT3_ARRAYED_LIST
			-- The control points of the patch

	count: INTEGER_64 is
			-- The number of items
		do
			Result := 1
		end

	extents: SCENE_BOUND
			-- The `extents' or bounds of the volume.

	mapping_count: INTEGER_64 is
			-- The number of elements with respect to mapping colors,
			-- normals, ...
		do
			inspect color_mapping
			when constant_color then
				Result := 1
			when per_object, per_element then
				Result := 4
			when per_vertex then
				Result := coordinates.count
			end
		end

feature -- Comparison
feature -- Status report

	has_extents: BOOLEAN is
			-- Does this volume have extents?
		do
			Result := extents /= Void
		end

	is_linear: BOOLEAN is
			-- Is this patch linear (as opposed to cubic)?
		do
			Result := coordinates.count = 4
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		local
			left_pt, right_pt, pt: POINT3
			min_vec, max_vec, dpdu, dpdv: VECTOR3
			normal: NORMAL
			sce: SCENE_COLOR_ELEMENT
		do
			left_pt := coordinates.item (1) + (coordinates.item (3) - coordinates.item (1)).scale (v)
			right_pt := coordinates.item (2) + (coordinates.item (4) - coordinates.item (2)).scale (v)

			pt := result_data.point
			pt.copy (left_pt + (right_pt - left_pt).scale (u))
			result_data.set_two_sided (two_sided)

			if not points_only then
				dpdu := result_data.dpdu
				dpdv := result_data.dpdv
				min_vec := coordinates.item (2) - coordinates.item (1)
				max_vec := coordinates.item (4) - coordinates.item (3)
				dpdu.copy (min_vec + (max_vec - min_vec).scale (v))
				min_vec := coordinates.item (3) - coordinates.item (1)
				max_vec := coordinates.item (4) - coordinates.item (2)
				dpdv.copy (min_vec + (max_vec - min_vec).scale (u))

				normal := result_data.geometric_normal
				normal.make_from_vector (dpdu * dpdv)
				normal.apply (object_to_world)
				normal.normalize
				--result_data.set_geometric_normal (normal)

				sce := get_colors
					check
						color_element_not_void: sce /= Void
					end
				result_data.set_color (sce.i_th (1))
			end
			-- Transform to world space. Done here, since we need the
			-- point in object space to compute the normal
			pt.apply (object_to_world)
		end

	tesselate is
			-- Transform the object into a polygonal structure
		do
				check
					implemented: false
				end
		end -- tesselate

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	update_extents is
			-- Compute new `extents' information
		local
			sbb: SCENE_BOX_BOUND
			left_pt, right_pt, pt1, pt2: POINT3
		do
			create {SCENE_BOX_BOUND} extents
			sbb ?= extents
			sbb.default_create

			left_pt := coordinates.item (1) + (coordinates.item (3) - coordinates.item (1)).scale (v_minimum)
			right_pt := coordinates.item (2) + (coordinates.item (4) - coordinates.item (2)).scale (v_minimum)

			pt1 := left_pt + (right_pt - left_pt).scale (u_minimum)
			pt2 := left_pt + (right_pt - left_pt).scale (u_maximum)
			sbb.make (pt1, pt2)

			left_pt := coordinates.item (1) + (coordinates.item (3) - coordinates.item (1)).scale (v_maximum)
			right_pt := coordinates.item (2) + (coordinates.item (4) - coordinates.item (2)).scale (v_maximum)

			sbb.expand_to_include_point (left_pt + (right_pt - left_pt).scale (u_minimum))
			sbb.expand_to_include_point (left_pt + (right_pt - left_pt).scale (u_maximum))
		end

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object.
		local
			sce: SCENE_COLOR_ELEMENT
			sne: SCENE_NORMAL_ELEMENT
			p: POINTER
		do

			if is_left_handed then
				renderer.front_face (renderer.rd_cw)
			else
				renderer.front_face (renderer.rd_ccw)
			end

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end

			sne := get_normals
			if sne = Void then
				--compute_normals
				sne := get_normals
			end

--			check
--				normal_element_not_void: sne /= Void
--			end

			renderer.color_material (renderer.rd_front_and_back,
							   renderer.rd_ambient_and_diffuse)
			renderer.enable (renderer.rd_color_material_constant)

			renderer.rgba_color (sce.i_th (1))
			p := coordinates.storage.item
			renderer.enable (renderer.rd_map2_vertex_3)
			renderer.enable (renderer.rd_auto_normal)
			renderer.enable (renderer.rd_normalize)

			if is_linear then
				renderer.map_2d (renderer.rd_map2_vertex_3, 0, 1, 3, 2, 0, 1, 6, 2, p)
			end
			renderer.map_grid_2d (20, 0, 1, 20, 0, 1)
			renderer.eval_mesh_2d (renderer.rd_fill, 0, 20, 0, 20)
		end

	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		do
				check
					implemented: false
				end
		end

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		do
				check
					implemented: false
				end
		end

end -- SCENE_PATCH

