

indexing
	description: "Sphere Primitive"
	patterns: "none"
	date: "$Date: 2006-12-19 16:00:24 -0500 (Tue, 19 Dec 2006) $"
	revision: "$Revision: 275 $"
	status: "unreviewed"




class

	SCENE_SPHERE

inherit

	SCENE_QUADRIC
		redefine
			compute_intermediates
		end

creation
	default_create, make_by_parameters

feature -- Basic operations

	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		local
			alpha: DOUBLE
			pt: POINT3
			dpdu, dpdv: VECTOR3
			normal: NORMAL
			theta: DOUBLE
			cosine_theta, cosine_alpha_radius, sine_theta, sine_alpha_radius: DOUBLE
			sce: SCENE_COLOR_ELEMENT
		do
			alpha := alphamin + v * alphadelta
			theta := u * sweep_angle
			cosine_theta := cosine (theta)
			cosine_alpha_radius := cosine (alpha) * radius
			sine_theta := sine (theta)
			sine_alpha_radius := sine (alpha) * radius
			pt := result_data.point
			pt.make (cosine_theta * cosine_alpha_radius,
							sine_theta * cosine_alpha_radius,
							sine_alpha_radius)
			result_data.set_two_sided (two_sided)

			if not points_only then
				dpdu := result_data.dpdu
				dpdv := result_data.dpdv
				dpdu.make (sweep_angle * -sine_theta * cosine_alpha_radius,
						  sweep_angle * cosine_theta * cosine_alpha_radius,
						  0)
				dpdv.make (cosine_theta * alphadelta * -sine_alpha_radius,
						  sine_theta * alphadelta * -sine_alpha_radius,
						  alphadelta * cosine_alpha_radius)

				--result_data.set_derivatives (dpdu, dpdv)

				normal := result_data.geometric_normal
				normal.make (pt.x, pt.y, pt.z)
				normal.apply (object_to_world)
				normal.normalize
				--result_data.set_geometric_normal (normal)

				sce := get_colors
					check
						color_element_not_void: sce /= Void
					end
				result_data.set_color (sce.i_th (1))
			end
			-- Transform to world space. Done here, since we need the 
			-- point in object space to compute the normal
			pt.apply (object_to_world)
		end

	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end

feature -- Element change

	update_extents is
			-- Recompute the extents of the object
		local
			rvmin, rvmax, rmin, rmax, tmin, tmax: DOUBLE
			b: SCENE_BOX_BOUND
		do
			rvmin := cosine (alphamin + v_minimum * alphadelta) * radius
			rvmax := cosine (alphamin + v_maximum * alphadelta) * radius
			rmin := rvmin.min (rvmax)
			if alphamin + v_minimum * alphadelta < 0 and
				alphamin + v_maximum * alphadelta > 0 then
				rmax := radius
			else
				rmax := rvmin.max (rvmax)
			end

			tmin := u_minimum * sweep_angle
			tmax := u_maximum * sweep_angle

			create {SCENE_BOX_BOUND} extents
			sweep_for_bounds (rmin, rmax, tmin, tmax)

			b ?= extents

			b.lower.put_z (sine (alphamin + v_minimum * alphadelta) * radius)
			b.upper.put_z (sine (alphamin + v_maximum * alphadelta) * radius)
		end

feature {NONE} -- Implementation

	radius_squared: DOUBLE
	alphamin, alphadelta: DOUBLE

	compute_intermediates is
			-- Compute intermediate values used internally
		do
			Precursor

			radius_squared := radius * radius
			alphamin := arc_sine (z_minimum / radius )
			alphadelta := arc_sine (z_maximum / radius ) - alphamin

			update_extents

		ensure then
			radius_squared_set: about_equal (radius_squared, radius * radius)
		end

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			sce: SCENE_COLOR_ELEMENT
		do
			renderer.push_matrix
			renderer.translate (origin.x, origin.y, origin.z)

			if is_left_handed then
				renderer.front_face (renderer.rd_ccw)
			else
				renderer.front_face (renderer.rd_ccw)
			end

			if get_orientation.consistent (is_left_handed) then
				renderer.orientation (renderer.rd_outside)
			else
				renderer.orientation (renderer.rd_inside)
			end

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
			renderer.enable (renderer.rd_color_material_constant)
			renderer.color_material (renderer.rd_front_and_back,
							   renderer.rd_ambient_and_diffuse)

			renderer.rgba_color (sce.i_th (1))

			renderer.sphere (radius, z_minimum, z_maximum, sweep_angle,
							 u_minimum, u_maximum, v_minimum, v_maximum)
			renderer.front_face (renderer.rd_ccw)
			renderer.pop_matrix
		end

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray',
			-- against this volume. It is assumed that the ray has
			-- been transformed into this object's coordinate system
		local
			A, B, C, inv2A, discrim, root_discrim: DOUBLE
			t0, t1, min_t, max_t: DOUBLE
			hit_pt: POINT3
			u, v, theta, phi, val: DOUBLE
		do
			intersected := false

			-- Eventually, min_t and max_t should be a parameter to
			-- this routine.
			min_t := 0
			max_t := maximum_double

			A := intersecter.ray.direction.x * intersecter.ray.direction.x +
				intersecter.ray.direction.y * intersecter.ray.direction.y +
				intersecter.ray.direction.z * intersecter.ray.direction.z
			B := 2.0 * (intersecter.ray.direction.x * intersecter.ray.origin.x +
						intersecter.ray.direction.y * intersecter.ray.origin.y +
						intersecter.ray.direction.z * intersecter.ray.origin.z)
			C := intersecter.ray.origin.x * intersecter.ray.origin.x +
				intersecter.ray.origin.y * intersecter.ray.origin.y +
				intersecter.ray.origin.z * intersecter.ray.origin.z - radius_squared

			discrim := B * B - 4 * A * C
			if discrim >= 0 then
				root_discrim := square_root (discrim)
				inv2A := 1.0 / (2.0 * A)
				t0 := (-B - root_discrim) * inv2A
				if t0 <= max_t then
					t1 := (-B + root_discrim) * inv2A
					if t1 >= min_t then
						-- At least one intersection is in range
						-- So, Check the first potential intersection

						if t0 >= min_t then
							hit_pt := intersecter.ray.project (t0)
							if hit_pt.z >= z_minimum and hit_pt.z <= z_maximum then
								phi := arc_cosine (-hit_pt.z / radius)
								v := phi / pi
								if v = 0 or v = 1 then
									u := 0
								else
									val := clamp_to_range (hit_pt.x / (radius * sine (phi)), -1, 1)

									theta := arc_cosine (val) / (2 * pi)
									if hit_pt.y > 0 then
										u := theta
									else
										u := 1 - theta
									end

									u := u * (2 * pi) / sweep_angle

									if u <= 1 then
										insert_intersection_information (intersecter, u, v, t0,
																		 hit_pt, intersecter.ray)
										intersected := true
									end
								end
							end
						end

						-- Now, check the second potential intersection
						if t1 <= max_t then
							hit_pt := intersecter.ray.project (t1)
							if hit_pt.z >= z_minimum and hit_pt.z <= z_maximum then
								phi := arc_cosine (-hit_pt.z / radius)
								v := phi / pi
								if v = 0 or v = 1 then
									u := 0
								else
									val := clamp_to_range (hit_pt.x / (radius * sine (phi)), -1, 1)

									theta := arc_cosine (val) / (2 * pi)
									if hit_pt.y > 0 then
										u := theta
									else
										u := 1 - theta
									end

									u := u * (2 * pi) / sweep_angle

									if u <= 1 then
										insert_intersection_information (intersecter, u, v, t1,
																		 hit_pt, intersecter.ray)
										intersected := true
									end
								end
							end
						end
					end
				end
			end
		end

	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		local
			vv, vmin, vmax: DOUBLE
			normal: NORMAL
			info: INTERSECTION_DATA
			sce: SCENE_COLOR_ELEMENT
		do
			vmin := arc_cosine (-z_minimum / radius)
			vmax := arc_cosine (-z_maximum / radius)
			vv := (v * pi - vmin) / (vmax / vmin)

			create normal.make_from_vector (point - point.zero)

			normal.normalize

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end

			create info.make (t, point, normal,
							  u, vv, sce.i_th (1), ray,
							  Current)
			action.intersection.force_last (info)
		end

end -- SCENE_SPHERE

