

indexing
	description: "Torus Primitive"
	patterns: "none"
	date: "$Date: 2007-02-09 16:45:54 -0500 (Fri, 09 Feb 2007) $"
	revision: "$Revision: 282 $"
	status: "unreviewed"




class

	SCENE_TORUS

inherit

	MEMORY
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		redefine
			dispose
		end

	SCENE_QUADRIC
		rename
			make_by_parameters as quadric_make_by_parameters,
			radius as major_radius,
			set_radius as set_major_radius,
			valid_radius as valid_major_radius,
			z_minimum as phi_minimum,
			set_z_minimum as set_phi_minimum,
			valid_z_minimum as valid_phi_minimum,
			z_maximum as phi_maximum,
			set_z_maximum as set_phi_maximum,
			valid_z_maximum as valid_phi_maximum,
			insert_intersection_information as quadric_insert_intersection_information
		export
			{NONE} quadric_make_by_parameters, quadric_insert_intersection_information
		redefine
			default_create, compute_intermediates, quadric_make_by_parameters,
			add_attributes, dispose, process_v_split
		end

creation
	default_create, make_by_parameters

feature -- Initialization

	default_create is
			-- Create a full torus of major_radius 1
		do
			Precursor {SCENE_QUADRIC}

			minor_radius := 0.5
			phi_minimum := 0
			phi_maximum := 2 * pi

			create dpdu
			create dpdv

			create coeff.make (5)
			create roots.make (8)
			gsl_poly := gsl_poly_complex_workspace_alloc (5)

			compute_intermediates

			set_v_wrapping
		end

	quadric_make_by_parameters (r, zmin, zmax, theta: DOUBLE) is
			-- Create a cone of radius `r', with height
			-- `zmax' - `zmin', and sweep angle `theta'
			-- (closed quadric = 360)
		do
			make_by_parameters (r, 0.5 * r, zmin, zmax, theta)
		end

	make_by_parameters (new_major_radius, new_minor_radius,
						new_phi_min, new_phi_max, theta: DOUBLE) is
			-- Create a torus of major radius `new_major_radius'
			-- (closed quadric = 360)
		do
			default_create

			major_radius := new_major_radius
			minor_radius := new_minor_radius
			phi_minimum := new_phi_min * pi / 180.0
			phi_maximum := new_phi_max * pi / 180.0

			sweep_angle := theta * pi / 180.0

			compute_intermediates

			set_v_wrapping
		end

feature -- Access

	minor_radius: DOUBLE
			-- The minor radius of the torus

feature -- Measurement
feature -- Comparison
feature -- Status report

	valid_minor_radius (r: DOUBLE): BOOLEAN is
		-- Is `r' valid for `minor_radius'?
		do
			Result := r > 0
		end

feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_minor_radius (new_minor_radius: DOUBLE) is
			-- Set `minor_radius' to `new_minor_radius'
		require
			new_minor_radius_is_valid: new_minor_radius > 0.0
		do
			minor_radius := new_minor_radius
			compute_intermediates
		ensure
			minor_radius_set: minor_radius = new_minor_radius
		end

	update_extents is
			-- Recompute the extents of the object
		local
			rmin, rmax, tmin, tmax, phimin, phimax: DOUBLE
			a, b: SCENE_BOX_BOUND
		do
			phimin := v_minimum * (phi_maximum - phi_minimum) + phi_minimum
			phimax := v_maximum * (phi_maximum - phi_minimum) + phi_minimum

			create {SCENE_BOX_BOUND} extents
			sweep_phi_for_bounds (minor_radius, phimin, phimax)
			a ?= extents

			rmin := a.lower.x + major_radius
			rmax := a.upper.x + major_radius

			tmin := u_minimum * sweep_angle
			tmax := u_maximum * sweep_angle

			create {SCENE_BOX_BOUND} extents
			sweep_for_bounds (rmin, rmax, tmin, tmax)
			b ?= extents

			b.lower.put_z (a.lower.y)
			b.upper.put_z (a.upper.y)
		end

feature -- Removal

	dispose is
			-- Dispose of the external polynomial solver
		do
--			if gsl_poly /= default_pointer then
--				gsl_poly_complex_workspace_free (gsl_poly)
--				gsl_poly := default_pointer
--			end
		end

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	evaluate (u, v: DOUBLE; points_only: BOOLEAN; result_data: SHADING_DATA) is
			-- What's the point at parametric coordinates (`u', `v')?
		local
			phi, dphi, sx, dsx: DOUBLE
			pt: POINT3
			t_dpdu, t_dpdv: VECTOR3
			normal: NORMAL
			distance, xm, ym, theta: DOUBLE
			sce: SCENE_COLOR_ELEMENT
		do
			phi := v * (phi_maximum - phi_minimum) + phi_minimum
			sx := cosine (phi) * minor_radius + major_radius
			theta := u * sweep_angle

			pt := result_data.point
			pt.make (sx * cosine (theta),
					 sx * sine (theta),
					 sine (phi) * minor_radius)
			pt.apply (object_to_world)
			result_data.set_point (pt)
			result_data.set_two_sided (two_sided)

			if not points_only then
				create t_dpdu.make (sx * sweep_angle * -sine (theta),
									sx * sweep_angle * cosine (theta),
									0)
				dphi := phi_maximum - phi_minimum
				dsx := dphi * -sine (phi) * minor_radius
				create t_dpdv.make (dsx * cosine (theta),
									dsx * sine (theta),
									dphi * cosine (phi) * minor_radius)


				result_data.set_derivatives (t_dpdu, t_dpdv)

				distance := square_root (pt.x * pt.x + pt.y * pt.y)
					check
						distance_greater_than_tolerance: distance > Tolerance
					end

				xm := major_radius * pt.x / distance
				ym := major_radius * pt.y / distance

				create normal.make (pt.x - xm,
									pt.y - ym,
									pt.z)
				normal.normalize
				result_data.set_geometric_normal (normal)

				sce := get_colors
					check
						color_element_not_void: sce /= Void
					end
				result_data.set_color (sce.i_th (1))
			end
		end

	tesselate is
			-- Transform the object into a polygonal structure
		do
			check
				implemented: false
			end
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	major_radius_squared, minor_radius_squared: DOUBLE

	coeff, roots: DOUBLE_ARRAYED_LIST
			-- Arrays for solving the roots of the quartic polynomial

	gsl_poly: POINTER
			-- Pointer to the external root finder

	u_value, v_value: DOUBLE
	dpdu, dpdv: VECTOR3
			-- For the calculation of `uv' and it's derivatives

	process_v_split is
			-- Set the neighbors for a split in v
		local
			first, second: SCENE_DRAWABLE_ELEMENT
		do
			if wrapped_in_v and then v_maximum - v_minimum > 0.5 then
				first := split_items.first
				second := split_items.last
				
				second.set_neighbor (first, below)
				second.set_neighbor (first, above)
				update_v_neighbors (first, below)
				update_u_neighbors (first, left)
				update_u_neighbors (first, right)
			
				update_v_neighbors (second, above)
				update_u_neighbors (second, left)
				update_u_neighbors (second, right)
			else
				Precursor
			end
		end

	set_v_wrapping is
			-- Set the wrapping parameter based on the `sweep_angle'
		do
			if about_equal ((phi_maximum - phi_minimum).abs, 2 * pi)  then
				set_neighbor (current, below)
				wrapped_in_v := true
			else
				remove_neighbor (current, below)
				wrapped_in_v := false
			end
		end 

	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add attributes for the GUI_ACTION
		local
			attribute: SCENE_ATTRIBUTE_INFO [DOUBLE]
		do
			create attribute.make ("major_radius", index_of_field ("major_radius"),
									agent valid_major_radius, agent set_major_radius)
			the_attributes.force_last (attribute)
			create attribute.make ("minor_radius", index_of_field ("minor_radius"),
									agent valid_minor_radius, agent set_minor_radius)
			the_attributes.force_last (attribute)
			create attribute.make ("phi_minimum", index_of_field ("phi_minimum"),
									agent valid_phi_minimum, agent set_phi_minimum)
			the_attributes.force_last (attribute)
			create attribute.make ("phi_maximum", index_of_field ("phi_maximum"),
									agent valid_phi_maximum, agent set_phi_maximum)
			the_attributes.force_last (attribute)
			create attribute.make ("sweep_angle", index_of_field ("sweep_angle"),
									agent valid_sweep_angle, agent set_sweep_angle)
			the_attributes.force_last (attribute)
		end

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		local
			sce: SCENE_COLOR_ELEMENT
		do
			renderer.push_matrix
			renderer.translate (origin.x, origin.y, origin.z)

			if not is_left_handed then
				renderer.front_face (renderer.rd_cw)
			else
				renderer.front_face (renderer.rd_ccw)
			end

			if get_orientation.consistent (is_left_handed) then
				renderer.orientation (renderer.rd_outside)
			else
				renderer.orientation (renderer.rd_inside)
			end

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end
			renderer.enable (renderer.rd_color_material_constant)
			renderer.color_material (renderer.rd_front_and_back,
							   renderer.rd_ambient_and_diffuse)

			renderer.rgba_color (sce.i_th (1))

			renderer.torus (minor_radius, major_radius, phi_minimum,
							phi_maximum, sweep_angle,
							u_minimum, u_maximum, v_minimum, v_maximum)
			renderer.front_face (renderer.rd_ccw)
			renderer.pop_matrix
		end

	compute_intermediates is
			--
		do
			Precursor

			major_radius_squared := major_radius * major_radius
			minor_radius_squared := minor_radius * minor_radius

			update_extents
		end

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray',
			-- against this volume. It is assumed that the ray has
			-- been transformed into this object's coordinate system
		local
			dd, de, val: DOUBLE
			point: POINT3
			i: INTEGER
		do
			intersected := false

			dd := intersecter.ray.direction # intersecter.ray.direction
			val := intersecter.ray.origin.x * intersecter.ray.origin.x +
				intersecter.ray.origin.y * intersecter.ray.origin.y +
				intersecter.ray.origin.z * intersecter.ray.origin.z -
				(major_radius_squared + minor_radius_squared)
			de := intersecter.ray.direction.x * intersecter.ray.origin.x + intersecter.ray.direction.y * intersecter.ray.origin.y + intersecter.ray.direction.z * intersecter.ray.origin.z

			coeff.put (dd * dd, 5)
			coeff.put (4 * dd * de, 4)
			coeff.put (2 * dd * val + 4 * de * de + 4.0 * major_radius_squared * intersecter.ray.direction.z * intersecter.ray.direction.z, 3)
			coeff.put (4.0 * de * val  +  8.0 * major_radius_squared * intersecter.ray.direction.z * intersecter.ray.origin.z, 2)
			coeff.put (val * val - 4.0 * major_radius_squared *
				(minor_radius_squared - intersecter.ray.origin.z * intersecter.ray.origin.z), 1)

			gsl_poly_complex_solve (coeff.storage.item, 5, gsl_poly, roots.storage.item)

			from
				i := 1
			until
				i > 8
			loop
				if roots.item (i + 1) = 0 then
					if roots.item (i) > 0 then
						intersected := true
						point := intersecter.ray.project (roots.item (i))
						insert_intersection_information (intersecter, roots.item (i),
														 point, intersecter.ray)

					end
				end
				i := i + 2
			end
		end

	insert_intersection_information (action: INTERSECT_ACTION;
									 t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		local
			normal: NORMAL
			distance, xm, ym: DOUBLE
			info: INTERSECTION_DATA
			sce: SCENE_COLOR_ELEMENT
		do

			sce := get_colors
			check
				color_element_not_void: sce /= Void
			end

			distance := square_root (point.x * point.x + point.y * point.y)

			check
				distance_greater_than_tolerance: distance > Tolerance
			end

			xm := major_radius * point.x / distance
			ym := major_radius * point.y / distance

			create normal.make (point.x - xm,
								point.y - ym,
								point.z)
			normal.normalize

			compute_uv_and_derivatives (point, normal)

			create info.make (t, point, normal,
							  u_value, v_value, sce.i_th (1), ray,
							  Current)
			action.intersection.force_last (info)
		end

	quadric_insert_intersection_information (action: INTERSECT_ACTION;
									 uu, vv, t: DOUBLE; point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information
			-- into `intersection'
		do
			insert_intersection_information (action, t, point, ray)
		end

	compute_uv_and_derivatives (pt: POINT3; normal: VECTOR3) is
			-- Compute `uv', `dp/du', and `dp/dv'
		require
			point_not_void: pt /= Void
			normal_not_void: normal /= Void
		local
			rad, cos_theta, sin_theta, cos_phi: DOUBLE
		do
			rad := square_root (pt.x * pt.x + pt.y * pt.y)
			cos_theta := pt.x / rad
			sin_theta := pt.y / rad
			if cos_theta > 1 then
				u_value := 0
			elseif cos_theta < -1 then
				u_value := 0.5
			else
				u_value := arc_cosine (cos_theta) / (2 * pi)
			end

			if sin_theta < 0 then
				u_value := 1 - u_value
			end

			dpdu.make (-pt.y, pt.x, 0)

			cos_phi := -(rad - major_radius) / minor_radius
			if cos_phi > 1 then
				v_value := 0
			elseif cos_phi < 1 then
				v_value := 0.5
			else
				v_value := arc_cosine (cos_phi) / (2 * pi)
			end

			if pt.z > 0 then -- sine (phi) > 0
				v_value := 1 - v_value
			end

			dpdv := normal * dpdu
		end

	gsl_poly_complex_workspace_alloc (num: INTEGER): POINTER is
		external
			"C use <gsl/gsl_poly.h>"
		end

	gsl_poly_complex_solve (a: POINTER; num: INTEGER; w, z: POINTER) is
		external
			"C use <gsl/gsl_poly.h>"
		end

	gsl_poly_complex_workspace_free (w: POINTER) is
		external
			"C use <gsl/gsl_poly.h>"
		end

	sweep_phi_for_bounds (rmin, phimin, phimax: DOUBLE) is
			-- Find the 2D bounds for part of the circle
			-- centered at the origin in the xy-plane
		local
			i: INTEGER
		do
			extents.make (polar (rmin, phimin), polar (rmin, phimin))
			extents.expand_to_include_point (polar (rmin, phimax))

			from
				i := -3
			until
				i >= 4
			loop
				if phimin < i * pi_over_two and phimax > i * pi_over_two then
					extents.expand_to_include_point (polar (rmin, i * pi_over_two))
				end
				i := i + 1
			end
		ensure
			default: true
		end
invariant

	dpdu_not_void: dpdu /= Void
	dpdv_not_void: dpdv /= Void
	wrapped_in_v: about_equal ((phi_maximum - phi_minimum).abs, 2 * pi) implies wrapped_in_v = true
	
end -- SCENE_TORUS

