

indexing
	description: "The shading data at a point of evaluation"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2006-12-10 07:10:42 -0500 (Sun, 10 Dec 2006) $"
	revision: "$Revision: 274 $"




class

	SHADING_DATA

inherit

	COMPARABLE
		undefine
			is_equal
		end

	BASIC_NUMERIC
		export
			{NONE} all
			{ANY} twin
		redefine
			is_equal
		end

creation
	make

feature -- Initialization

	make (new_pool: SHADING_DATA_POOL) is
			-- Create the data. All instances must have a valid `point',
			-- so create it by default
		do
			pool := new_pool

			create point
			create raster_point

			create dpdu
			create dpdv

			create geometric_normal
			create shading_normal

			create color
			create opacity

			create shaded_color
			create shaded_opacity

			reference_count := 0

			-- For debugging
			debug ("debug_referers")
				create debug_referers.make (10)
			end
		end

feature -- Access

	point, raster_point: POINT3
			-- The coordinates of the evaluation point

	dpdu, dpdv: VECTOR3
			-- The derivatives of `point' in terms of u, v at `point'

	du, dv: DOUBLE
			-- The change in u and v at `point'

	geometric_normal: NORMAL
			-- The vector at `point' tangent to the plane defined by
			-- `dpdu' and `dpdv'

	shading_normal: NORMAL
			-- A user provided normal at `point', if any

	color: RGBA_COLOR
	opacity: RGB_COLOR
			-- The color and opacity

	shaded_color: RGBA_COLOR
	shaded_opacity: RGB_COLOR
			-- The color and opacity after shading

	two_sided: BOOLEAN
			-- Is the surface attached to this point two sided?

	reference_count: INTEGER
			-- The number of references

feature -- Measurement
feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is `Current' less than `other'?
		do
			Result := reference_count < other.reference_count
		end

	is_equal (other: like Current): BOOLEAN is
			-- Do `Current' and other have equal-valued items?
		do
			Result := reference_count.is_equal (other.reference_count)
		end

feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_point (new_point: POINT3) is
			-- Set `point' to `new_point'
		require
			new_point_not_void: new_point /= Void
		do
			point.copy (new_point)
		ensure
			point_set: point.is_equal (new_point)
		end

	set_raster_point (new_point: POINT3) is
			-- Set `point' to `new_point'
		require
			new_point_not_void: new_point /= Void
		do
			raster_point.copy (new_point)
		ensure
			raster_point_set: raster_point.is_equal (new_point)
		end

	set_deltas (new_du, new_dv: DOUBLE) is
			-- Set `du' and `dv' respectively
		do
			du := new_du
			dv := new_dv
		ensure
			du_set: within_tolerance (du, new_du, Tolerance)
			dv_set: within_tolerance (dv, new_dv, Tolerance)
		end

	set_derivatives (new_dpdu, new_dpdv: VECTOR3) is
			-- Set `dpdu' and `dpdv' respectively
		require
			new_dpdu_not_void: new_dpdu /= Void
			new_dpdv_not_void: new_dpdv /= Void
		do
			dpdu := new_dpdu
			dpdv := new_dpdv
		ensure
			dpdu_set: dpdu = new_dpdu
			dpdv_set: dpdv = new_dpdv
		end

	set_geometric_normal (new_normal: NORMAL) is
			-- Set `geometric_normal' to `new_normal'
		require
			new_normal_not_void: new_normal /= Void
		do
			geometric_normal.copy (new_normal)
		ensure
			geometric_normal_set: geometric_normal.is_equal (new_normal)
		end

	set_shading_normal (new_normal: NORMAL) is
			-- Set `shading_normal' to `new_normal'
		require
			new_normal_not_void: new_normal /= Void
		do
			shading_normal.copy (new_normal)
		ensure
			shading_normal_set: shading_normal.is_equal (new_normal)
		end

	set_shaded_color (new_color: RGBA_COLOR) is
			-- Set `shaded_color' to `new_color'
		require
			new_color_not_void: new_color /= Void
		do
			shaded_color.copy (new_color)
		ensure
			shaded_color_set: shaded_color.is_equal (new_color)
		end

	set_color (new_color: RGBA_COLOR) is
			-- Set `color' to `new_color'
		require
			new_color_not_void: new_color /= Void
		do
			color.copy (new_color)
		ensure
			color_set: color.is_equal (new_color)
		end

	set_shaded_opacity (new_opacity: RGB_COLOR) is
			-- Set `shaded_opacity' to `new_opacity'
		require
			new_opacity_not_void: new_opacity /= Void
		do
			shaded_opacity.copy (new_opacity)
		ensure
			shaded_opacity_set: shaded_opacity.is_equal (new_opacity)
		end

	set_opacity (new_opacity: RGB_COLOR) is
			-- Set `opacity' to `new_opacity'
		require
			new_opacity_not_void: new_opacity /= Void
		do
			opacity.copy (new_opacity)
		ensure
			opacity_set: opacity.is_equal (new_opacity)
		end

	set_two_sided (on_or_off: BOOLEAN) is
			-- Set `two_sided' to `on_or_off'
		do
			two_sided := on_or_off
		ensure
			two_sided_set: two_sided = on_or_off
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	add_reference (item: ANY) is
			-- Add a reference
		do
			reference_count := reference_count + 1
			debug ("debug_referers")
				debug_referers.force_last (item)
			end
		ensure
			count_increased: reference_count = old reference_count + 1
		end

	remove_reference (item: ANY) is
			-- Remove a reference
		require
			reference_count_positive: reference_count > 0
		do
			reference_count := reference_count - 1
			if reference_count = 0 then
				release
			end

			debug ("debug_referers")
					check
						has_item: debug_referers.has (item)
					end
				debug_referers.start
				debug_referers.search_forth (item)
				debug_referers.remove_at
			end
		ensure
			count_increased: reference_count = old reference_count - 1
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SHADING_DATA_POOL} -- Implementation

	debug_referers: DS_ARRAYED_LIST [ANY]
			-- List of objects that refer to this element

feature {NONE} -- Implementation

	pool: SHADING_DATA_POOL
		-- Memory pool

	release is
			-- Free the data
		do
			pool.return (Current)
		end

	referer_list_correct: BOOLEAN is
			-- If the data is listed more than once, the refering
			-- object better have it more than one entry for the data
		local
			mp: MICROPOLYGON
			item_count, sd_count: INTEGER
		do
			Result := true
			debug ("debug_referers")
				from
					debug_referers.start
				until
					debug_referers.after or not Result
				loop
					item_count := debug_referers.occurrences (debug_referers.item_for_iteration)
					mp ?= debug_referers.item_for_iteration
					if mp /= Void then
						sd_count := 0
						if Current = mp.lower_left then
							sd_count := sd_count + 1
						end
						if Current = mp.upper_left then
							sd_count := sd_count + 1
						end
						if Current = mp.lower_right then
							sd_count := sd_count + 1
						end
						if Current = mp.upper_right then
							sd_count := sd_count + 1
						end

						Result := sd_count >= item_count
					end

					debug_referers.forth
				end
			end
		end

invariant

	point_not_void: point /= Void
	pool_not_void: pool /= Void
	reference_count_positive: reference_count >= 0
	debug_referers_count_is_correct: (debug_referers /= Void implies
									  debug_referers.count = reference_count)
	--a_referer_is_only_listed_once: referer_list_correct

end -- SHADING_DATA

