

indexing
	description: "A memory pool for SHADING_DATA"
	patterns: "singelton"
	status: "unreviewed"
	date: "$Date: 2006-09-03 16:32:39 -0400 (Sun, 03 Sep 2006) $"
	revision: "$Revision: 261 $"



class

	SHADING_DATA_POOL

inherit

	SINGLETON
		redefine
			default_create
		end

creation
	default_create

feature -- Initialization

	default_create is
			-- Create a pool with a default number of elements
		do
			total_allocated := 0
			fill
		end

feature -- Access

	item: SHADING_DATA is
			-- An unused element from the pool
		do
			Result := pool.last
			pool.remove_last
			debug ("shading_data_pool_allocation")
				allocated.force_last (Result)
			end

			if pool.count = 0 then
				fill
				debug ("shading_data_pool_allocation")
					allocated.sort
				end
				debug ("print_reference_counting")
					print_histogram
					print_active_grids
				end
			end
		ensure
			Result_not_void: Result /= Void
			Result_reference_count_is_zero: Result.reference_count = 0
		end

	total_allocated: INTEGER
			-- The total number of items allocated from the pool

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	return (data: SHADING_DATA) is
			-- `Return' `data' to the pool
		require
			data_not_void: data /= Void
		do
			pool.force_last (data)
			debug ("shading_data_pool_allocation")
				allocated.start
				allocated.search_forth (data)
				allocated.remove_at
			end
		ensure
			data_returned: pool.last = data
		end

feature -- Miscellaneous
	
	print_active_grids is
			-- Print a list of currently active grids in `allocated'
		local
			active_grids: ARRAY [INTEGER]
			referers: DS_ARRAYED_LIST [ANY]
			mg: MICROPOLYGON_GRID
			i: INTEGER
		do
			create active_grids.make (1, 100)

			from
				allocated.start
			until
				allocated.after
			loop
				referers := allocated.item_for_iteration.debug_referers
				from
					referers.start
				until
					referers.after
				loop
					mg ?= referers.item_for_iteration
					if mg /= Void then
						if mg.parent_element /= Void then
							active_grids.force (mg.parent_element.bucket, mg.identifier)
						else
							active_grids.force (-1, mg.identifier)
						end
					end
					referers.forth
				end

				allocated.forth
			end

			io.put_string ("%N%N-- Active Grids -- %N")
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + active_grids.upper
			variant
				i_increasing: 1 + active_grids.upper - i
			until
				i > active_grids.upper
			loop
				if active_grids.item (i) /= 0 then
					io.put_integer (i)
					io.put_string (" (")
					io.put_integer (active_grids.item (i) )
					io.put_string (") ")
				end
				i := i + 1
			end

			io.new_line
			io.new_line
		end

feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	default_count: INTEGER is 1000
			-- The default number of items to allocate

	pool: DS_ARRAYED_LIST [SHADING_DATA] is
			-- The memory pool
		once
			create Result.make (default_count)
		end

	allocated: SORTABLE_ARRAYED_LIST [SHADING_DATA] is
			-- Allocated items memory pool (debugging)
		once
			create Result.make (default_count)
		end

	fill is
			-- Fill the pool with `default_count' items
		local
			i: INTEGER
			sd: SHADING_DATA
		do
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + default_count
			variant
				i_increasing: 1 + default_count - i
			until
				i > default_count
			loop
				create sd.make (Current)
				pool.force_last (sd)

				i := i + 1
			end

			total_allocated := total_allocated + default_count
			io.put_string ("Total Allocated to Pool = ")
			io.put_integer (total_allocated)
			io.new_line
		ensure
			pool_has_at_least_default_count_items: pool.count >= default_count
		end

	print_histogram is
			-- Print a histogram of `allocated' by `reference_count'
		local
			h: ARRAY [INTEGER]
			i, ref: INTEGER
		do
			create h.make (0, 100)
			from
				allocated.start
			until
				allocated.after
			loop
				ref := allocated.item_for_iteration.reference_count
				h.put (h.item (ref) + 1, ref)

				allocated.forth
			end

			from
				i := 0
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + 100 + 1
			variant
				i_increasing: 0 + 100 - i + 1
			until
				i > 100
			loop
				io.put_integer (i)
				io.put_string (" - ")
				io.put_integer (h.item (i))
				io.new_line

				i := i + 1
			end
		end

invariant

	pool_not_void: pool /= Void

end -- SHADING_DATA_POOL

