

indexing
	description: "Lights that have constant attenuation and %
	% parallel rays, i.e., sunlight."
	patterns: "none"
	date: "$Date: 2007-01-09 16:27:59 -0500 (Tue, 09 Jan 2007) $"
	revision: "$Revision: 278 $"
	status: "unreviewed"




class

	DISTANT_LIGHT_SOURCE

inherit

	LIGHT_SOURCE
		redefine
			default_create, make, draw, print_features, diffuse
		end

	RI_CONSTANTS
		undefine
			default_create, is_equal, copy
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize the light pointing along positive z
			-- with constant attenuation
		do
			Precursor {LIGHT_SOURCE}

			if position /= Void then
				position.make (0, 0, 0, 1)
			else
				create position.make (0, 0, 0, 1)
			end

			if aim_point /= Void then
				aim_point.make (0, 0, 1, 1)
			else
				create aim_point.make (0, 0, 1, 1)
			end

			if shader /= Void then
				shader.make_from_string ("distantlight")
			else
				create shader.make_from_string ("distantlight")
			end

			if direction_point /= Void then
				direction_point.default_create
			else
				create direction_point
			end

			if tmp_point /= Void then
				tmp_point.default_create
			else
				create tmp_point
			end
		end

	make (the_handle: INTEGER; the_shader: STRING;
		  parameters: PARAMETER_LIST) is
			-- Initialize a light
		do
			Precursor (the_handle, the_shader, parameters)

			if parameters /= Void then
				from
					parameters.start
				until
					parameters.after
				loop
					if parameters.item_for_iteration.first.is_equal ("%"from%"") then
						position.make (parameters.item_for_iteration.second.second @ 1,
										 parameters.item_for_iteration.second.second @ 2,
										 parameters.item_for_iteration.second.second @ 3,
										 1)
					elseif parameters.item_for_iteration.first.is_equal ("%"to%"") then
						aim_point.make (parameters.item_for_iteration.second.second @ 1,
									   parameters.item_for_iteration.second.second @ 2,
									   parameters.item_for_iteration.second.second @ 3,
									   1)
					end
					parameters.forth
				end
			end
		end

feature -- Access

	direction: VECTOR4 is
			-- The light direction
		do
			Result := position - aim_point
			Result.normalize
		end

	position: POINT4
			-- The location of the light

	aim_point: POINT4
			-- Where the light is pointing

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_position (new_position: POINT4) is
			-- Set `position` to `new_position`
		require
			new_position_not_void: new_position /= Void
		do
			position.copy (new_position)
		ensure
			position_set: position.is_equal (new_position)
		end

	set_aim_point (new_aim_point: POINT4) is
			-- Set `aim_point` to `new_aim_point`
		require
			new_aim_point_not_void: new_aim_point /= Void
		do
			aim_point.copy (new_aim_point)
		ensure
			aim_point_set: aim_point.is_equal (new_aim_point)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	draw (renderer: RENDERER) is
			-- Apply the renderer to this object.
		do
			Precursor (renderer)

			direction_point.make (direction.x, direction.y, direction.z, direction.w)
			renderer.set_light_position (dynamic_id, direction_point)
		end

	generate_sample_point: POINT3 is
			-- Generate a point on the light
		do
			create Result.make (direction.x * infinity,
								direction.y * infinity,
								direction.z * infinity)
		end

	intersect (scene: SCENE_ACCELERATOR [SCENE_DRAWABLE]; origin: POINT3;
			   info: INTERSECTION_DATA) is
			-- Intersect the `scene' with a ray at `origin'
		local
			light_pt: POINT3
			cos_theta, distance_to_light: DOUBLE
		do
			tmp_point.make (position.x, position.y, position.z)
			tmp_point := light_to_world.apply (tmp_point)
			distance_to_light := (tmp_point - origin).magnitude

			light_probe.reset
			light_pt := tmp_point
			light_direction := light_pt - origin
			light_direction.normalize
			light_ray.set_direction (light_direction)
			light_ray.set_origin (origin)
			light_probe.set_ray (light_ray)

			scene.intersect (light_probe)

			illumination_at_point.fill (0)

			if not light_probe.intersected or else
				not occluded (light_probe, distance_to_light) then
				was_occluded := false
				light_pt := light_to_world.apply (generate_sample_point)
				light_direction := light_pt - origin
				light_direction.normalize
				cos_theta := info.normal.dot_with_vector (
					info.geometry.object_to_world.inverse.apply_as_direction (light_direction))
				if cos_theta > 0 then
					illumination_at_point := emission (light_pt, origin).scale (cos_theta)
				end
			else
				was_occluded := true
			end
		end

	compute_illumination (point: SHADING_DATA; camera: SCENE_CAMERA) is
			-- Compute the illumination at `point'
		local
			light_pt: POINT3
			normal: NORMAL
			cos_theta, distance_to_light: DOUBLE
		do
			tmp_point.make (position.x, position.y, position.z)
			tmp_point := light_to_world.apply (tmp_point)
			distance_to_light := (tmp_point - point.point).magnitude

			light_probe.reset
			light_pt := tmp_point
			light_direction := light_pt - point.point
			light_direction.normalize
			light_ray.set_direction (light_direction)
			light_ray.set_origin (point.point)
			light_probe.set_ray (light_ray)

			light_pt := light_to_world.apply (generate_sample_point)
			light_direction := light_pt - point.point
			light_direction.normalize

			illumination_at_point.fill (0)
			illumination_at_point.put_alpha (1.0)

			normal := point.geometric_normal
			cos_theta := normal.dot_with_vector (light_direction)

			if cos_theta > 0 then
				illumination_at_point := emission (light_pt, point.point).scale (cos_theta)
			else
				if point.two_sided then
					cos_theta := -normal.dot_with_vector (light_direction)
					if cos_theta > 0 then
						illumination_at_point := emission (light_pt, point.point).scale (cos_theta)
					end
				end
			end
		end

	diffuse (n: NORMAL; shading_point: POINT3): RGBA_COLOR is
			-- Compute the `diffuse' illumination
		do
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	direction_point: POINT4
			-- Temporary to speed drawing

	tmp_point: POINT3
			-- Temporary for light intersection

	print_features (action: PRINT_ACTION) is
			-- print the light's features
		do
			Precursor (action)

			action.io.put_string ("	 position = ")
			action.io.put_string (position.out)
			action.io.put_new_line

			action.io.put_string ("	 aim_point = ")
			action.io.put_string (aim_point.out)
			action.io.put_new_line

			action.io.put_string ("	 direction = ")
			action.io.put_string (direction.out)
			action.io.put_new_line
		end

invariant

	position_not_void: position /= Void
	aim_point_not_void: aim_point /= Void
	direction_not_zero: not about_equal (direction.magnitude, 0.0)
	direction_point_not_void: direction_point /= Void

end -- DISTANT_LIGHT_SOURCE

