


indexing
   description: "Point light sources"
	patterns: "none"
	date: "$Date: 2007-01-09 16:27:59 -0500 (Tue, 09 Jan 2007) $"
	revision: "$Revision: 278 $"
	status: "unreviewed"




class

	POINT_LIGHT_SOURCE

inherit

	LIGHT_SOURCE
		redefine
			default_create, emission, make, draw, print_features, diffuse
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize the light at the origin with intensity 1
			-- and quadratic attenuation
		do
			Precursor

			if position /= Void then
				position.make (0, 0, 0, 1)
			else
				create position.make (0, 0, 0, 1)
			end

			if shader /= Void then
				shader.make_from_string ("pointlight")
			else
				create shader.make_from_string ("pointlight")
			end

			constant_attenuation := 0.0
			quadratic_attenuation := 1.0
		end

	make (the_handle: INTEGER; the_shader: STRING;
		  parameters: PARAMETER_LIST) is
			-- Initialize a light
		do
			Precursor (the_handle, the_shader, parameters)

			if parameters /= Void then
				from
					parameters.start
				until
					parameters.after
				loop
					if parameters.item_for_iteration.first.is_equal ("%"from%"") then
						position.make (parameters.item_for_iteration.second.second @ 1,
									   parameters.item_for_iteration.second.second @ 2,
									   parameters.item_for_iteration.second.second @ 3,
									   1)
					end
					parameters.forth
				end

			end
		end

feature -- Access

	position: POINT4
			-- The center of the light

	emission (light_pt, geom_pt: POINT3): RGBA_COLOR is
			-- The emissive property of the light
		do
			Result := light_color.i_th (1).scale (intensity / light_pt.distance_squared (geom_pt))
		end

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_position (new_position: POINT4) is
			-- Set `position` to `new_position`
		require
			new_position_not_void: new_position /= Void
		do
			position.copy (new_position)
		ensure
			position_set: position.is_equal (new_position)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	draw (renderer: RENDERER) is
			-- Apply the renderer to this object.
		do
			Precursor (renderer)

			renderer.set_light_position (dynamic_id, position)
			renderer.set_spotlight_exponent (dynamic_id, 0.0)
			renderer.set_spotlight_cutoff (dynamic_id, 180.0)
		end

	generate_sample_point: POINT3 is
			-- Generate a point on the light
		do
			create Result.make (position.x, position.y, position.z)
		end

	intersect (scene: SCENE_ACCELERATOR [SCENE_DRAWABLE]; origin: POINT3;
			   info: INTERSECTION_DATA) is
			-- Intersect the `scene' with a ray at `origin'
		local
			light_pt: POINT3
			cos_theta, distance_to_light: DOUBLE
		do
			light_probe.reset
			light_pt := light_to_world.apply (generate_sample_point)
			light_direction := light_pt - origin
			distance_to_light := light_direction.magnitude
			light_direction.normalize
			light_ray.set_direction (light_direction)
			light_ray.set_origin (origin)
			light_probe.set_ray (light_ray)

			scene.intersect (light_probe)

			illumination_at_point.fill (0)

			if not light_probe.intersected or else
				not occluded (light_probe, distance_to_light) then
				was_occluded := false
				cos_theta := info.normal.dot_with_vector (
					info.geometry.object_to_world.inverse.apply_as_direction (light_direction))
				if cos_theta > 0 then
					illumination_at_point := emission (light_pt, origin).scale (cos_theta)
				end
			else
				was_occluded := true
			end
		end

	compute_illumination (point: SHADING_DATA; camera: SCENE_CAMERA) is
			-- Compute the illumination at `point'
		local
			light_pt: POINT3
			normal: NORMAL
			cos_theta, distance_to_light: DOUBLE
		do
			light_pt := light_to_world.apply (generate_sample_point)
			light_direction := light_pt - point.point
			distance_to_light := light_direction.magnitude
			light_direction.normalize

			illumination_at_point.fill (0)
			illumination_at_point.put_alpha (1.0)

			normal := point.geometric_normal
			cos_theta := normal.dot_with_vector (light_direction)
			if cos_theta > 0 then
				illumination_at_point := emission (light_pt, point.point).scale (cos_theta)
			else
				if point.two_sided then
					cos_theta := -normal.dot_with_vector (light_direction)
					if cos_theta > 0 then
						illumination_at_point := emission (light_pt, point.point).scale (cos_theta)
					end
				end
			end
		end

	diffuse (n: NORMAL; shading_point: POINT3): RGBA_COLOR is
			-- Compute the `diffuse' illumination
		local
			light_pt: POINT3
			cos_theta, distance_to_light: DOUBLE
		do
			create Result.make (0, 0, 0, 1)
			light_pt := light_to_world.apply (generate_sample_point)
			light_direction := light_pt - shading_point
			distance_to_light := light_direction.magnitude
			light_direction.normalize

			cos_theta := n.dot_with_vector (light_direction)
			if cos_theta > 0 then
				Result := emission (light_pt, shading_point).scale (cos_theta)
			end
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	print_features (action: PRINT_ACTION) is
			-- print the light's features
		do
			Precursor (action)

			action.io.put_string ("	 position = ")
			action.io.put_string (position.out)
			action.io.put_new_line
		end

invariant

	position_not_void: position /= Void

end -- POINT_LIGHT_SOURCE

