

indexing
	description: "Spot lights"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class
	
	SPOT_LIGHT_SOURCE
	
inherit
	
	POINT_LIGHT_SOURCE
		redefine
			default_create, make, draw, print_features,
			add_attributes, intersect
		end
	
creation
	default_create, make

feature -- Initialization

	default_create is
			-- Initialize the light pointing along positive z
			-- with constant attenuation
		do
			Precursor
			
			shader.make_from_string ("spotlight")
			
			if aim_point /= Void then
				aim_point.make (0, 0, 1, 1)
			else
				create aim_point.make (0, 0, 1, 1)
			end
			
			radians_to_degrees := real_64_truncated_to_real_32 (180.0 / pi)
			cone_angle := 30.0 / radians_to_degrees
			cone_delta := 5.0 / radians_to_degrees
			constant_attenuation := 0.0
			quadratic_attenuation := 1.0
		end
	
	make (the_handle: INTEGER; the_shader: STRING;
		  parameters: PARAMETER_LIST) is
			-- Initialize a light
		do
			Precursor (the_handle, the_shader, parameters)
			
			if parameters /= Void then
				from
					parameters.start
				until
					parameters.after
				loop
					if parameters.item_for_iteration.first.is_equal ("%"coneangle%"") then
						cone_angle := real_64_truncated_to_real_32 (parameters.item_for_iteration.second.second @ 1)
					elseif parameters.item_for_iteration.first.is_equal ("%"conedeltaangle%"") then
						cone_delta := real_64_truncated_to_real_32 (parameters.item_for_iteration.second.second @ 1)
					elseif parameters.item_for_iteration.first.is_equal ("%"to%"") then
						aim_point.make (parameters.item_for_iteration.second.second @ 1,
									   parameters.item_for_iteration.second.second @ 2,
									   parameters.item_for_iteration.second.second @ 3,
									   1)
					end
					parameters.forth
				end
			end
		end

feature -- Access
	
	cone_angle: REAL
			-- The angle of the spotlight
	
	cone_delta: REAL
			-- The angle of the spotlight
	
	direction: VECTOR4 is
			-- The light direction
		do
			Result := position - aim_point
			Result.normalize
		end
	
	aim_point: POINT4
			-- Where the light is pointing
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	valid_cone_angle (new_cone_angle: REAL): BOOLEAN is
			-- Is `new_cone_angle` valid?
		do 
			Result := new_cone_angle > 0
		end 
	
	valid_cone_delta (new_cone_delta: REAL): BOOLEAN is
			-- Is `new_cone_delta` valid?
		do
			Result := new_cone_delta > 0
		end
	
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_cone_angle (new_cone_angle: REAL) is
			-- Set `cone_angle` to `new_cone_angle`
		require 
			new_cone_angle_not_void: new_cone_angle > 0
		do
			cone_angle := new_cone_angle
		ensure
			cone_angle_set: cone_angle = new_cone_angle
		end 
	
	set_cone_delta (new_cone_delta: REAL) is
			-- Set `cone_delta` to `new_cone_delta`
		require 
			new_cone_delta_not_void: new_cone_delta > 0
		do
			cone_delta := new_cone_delta
		ensure
			cone_delta_set: cone_delta = new_cone_delta
		end 
	
	set_aim_point (new_aim_point: POINT4) is
			-- Set `aim_point` to `new_aim_point`
		require 
			new_aim_point_not_void: new_aim_point /= Void
		do
			aim_point.copy (new_aim_point)
		ensure
			aim_point_set: aim_point.is_equal (new_aim_point)
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	draw (renderer: RENDERER) is
			-- Apply the renderer to this object. By default, a 
			-- no-op
		do
			Precursor (renderer)
			
			renderer.set_spotlight_cutoff (dynamic_id, 
									   cone_angle * radians_to_degrees)
			renderer.set_light_position (dynamic_id, position)
			renderer.set_light_direction (dynamic_id, -direction)
		end
	
	intersect (scene: SCENE_ACCELERATOR [SCENE_DRAWABLE]; origin: POINT3;
			   info: INTERSECTION_DATA) is
			-- Intersect the `scene' with a ray at `origin'
		local
			light_pt: POINT3
			atten, cos_theta, distance_to_light: DOUBLE
		do
			light_probe.reset
			light_pt := light_to_world.apply (generate_sample_point)
			light_direction := light_pt - origin
			distance_to_light := light_direction.magnitude
			light_direction.normalize
			light_ray.set_direction (light_direction) 
			light_ray.set_origin (origin)
			light_probe.set_ray (light_ray)
			
			scene.intersect (light_probe)
			
			illumination_at_point.fill (0)
			
			if not light_probe.intersected or else 
				not occluded (light_probe, distance_to_light) then
				was_occluded := false
				cos_theta := info.geometry.object_to_world.inverse.apply_as_direction (light_direction) # direction.dehomogenize
				if cos_theta >= cosine (cone_angle) then
					atten := smoothstep (cosine (cone_angle), cosine (cone_angle - cone_delta), cos_theta)
					cos_theta := info.normal.dot_with_vector (
						info.geometry.object_to_world.inverse.apply_as_direction (light_direction))
					if cos_theta > 0 then
						illumination_at_point := emission (light_pt, origin).scale (atten * cos_theta)
					end
				end
			else
				was_occluded := true
			end
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	radians_to_degrees: REAL
	
	smoothstep (lower, upper, value: DOUBLE): DOUBLE is
			-- Generate a smooth Hermite interpolate between lower and upper
		require
			correct_order: lower <= upper
		local
			v: DOUBLE
		do
			if value >= upper then
				Result := 1
			elseif value <= lower then
				Result := 0
			else
				v := (value - lower) / (upper - lower)
				Result := v * v * (3 - 2 * v)
			end
		end
	
	print_features (action: PRINT_ACTION) is
			-- print the group's features
		do
			Precursor (action)
			
			action.io.put_string ("	 aim_point = ")
			action.io.put_string (aim_point.out)
			action.io.put_new_line
			
			action.io.put_string ("	 direction = ")
			action.io.put_string (direction.out)
			action.io.put_new_line
			
			action.io.put_string ("	 cone_angle = ")
			action.io.put_real (cone_angle)
			action.io.put_new_line
			
			action.io.put_string ("	 cone_delta = ")
			action.io.put_real (cone_delta)
			action.io.put_new_line
		end
	
	add_attributes (the_attributes: DS_ARRAYED_LIST [SCENE_ATTRIBUTE]) is
			-- Add public modifiable attributes to `attributes'
		local
			double_attribute: SCENE_ATTRIBUTE_INFO [REAL]
		do
			Precursor (the_attributes)
			
			create double_attribute.make ("cone angle", index_of_field ("cone_angle"), 
										  agent valid_cone_angle, agent set_cone_angle)
			the_attributes.force_last (double_attribute)
			
			create double_attribute.make ("cone delta", 
										  index_of_field ("cone_delta"), 
										  agent valid_cone_delta, 
										  agent set_cone_delta)
			the_attributes.force_last (double_attribute)
		end
	
invariant
	
	cone_angle_is_positive: cone_angle >= 0.0
	cone_delta_is_positive: cone_delta >= 0.0
	aim_point_not_void: aim_point /= Void
	direction_not_zero: not about_equal (direction.magnitude, 0.0)
	
end -- SPOT_LIGHT_SOURCE

