

indexing
	description: "Abstract class for bounding volumes"
	patterns: "template"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 

	SCENE_BOUND
	
inherit
	
	INTERSECTABLE
		rename
			intersect as intersect_ray
		undefine
			out
		redefine
			default_create
		end
	
feature -- Initialization

	default_create is
			-- Create an empty bounding box
		do
			Precursor			
		ensure then
			is_empty: empty
		end
	
	make (first_corner, second_corner: POINT3) is
			-- Arbitrary-ordered vertices to be bounded.
		require
			first_corner_not_void: first_corner /= Void
			second_corner_not_void: second_corner /= Void
		deferred
		end

feature -- Access
	
	centroid: POINT3 is
			-- The center of the bounds
		deferred
		end
	
	length: DOUBLE is
			-- The longest distance from the center to the outside of 
			-- the bounds. A sphere of center `centroid` with radius 
			-- `length` is guaranteed to encompass the bounds
		require
			not_infinite: not infinite
			not_empty: not empty
		deferred
		ensure
			Result_strictly_positive: Result > 0.0
		end
	
feature -- Measurement
feature -- Comparison
feature -- Status report
	
	contains (point: POINT3): BOOLEAN is
			-- Does this object contain `point'?
		require 
			point_not_void: point /= Void
		deferred
		end
	
	outside: BOOLEAN
			-- If not `intersected', am I `outside' the intersection, 
			-- where `outside' is defined by the intersecting object.
	
	inside: BOOLEAN
			-- If not `intersected', am I `inside' the intersection, 
			-- where `inside' is defined by the intersecting object.
	
	
	empty: BOOLEAN is
			-- Is the bounds empty?
		deferred
		end 

	infinite: BOOLEAN is
			-- Is the bounds infinite?
		deferred
		end 

feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	expand_to_include (bound: SCENE_BOUND) is
			-- Adapt this bounds to include `bound'
		require 
			bound_not_void: bound /= Void
		deferred
		ensure 
			-- encompasses_bound
		end 

	expand_to_include_point (point: POINT3) is
			-- Adapt this bounds to include `point'
		require
			point_not_void: point /= Void
		deferred
		ensure
			includes_point: contains (point)
		end
	
	expand_to_include_points (points: POINT3_ARRAYED_LIST) is
		require
			points_not_void: points/= Void
		deferred
		end
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
	
	circumscribed_box: AXIS_ALIGNED_BOX is
			-- A box that circumscribes the sphere, i.e., the sphere is 
			-- inscribed in the box; radius of box 2*R/sqrt(3)
		deferred
		end
	
	out: STRING is
			-- Stringified version of `Current'
		deferred
		ensure then
			Result_not_void: Result /= Void
		end 

feature -- Duplication
feature -- Basic operations
	
	apply (transform: TRANSFORM): like Current is
			-- Apply `transform' to `Current'
		require 
			transform_not_void: transform /= Void
		deferred
		ensure 
			Result_not_void: Result /= Void
		end 

	draw_immediate (renderer: RENDERER) is
			-- Apply the renderer to this object
		deferred
		end
	
	intersect_plane (plane: PLANE) is
			-- Intersect with `plane'.
			-- One and only one of the following will be true, 
			-- `inside', `outside', or `intersected'. `inside' is 
			-- defined by the normal direction
		require
			plane_not_void: plane /= Void
		deferred
		ensure
			only_one_true: (inside implies (not outside and not intersected)) or
						   (outside implies (not inside and not intersected)) or
						   (intersected implies (not outside and not inside))
		end

	intersect_polyhedron (poly: POLYHEDRON) is
			-- Intersect with `plane'.
			-- One and only one of the following will be true, 
			-- `inside', `outside', or `intersected'. `inside' is 
			-- defined by the normal direction
		require
			poly_not_void: poly /= Void
		deferred
		ensure
			only_one_true: (inside implies (not outside and not intersected)) or
						   (outside implies (not inside and not intersected)) or
						   (intersected implies (not outside and not inside))
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	green: RGBA_COLOR is
			-- A bluish-green color
		once
			create Result.make (0, 1, 0.5, 1)
		end
	
invariant
	siding_is_valid: ((inside implies (not outside and not intersected)) or
					  (outside implies (not inside and not intersected)) or
					  (intersected implies (not outside and not inside)) or
					  (not inside and not outside and not intersected))
	centroid_is_not_void: centroid /= Void
	length_strictly_positive: (not infinite and not empty) implies length > 0.0
	
end -- SCENE_BOUND

