

indexing
    description: "Abstract class for nodes that contain other nodes"
    names: "group", "children"
    representation: "DS_ARRAYED_LIST"
    access: "index"
    size: "resizable"
    contents: "SCENE_VOLUME"
    patterns: "none"
    status: "unreviewed"




deferred class 

	SCENE_COLLECTION [G-> SCENE_VOLUME]
	
inherit
	
	SCENE_NODE
		redefine
			default_create, gather_extents
		end
	
feature -- Initialization

	default_create is
			-- Create the node.
		do
			Precursor
			
			create children.make (1)
			create {SCENE_BOX_BOUND} gathered_extents
		ensure then
			children_created: children /= Void
		end

feature -- Access
	
	extents: SCENE_BOUND is
			-- The `extents' or bounds of the volume
		do
			if update_needed then
				gather_extents
			end
			
			Result := gathered_extents
		end
	
	item (i: INTEGER): G is
			-- 
		require 
			valid_key: i >= 1 and i <= count
		do
			Result := children.item (i)
		end 

	has (v: G): BOOLEAN is
			-- Does `v' appear in array?
		do
			Result := children.has (v)
		end
	
feature -- Measurement
	
	count: INTEGER is
			-- The number of children
		do
			Result := children.count
		ensure 
			Result_is_positive: Result >= 0
		end

feature -- Comparison
feature -- Status report
	
	has_extents: BOOLEAN is
			-- Does this volume have extents?
		do
			Result := gathered_extents /= Void
		end
	
feature -- Cursor movement
feature -- Element change
	
	add_child (node: G) is
			-- add `node' as a new child
		require 
			node_not_void: node /= Void
			not_already_a_child: not has (node)
		do
			children.force_last (node)
		ensure 
			count_increased_by_one: count = old count + 1
			node_is_a_child: has (node)
		end
	
	insert_child (node: G; index: INTEGER) is
			-- insert `node' at `index'
		require 
			index_is_positive: index >= 1
			index_less_than_count: index <= count
			node_not_void: node /= Void
			not_already_a_child: not has (node)
		do
			children.go_i_th (index)
			children.force_left (node)
		ensure 
			count_increased_by_one: count = old count + 1
			node_at_index: node = item (index)
		end
	
	replace_child (node: G; index: INTEGER) is
			-- replace the child at `index' with `node'
		require 
			index_is_positive: index >= 1
			index_less_than_count: index <= count
			node_not_void: node /= Void
			not_already_a_child: not has (node)
		do
			children.go_i_th (index)
			children.replace_at (node)
		ensure 
			node_at_index: node = item (index)
			count_unchanged: count = old count
		end

	update_extents is
			-- Recompute the extents of the object
		do
			gather_extents
		end
		
feature -- Removal
	
	remove_child (node: G) is
			-- remove the child corresponding to `node'
		require 
			node_not_void: node /= Void
			node_is_child: has (node)
		do
			children.start

			from
				children.search_forth (node)
			until
				children.after
			loop
				children.remove_at
				children.search_forth (node)
			end
		ensure 
			node_is_removed: not has (node)
		end
	
	remove_all_children is
			-- remove all children
		do
			children.wipe_out
		ensure 
			count_is_zero: count = 0
		end

feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	children: DS_ARRAYED_LIST [G]
	
	gathered_extents: SCENE_BOUND
			-- The bounding box of all my children
	
	gather_extents is
			-- Gather together all bounding information.
		do
			update_needed := false
			
			if count > 0 then
				children.start
				
				gathered_extents.expand_to_include (children.item_for_iteration.extents)
				
				children.forth
				
				from
				until
					children.after
				loop
					gathered_extents.expand_to_include (children.item_for_iteration.extents)

					children.forth
				end
			end
		end
	
invariant
	
	children_not_void: children /= Void
	
end -- SCENE_COLLECTION

