

indexing
	description: "A drawable is defined by its geometry and state"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2006-09-03 16:32:39 -0400 (Sun, 03 Sep 2006) $"
	revision: "$Revision: 261 $"




class

	SCENE_DRAWABLE

inherit

	TRIPLE [SCENE_DRAWABLE_ELEMENT, SCENE_STATE_ELEMENT_TABLE, TRANSFORM]
		rename
			first as geometry,
			second as elements,
			third as transform
		undefine
			default_create
		redefine
			make
		end

	SINGLETON_ACCESSOR
		undefine
			is_equal
		redefine
			singleton, default_create
		end
creation

	default_create

feature -- Initialization

	default_create is
			-- Initialize the action
		do
			singleton.next_id
			identifier := singleton.last_id
			initialized := false
		end

	make (the_first: SCENE_GEOMETRY; the_second: SCENE_STATE_ELEMENT_TABLE; the_third: TRANSFORM) is
			-- Make triple with the given components.
		do
			default_create
			initialized := true
			Precursor (the_first, the_second, the_third)
		end

feature -- Access

	identifier: INTEGER
			-- A guaranteed unique value to identify this instance of
			-- an action

	bucket: INTEGER
			-- Which bucket does this object fall
	
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change

	release is
			-- Release memory
		do
			initialized := false
			elements := Void
			transform := Void
			geometry.release
			geometry := Void
		end

	set_bucket (new_bucket: INTEGER) is
			-- Set `bucket' to `new_bucket'
		require 
			new_bucket_positive: new_bucket >= 0
		do
			bucket := new_bucket
		ensure 
			bucket_set: bucket = new_bucket
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	draw (op: RENDERER) is
			-- Draw `Current'
		local
			left_handed: BOOLEAN
		do
			op.push_matrix
			op.multiply_matrix (transform)

			left_handed := (op.viewing_camera.world_to_camera * transform).determinant >= 0
			if not geometry.handness_set or else geometry.is_left_handed /= left_handed then
				geometry.set_handness (left_handed)
			end

			op.global_lights.illuminate (false)

			if elements /= Void then
				elements.apply (op)
			end

			op.draw_lights

			geometry.draw (op)
			op.pop_matrix
		end

	intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect the `drawable'
		require
			intersecter_not_void: intersecter /=Void
		do
			if transform_node = Void then
				create transform_node
			end
			transform_node.load_transform (transform)

			intersecter.push (transform_node)
--			geometry.intersect (intersecter)
			intersecter.apply (geometry)
			intersecter.pop (transform_node)
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	initialized: BOOLEAN

	singleton: OBJECT_IDENTIFIER is
			-- Create the id generator
		once
			create Result
		end

	transform_node: SCENE_TRANSFORM
			-- Used for pushing and poping of `transform' when `intersect'ing

invariant

	geometry_not_void: initialized implies geometry /= Void
	transform_not_void: initialized implies transform /= Void
	identifier_strictly_positive: identifier > 0

end -- SCENE_DRAWABLE

