

indexing
	description: "An element that is capable of drawing"
	patterns: "none"
	status: "unreviewed"
	date: "$Date: 2007-04-07 10:37:11 -0400 (Sat, 07 Apr 2007) $"
	revision: "$Revision: 287 $"



deferred class

	SCENE_DRAWABLE_ELEMENT

inherit

	SCENE_STATE_ELEMENT_TABLE
		redefine
			default_create, is_equal, copy
		end

	SCENE_VOLUME
		undefine
			detailed_memory_usage
		redefine
			copy, default_create, is_equal
		end

	BASIC_NUMERIC
		export
			{NONE} all
		undefine
			copy, is_equal, default_create
		end

feature -- Initialization

	default_create is
			-- Initialize common geometry features
		local
			i: INTEGER
			the_item: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
		do
			Precursor {SCENE_VOLUME}
			Precursor {SCENE_STATE_ELEMENT_TABLE}

			is_annotation := false
			is_split := false
			shading_rate := 1.0

			u_minimum := 0
			u_maximum := 1
			v_minimum := 0
			v_maximum := 1
			wrapped_in_u := false
			wrapped_in_v := false

			updating_neighbors := false

			create tester

			create split_items.make (2)
			
			from
				create neighbors.make (0, 3)
				i := 0
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 4
			variant
				i_increasing: 4 - i
			until
				i > 3
			loop
				create the_item.make (1)
				the_item.set_equality_tester (tester)
				neighbors.put (the_item, i)
				i := i + 1
			end
		end

feature -- Access

	split_items: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
			-- The result of a split operation

	left: INTEGER is 0
	right: INTEGER is 2
	above: INTEGER is 1
	below: INTEGER is 3
			-- Indices to neighbors

	object_to_world: TRANSFORM is
			-- Conversion from object coordinates to world coordinates
		deferred
		end

	shading_rate: DOUBLE
			-- How many points per pixel to shade at

	u_minimum: DOUBLE
			-- The minimum value of the `u' parameter

	u_maximum: DOUBLE
			-- The maximum value of the `u' parameter

	v_minimum: DOUBLE
			-- The minimum value of the `v' parameter

	v_maximum: DOUBLE
			-- The maximum value of the `v' parameter

	wrapped_in_u, wrapped_in_v: BOOLEAN
			-- Does this object wrap around on itself in the
			-- specified direction?

	grid: MICROPOLYGON_GRID
			-- The grid object after we've been diced.

	neighbor (which: INTEGER): DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT] is
			-- Return the neighbor at `which'
		require
			has_neighbor: has_neighbor (which)
			which_in_range: which >= left and which <= below
		do
			Result := neighbors.item (which)
		ensure
			Result_not_void: Result /= Void
			Result_not_empty: Result.count > 0
		end

	all_neighbors: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT] is
			-- A single list containing all neighbors
		do
			create Result.make(1)
			if has_neighbor (left) then
				Result.append_last (neighbor (left))
			end
			if has_neighbor (right) then
				Result.append_last (neighbor (right))
			end
			if has_neighbor (above) then
				Result.append_last (neighbor (above))
			end
			if has_neighbor (below) then
				Result.append_last (neighbor (below))
			end
		end

	drawable: SCENE_DRAWABLE
			-- The drawable for this object?

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Are we equal?
		do
			Result := u_minimum = other.u_minimum and u_maximum = other.u_maximum and
				v_minimum = other.v_minimum and v_maximum = other.v_maximum
			if Result then
				Result := Precursor {SCENE_VOLUME} (other)
				if Result then
					Result := Precursor {SCENE_STATE_ELEMENT_TABLE} (other)
				end
			end
		end

feature -- Status report

	is_split: BOOLEAN
			-- Has this object been split into smaller primitives?

	split_in_u: BOOLEAN
			-- is the next split in the u direction?

	is_annotation: BOOLEAN
		-- Is this object to be considered an annotation in the window?

	two_sided: BOOLEAN is
			-- Does the object have two sides
		deferred
		end

	diced: BOOLEAN
			-- Has this object been diced?

	diceable: BOOLEAN
			-- Is this object "small enough" to be `diced'

	test_diceability (the_renderer: MICROPOLYGON_RENDERER) is
			-- Is the resulting of dicing this object "small enough"?
		require
			the_renderer_not_void: the_renderer /= Void
		deferred
		ensure
			diced_is_false: not diced
		end

	requested_grid_x_dim, requested_grid_y_dim: INTEGER
			-- The rate at which to dice a new grid

	has_neighbor (index: INTEGER): BOOLEAN is
			-- Does this object have a neigbor at `index'
		require
			index_in_range: index >= left and index <= below
		do
			Result := neighbors.item (index).count > 0
		end

feature -- Status setting

	mark_as_annotation (true_or_false: BOOLEAN) is
			-- Mark this object as an annotation
		do
			is_annotation := true_or_false
		ensure
			is_annotation_set: is_annotation = true_or_false
		end

	set_split_direction (in_u: BOOLEAN) is
			-- Set the split directions
		do
			split_in_u := in_u
		ensure
			split_in_u_set: split_in_u = in_u
		end

	set_drawable (new_drawable: SCENE_DRAWABLE) is
			-- Set `drawable' to `new_drawable'
		do
			drawable := new_drawable
		ensure
			drawable_set: drawable = new_drawable
		end

feature -- Cursor movement
feature -- Element change

	copy (other: like Current) is
			-- Make `Current' a copy of `other'
		do
			Precursor {SCENE_VOLUME} (other)
			Precursor {SCENE_STATE_ELEMENT_TABLE} (other)
		end

	set_shading_rate (new_shading_rate: DOUBLE) is
			-- Set `shading_rate' to `new_shading_rate'
		require
			new_shading_rate_positive: new_shading_rate > 0
		do
			shading_rate := new_shading_rate
		ensure
			shading_rate_set: shading_rate = new_shading_rate
		end

	release_neighbors is
			-- Break the linkage to neighbors that were previously
			-- diced
		local
			i: INTEGER
			the_list: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
		do
			from
				i := left
			invariant
				i_at_least_one: i >= left
				i_small_enough: i <= left + below + 1
			variant
				i_increasing: left + 1 + below - i
			until
				i > below
			loop
				from
					the_list := neighbors.item (i)
					the_list.start
				until
					the_list.after
				loop
					remove_neighbor (the_list.item_for_iteration, i)
					the_list.start
					--the_list.forth
				end

				i := i + 1
			end
		ensure
			no_neighbors: all_neighbors.count = 0
		end

	set_neighbor (new_part: SCENE_DRAWABLE_ELEMENT; index: INTEGER) is
			-- Set the neighbor for `index' to `new_part'
		require
			new_part_not_void: new_part /= Void
			index_valid: index >= left and index <= below
		local
			other: INTEGER
		do
			neighbors.item (index).wipe_out
			neighbors.item (index).force_last (new_part)

			other := (index + 2) \\ 4
			new_part.neighbors.item (other).wipe_out
			new_part.neighbors.item (other).force_last (Current)
		ensure
			neighbor_set: neighbors.item (index).item (1) = new_part
			new_part_has_current: new_part.neighbors.item ((index + 2) \\ 4).has (Current)
		end

	remove_neighbor (part: SCENE_DRAWABLE_ELEMENT; index: INTEGER) is
			-- Remove the `part' from the neighbor at `index'
		require
			part_not_void: part /= Void
			index_valid: index >= left and index <= below
			has_neighbor: has_neighbor (index)
			neighbor_has_part: neighbor (index).has (part)
		local
			other: INTEGER
		do
			other := (index + 2) \\ 4
				check
					has_current: part.has_neighbor (other)
					neighbor_has_part: part.neighbor (other).has (Current)
				end
			part.neighbor (other).delete (Current)
			neighbor (index).delete (part)
		ensure
			neighbors_removed: not neighbor (index).has (part)
			part_neighbors_removed: not part.neighbor ((index + 2) \\ 4).has (Current)
		end

	add_neighbor (new_part: SCENE_DRAWABLE_ELEMENT; index: INTEGER) is
			-- Add `new_part' to `left'
		require
			new_part_not_void: new_part /= Void
			index_valid: index >= left and index <= below
		do
			neighbors.item (index).force_last (new_part)
				check
					new_part_has_current: new_part.neighbors.item ((index + 2) \\ 4).has (Current)
				end
		ensure
			neighbor_set: neighbors.item (index).item (neighbors.item (index).count) = new_part
		end

	set_grid (new_grid: MICROPOLYGON_GRID) is
			-- Set `grid' to `new_grid'
		require
			new_grid_not_void: new_grid /= Void
		do
			grid := new_grid
		ensure
			grid_set: grid = new_grid
		end

	release is
			-- Release all memory used by this object
		do
			is_split := false
			split_items.wipe_out
			grid := Void
			release_neighbors
		end

	release_grid is
			-- Release all memory used by the grid of this object
		require
			diced: diced
		local
			patches: DS_ARRAYED_LIST [PATCH_OBJECT]
		do
			patches := grid.all_patches
			from
				patches.start
			until
				patches.after
			loop
				if patches.item_for_iteration.busted then
					patches.item_for_iteration.release
				end
				patches.forth
			end

			if not grid.released then
				grid.release
			end
		end

	toggle_u_wrapping is
			-- Toggle `wrapped_in_u'
		do
			wrapped_in_u := not wrapped_in_u
		ensure
			wrapped_in_u_set: wrapped_in_u /= old wrapped_in_u
		end
	
	toggle_v_wrapping is
			-- Toggle `wrapped_in_v'
		do
			wrapped_in_v := not wrapped_in_v
		ensure
			wrapped_in_v_set: wrapped_in_v /= old wrapped_in_v
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	dice (the_grid: MICROPOLYGON_GRID; points_only: BOOLEAN) is
			-- Dice this object onto `the_grid'
		require
			the_grid_not_void: the_grid /= Void
		deferred
		ensure
			diced: diced
			grid_set: grid /= Void
		end

	patch_grid is
			-- For the grid associatied with this object, create a
			-- filler to deal with displacement cracks. If the
			-- neighbors haven't been diced, wait and patch from the
			-- other side.
		require
			is_diced: diced and grid /= Void
		deferred
		end

	draw (renderer: RENDERER) is
			-- Apply the renderer to this object.
		require
			renderer_not_void: renderer /= Void
		deferred
		end

	split is
			-- Split the object into smaller pieces
		require
			split_items_empty: split_items.count = 0
		deferred
		ensure
			consistant: consistancy_check
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SCENE_DRAWABLE_ELEMENT} -- Implementation

	neighbors: ARRAY [DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]]
			-- Neighbors, typically after a split

	updating_neighbors: BOOLEAN
			-- Flag used to control checking of invariants

	toggle_updating_neighbors is
			-- Flip the value of `updating_neighbors'
		do
			updating_neighbors := not updating_neighbors
		end

	consistancy_check: BOOLEAN is
			-- See if all of the split parts are valid
		local
			i: INTEGER
			split_thingie: like Current
		do
			from
				split_items.start
				Result := true
			until
				split_items.after or not Result
			loop
				if split_items.item_for_iteration.updating_neighbors then
					Result := false
				else
					split_thingie := split_items.item_for_iteration
					from
						i := left 
					invariant
						i_at_least_one: i >= left
						i_small_enough: i <= left + below
					variant
						i_increasing: left + below - i
					until
						i > below or not Result
					loop
						Result := split_thingie.neighbor_consistancy_check (i)
						if not Result then
							io.put_string ("consistancy_check failed on ")
							io.put_string (i.out)
							io.new_line
						end
						i := i + 1
					end
					split_items.forth
				end
			end
		end
	
	neighbor_consistancy_check (index: INTEGER): BOOLEAN is
			-- Verify that the pointers are correct
		require
			index_valid: index >= left and index <= below
		local
			other: INTEGER
		do
			from
				Result := true
				neighbors.item (index).start
				other := (index + 2) \\ 4
			until
				neighbors.item (index).after or not Result
			loop
				Result := neighbors.item (index).item_for_iteration.neighbors.item (other).has (Current)
				neighbors.item (index).forth
			end

		end

	no_duplicates (index: INTEGER): BOOLEAN is
			-- Are there any duplicates in `neighbors' at `index'
		require
			index_valid: index >= left and index <= below
		local
			the_list: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
		do
			from
				Result := true
				the_list := neighbors.item (index)
				the_list.start
			until
				the_list.after or not Result
			loop
				if the_list.occurrences (the_list.item_for_iteration) /= 1 then
					Result := false
				end
				the_list.forth
			end
		end

	u_neighbors_bounded: BOOLEAN is
			-- Are the neigbors to the `left' and `right' bounded by
			-- or bound this element
		local
			the_list: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
			sde: SCENE_DRAWABLE_ELEMENT
		do
			from
				Result := true
				the_list := neighbors.item (left)
				the_list.start
			until
				the_list.after or not Result
			loop
				sde := the_list.item_for_iteration
				Result :=
					-- Case 1 - Ranges are exact
					(the_list.count = 1 and sde.v_minimum = v_minimum and
					 sde.v_maximum = v_maximum) or else
					-- No overlap with just endpoints
					(not (sde.v_minimum = v_maximum or
						  v_minimum = sde.v_maximum) and
					-- Case 2 - Ranges overlap
					(((within_range (sde.v_minimum, v_minimum, v_maximum) or
					 within_range (sde.v_maximum, v_minimum, v_maximum)) and
					(within_range (v_minimum, sde.v_minimum, sde.v_maximum) or
					 within_range (v_maximum, sde.v_minimum, sde.v_maximum))) or else
					-- Case 3 - One range is wholly contained
					-- within the other
					((within_range (sde.v_minimum, v_minimum, v_maximum) and
					 within_range (sde.v_maximum, v_minimum, v_maximum)) or
					(within_range (v_minimum, sde.v_minimum, sde.v_maximum) and
					 within_range (v_maximum, sde.v_minimum, sde.v_maximum)))))
				the_list.forth
			end

			from
				the_list := neighbors.item (right)
				the_list.start
			until
				the_list.after or not Result
			loop
				sde := the_list.item_for_iteration
				Result :=
					-- Case 1 - Ranges are exact
					(sde.v_minimum = v_minimum and
					 sde.v_maximum = v_maximum) or else
					-- No overlap with just endpoints
					(not (sde.v_minimum = v_maximum or
						  v_minimum = sde.v_maximum) and
					-- Case 2 - Ranges overlap
					(((within_range (sde.v_minimum, v_minimum, v_maximum) or
					 within_range (sde.v_maximum, v_minimum, v_maximum)) and
					(within_range (v_minimum, sde.v_minimum, sde.v_maximum) or
					 within_range (v_maximum, sde.v_minimum, sde.v_maximum))) or else
					-- Case 3 - One range is wholly contained
					-- within the other
					((within_range (sde.v_minimum, v_minimum, v_maximum) and
					 within_range (sde.v_maximum, v_minimum, v_maximum)) or
					(within_range (v_minimum, sde.v_minimum, sde.v_maximum) and
					 within_range (v_maximum, sde.v_minimum, sde.v_maximum)))))
				the_list.forth
			end
		end

	v_neighbors_bounded: BOOLEAN is
			-- Are the neigbors to the `above' and `below' bounded by
			-- or bound this element
		local
			the_list: DS_ARRAYED_LIST [SCENE_DRAWABLE_ELEMENT]
			sde: SCENE_DRAWABLE_ELEMENT
		do
			from
				Result := true
				the_list := neighbors.item (above)
				the_list.start
			until
				the_list.after or not Result
			loop
				sde := the_list.item_for_iteration
				Result :=
					-- Case 1 - Ranges are exact
					(sde.u_minimum = u_minimum and
					 sde.u_maximum = u_maximum) or else
					-- No overlap with just endpoints
					(not (sde.u_minimum = u_maximum or
						  u_minimum = sde.u_maximum) and
					-- Case 2 - Ranges overlap
					(((within_range (sde.u_minimum, u_minimum, u_maximum) or
					 within_range (sde.u_maximum, u_minimum, u_maximum)) and
					(within_range (u_minimum, sde.u_minimum, sde.u_maximum) or
					 within_range (u_maximum, sde.u_minimum, sde.u_maximum))) or else
					-- Case 3 - One range is wholly contained
					-- within the other
					((within_range (sde.u_minimum, u_minimum, u_maximum) and
					 within_range (sde.u_maximum, u_minimum, u_maximum)) or
					(within_range (u_minimum, sde.u_minimum, sde.u_maximum) and
					 within_range (u_maximum, sde.u_minimum, sde.u_maximum)))))
				the_list.forth
			end

			from
				the_list := neighbors.item (below)
				the_list.start
			until
				the_list.after or not Result
			loop
				sde := the_list.item_for_iteration
				Result :=
					-- Case 1 - Ranges are exact
					(sde.u_minimum = u_minimum and
					 sde.u_maximum = u_maximum) or else
					-- No overlap with just endpoints
					(not (sde.u_minimum = u_maximum or
						  u_minimum = sde.u_maximum) and
					-- Case 2 - Ranges overlap
					(((within_range (sde.u_minimum, u_minimum, u_maximum) or
					   within_range (sde.u_maximum, u_minimum, u_maximum)) and
					  (within_range (u_minimum, sde.u_minimum, sde.u_maximum) or
					   within_range (u_maximum, sde.u_minimum, sde.u_maximum))) or else
					 -- Case 3 - One range is wholly contained
					 -- within the other
					 ((within_range (sde.u_minimum, u_minimum, u_maximum) and
					   within_range (sde.u_maximum, u_minimum, u_maximum)) or
					  (within_range (u_minimum, sde.u_minimum, sde.u_maximum) and
					   within_range (u_maximum, sde.u_minimum, sde.u_maximum)))))
				the_list.forth
			end
		end

feature {NONE} -- Implementation

	tester: KL_EQUALITY_TESTER [SCENE_DRAWABLE_ELEMENT]

invariant

	shading_rate_positive: shading_rate > 0
	u_parameter_ordered: u_minimum < u_maximum
	v_parameter_ordered: v_minimum < v_maximum
	split_items_not_void: split_items /= Void
	is_split_implies_split_items_not_zero: (is_split implies split_items.count > 0)
	no_left_duplicates: no_duplicates (left)
	no_right_duplicates: no_duplicates (right)
	no_above_duplicates: no_duplicates (above)
	no_below_duplicates: no_duplicates (below)
	left_neighbors_correct: (not updating_neighbors implies
							 neighbor_consistancy_check (left))
	right_neighbors_correct: (not updating_neighbors implies
							  neighbor_consistancy_check (right))
	below_neighbors_correct: (not updating_neighbors implies
							  neighbor_consistancy_check (below))
	above_neighbors_correct: (not updating_neighbors implies
							  neighbor_consistancy_check (above))

	bounded_in_u: not updating_neighbors implies u_neighbors_bounded
	bounded_in_v: not updating_neighbors implies v_neighbors_bounded

end -- SCENE_DRAWABLE_ELEMENT

