


indexing
	description: "Abstract class for all scene graph nodes that %
				 %use traversal"
	patterns: "template"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 

	SCENE_NODE
	
inherit
	
	SCENE_VOLUME
		redefine
			default_create, print_features
		end
	
feature -- Initialization
	
	default_create is
			-- Initalize a node my calling `make'
		do
			Precursor
			
			create parents.make (1)
			traversable := true
			update_needed := true
		end
	
	make is
			-- initialize a node
		do
			default_create
		end

feature -- Access
	
	parent (i_th: INTEGER): SCENE_GROUP is
			-- Return the `i_th' parent of this node
		require
			i_th_at_least_one: i_th >= 1
			i_th_less_than_parent_count: i_th <= parent_count
		do
			Result := parents.item (i_th)
		ensure
			Result_not_void: Result /= Void
		end
	
feature -- Measurement
	
	parent_count: INTEGER is
			-- The number of parents for this node
		do
			Result := parents.count
		ensure 
			in_range: Result >= 0 and Result <= parents.count
		end 

feature -- Comparison
feature -- Status report
	
	ancestor (node: SCENE_NODE): BOOLEAN is
			-- Is `node' an ancestor?
		require 
			node_not_void: node /= Void
		local
			g: SCENE_GROUP
		do
			g ?= node
			if g /= Void then
				Result := has_parent (g)
				if not Result then
					from
						parents.start
					until
						Result or parents.after
					loop
						Result := parents.item_for_iteration.ancestor (node)
						parents.forth
					end
				end
			end
		end 

	traversable: BOOLEAN
			-- Is this node traversable
	
	has_parent (the_parent: SCENE_GROUP): BOOLEAN is
			-- Is 'the_parent' a parent of this node
		require 
			the_parent_not_void: the_parent /= Void
		do
			Result := parents.has (the_parent)
		end 

feature -- Status setting
	
	set_traversable (status: BOOLEAN) is
			-- set `traversable' to `status'
		do
			traversable := status
		ensure 
			traversable_set: traversable = status
		end -- set_traversable

feature -- Cursor movement
feature -- Element change
	
	add_parent (new_parent: SCENE_GROUP) is
			-- Add `new_parent' to `parents'
		require 
			new_parent_not_void: new_parent /= Void
			new_parent_not_in_list: not has_parent (new_parent)
		do
			parents.force_last (new_parent)
		ensure 
			one_more_parent: parents.count = old (parent_count) + 1
			new_parent_in_parents: has_parent (new_parent)
		end 
	
	remove_parent (the_parent: SCENE_GROUP) is
			-- Remove `the_parent' from `parents'
		require 
			the_parent_not_void: the_parent /= Void
			in_parents: has_parent (the_parent)
		do
			parents.start
			parents.search_forth (the_parent)
			parents.remove_at
		ensure 
			one_less_parent: parents.count = old(parent_count) - 1
			the_parent_not_in_list: not has_parent (the_parent)
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	pre_draw is
			-- A traversal function that is used before the draw 
			-- visitor to ensure necessary clean-up and rebuilding 
			-- has been done
		do
			if update_needed then
				gather_extents
				update_needed := false
			end
		end 
	
	transform_to_parent (t: TRANSFORM) is
			-- Compute a transform to parent and append the transform
			-- to the parameter passed in.	This is the default
			-- implementation for all nodes that do not affect this
			-- matrix and therefore this function does nothing
		require
			t_not_void: t /= Void
		do
		end 

	traverse (action: SCENE_ACTION) is
			-- Traverse this node
		require 
			action_not_void: action /= Void
		do
			if traversable then
				traverse_node (action)
			end
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {SCENE_NODE} -- Implementation
	
	update_needed: BOOLEAN
			-- A flag that is used to mark when the bounding box of 
			-- the node has been changed
	
	force_update is
			-- Force a recomputation of the bounds by setting 
			-- `update_needed' to `true'
		do
			update_needed := true
			
			from
				parents.start
			until
				parents.off
			loop
				parents.item_for_iteration.force_update
				
				parents.forth
			end

		ensure 
			update_requested: update_needed
			parents_informed: parents.count > 0 implies 
							  parents.item (1).update_needed
		end 
	
feature {SCENE_NODE} -- Implementation

	parents: DS_ARRAYED_LIST [SCENE_GROUP]
			-- Who are the parents of this container?
	
feature {NONE} -- Implementation

	traverse_node (action: SCENE_ACTION) is
			-- Traverse this node
		require 
			is_traversable: traversable
			action_not_void: action /= Void
		deferred
		end
	
	gather_extents is
			-- Gather together all bounding information.
		require 
			is_traversable: traversable
		do
		end

	print_features (action: PRINT_ACTION) is
			-- print the node's features
		require else
			is_traversable: traversable
		do
			Precursor (action)
			
			action.io.put_string ("	 intersected = ")
			action.io.put_boolean (intersected)
			action.io.put_new_line
			
			action.io.put_string ("	 traversable = ")
			action.io.put_boolean (traversable)
			action.io.put_new_line
		end 

	is_acyclic: BOOLEAN is
			-- Is the graph acyclic
		local
			p_list: DS_ARRAYED_LIST [SCENE_NODE]
		do
			Result := false

			create p_list.make (10)
			p_list.force_last (Current)
			Result := is_parent_acyclic (Current, p_list)
			p_list.remove_last
		end 

	is_parent_acyclic (p: SCENE_NODE;
					   p_list: DS_ARRAYED_LIST [SCENE_NODE]): BOOLEAN is
			-- Is the parent graph acyclic
--		require
--			not_in_list: not p_list.has (p)
		local
			sg: SCENE_NODE
			index: INTEGER
		do
			Result := false
			
			index := p.parents.index
			
			from
				p.parents.start
			until
				p.parents.after or Result
			loop
				sg := p.parents.item_for_iteration

				if p_list.has (sg) then
					Result := true
				else
					p_list.force_last (sg)
					Result := is_parent_acyclic (sg, p_list)
					p_list.remove_last
				end
				
				p.parents.forth
			end

			p.parents.go_i_th (index)
		end
	
invariant
	
	parents_not_void: parents /= Void
	graph_is_not_acyclic: not is_acyclic
	
end -- SCENE_NODE

