

indexing
	description: "Orthographic camera for viewing scene graphs"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	SCENE_ORTHOGRAPHIC_CAMERA
	
inherit
	
	SCENE_CAMERA
		rename
			make as camera_make
		undefine
			default_create
		redefine
			update_view_volume
		end

	BASIC_NUMERIC
		undefine
			default_create
		end
	
creation
	default_create, make

feature -- Initialization

	default_create, make is
			-- Initialize the camera
		do
			camera_make
			
			height := 2.0
			
			create the_view_volume
			
			update_view_volume
		end

feature -- Access
	
	height: DOUBLE
			-- The height of the camera 
	
	view_volume: VIEW_VOLUME is
			-- The camera's view volume
		do
			Result := the_view_volume
		end 

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	draw (pool: SCENE_DRAWABLE_POOL; renderer: RENDERER) is
			-- `Draw' the scene through the camera's lens
		local
			shear: TRANSFORM
			focal_dist: DOUBLE
		do
			renderer.viewport (0, 0, x_resolution, y_resolution)
			renderer.matrix_mode (renderer.rd_projection)
			renderer.load_identity
			if about_equal (near_clipper, epsilon) or
				about_equal (far_clipper, infinity) then
				--update_clipping_planes (pool)
				set_clipping (0.05, 100000)
			end
			
			the_view_volume.scale (screen_scale)
			renderer.load_matrix (camera_to_screen)
			the_view_volume.scale (1.0 / screen_scale)
			
			focal_dist := position.distance (focal_point)
			if not about_equal (screen_window @ 2 + screen_window @ 1, 0) or
				not about_equal (screen_window @ 4 + screen_window @ 3, 0) then
				create shear.make_with_elements (1, 0, (screen_window @ 2 + screen_window @ 1)/(2 * focal_dist), 0,
												 0, 1, (screen_window @ 4 + screen_window @ 3)/(2* focal_dist), 0,
												 0, 0, 1, 0,
												 0, 0, 0, 1)
				renderer.multiply_matrix (shear)
			end
			
			renderer.scale (1.0 / frame_aspect_ratio, 1, 1)
			renderer.matrix_mode (renderer.rd_modelview)
			renderer.load_identity
			
			renderer.load_matrix (view_volume.affine_transformation)
			renderer.multiply_matrix (world_to_camera)
			renderer.set_viewing_camera (Current)
			-- Turn on all lights
			renderer.global_lights.illuminate (true)
			renderer.draw_lights
			
--			renderer.front_face (renderer.rd_ccw)
			pool.draw (renderer)
		end 

	update_clipping_planes (node: SCENE_NODE) is
			-- Update the clipping planes to surround the object
		local
			sphere_bound: SCENE_SPHERE_BOUND
		do
			-- Bound the scene graph with a sphere
			create sphere_bound.make_with_point_and_radius	(node.extents.centroid,
															 node.extents.length)
			
			set_clipping (-2*sphere_bound.length, 2*sphere_bound.length)
		end 
	
	view_all (node: SCENE_NODE) is
			-- Center the scene graph in the viewer
		local
			pt: POINT3
			v: VECTOR3
			rotation: TRANSFORM
			sphere_bound: SCENE_SPHERE_BOUND
		do
			-- Bound the scene graph with a sphere
			create sphere_bound.make_with_point_and_radius	(node.extents.centroid,
															 node.extents.length)
			
			
			-- The position will be such that the "camera" touches 
			-- the bounding sphere
			rotation := orientation.rotation_matrix
			create pt.make (0, 0, sphere_bound.length)
			pt := rotation.apply (pt)
			create v.make (sphere_bound.centroid.x, sphere_bound.centroid.y,
						   sphere_bound.centroid.z)
			pt := pt + v
			position.make (pt.x, pt.y, pt.z)
			
			-- Set the clipping planes to the bounds of the sphere and 
			-- the focal point to the center of the sphere.
			set_clipping (-2*sphere_bound.length, 2*sphere_bound.length)
			camera_focal_point.copy (sphere_bound.centroid)
			
			-- Find the height necessary to fit the object
			if frame_aspect_ratio < 1 then
				height := 2 * sphere_bound.length / frame_aspect_ratio
			else
				height := 2 * sphere_bound.length
			end
		end 
	
	generate_ray (sample_point: GENERAL_POINT): RAY is
			-- Generate a ray from the eye through `sample_point'
		local
			p_camera: POINT3
			sample: POINT3
		do
			create sample.make (sample_point.item (1), sample_point.item (2), 0)
			p_camera := raster_to_camera.apply (sample)
			
			create Result
			Result.set_origin (p_camera)
			Result.apply (world_to_camera.inverse)
		end 
	
	update_view_volume is
			-- The camera's view volume
		local
			v: VECTOR3
			x_radius, y_radius: DOUBLE
		do
			Precursor
			
			x_radius := 0.5 * height * frame_aspect_ratio
			y_radius := 0.5 * height
			
			the_view_volume.make_as_orthographic (-x_radius, x_radius, 
												-y_radius, y_radius,
												near_clipper, far_clipper)
			
			the_view_volume.rotate_camera (orientation.rotation_matrix)
			create v.make (position.x, position.y, position.z)
			the_view_volume.translate_camera (v)
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	the_view_volume: VIEW_VOLUME
	
end -- SCENE_ORTHOGRAPHIC_CAMERA

