

indexing
	description: "Perspective camera for viewing scene graph"
	patterns: "none"
	date: "$Date: 2007-02-20 16:28:19 -0500 (Tue, 20 Feb 2007) $"
	revision: "$Revision: 283 $"
	status: "unreviewed"




class

	SCENE_PERSPECTIVE_CAMERA

inherit

	SCENE_CAMERA
		rename
			make as camera_make
		undefine
			default_create
		redefine
			update_view_volume
		end

	BASIC_NUMERIC
		undefine
			default_create
		end

creation
	make, make_with_fov, default_create

feature -- Initialization

	default_create, make is
			--
		do
			camera_make

			field_of_view := 45 * pi / 180.0

			create the_view_volume

			update_view_volume

			update_matrices
		end

	make_with_fov (fov: DOUBLE) is
			-- Make a camera with a field of view of `fov' (in degrees)
		require
			fov_is_positive: fov > 0.0
		do
			make
			set_field_of_view (fov)

			update_view_volume

			update_matrices
		end

feature -- Access

	field_of_view: DOUBLE
			-- The field of view (in radians) in the camera's 'y' direction

	view_volume: VIEW_VOLUME is
			-- The camera's view volume
		do
			Result := the_view_volume
		end

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change

	set_field_of_view (fov: DOUBLE) is
			-- Set `field_of_view' to `fov' (in degrees)
		require
			fov_is_positive: fov > 0.0
		do
			field_of_view := fov * pi / 180.0
		ensure
			field_of_view_set: about_equal (field_of_view, fov * pi / 180.0)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations

	draw (pool: SCENE_DRAWABLE_POOL; renderer: RENDERER) is
			-- `Draw' the scene through the camera's lens
		local
			shear: TRANSFORM
			focal_dist: DOUBLE
		do
			-- set up viewing according to the camera parameters
			renderer.viewport (0, 0, x_resolution, y_resolution)
			renderer.matrix_mode (renderer.rd_projection)
			renderer.load_identity
			if about_equal (near_clipper, epsilon) or
				about_equal (far_clipper, infinity) then
				--update_clipping_planes (pool)
				set_clipping (0.05, 100000)
			end

			--the_view_volume.scale (screen_scale)
			shear := clone (camera_to_screen)
			shear.scale (1/screen_scale, 1/screen_scale, 1.0)
			renderer.load_matrix (shear)
			--the_view_volume.scale (1.0 / screen_scale)
			focal_dist := position.distance (focal_point)
			if not about_equal (screen_window @ 2 + screen_window @ 1, 0) or
				not about_equal (screen_window @ 4 + screen_window @ 3, 0) then
				create shear.make_with_elements (1, 0, (screen_window @ 2 + screen_window @ 1)/(2 * focal_dist), 0,
												 0, 1, (screen_window @ 4 + screen_window @ 3)/(2* focal_dist), 0,
												 0, 0, 1, 0,
												 0, 0, 0, 1)
				renderer.multiply_matrix (shear)
			end

			renderer.scale (1.0/frame_aspect_ratio, 1, 1)
			renderer.matrix_mode (renderer.rd_modelview)
			renderer.load_matrix (world_to_camera)
			renderer.set_viewing_camera (Current)
			-- Turn on all lights
			renderer.global_lights.illuminate (true)
			renderer.draw_lights

--			renderer.front_face (renderer.rd_ccw)
			pool.draw (renderer)
		end

	extents_to_camera_space (node: SCENE_NODE): SCENE_BOUND is
			-- Transform `extents' into camera space
		require
			node_not_void: node /= Void
		do
			Result := node.extents.apply (world_to_camera)
		ensure
			Result_not_void: Result /= Void
		end

	update_clipping_planes (node: SCENE_NODE) is
			-- Update the clipping planes to surround the object
		local
			h_over_d: DOUBLE
			distance_to_center, tmp_dist: DOUBLE
			sphere_bound: SCENE_SPHERE_BOUND
		do
			-- Bound the scene graph with a sphere
			create sphere_bound.make_with_point_and_radius	(node.extents.centroid,
															 node.extents.length)

			h_over_d := tangent (field_of_view / 2)
			if frame_aspect_ratio < 1 then
				h_over_d := h_over_d * frame_aspect_ratio
			end

			distance_to_center := sphere_bound.length / h_over_d

			-- Set the clipping planes to the bounds of the sphere and
			-- the focal point to the center of the sphere.
			tmp_dist := distance_to_center - sphere_bound.length
			if tmp_dist < 0 or
				tmp_dist < minimum_near_plane_distance * distance_to_center then
				tmp_dist := minimum_near_plane_distance * distance_to_center
			end

			set_clipping (tmp_dist, distance_to_center + sphere_bound.length)
		end

	view_all (node: SCENE_NODE) is
			-- Center the scene graph in the viewer
		local
			h_over_d: DOUBLE
			distance_to_center, tmp_dist: DOUBLE
			v: VECTOR3
			rotation: TRANSFORM
			pt: POINT3
			sphere_bound: SCENE_SPHERE_BOUND
		do
			-- Bound the scene graph with a sphere
			create sphere_bound.make_with_point_and_radius	(internal_world_to_camera.apply (node.extents.centroid),
															 node.extents.length)

			h_over_d := tangent (field_of_view / 2)
			if frame_aspect_ratio < 1 then
				h_over_d := h_over_d * frame_aspect_ratio
			end

			distance_to_center := sphere_bound.length / h_over_d

			-- The position will be such that the "camera" touches
			-- the bounding sphere
			rotation := orientation.rotation_matrix
			create pt.make (0, 0, distance_to_center)
			pt := rotation.apply (pt)
			create v.make (sphere_bound.centroid.x, sphere_bound.centroid.y,
						   sphere_bound.centroid.z)
			pt := pt + v
			position.make (pt.x, pt.y, pt.z)

			-- Set the clipping planes to the bounds of the sphere and
			-- the focal point to the center of the sphere.
			tmp_dist := distance_to_center - sphere_bound.length
			if tmp_dist < 0 or
				tmp_dist < minimum_near_plane_distance * distance_to_center then
				tmp_dist := minimum_near_plane_distance * distance_to_center
			end

			set_clipping (tmp_dist, distance_to_center + sphere_bound.length)
			camera_focal_point.copy (internal_world_to_camera.apply (node.extents.centroid))

			debug ("gui")
				io.put_string ("%NNew Camera position%N")
				io.put_string ("  sphere_radius = ")
				io.put_double (sphere_bound.length)
				io.put_string ("%N	sphere_center = ")
				io.put_string (sphere_bound.centroid.out)
				io.put_string ("%N	position = ")
				io.put_string (position.out)
				io.put_string ("%N	camera_focal_point = ")
				io.put_string (camera_focal_point.out)
				io.put_string ("%N	distance_to_center = ")
				io.put_double (distance_to_center)
				io.put_string ("%N	clipping planes = [")
				io.put_double (near_clipper)
				io.put_string (" ")
				io.put_double (far_clipper)
				io.put_string ("]%N")
			end
		end

	generate_ray (sample_point: POINT): RAY is
			-- Generate a ray from the eye through `sample_point'
		local
			p_camera: VECTOR3
			sample: POINT3
		do
			create sample.make (sample_point.item (1), sample_point.item (2), 0)
			sample := raster_to_camera.apply (sample)

			create p_camera.make (sample.x, sample.y, sample.z)
			p_camera.normalize

			create Result
			Result.set_direction (p_camera)
			Result.apply (world_to_camera.inverse)

			debug ("ray")
				io.put_string (sample_point.out)
				io.put_string (" ")
				io.put_string (Result.out)
				io.put_string ("%N")
			end
		end

	update_view_volume is
			-- The camera's view volume
		local
			v: VECTOR3
		do
			Precursor

			the_view_volume.make_as_perspective (field_of_view, pixel_aspect_ratio,
											   near_clipper, far_clipper)

			the_view_volume.rotate_camera (orientation.rotation_matrix)
			create v.make (position.x, position.y, position.z)
			the_view_volume.translate_camera (v)

			update_matrices
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	minimum_near_plane_distance: DOUBLE is 0.01

	the_view_volume: VIEW_VOLUME

end -- SCENE_PERSPECTIVE_CAMERA

