

indexing
	description: "Abstract class for scene graph objects with a volume"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 

	SCENE_VOLUME
	
inherit
	
	SCENE_OBJECT
		redefine
			print_features, default_create, is_equal
		end
		
	INTERSECTABLE
		rename
			intersect as intersectable_intersect
		export
			{NONE} intersectable_intersect
		redefine
			intersectable_intersect, default_create, is_equal
		end
	
feature -- Initialization
	
	default_create is
			-- Initalize a node my calling `make'
		do
			Precursor {SCENE_OBJECT}
			Precursor {INTERSECTABLE}

			handness_set := false
			is_left_handed := false
		end
	
feature -- Access
	
	extents: SCENE_BOUND is
			-- The `extents' or bounds of the volume
		deferred
		end
	
	is_left_handed: BOOLEAN
			-- Is this object left-handed?

	handness_set: BOOLEAN
			-- Has `is_left_handed' been set?
	
feature -- Measurement
feature -- Comparison
	
	is_equal (other: like Current): BOOLEAN is
			-- Are we equal?
		do
			Result := handness_set = other.handness_set and 
						is_left_handed = other.is_left_handed and
						 extents.is_equal (other.extents)
			if Result then
				Result := has_extents = other.has_extents
			end
		end
	
feature -- Status report
	
	has_extents: BOOLEAN is
			-- Does this volume have extents?
		deferred
		end
	
feature -- Status setting

	set_handness (left_handed: BOOLEAN) is
			-- Set the `is_left_handed' to `left_handed'
		do
			is_left_handed := left_handed
			handness_set := true
		ensure 
			is_left_handed_is_set: is_left_handed = left_handed
			is_set: handness_set
		end 

feature -- Cursor movement
feature -- Element change

	update_extents is
			-- Recompute the extents of the object
		deferred
		end
		
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	cull (action: CULLING_ACTION) is
			-- A traversal that culls geometry not in the viewing 
			-- frustrum
		require 
			action_not_void: action /= Void
		deferred
		end 

	intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		require
			intersecter_not_void: intersecter /= Void
			valid_ray: intersecter.ray /= Void
		do
			intersected := false
			intersection_count := intersection_count + 1
			
			extents.intersect_ray (intersecter.ray)
			if not extents.outside then
				object_intersect (intersecter)
			
				if intersected then
					intersection_hit := intersection_hit + 1
				end
			end
		end

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		require
			intersecter_not_void: intersecter /= Void
			valid_ray: intersecter.ray /= Void
			not_intersected: not intersected
		deferred
		end

	print_features (action: PRINT_ACTION) is
			-- print the node's features
		do
			Precursor (action)
			
			action.io.put_string ("	 extents = ")
			action.io.put_string (extents.out)
			action.io.put_new_line
		end 
	
	intersectable_intersect (ray: RAY) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		do
			check
				never_called: false
			end
		end

invariant
	
	extents_not_void: has_extents
	
end -- SCENE_VOLUME

