

indexing
	description: "Tesselates any primitive representable as a parametric square"
	patterns: "template"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	status: "unreviewed"




deferred class 

	SCENE_TESSELATOR

inherit
	
	SCENE_COORDINATES
		redefine
			default_create
		end
	
feature -- Initialization

	default_create is
			-- 
		do
			Precursor {SCENE_COORDINATES}
			
			threshold := 0.005
			bias := 0.7
			skewness := 0.01
			
			tesselated := false
		end

feature -- Access
	
	tesselated: BOOLEAN
			-- Has the object been tesselated
	
	threshold: DOUBLE
			-- Factor for terminating splitting based on edge length
	
	bias: DOUBLE
			-- Factor for determining when an edge should be split
	
	skewness: DOUBLE
			-- Factor for terminating splitting based on tile degeneracy
	
feature -- Measurement

	edge_length: DOUBLE
			-- The length of the edge computed by `split_edge'
	
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	split_edge (pt1, pt2: POINT2): BOOLEAN is
			-- determine whether an edge should be split
		require
			pt1_not_void: pt1 /= Void
			pt2_not_void: pt2 /= Void
		deferred
		end
	
	split_center (pts: ARRAY [POINT2]): BOOLEAN is
			-- determine whether a tile should be split in the center
		require
			pts_not_void: pts /= Void
		deferred
		end
	
	evaluate_pt (pt: POINT2): POINT3 is
			-- determine whether a tile should be split in the center
		require
			pt_not_void: pt /= Void
		deferred
		ensure
			Result_not_void: Result /= Void
		end
	
	normal (u, v: DOUBLE): NORMAL is
			-- Compute the normal at the parametric coordinates 
			-- (`u',  `v') 
		require 
			u_in_range: u >= 0 and u <= 1
			v_in_range: v >= 0 and v <= 1
		deferred
		ensure 
			Result_not_void: Result /= Void
			Result_is_unit: Result.is_unit
		end 
			
	begin_tesselation (geom: SCENE_GEOMETRY) is
			-- Tesselate the object associated with the splitting functions
		require
			coordinates_not_void: coordinates /= Void
			coordinate_indices_not_void: coordinate_indices /= Void
		local
			work: DS_LINKED_LIST [TESSELATION_TILE]
			finished: DS_ARRAYED_LIST [TESSELATION_TILE]
			tile: TESSELATION_TILE
			pt: POINT2
			i, counter: INTEGER
			indices: INTEGER_64_ARRAYED_LIST
			ine: SCENE_INDEXED_NORMAL_ELEMENT
		do
			create work.make
			create finished.make (50)
			
			create tile
			create pt.make (geom.u_minimum, geom.v_minimum)
			tile.set_vertex (1, pt)
			create pt.make (geom.u_maximum, geom.v_minimum)
			tile.set_vertex (2, pt)
			create pt.make (geom.u_minimum, geom.v_maximum)
			tile.set_vertex (3, pt)
			work.force_last (tile)
			
			create tile
			create pt.make (geom.u_minimum, geom.v_maximum)
			tile.set_vertex (1, pt)
			create pt.make (geom.u_maximum, geom.v_minimum)
			tile.set_vertex (2, pt)
			create pt.make (geom.u_maximum, geom.v_maximum)
			tile.set_vertex (3, pt)
			work.force_last (tile)
			
			from
				work.start
			until
				work.is_empty
			loop
				tile := work.item_for_iteration
				work.remove_at
				
				if tile.refine (Current) then
					from
						tile.refined.start
					until
						tile.refined.after
					loop
						work.force_last (tile.refined.item_for_iteration)
						tile.refined.forth
					end
				else
					finished.force_last (tile)
				end
			end
			
			ine ?= geom.element_item ("SCENE_INDEXED_NORMAL_ELEMENT")
			if ine = Void then
				create ine
			else
				ine.remove_all_values
			end
			
			create coordinates.make (finished.count * 3)
			create coordinate_indices.make (finished.count * 3)
			
			from
				finished.start
				counter := 1
			until
				finished.is_empty
			loop
				tile := finished.item_for_iteration
				finished.remove_at
				
				create indices.make (3)
				from
					i := 1 
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + tile.count
				variant
					i_increasing: 1 + tile.count - i
				until
					i > tile.count
				loop
					ine.add_item (normal (tile.vertex (i).x, tile.vertex (i).y))
					
					coordinates.force_last (evaluate_pt (tile.vertex (i)))
					indices.force_last (counter)
					counter := counter + 1
					
					i := i + 1
				end
				
				ine.create_strip
				coordinate_indices.force_last (indices)
			end
			ine.set_strip_index (1)
			geom.replace_element (ine)
			
			geom.set_normal_mapping (geom.per_vertex)
			
			debug ("tesselator")
				from
					coordinates.start
				until
					coordinates.after
				loop
					io.put_string (coordinates.index.out)
					io.put_string (" - ")
					io.put_string (coordinates.item_for_iteration.out)
					io.put_new_line
					coordinates.forth
				end
				from
					coordinate_indices.start
				until
					coordinate_indices.after
				loop
					io.put_integer (coordinate_indices.index)
					io.put_string (" - ")
					from
						coordinate_indices.item_for_iteration.start
					until
						coordinate_indices.item_for_iteration.after
					loop
						io.put_string (coordinate_indices.item_for_iteration.item_for_iteration.out)
						io.put_string (", ")
						coordinate_indices.item_for_iteration.forth
					end
					io.put_new_line
					coordinate_indices.forth
				end
			end
					
			tesselated := true		
		ensure
			tesselated: tesselated
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

end -- SCENE_TESSELATOR

