

indexing
	description: "Intermediary structure generated by the SCENE_TESSELATOR"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




class 

	TESSELATION_TILE

inherit
	
	ANY
		redefine
			default_create
		end

creation
	default_create, make

feature -- Initialization

	default_create is
			-- 
		do
			create verticies.make (1, count)			
		end

	make (pts: ARRAY [POINT2]) is
			-- Initialize the tile to `pts'
		require
			pts_not_void: pts /= Void
			correct_size: pts.count = count
		do
			verticies := pts
		end

feature -- Access
	
	vertex (index: INTEGER): POINT2 is
			-- The `vertex' at `index'
		require 
			index_in_range: index >= 1 and index <= count
		do
			Result := verticies.item (index)
		ensure 
			Result_not_void: Result /= Void
		end 
	
	refined: DS_ARRAYED_LIST [like Current]
			-- The result of `refine'
	
feature -- Measurement
	
	count: INTEGER is 3
			-- The number of verticies that make up a tile
	
	area: DOUBLE is
			-- The `area' of the tile
		local
			pt1, pt2, pt3: POINT2
		do
			pt1 := verticies.item (1)
			pt2 := verticies.item (2)
			pt3 := verticies.item (3)
			
			Result := pt1.x * pt2.y - pt2.x * pt1.y +
				pt2.x * pt3.y - pt3.x * pt2.y +
				pt3.x * pt1.y - pt1.x * pt3.y
			
			if Result < 0 then
				Result := -Result
			end
		ensure
			area_is_positive: Result > 0
		end
	
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	set_vertex (index: INTEGER; vtx: POINT2) is
			-- Set `vtx' as the `index'th vertex
		require 
			index_in_range: index >= 1 and index <= count
			vertex_not_void: vtx /= Void
		do
			verticies.put (vtx, index)
		ensure 
			set: vtx = verticies.item (index)
		end 

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	refine (tesselator: SCENE_TESSELATOR): BOOLEAN is
			-- 
		require 
			tesselator_not_void: tesselator /= Void
		local
			perimeter_squared: DOUBLE
			pt, pt1, pt2: POINT2
			i, j, j1, j2, co: INTEGER
			eds: ARRAY [DOUBLE]
			m: ARRAY [INTEGER]
			mv: ARRAY [POINT2]
			tile: like Current
		do
			from
				i := 2
				pt := verticies.item (1)
			until
				i > count
			loop
				perimeter_squared := perimeter_squared + pt.distance_squared (verticies.item (i))
				pt := verticies.item (i)
			
				i := i + 1
			end
			
			perimeter_squared := perimeter_squared + pt.distance_squared (verticies.item (1))
			
			if area / perimeter_squared > tesselator.skewness then
				create eds.make (1, count)
				
				from
					i := 1
					perimeter_squared := 0
				invariant
					i_at_least_one: i >= 1
					i_small_enough: i <= 1 + count
				variant
					i_increasing: 1 + count - i
				until
					i > count
				loop
					j1 := i \\ count + 1
					j2 := (i + 1) \\ count + 1
					
					pt1 := verticies.item (j1)
					pt2 := verticies.item (j2)
					if tesselator.split_edge (pt1, pt2) then
						eds.put (tesselator.edge_length, i)
						if perimeter_squared < tesselator.edge_length then
							perimeter_squared := tesselator.edge_length
						end
					end
					
					i := i + 1
				end
				
				if perimeter_squared > 0 then
					perimeter_squared := perimeter_squared * tesselator.bias
					
					from
						i := 1
						co := 0
						create m.make (1, count)
						create mv.make (1, 3)
					invariant
						i_at_least_one: i >= 1
						i_small_enough: i <= 1 + count
					variant
						i_increasing: 1 + count - i
					until
						i > count
					loop
						j1 := i \\ count + 1
						j2 := (i + 1) \\ count + 1
						
						pt1 := verticies.item (j1)
						pt2 := verticies.item (j2)
						
						if eds.item (i) > tesselator.threshold and
							eds.item (i) >= perimeter_squared then
							co := co + 1
							mv.put (pt1.interpolate (pt2, 0.5), i)
							m.put (i - count, i)
						else
							if eds.item (j1) > eds.item (j2) then
								mv.put (verticies.item (j1), i)
								m.put (j1, i)
							else
								mv.put (verticies.item (j2), i)
								m.put (j2, i)
							end
						end
						
						i := i + 1
					end
					
					create refined.make (4)
					
					if co > 0 then
						co := co + 1
						
						-- Process corner tiles
						from
							i := 1
							j := co
						invariant
							i_at_least_one: i >= 1
							i_small_enough: i <= 1 + count
						variant
							i_increasing: 1 + count - i
						until
							i > count
						loop
							j1 := i \\ count + 1
							j2 := (i + 1) \\ count + 1
							
							if m @ j1 /= i and m @ j2 /= i then
								j := j - 1
								create tile
								tile.set_vertex (1, verticies.item (i))
								tile.set_vertex (2, mv.item (j2))
								tile.set_vertex (3, mv.item (j1))
								refined.force_last (tile)
							end
							i := i + 1
						end
						
						Result := true
						if j > 0 then
							-- Add a center tile
							create tile.make (mv)
							refined.force_last (tile)
						end
					elseif tesselator.split_center (mv) then
						create pt1.make ((verticies.item (1).x + verticies.item (2).x + verticies.item (3).x) / 3,
										 (verticies.item (1).y + verticies.item (2).y + verticies.item (3).y) / 3)
						
						from
							i := 1
						invariant
							i_at_least_one: i >= 1
							i_small_enough: i <= 1 + count
						variant
							i_increasing: 1 + count - i
						until
							i > count
						loop
							create tile
							
							tile.set_vertex (1, verticies.item (i))
							tile.set_vertex (2, verticies.item ((i + 1) \\ (count - 1)))
							tile.set_vertex (3, pt1)
							refined.force_last (tile)
							
							i := i + 1
						end
						Result := true
					else
						Result := false
					end
				end
			else
				check
					dice_needed: false
				end
			end
		ensure 
			if_refined: Result implies refined.count > 0
		end 

feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	verticies: ARRAY [POINT2]
			-- The tile's verticies
	
invariant
	
	count_at_least_3: count >= 3
	verticies_not_void: verticies /= Void
	
end -- TESSELATOR_TILE

