

indexing
	description: "Mixin for intersecting lists of triangles"
	patterns: "none"
	date: "$Date: 2007-03-29 13:43:21 -0400 (Thu, 29 Mar 2007) $"
	revision: "$Revision: 286 $"
	status: "unreviewed"




deferred class 

	TRIANGLE_INTERSECTOR

inherit
	
	BASIC_NUMERIC
		export
			{ANY} Tolerance
		undefine
			copy, is_equal, default_create
		end
	
	SCENE_INDEXED_GEOMETRY
		undefine
			print_features, create_color_element
		redefine
			 i_th_bound, intersect
		end
	
feature -- Initialization
feature -- Access
	
	i_th_bound (index: INTEGER): AXIS_ALIGNED_BOX is
			-- Return a bound around the `index'th element
		local
			indices: INTEGER_64_ARRAYED_LIST
		do
			indices := coordinate_indices.item (index)
			
			create Result.make (coordinates.item (indices.item (1)),
								coordinates.item (indices.item (2)))
			from
				indices.start
				indices.forth
				indices.forth
			until
				indices.after
			loop
				Result.grow (coordinates.item (indices.item_for_iteration))
				indices.forth
			end		
		end 

feature -- Access
feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
	
	set_intersection_status (status: BOOLEAN) is
			-- Set the intersection status
		deferred
		end
	
feature -- Cursor movement
feature -- Element change	
feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
	
	intersect (intersecter: INTERSECT_ACTION) is
			-- Intersect with `intersecter'.
			-- If `intersecter' does not intersect then `intersected' will be False
			-- else `intersected' will be True and `intersection' will yield
			-- the ordered parametric coordinates along `ray' of each
			-- successive point of intersection.
		do
			intersected := false
			set_intersection_status (false)
			
			extents.intersect_ray (intersecter.ray)
			if not extents.outside then
				object_intersect (intersecter)
			end
		end

	object_intersect (intersecter: INTERSECT_ACTION) is
			-- Generate the intersection of the ray, `intersecter.ray', 
			-- against this volume. It is assumed that the ray has 
			-- been transformed into this object's coordinate system
		do
			set_intersection_status (false)
			check
				never_called: false
			end
		end
	
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	color_element: SCENE_INDEXED_COLOR_ELEMENT
	normal_element: SCENE_INDEXED_NORMAL_ELEMENT
	
	intersect_item (intersecter: INTERSECT_ACTION; index: INTEGER_64) is
			-- Test the intersection against the item at `index'
		local
			indices: INTEGER_64_ARRAYED_LIST
			e1, e2, s1, t, s2: VECTOR3
			pt: POINT3
			i: INTEGER
			intersected_flag: BOOLEAN
			divisor, inv_divisor, u, v, tt: DOUBLE
		do
			indices := coordinate_indices.item (index.to_integer)
			
			from
				i := 1 
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 2 + indices.count
			variant
				i_increasing: 2 + indices.count - i
			until
				i > indices.count - 1
			loop
				intersected_flag := false
				e1 := coordinates.item (indices.item (i + 1)) - 
					coordinates.item (indices.item (i)) 
				e2 := coordinates.item (indices.item (i + 2)) - 
					coordinates.item (indices.item (i))
				s1 := intersecter.ray.direction * e2
				divisor := s1 # e1
				
				if divisor /= 0 then
					inv_divisor := 1.0 / divisor
					
					t := intersecter.ray.origin - coordinates.item (indices.item (i))
					u := (t # s1) * inv_divisor
					
					if u >= 0 and u <= 1 then
						s2 := t * e1
						v := (intersecter.ray.direction # s2) * inv_divisor
						
						if v >= 0 and u + v <= 1 then
							tt := (e2 # s2) * inv_divisor
							if tt > Tolerance then
								pt := intersecter.ray.project (tt)
								set_intersection_status (true)
								intersected_flag := true
								normal_element.set_strip_index (index)
								if normal_mapping = per_object or normal_mapping = per_element then
									normal_element.go_i_th (i\\3)
								else
									normal_element.go_i_th (i)
								end
								if not (color_mapping = constant_color) then
									color_element.set_strip_index (index)
									if color_mapping = per_object or color_mapping = per_element then
										color_element.go_i_th (i\\3)
									else
										color_element.go_i_th (i)
									end
								end
								
								debug ("triangle")
									io.put_string ("Triangle Intersection - ")
									io.put_string (index.out)
									io.put_string ("%N%T t = ")
									io.put_double (tt)
									io.put_string ("%N%T u = ")
									io.put_double (u)
									io.put_string ("%N%T v = ")
									io.put_double (v)
									io.put_string ("%N%T point = ")
									io.put_string ("%N%T%T")
									io.put_string (pt.out)
									io.put_string ("%N%T ray = ")
									io.put_string ("%N%T%T")
									io.put_string (intersecter.ray.out)
									io.put_string ("%N%T triangle = ")
									io.put_string ("%N%T%T")
									io.put_string (coordinates.item (indices.item (1)).out)
									io.put_string ("%N%T%T")
									io.put_string (coordinates.item (indices.item (2)).out)
									io.put_string ("%N%T%T")
									io.put_string (coordinates.item (indices.item (3)).out)
									io.put_string ("%N")
								end
								
								if two_sided or (not two_sided and 
											  normal_element.item.dot_with_vector (intersecter.ray.direction) < 0) then
									insert_intersection_information (intersecter, u, v, tt, 
																	 color_element, normal_element, pt, intersecter.ray)
								end
							end
						end
					end
				end
				
				i := i + 3
			end								
		end 

	insert_intersection_information (action: INTERSECT_ACTION;
									 u, v, t: DOUBLE; 
									 ice: SCENE_INDEXED_COLOR_ELEMENT;
									 ine: SCENE_INDEXED_NORMAL_ELEMENT;
									 point: POINT3;
									 ray: RAY) is
			-- Insert `t', `point', and other computed information 
			-- into `intersection'
		local
			info: INTERSECTION_DATA
			normal: NORMAL
			color: RGBA_COLOR
			c1, c2, c3, c4: RGBA_COLOR
			scale_val: DOUBLE
		do
			scale_val := 1 - u - v
			if scale_val < 0 then
				scale_val := 0
			end
			
			if color_mapping = constant_color then
				color := ice.i_th (1)
			elseif color_mapping = per_object or color_mapping = per_element then
				color := ice.item
				ice.forth
			else
				c1 := ice.item
				ice.forth
				c2 := ice.item
				ice.forth
				c3 := ice.item
				ice.forth
				c4 := c2
								
				color := c1.scale (scale_val) + c2.scale (u) + c3.scale (v)
--				color := c1.scale ((1-u)*(1-v)) + c2.scale (u*(1-v)) + c3.scale ((1-u)*v) + c4.scale (u*v)
			end
			
			if normal_mapping = per_object or normal_mapping = per_element then
				normal := ine.item
				ine.forth
			else
				normal := ine.item.scale (scale_val) 
				ine.forth
				normal := normal + ine.item.scale (u)
				ine.forth
				normal := normal + ine.item.scale (v)
				ine.forth
			end
			
			if -action.ray.direction # normal < 0 then
				normal := -normal
			end
			
			create info.make (t, point, normal,
							  u, v, color, ray, 
							  Current)
			
			action.intersection.force_last (info)
		end
	
end -- TRIANGLE_INTERSECTOR

