 

indexing
	description: "A multi-dimensional area for sampling"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"




deferred class 
	
	DOMAIN 

inherit
	
	ANY
		redefine
			out, default_create
		end
	
feature -- Initialization

	default_create is
			-- Create a default domain
		do
			make
		end

	make is
			-- Create a default domain
		deferred
		end

	make_from_intervals (vals: DS_ARRAYED_LIST [DOMAIN_INTERVAL]) is
			-- Create a domain of specified intervals
		require
			valid_array: vals /= Void and vals.count > 0
		deferred
		ensure
			dimensions_correct: dimensions = vals.count
		end

feature -- Access
	
	buckets (i: INTEGER): INTEGER is
			-- The number of buckets for dimension `i'
		require 
			good_index: i >= 1 and i <= dimensions
		deferred
		ensure
			Result_positive: Result > 0
		end

	bucket_number (point: POINT): INTEGER is
			-- Compute a number to represent the point. All values 
			-- inside a single bucket map into one `bucket number'
		require 
			not_void: point /= Void
			correct_size: point.count = dimensions
		deferred
		ensure 
			good_bucket: Result >= 1 and Result <= bucket_count
		end

	dimensions: INTEGER is
			-- The `dimensions' of the domain
		deferred
		end

	bucket_width (i: INTEGER): DOUBLE is
			-- the `width' of a bucket in interval `i'
		require 
			good_index: i >= 1 and i <= dimensions
		deferred
		end
	
	bucket_widths: ARRAY [DOUBLE] is
			-- the `width' of all buckets
		deferred
		ensure
			Result_not_void: Result /= Void
			correct_dimensions: Result.count = dimensions
		end
   
	i_th (i: INTEGER): DOMAIN_INTERVAL is
			-- the interval `i'
		require 
			good_index: i >= 1 and i <= dimensions
		deferred
		ensure
			Result_not_void: Result /= Void
		end
	
	item: POINT is
			-- The current point in the domain
		deferred
		ensure
			Result_not_void: Result /= Void
			Result_matches_dimensions: Result.count = dimensions
		end
	
	indexed_point (indicies: ARRAY [INTEGER]): POINT is
			-- Return the point defined by `indicies'
		require 
			indicies_not_void: indicies /= Void
			indicies_matches_dimensions: indicies.count = dimensions
		deferred
		ensure 
			Result_not_void: Result /= Void
			Result_matches_dimensions: Result.count = dimensions
		end 

	interpolation (point: POINT): POINT is
			-- 
		require 
			not_void: point /= Void
			correct_size: point.count = dimensions
		deferred
		ensure 
			Result_not_void: Result /= Void
			Result_matches_dimensions: Result.count = dimensions
		end
	
	bounds: ARRAY [ARRAY [DOUBLE]] is
			-- The `bounds' for `item'
		deferred
		ensure 
			not_void: Result /= Void
			correct_size: Result.count = dimensions
		end

feature -- Measurement
	
	bucket_count: INTEGER
			-- The total number of buckets in the `domain'

feature -- Comparison
feature -- Status report
	
	before: BOOLEAN is
			-- Is this the first point in the domain?
		deferred
		end

	after: BOOLEAN is
			-- Is this the last point in the domain?
		deferred
		end
	
feature -- Status setting
	
feature -- Cursor movement
	
	start is
			-- Move cursor to the beginning of the domain
		deferred
		ensure 
			before: before
		end
	
	finish is
			-- Move cursor to the end of the domain
		deferred
		ensure 
			after: after
		end
	
	forth is
			-- Move forward one cell
		deferred
		ensure 
			default: true
		end

feature -- Element change
	
	extend (v: DOMAIN_INTERVAL) is
			-- Add `v' to end. Implicitly calls `start'
		require 
			good_interval: v /= Void
		deferred
		ensure 
			extended: old dimensions = dimensions + 1
		end

	extend_by_values (minimum, maximum: DOUBLE; num_buckets: INTEGER) is
			-- Create a new rectilinear interval and add to end.
		require 
			good_interval: minimum < maximum and num_buckets > 0
		deferred
		ensure 
			--extended: old intervals.count = intervals.count + 1
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
	
	out: STRING is
			-- Printable representation of identification of `Current'
		do
		end

feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation
	
	update_item is
			-- Update `domain_pt' to the values specified by `bucket_index'
		deferred
		end

invariant
	
	buckets_are_positive: bucket_count > 0
						
end -- DOMAIN

