

indexing
	description: "A two-dimensional sampling area"
	names: "domain", "sampling"
	representation: "array"
	access: "index", "cursor"
	size: "resizable"
	contents: "DS_ARRAYED_LIST"
	patterns: "none"
	date: "$Date: 2006-08-14 17:33:52 -0400 (Mon, 14 Aug 2006) $"
	revision: "$Revision: 251 $"
	author: "Mark A Bolstad"
	status: "unreviewed"




class

	DOMAIN_2D

inherit

	DOMAIN
		export
			{NONE} default_create, make, extend, extend_by_values
		redefine
			out
		end

creation
	make_from_intervals

feature -- Initialization

	make_from_intervals (vals: DS_ARRAYED_LIST [DOMAIN_INTERVAL]) is
			-- Create a domain of specified intervals
		do
			if domain_pt = Void then
				create domain_pt
			else
				domain_pt.default_create
			end

			if tmp_pt = Void then
				create tmp_pt
			else
				tmp_pt.default_create
			end

			if intervals = Void then
				create intervals.make (1, 2)
			end

			d1 := vals.item (1).standard_twin
			intervals.put (d1, 1)

			d2 := vals.item (2).standard_twin
			intervals.put (d2, 2)

			bucket_count := d1.buckets * d2.buckets
			b1 := 1
			b2 := 1
		end

feature -- Access

	buckets (i: INTEGER): INTEGER is
			-- The number of buckets for dimension `i'
		do
			Result := intervals.item (i).buckets
		end

	bucket_number (point: POINT): INTEGER is
			-- Compute a number to represent the point. All values
			-- inside a single bucket map into one `bucket number'
		do
			Result := d1.bucket_interpolation(point.item (1)) - 1
			Result := Result +
				(d2.bucket_interpolation (point.item (2))-1) * d1.buckets
		end

	dimensions: INTEGER is 2
			-- The `dimensions' of the domain

	bucket_width (i: INTEGER): DOUBLE is
			-- the `width' of a bucket in interval `i'
		do
			Result := intervals.item (i).bucket_width
		end

	bucket_widths: ARRAY [DOUBLE] is
			-- the `width' of all buckets
		do
			create Result.make (1, dimensions)
			Result.put (d1.bucket_width, 1)
			Result.put (d2.bucket_width, 2)
		end

	i_th (i: INTEGER): DOMAIN_INTERVAL is
			-- the interval `i'
		do
			Result := intervals.item (i)
		end

	item: POINT2 is
			-- The current point in the domain
		do
			Result := domain_pt
		end

	indexed_point (indicies: ARRAY [INTEGER]): POINT2 is
			-- Return the point defined by `indicies'
		do
			create Result
			Result.put (d1.item (indicies.item (1)), 1)
			Result.put (d2.item (indicies.item (2)), 2)
		end

	interpolation (point: POINT2): POINT2 is
			--
		do
			tmp_pt.put (d1.interpolation (point.x), 1)
			tmp_pt.put (d2.interpolation (point.y), 2)

			Result := tmp_pt
		end

	bounds: ARRAY [ARRAY [DOUBLE]] is
			-- The `bounds' for `item'
		do
			create Result.make (1, dimensions)
			Result.put (d1.bounds (b1), 1)
			Result.put (d2.bounds (b2), 2)
		end

feature -- Measurement
feature -- Comparison
feature -- Status report

	before: BOOLEAN is
			-- Is this the first point in the domain?
		do
			Result := b1 = 1 and b2 = 1
		end

	after: BOOLEAN is
			-- Is this the last point in the domain?
		do
			Result := b1 = 1 and b2 = d2.buckets + 1
		end

feature -- Status setting
feature -- Cursor movement

	start is
			-- Move cursor to the beginning of the domain
		do
			b1 := 1
			b2 := 1
			update_item
		end

	finish is
			-- Move cursor to the end of the domain
		do
			b1 := d1.buckets
			b2 := d2.buckets
			update_item
		end

	forth is
			-- Move forward one cell
		do
			b1 := b1 + 1
			if b1 > d1.buckets then
				b1 := 1
				b2 := b2 + 1
			end

			if not after then
				update_item
			end
		ensure then
			advanced: (b1 = old b1 + 1) or (b1 = 1 and b2 = old b2 + 1)
		end

feature -- Element change

	extend (v: DOMAIN_INTERVAL) is
			-- Add `v' to end. Implicitly calls `start'
		do
			-- Not implemented; Should be never called
		end

	extend_by_values (minimum, maximum: DOUBLE; num_buckets: INTEGER) is
			-- Create a new rectilinear interval and add to end.
		do
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion

	out: STRING is
			-- Printable representation of identification of `Current'
		local
			fi: INTEGER_FORMATTER
			i: INTEGER
		do
			create Result.make (20)
			create fi.make (4)

			Result.append("Dimension	 Min		  Max		Buckets%N")
			from
				i := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + dimensions
			variant
				i_increasing: 1 + dimensions - i
			until
				i >= dimensions
			loop
				Result.append (fi.formatted (i))
				Result.append ("	")
				Result.append (intervals.item (i).out)

				i := i + 1
			end
		end

feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	d1, d2: DOMAIN_INTERVAL
	intervals: ARRAY [DOMAIN_INTERVAL]
	b1, b2: INTEGER
	domain_pt: POINT2
	tmp_pt: POINT2

	update_item is
			-- Update `domain_pt' to the values specified by `bucket_index'
		do
			domain_pt.put (d1.item (b1), 1)
			domain_pt.put (d2.item (b2), 2)
		end

	make is
			-- Create a default 2-dimensional domain
		do
		end

end -- DOMAIN_2D

