
indexing
	description: "Abstract class describing an interval"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"


deferred class 
	
	DOMAIN_INTERVAL

inherit
	
	ANY
		redefine
			out
		end

feature -- Access
	
	minimum: DOUBLE
			-- The `minimum' of the interval
	
	maximum: DOUBLE
			-- The `maximum' of the interval
	
	buckets: INTEGER
			-- The number of `buckets' in the interval
	
	bucket_width: DOUBLE
			-- The width of each bucket
	
	width: DOUBLE
			-- The width of the interval
	
	interpolation (parameterization: DOUBLE): DOUBLE is
		-- Compute the value at `parameterization' in the range
		-- [`minimum', `maximum']
		require 
			in_range: parameterization >= 0.0 and
					  parameterization <= 1.0
		deferred
		ensure 
			in_range: Result >= minimum and Result <= maximum
		end
	
	bucket_interpolation (point: DOUBLE): INTEGER is
			-- Given a `point' return the bucket it is in
		require 
			good_range: point >= minimum and point <= maximum
		deferred
		ensure 
			good_bucket: Result >= 1 and Result <= buckets
		end

	item (bucket: INTEGER): DOUBLE is
			-- The value at `bucket'
		require 
			in_range: bucket >= 1 and bucket <= buckets
		deferred
		ensure 
			in_range: Result >= minimum and Result <= maximum
		end
	
	bounds (bucket: INTEGER): ARRAY [DOUBLE] is
			-- 
		require 
			in_range: bucket >= 1 and bucket <= buckets
		deferred
		ensure 
			correct_size: Result.count = 2
			in_range: Result @ 1 >= minimum and Result @ 1 <= maximum and
					  Result @ 2 >= minimum and Result @ 2 <= maximum and
					  Result @ 1 < Result @ 2
		end
	
	out: STRING is
			-- 
		do
			create Result.make (20)
			
			Result.append (minimum.out)
			Result.append (" ")
			Result.append (maximum.out)
			Result.append (" ")
			Result.append (buckets.out)
			Result.append ("%N")
		ensure then
			not_void: Result /= Void
		end

end -- DOMAIN_INTERVAL

