

indexing
	description: "Intervals where intermediate values are interpreted linearly"
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"

class 
	
	LINEAR_INTERVAL

inherit
	
	DOMAIN_INTERVAL
	
	BASIC_NUMERIC
		undefine
			out
		end

creation
	make

feature -- Initialization

	make (min, max: DOUBLE; num_buckets: INTEGER) is
			-- Create a new interval
		do
			minimum := min
			maximum := max
			buckets := num_buckets
			
			-- Computed items
			bucket_width := (maximum - minimum)/buckets
			width := maximum - minimum
		ensure
			min_correct: minimum = min
			max_correct: maximum = max
			buckets_correct: buckets = num_buckets
		end

feature -- Access
	
	interpolation (parameterization: DOUBLE) : DOUBLE is
		-- Compute the value at `parameterization' in the range
		-- [`minimum', `maximum']
		do
			Result := minimum + width * parameterization
		end
	
	bucket_interpolation (point: DOUBLE): INTEGER is
			-- Given a `point' return the bucket it is in. Inverse of `item'
		do
			Result := truncate(((point - minimum) / bucket_width).floor) + 1
		end

	item (bucket: INTEGER) : DOUBLE is
			-- The value at `bucket'
		do
			Result := minimum + bucket_width * (bucket-1) + bucket_width/2.0
		end
	
	bounds (bucket: INTEGER): ARRAY [DOUBLE] is
			-- Compute the min and max bounds for `bucket'
		do
			create Result.make (1, 2)
			
			Result.put (item (bucket) - bucket_width/2.0, 1)
			Result.put (item (bucket) + bucket_width/2.0, 2)
		end
	
feature {NONE} -- Implementation

end -- LINEAR_INTERVAL

