
indexing
	description: "Samplers wiil generate points inside of a DOMAIN."
	patterns: "none"
	date: "$Date: 2006-12-06 13:14:08 -0500 (Wed, 06 Dec 2006) $"
	revision: "$Revision: 271 $"
	status: "unreviewed"





deferred class 
	
	SAMPLER
	
inherit
	
	ANY
		redefine
			default_create
		end

	NUMERIC_CONVERSION
		redefine
			default_create
		end
	
feature -- Initialization
	
	default_create is
			-- Initialize
		do
			reporting_interval := 256
		end
	
feature -- Access
	
	max_number_of_samples: INTEGER
			-- The maximum number of samples to aquire
	
	min_number_of_samples: INTEGER
			-- The minimum number of samples to aquire
	
	use_number_of_samples: BOOLEAN
			-- Sample without exceeding `max_number_of_samples'
	
	use_equality_procedure: BOOLEAN
			-- Use `sample_equality' from the SAMPLER_TARGET. Can be used 
			-- in conjunction with `use_number_of_samples'
	
	item: POINT is
			-- Return the element at the cursor
		require
			not_after: not after
		deferred
		ensure
			result_not_void: Result /= Void
			result_correct_dimension: Result.count = domain.dimensions
		end 
	
	reporting_interval: INTEGER
			-- After how many samples should I report?
	
	domain: DOMAIN is
			-- The domain to be sampled
		deferred
		end
	
feature -- Measurement
	
	number_of_samples: INTEGER
			-- How many samples have been generated

feature -- Status report
	
	after: BOOLEAN is
			-- Is the cursor off the end?
		deferred
		end
	
	end_of_reporting_interval: BOOLEAN is
			-- Is this sample at the end of the first dimension in `domain'?
		deferred
		end 
	
	dimensions: INTEGER is
			-- The number of dimension in the sample, i.e., item.count = dimensions
		do
			Result := domain.dimensions
		ensure
			Result_strictly_positive: Result >= 1
		end
	
feature -- Status setting
	
	set_reporting_interval (num_samples: INTEGER) is
			-- Set the `reporting_interval' to `num_samples'
		require 
			num_samples_positive: num_samples >= 1
		do
			reporting_interval := num_samples
		ensure 
			default: true
		end 

	set_number_of_samples (min, max: INTEGER) is
			-- Set the number of samples taken to between 
			-- `min' and `max'. `min' is used only if both 
			-- `use_number_of_samples' and `use_equality_procedure' are 
			-- true. 
		require 
			n_positive: min > 0 and max > 0
			ordered: min <= max
			max_not_exceeded: max >= number_of_samples
		do
			min_number_of_samples := min
			max_number_of_samples := max
		ensure 
			min_samples_set: min_number_of_samples = min
			max_samples_set: max_number_of_samples = max
		end
	
	set_max_samples (n: INTEGER) is
			-- Set the maximum number of samples to use
		require 
			n_positive: n > 0
			max_not_exceeded: n >= number_of_samples
			greater_than_min: n >= min_number_of_samples
		do
			max_number_of_samples := n
		ensure 
			samples_set: max_number_of_samples = n
		end
		
	set_min_samples (n: INTEGER) is
			-- Set the minimum number of samples to use. This is used 
			-- iff both `use_number_of_samples' and `use_equality_procedure' 
			-- are true
		require 
			n_positive: n > 0
			less_than_max: n <= max_number_of_samples
		do
			min_number_of_samples := n
		ensure 
			samples_set: min_number_of_samples = n
		end
		
	sample_with_max_samples (b: BOOLEAN) is
			-- 
		do
			use_number_of_samples := b
		ensure 
			use_samples_set: use_number_of_samples = b
		end

	sample_with_equality (b: BOOLEAN) is
			-- 
		do
			use_equality_procedure := b
		ensure 
			use_equality_set: use_equality_procedure = b
		end
	
	change_domain (new_domain: DOMAIN) is
			-- Change `domain' to `new_domain'
		require 
			valid_domain: new_domain /= Void
		deferred
		ensure 
			domain_set: domain = new_domain
		end

feature -- Cursor movement
	
	start is
			-- Move to the beginning of the elements
		deferred
		end
	
	forth is
			-- Move to the next element
		require
			not_after: not after
		deferred
		end
	
feature -- Basic operations
	
invariant
	positive_sample_limit: (use_number_of_samples implies 
							min_number_of_samples > 0 and 
							max_number_of_samples > 0 and
							min_number_of_samples <= max_number_of_samples)
	good_domain: domain /= Void
	reporting_interval_positive: reporting_interval >= 1
	
end -- SAMPLER

