

indexing
	description: "A 2D-sampler that generate samples by taking a single sample in each strata of a DOMAIN"
	names: "sampler", "stratification", "2d"
	representation: "list"
	access: "index", "cursor"
	size: "resizable"
	contents: "POINT2"
	patterns: "none"
	date: "$Date: 2006-08-07 17:35:02 -0400 (Mon, 07 Aug 2006) $"
	revision: "$Revision: 244 $"
	author: "Mark A Bolstad"
	status: "unreviewed"




class 

	STRATIFIED_SAMPLER_2D

inherit

	STRATIFIED_SAMPLER
		redefine
			item, current_point
		end
creation
	make

feature -- Initialization

	make (d: DOMAIN_2D) is
			-- Create a `SAMPLER' object on the domain `d'
		local
			x, y: INTEGER
		do
			default_create

			x := d.buckets (1)
			y :=  d.buckets (2)

			if sample_points = Void then
				create sample_points.make (1, 2 * x * y)
			else
				-- Usually, the size of the domain won't vary from 
				-- invocation to invocation
				if sample_points.count /= 2 * x * y then
					sample_points.make (1, 2 * x * y)
				end
			end
			
			domain := d
			index := 1
			stride_x := 1.0 / x
			stride_y := 1.0 / y
			use_number_of_samples := true
			number_of_samples := 0

			-- Remove later
			min_number_of_samples := 1
			max_number_of_samples := sample_points.count

			if current_point = Void then
				create current_point
			end

			-- Have to create a new to reset the generator
			create {UNIFORM_RANDOM_GENERATOR} random.make_unit_distribution
		end

feature -- Access
	
	domain: DOMAIN_2D
			-- The domain to be sampled
	
	item: POINT2 is
			-- The current sample point
		do
			current_point.make (sample_points.item (index),
								sample_points.item (index+1))
			Result := domain.interpolation (current_point)
		end 

feature -- Measurement
feature -- Comparison
feature -- Status report
feature -- Status setting
feature -- Cursor movement
feature -- Element change
	
	change_domain (new_domain: DOMAIN_2D) is
			-- Change `domain' to `new_domain'
		do
			make (new_domain)
		end

feature -- Removal
feature -- Resizing
feature -- Transformation
feature -- Conversion
feature -- Duplication
feature -- Basic operations
feature -- Miscellaneous
feature -- Obsolete
feature -- Inapplicable
feature {NONE} -- Implementation

	current_point: POINT2
	stride_x, stride_y: DOUBLE
			-- The distance between samples (parameter space)
	
	generate_uniform_samples is
			-- Generate the initial samples
		local
			i, j, ind: INTEGER
			dx, dy: DOUBLE
		do
			from
				i := 0
				ind := 1
				if not use_jittered_samples then
					dx := 0.5
					dy := 0.5
				end
			invariant
				i_at_least_one: i >= 0
				i_small_enough: i <= 0 + domain.buckets (1)
			variant
				i_increasing: 0 + domain.buckets (1) - i
			until
				i >= domain.buckets (1)
			loop
				from
					j := 0
				invariant
					j_at_least_one: j >= 0
					j_small_enough: j <= 0 + domain.buckets (2)
				variant
					j_increasing: 0 + domain.buckets (2) - j
				until
					j >= domain.buckets (2)
				loop
					if use_jittered_samples then
						dx := random.item
						random.forth
						dy := random.item
						random.forth
					end
					sample_points.put ((i + dx) * stride_x, ind)
					sample_points.put ((j + dy) * stride_y, ind + 1)
					ind := ind + 2
					j := j + 1
				end
				i := i + 1
			end
			
			number_of_samples := sample_points.count // 2
		end 

	generate_hilbert_samples is
			-- Generate the initial samples using a hilbert 
			-- space-filling curve as the access pattern
		local
			next_power_of_two: INTEGER
			ind, pow_of_two: INTEGER
			i, db1, db2: INTEGER
			x1, y1: DOUBLE
		do
			db1 := domain.buckets (1)
			db2 := domain.buckets (1) 
			if db1 > db2 then
				pow_of_two := real_64_truncated_to_integer (log10 (db1 - 1) / log10 (2))
			else
				pow_of_two := real_64_truncated_to_integer (log10 (db2 - 1) / log10 (2))
			end

			next_power_of_two := power_of_two (pow_of_two + 1)
			next_power_of_two := next_power_of_two * next_power_of_two
			
			from
				i := 1
				ind := 1
			invariant
				i_at_least_one: i >= 1
				i_small_enough: i <= 1 + next_power_of_two
			variant
				i_increasing: 1 + next_power_of_two - i
			until
				i >= next_power_of_two
			loop
				x1 := hilbert_coordinates (2, i - 1, 0)
				y1 := hilbert_coordinates (2, i - 1, 1)

				if x1 < db1 and y1 < db2 then
					sample_points.put ((x1 + 0.5)/ db1, ind)
					sample_points.put ((y1 + 0.5)/ db2, ind + 1)
					ind := ind + 2
				end
				i := i + 1
			end
		end 

end -- STRATIFIED_SAMPLER_2D

