

indexing
	description: "Test class for real-time shading language"

class 

	SPLIT_TEST
	
inherit

	GTK_SHARED_MAIN
		export
			{NONE} all
		undefine
			default_create
		end
	
	KL_SHARED_EXCEPTIONS
		export
			{NONE} all
		undefine
			default_create
		end

	GTK_CLICKED_SIGNAL_RECEIVER
		rename
			make as make_gtk_clicked_signal_receiver
		undefine
			default_create
		end

	RIB_PARSER
		rename
			make as rib_parser_make,
			camera as the_camera,
			root_node as rib_root_node
		undefine
			copy
		end

creation
	make

feature -- Initialization

	make is
			-- 
		do
			rib_parser_make
			default_create

			if prepare then
				create micropoly_renderer.make (options)

				micropoly_renderer.set_camera (the_camera)
				micropoly_renderer.set_scene_graph (rib_root_node)

				setup_display
				
				gtk_main.run_main_loop
			end
		end

feature -- Basic operations
	
feature -- Attributes
	
	main_window: GTK_WINDOW
			-- Wrapper for GTK window

feature {NONE} -- Implementation
	
	setup_display is
			-- Initialize the display
		local
			box_widget: GTK_VBOX
		do
			gtk_main.set_locale

			-- Initialize GTK
			gtk_main.initialize_toolkit

			-- If initialization went wrong for some reason,
			-- quit the application
			if not gtk_main.is_toolkit_initialized then
				print ("Unable to init GTK%N")
				Exceptions.die (1)
			end

			make_gtk_clicked_signal_receiver

			-- Create the default DoubleBuffered, Depth, RGB configuration
			create gl_widget.make (micropoly_renderer)
			print (gl_widget.config.out)

			-- Create a GTK window toplevel window
			create window.make_top_level

			-- Create a signal receiver for the 'destroy' signal
			create destroy_event_signal_receiver.make

			-- Connect signal receiver to `window'
			window.connect_destroy_signal_receiver (destroy_event_signal_receiver)
			create key_press_event_signal_receiver.make (gl_widget)
			window.connect_key_press_event_signal_receiver (key_press_event_signal_receiver)

			-- Set the border width of our new window to 2
			window.set_border_width (2)

			create box_widget.make (5)

			-- Add the box as single child to the window
			window.add (box_widget)

			box_widget.pack_start (gl_widget, True, True, 0)

			-- Create a GTK button with a text
			create button.make_with_label (hello_world_string)

			-- Connect the signal receiver to the button
			-- This means that everytime `button' emits
			-- a 'clicked' signal, `clicked_signal_receiver.on_clicked' will be called
			button.connect_clicked_signal_receiver (Current)

			-- Add the GTK button to the GTK window
			box_widget.pack_start (button, False, False, 0)

			-- Show the button
			button.show

			-- Show the GL widget
			gl_widget.show

			box_widget.show

			-- Show the window
			window.show

			gl_widget.set_scene_graph (rib_root_node)
		end

	prepare: BOOLEAN is
			-- Parse RIB files `arguments (1..argument_count)'.
		local
			j, n: INTEGER
			a_filename: STRING
			a_file: KL_TEXT_INPUT_FILE
		do
			Result := true
			n := Arguments.argument_count
			if n = 0 then
				std.error.put_string ("usage: parser_test filename ...%N")
				Exceptions.die (1)
			else
				from
					j := 1
				until
					j > n or not Result
				loop
					a_filename := Arguments.argument (j)
					create a_file.make (a_filename)
					a_file.open_read
					if a_file.is_open_read then
						reset
						set_input_buffer (new_file_buffer (a_file))
						parse
						a_file.close
					else
						std.error.put_string (Arguments.program_name)
						std.error.put_string (": cannot read %'")
						std.error.put_string (a_filename)
						std.error.put_string ("%'%N")
						
						Result := false
					end
					
					j := j + 1
				end
			end
		end
	
feature {CLICKED_SIGNAL_RECEIVER} -- Implementation

	on_clicked (a_button: GTK_BUTTON) is
		do
		end

feature {NONE} -- Implementation
	
	gl_widget: BUCKET_VIEWER
			-- The Viewer
	
	on_destroy (a_gtk_object: GTK_OBJECT) is
		do
			print ("on destroy has been called%N")
			gtk_main.quit_main_loop
		end

	hello_world_string: STRING is "Render"
			-- Text for the GTK button

	window: GTK_WINDOW
			-- Wrapper for GTK window

	button: GTK_BUTTON
			-- Wrapper for GTK button

	clicked_signal_receiver: CLICKED_SIGNAL_RECEIVER
			-- Receiver for 'clicked' signal

	key_press_event_signal_receiver: KEY_PRESS_EVENT_SIGNAL_RECEIVER
			-- Receiver for 'key-press' signal

	destroy_event_signal_receiver: DESTROY_SIGNAL_RECEIVER
			-- Receiver for 'destroy' signal

	gl_drawer: GL

	micropoly_renderer: MICROPOLYGON_RENDERER

end -- SCENE_GRAPH_TEST

